###############################################################################
# All common defines, variables, definitions, and such for everything in the
# GLES 1.1 Conformance tests from Khronos
###############################################################################

#"$(call my-dir)" returns the directory that the Android.mk found by "mm" is in (Android specific)
LOCAL_PATH := $(call my-dir)

include $(CLEAR_VARS)   #Clears all variables that the Android build system looks for

COMMON_LOCAL_C_INCLUDES := 
COMMON_LOCAL_C_INCLUDES += frameworks/base/include
COMMON_LOCAL_CFLAGS := -w -fpermissive


########################################
# Linking and versioning info
########################################

#See 2.0/Android.mk for comments on this section
COMMON_LOCAL_CFLAGS += -DCURRENT_CONFORMANCE_VERSION="\"$(shell cd $(LOCAL_PATH); git show --pretty=oneline --abbrev-commit HEAD | grep '^[0-9a-f]\{7\}' | sed 's/[^0-9a-f]* .*$$//')\""   #Works

#NO_DISPLAY := 1

ifdef NO_DISPLAY
	link_directly_to_HW_libs := a non-empty string
	COMMON_LOCAL_CFLAGS += -DNO_DISPLAY
endif

ifdef link_directly_to_HW_libs
    COMMON_LOCAL_CFLAGS += -DOS_ANDROID__LINK_METHOD="\"linked directly to HW driver\""
else
    COMMON_LOCAL_CFLAGS += -DOS_ANDROID__LINK_METHOD="\"normal\""
endif

COMMON_LOCAL_CFLAGS += -DOS_ANDROID__HW_TARGET="\"$(TARGET_PRODUCT)\""

ifdef link_directly_to_HW_libs 
    #Works (directly linked with HW libs):
    #   See 2.0/Android.mk for more comments and notes about this
    LOCAL_SHARED_LIBRARIES__executables := \
        libGLESv1_CM_adreno \
        libEGL_adreno
else
    #Works (links with SW lib which interfaces with HW lib)
    LOCAL_SHARED_LIBRARIES__executables := \
        libGLESv1_CM \
        libEGL
endif
#LOCAL_SHARED_LIBRARIES__executables += libui


########################################
# Things absolutely required to make conformance work
########################################

COMMON_LOCAL_CFLAGS += -DOS_ANDROID
COMMON_LOCAL_CFLAGS += -DATI_COMP_VC -DOGLES_USE_COMMON_PROFILE -D_CRT_SECURE_NO_WARNINGS -DGL_OES_VERSION_1_1 -DTK_DISPATCH_OES_query_matrix -D_UNICODE
COMMON_LOCAL_CFLAGS += -DUSE_PBUFFER_SURFACE
COMMON_LOCAL_CFLAGS += -DGL_GLEXT_PROTOTYPES

#COMMON_LOCAL_C_INCLUDES += $(LOCAL_PATH)
COMMON_LOCAL_C_INCLUDES += $(LOCAL_PATH)/include


###############################################################################
# Main app
###############################################################################

########################################
# conformshell_cl
# &
# conformshell_cm
########################################

LOCAL_SRC_FILES__conformshell_cx := \
    conform/conformshell/driver.c \
    conform/conformshell/path.c \
    conform/conformshell/pathdata.c \
    conform/conformshell/paths.c \
    conform/conformshell/shell.c \
    conform/conformshell/util.c \
    conform/conformshell/utilg.c \
    conform/conformshell/utilm.c \
    conform/conformshell/utilp.c \
    conform/conformshell/utilru.c \
    conform/conformshell/utils.c \
    conform/conformshell/utilt.c

LOCAL_C_INCLUDES__conformshell_cx := $(COMMON_LOCAL_C_INCLUDES)
#LOCAL_C_INCLUDES__conformshell_cx += \
#   $(LOCAL_PATH)/conform \
#   $(LOCAL_PATH)/ctk
#LOCAL_C_INCLUDES__conformshell_cx += frameworks/base/include    #Required for standlone app requirements like libui and ui/FramebufferNativeWindow.h


####################
# conformshell_cl
####################

include $(CLEAR_VARS)

LOCAL_CFLAGS := $(COMMON_LOCAL_CFLAGS)
LOCAL_CFLAGS += -DCOMMON_LITE

LOCAL_C_INCLUDES := $(LOCAL_C_INCLUDES__conformshell_cx)

LOCAL_SRC_FILES := $(LOCAL_SRC_FILES__conformshell_cx)

LOCAL_MODULE_TAGS := tests

LOCAL_MODULE := libconformshell_cl

include $(BUILD_STATIC_LIBRARY)


####################
# conformshell_cm
####################

include $(CLEAR_VARS)

LOCAL_CFLAGS := $(COMMON_LOCAL_CFLAGS)
LOCAL_CFLAGS += -DCOMMON

LOCAL_C_INCLUDES := $(LOCAL_C_INCLUDES__conformshell_cx)

LOCAL_SRC_FILES := $(LOCAL_SRC_FILES__conformshell_cx)

LOCAL_MODULE_TAGS := tests

LOCAL_MODULE := libconformshell_cm

include $(BUILD_STATIC_LIBRARY)


########################################
# ctk
########################################

include $(CLEAR_VARS)

LOCAL_CFLAGS := $(COMMON_LOCAL_CFLAGS)
LOCAL_CFLAGS += -DCOMMON   #This is needed, but only to satify platform/platform.h.  The part this satisfies isn't used here so it's fine that we don't have a "_cl" and "_cm" version

LOCAL_C_INCLUDES := $(COMMON_LOCAL_C_INCLUDES)
#LOCAL_C_INCLUDES += \
#   $(LOCAL_PATH)/conform/conformshell

LOCAL_SRC_FILES := \
    ctk/ctke.c \
    ctk/glext.c

LOCAL_MODULE_TAGS := tests

LOCAL_MODULE := libctk

include $(BUILD_STATIC_LIBRARY)


########################################
# fixed
########################################

include $(CLEAR_VARS)

LOCAL_CFLAGS := $(COMMON_LOCAL_CFLAGS)
LOCAL_CFLAGS += -DCOMMON   #This is needed, but only to satify platform/platform.h.  The part this satisfies isn't used here so it's fine that we don't have a "_cl" and "_cm" version

LOCAL_C_INCLUDES := $(COMMON_LOCAL_C_INCLUDES)

LOCAL_SRC_FILES := \
    fixed/fixed.c \
    fixed/float64.c \
    fixed/int32.c \
    fixed/int64.c

LOCAL_MODULE_TAGS := tests

LOCAL_MODULE := libxm

include $(BUILD_STATIC_LIBRARY)


########################################
# platform
########################################

include $(CLEAR_VARS)

LOCAL_CFLAGS := $(COMMON_LOCAL_CFLAGS)

LOCAL_C_INCLUDES := $(COMMON_LOCAL_C_INCLUDES)

LOCAL_SRC_FILES := \
    platform/random.c

LOCAL_MODULE_TAGS := tests

LOCAL_MODULE := libplatform

include $(BUILD_STATIC_LIBRARY)


########################################
# covegl
########################################

include $(CLEAR_VARS)

LOCAL_CFLAGS := $(COMMON_LOCAL_CFLAGS)
LOCAL_CFLAGS += -DCOMMON   #This is needed, but only to satify platform/platform.h.  The part this satisfies isn't used here so it's fine that we don't have a "_cl" and "_cm" version

LOCAL_C_INCLUDES := $(COMMON_LOCAL_C_INCLUDES)
LOCAL_EXPORT_C_INCLUDES := frameworks/native/include/android
#LOCAL_C_INCLUDES += $(LOCAL_PATH)/../ctk

LOCAL_SRC_FILES := \
    conform/covegl/enum.c \
    conform/covegl/shell.c \
    conform/covegl/native_ANDROID_LINUX_CBSP2.0.cpp \
    conform/covegl/test.c

LOCAL_SHARED_LIBRARIES := $(LOCAL_SHARED_LIBRARIES__executables)
LOCAL_SHARED_LIBRARIES += \
                         libcutils \
                         libutils \
                         libbinder \
                         libui \
                         libgui \
                         libandroid
                        
LOCAL_MODULE_TAGS := tests

LOCAL_MODULE := covegl

include $(BUILD_EXECUTABLE)


########################################
# covgl_cl
# &
# covgl_cm
########################################

LOCAL_SRC_FILES__covgl_cx := \
    conform/covgl/a.c \
    conform/covgl/b.c \
    conform/covgl/c.c \
    conform/covgl/d.c \
    conform/covgl/e.c \
    conform/covgl/f.c \
    conform/covgl/g.c \
    conform/covgl/h.c \
    conform/covgl/i.c \
    conform/covgl/l.c \
    conform/covgl/m.c \
    conform/covgl/n.c \
    conform/covgl/o.c \
    conform/covgl/p.c \
    conform/covgl/q.c \
    conform/covgl/r.c \
    conform/covgl/s.c \
    conform/covgl/t.c \
    conform/covgl/v.c \
    conform/covgl/w.c \
    conform/covgl/data.c \
    conform/covgl/enum.c \
    conform/covgl/shell.c

LOCAL_C_INCLUDES__covgl_cx := $(COMMON_LOCAL_C_INCLUDES)
#LOCAL_C_INCLUDES__covgl_cx += \
#   $(LOCAL_PATH)/ctk

LOCAL_STATIC_LIBRARIES__covgl_cx := \
    libctk \
    libxm


####################
# covgl_cl
####################

include $(CLEAR_VARS)

LOCAL_CFLAGS := $(COMMON_LOCAL_CFLAGS)
LOCAL_CFLAGS += -DCOMMON_LITE

LOCAL_C_INCLUDES := $(LOCAL_C_INCLUDES__covgl_cx)

LOCAL_SRC_FILES := $(LOCAL_SRC_FILES__covgl_cx)

LOCAL_STATIC_LIBRARIES := $(LOCAL_STATIC_LIBRARIES__covgl_cx)

LOCAL_SHARED_LIBRARIES := $(LOCAL_SHARED_LIBRARIES__executables)

LOCAL_MODULE_TAGS := tests

LOCAL_MODULE := covgl_cl

include $(BUILD_EXECUTABLE)


####################
# covgl_cm
####################

include $(CLEAR_VARS)

LOCAL_CFLAGS := $(COMMON_LOCAL_CFLAGS)
LOCAL_CFLAGS += -DCOMMON



LOCAL_C_INCLUDES := $(LOCAL_C_INCLUDES__covgl_cx)

LOCAL_SRC_FILES := $(LOCAL_SRC_FILES__covgl_cx)

LOCAL_STATIC_LIBRARIES := $(LOCAL_STATIC_LIBRARIES__covgl_cx)

LOCAL_SHARED_LIBRARIES := $(LOCAL_SHARED_LIBRARIES__executables)

LOCAL_MODULE_TAGS := tests

LOCAL_MODULE := covgl_cm

include $(BUILD_EXECUTABLE)


########################################
# primtest_cl
# &
# primtest_cm
########################################

LOCAL_SRC_FILES__primtest_cx := \
    conform/primtest/alias.c \
    conform/primtest/alpha.c \
    conform/primtest/blend.c \
    conform/primtest/depth.c \
    conform/primtest/dither.c \
    conform/primtest/driver.c \
    conform/primtest/fog.c \
    conform/primtest/hint.c \
    conform/primtest/light.c \
    conform/primtest/logic.c \
    conform/primtest/prim.c \
    conform/primtest/scissor.c \
    conform/primtest/shade.c \
    conform/primtest/shell.c \
    conform/primtest/stencil.c \
    conform/primtest/texture.c

LOCAL_C_INCLUDES__primtest_cx := $(COMMON_LOCAL_C_INCLUDES)
#LOCAL_C_INCLUDES__primtest_cx += \
#   $(LOCAL_PATH)/ctk

LOCAL_STATIC_LIBRARIES__primtest_cx := \
    libctk \
    libxm


####################
# primtest_cl
####################

include $(CLEAR_VARS)

LOCAL_CFLAGS := $(COMMON_LOCAL_CFLAGS)
LOCAL_CFLAGS += -DCOMMON_LITE

LOCAL_C_INCLUDES := $(LOCAL_C_INCLUDES__primtest_cx)

LOCAL_SRC_FILES := $(LOCAL_SRC_FILES__primtest_cx)

LOCAL_STATIC_LIBRARIES := $(LOCAL_STATIC_LIBRARIES__primtest_cx)

LOCAL_SHARED_LIBRARIES := $(LOCAL_SHARED_LIBRARIES__executables)

LOCAL_MODULE_TAGS := tests

LOCAL_MODULE := primtest_cl

include $(BUILD_EXECUTABLE)


####################
# primtest_cm
####################

include $(CLEAR_VARS)

LOCAL_CFLAGS := $(COMMON_LOCAL_CFLAGS)
LOCAL_CFLAGS += -DCOMMON

LOCAL_C_INCLUDES := $(LOCAL_C_INCLUDES__primtest_cx)

LOCAL_SRC_FILES := $(LOCAL_SRC_FILES__primtest_cx)

LOCAL_STATIC_LIBRARIES := $(LOCAL_STATIC_LIBRARIES__primtest_cx)

LOCAL_SHARED_LIBRARIES := $(LOCAL_SHARED_LIBRARIES__executables)

LOCAL_MODULE_TAGS := tests

LOCAL_MODULE := primtest_cm

include $(BUILD_EXECUTABLE)


########################################
# conform_cl
# &
# conform_cm
########################################

LOCAL_SRC_FILES__conform_cx := \
    conform/conform/apfunc.c \
    conform/conform/appl.c \
    conform/conform/bclear.c \
    conform/conform/bcolor.c \
    conform/conform/bcorner.c \
    conform/conform/blend.c \
    conform/conform/bufobj.c \
    conform/conform/clip.c \
    conform/conform/colramp.c \
    conform/conform/copytex.c \
    conform/conform/dither.c \
    conform/conform/divzero.c \
    conform/conform/drawtex.c \
    conform/conform/fogexp.c \
    conform/conform/foglin.c \
    conform/conform/gets.c \
    conform/conform/lineaa.c \
    conform/conform/linehv.c \
    conform/conform/linerast.c \
    conform/conform/logicop.c \
    conform/conform/l_ac.c \
    conform/conform/l_al.c \
    conform/conform/l_am.c \
    conform/conform/l_ap.c \
    conform/conform/l_as.c \
    conform/conform/l_dl.c \
    conform/conform/l_dm.c \
    conform/conform/l_dmn.c \
    conform/conform/l_dmp.c \
    conform/conform/l_em.c \
    conform/conform/l_se.c \
    conform/conform/l_sed.c \
    conform/conform/l_sen.c \
    conform/conform/l_sep.c \
    conform/conform/l_sl.c \
    conform/conform/l_sm.c \
    conform/conform/l_sn.c \
    conform/conform/l_sp.c \
    conform/conform/mask.c \
    conform/conform/mget.c \
    conform/conform/mipgen.c \
    conform/conform/miplevels.c \
    conform/conform/miplin.c \
    conform/conform/mipsel.c \
    conform/conform/mpalette.c \
    conform/conform/mquery.c \
    conform/conform/mstack.c \
    conform/conform/multitex.c \
    conform/conform/mustpass.c \
    conform/conform/packpix.c \
    conform/conform/pntaa.c \
    conform/conform/pntrast.c \
    conform/conform/pntsprt.c \
    conform/conform/pntszary.c \
    conform/conform/polycull.c \
    conform/conform/readfmt.c \
    conform/conform/rescalen.c \
    conform/conform/scissor.c \
    conform/conform/spclear.c \
    conform/conform/spcorner.c \
    conform/conform/spfunc.c \
    conform/conform/spop.c \
    conform/conform/tests.c \
    conform/conform/texcombine.c \
    conform/conform/texdecal.c \
    conform/conform/texedge.c \
    conform/conform/texpalet.c \
    conform/conform/trirast.c \
    conform/conform/tritile.c \
    conform/conform/userclip.c \
    conform/conform/vorder.c \
    conform/conform/vpclamp.c \
    conform/conform/xform.c \
    conform/conform/xformmix.c \
    conform/conform/xformn.c \
    conform/conform/xformvp.c \
    conform/conform/xformw.c \
    conform/conform/zbclear.c \
    conform/conform/zbfunc.c

LOCAL_C_INCLUDES__conform_cx := $(COMMON_LOCAL_C_INCLUDES)
#LOCAL_C_INCLUDES__conform_cx += \
#   $(LOCAL_PATH)/ctk \
#   $(LOCAL_PATH)/conform \
#   $(LOCAL_PATH)/conform/conformshell
LOCAL_C_INCLUDES__conform_cx += \
    $(LOCAL_PATH)/conform/conformshell

#LOCAL_STATIC_LIBRARIES__conform_cx := \
#   libplatform \
#   libxm \
#   libctk


####################
# conform_cl
####################

include $(CLEAR_VARS)

LOCAL_CFLAGS := $(COMMON_LOCAL_CFLAGS)
LOCAL_CFLAGS += -DCOMMON_LITE

LOCAL_C_INCLUDES := $(LOCAL_C_INCLUDES__conform_cx)

LOCAL_SRC_FILES := $(LOCAL_SRC_FILES__conform_cx)

#LOCAL_STATIC_LIBRARIES := $(LOCAL_STATIC_LIBRARIES__conform_cx)
#LOCAL_STATIC_LIBRARIES += libconformshell_cl  #Apparently libconformshell_c* this needs to be included in the proper order
LOCAL_STATIC_LIBRARIES := \
    libplatform \
    libconformshell_cl \
    libxm \
    libctk

LOCAL_SHARED_LIBRARIES := $(LOCAL_SHARED_LIBRARIES__executables)

LOCAL_MODULE_TAGS := tests

LOCAL_MODULE := conform_cl

include $(BUILD_EXECUTABLE)


####################
# conform_cm
####################

include $(CLEAR_VARS)

LOCAL_CFLAGS := $(COMMON_LOCAL_CFLAGS)
LOCAL_CFLAGS += -DCOMMON

LOCAL_C_INCLUDES := $(LOCAL_C_INCLUDES__conform_cx)

LOCAL_SRC_FILES := $(LOCAL_SRC_FILES__conform_cx)

#LOCAL_STATIC_LIBRARIES := $(LOCAL_STATIC_LIBRARIES__conform_cx)
#LOCAL_STATIC_LIBRARIES += libconformshell_cm  #Apparently libconformshell_c* this needs to be included in the proper order
LOCAL_STATIC_LIBRARIES := \
    libplatform \
    libconformshell_cm \
    libxm \
    libctk

LOCAL_SHARED_LIBRARIES := $(LOCAL_SHARED_LIBRARIES__executables)

LOCAL_MODULE_TAGS := tests

LOCAL_MODULE := conform_cm

include $(BUILD_EXECUTABLE)
