/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
*/

#include "native.h"

#ifdef OS_LE__LINUX_CBSP_2_0
extern void* updater_get_native_window(void);
extern void* updater_destroy_native_window(void *);
#endif
#ifdef OS_ANDROID
#define GL_TEXTURE_EXTERNAL_OES 0x8D65

#include <binder/IMemory.h>
#include <gui/ISurfaceComposer.h>
#include <gui/Surface.h>
#include <gui/SurfaceComposerClient.h>
#include <utils/String8.h>

#include <private/gui/ComposerService.h>
#include <binder/ProcessState.h>
#include <android/native_window.h>

using namespace android;
#endif

NativeDisplay nativeCreateDisplay(void) {
    return (NativeDisplay)0;
}

void nativeDestroyDisplay(NativeDisplay display) {
    //Empty
}

static sp<ANativeWindow> ANWindow;
static sp<SurfaceComposerClient> ComposerClient;
static sp<SurfaceControl> Control;


NativeWindow nativeCreateWindow(const char *title, int width, int height, EGLint visualId) {
    NativeWindowType win = NULL;
#ifdef OS_ANDROID && !defined(NO_DISPLAY)
    ComposerClient = new SurfaceComposerClient;
    Control = ComposerClient->createSurface(
            String8("Gles Conformance 1.0"), width, height, PIXEL_FORMAT_RGBA_8888, 0);

    SurfaceComposerClient::openGlobalTransaction();
    Control->setLayer(0x7fffffff);
    Control->show();
    SurfaceComposerClient::closeGlobalTransaction();

    ANWindow = Control->getSurface();
    win = ANWindow.get();
    ANativeWindow_setBuffersGeometry(win, width, height,  WINDOW_FORMAT_RGBA_8888);
#else
#ifdef OS_LE__LINUX_CBSP_2_0
    win = updater_get_native_window();
#elif OS_WEBOS
    win = NULL;
#endif
#endif
    return win;
}

void nativeDestroyWindow(NativeWindow window) {
#ifdef OS_ANDROID
	ANWindow = NULL;
	ComposerClient = NULL;
	Control = NULL;
#endif
#ifdef OS_LE__LINUX_CBSP_2_0
    updater_destroy_native_window(window);
#endif
}

NativePixmap nativeCreatePixmap(int width, int height, EGLint visualId) {
    return (NativePixmap)0;
}

void nativeDestroyPixmap(NativePixmap pixmap) {
    //Empty
}
