/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /export/home/cvs/conformes/conformes_patch/linux_nws_patch.txt,v 1.2 2005/07/27 10:23:23 mah Exp $
*/

#include <GLES/egl.h>
#include "ctk.h"

#if defined(__cplusplus) || defined(c_plusplus)
#define class c_class
#endif

/******************************************************************************/

static EGLDisplay egl_display = EGL_NO_DISPLAY;
static EGLSurface egl_surface = EGL_NO_SURFACE;
static EGLContext context = EGL_NO_CONTEXT;
static EGLConfig config;
static int majorVersion, minorVersion;

static TK_EventRec event;

/******************************************************************************/

static EGLConfig FindConfig(long type)
{
    EGLConfig config;
    EGLint num_config;
    EGLint list[32];
    int i;

    i = 0;

    list[i++] = EGL_LEVEL;
    list[i++] = 0;

    list[i++] = EGL_RED_SIZE;
    list[i++] = 1;

    list[i++] = EGL_DEPTH_SIZE;
    list[i++] = 12;

    if (type & TK_WIND_STENCIL) {
    list[i++] = EGL_STENCIL_SIZE;
    list[i++] = 1;
    }

    list[i++] = EGL_NONE;

    if (!eglChooseConfig(egl_display, list, &config, 1, &num_config) || num_config != 1) {
    /* XXX what is a null config? */
    return 0;
    }

    return config;
}

static long MakeConfigType(EGLConfig config)
{
    int stencilSize, depthSize;
    long mask;

    mask = 0;

    eglGetConfigAttrib(egl_display, config, EGL_STENCIL_SIZE, &stencilSize);
    if (stencilSize > 0) {
    mask |= TK_WIND_STENCIL;
    }

    eglGetConfigAttrib(egl_display, config, EGL_DEPTH_SIZE, &depthSize);
    if (depthSize > 0) {
    mask |= TK_WIND_Z;
    }

    return mask;
}

static long NoOpExecFunc( TK_EventRec *pEvent )
{
    pEvent = pEvent;
    return(1);
}

static long (*ExecFunc)(TK_EventRec *pEvent ) = NoOpExecFunc;


long tkNewWindow(TK_WindowRec *wind)
{
    egl_display = eglGetDisplay(0);

    if (!eglInitialize(egl_display, &majorVersion, &minorVersion))
    {
        FPRINT(stderr, "eglInitialize failed!\n");
        return 0;
    }

    if (wind->type == TK_WIND_REQUEST)
    {
        config = FindConfig(wind->info);
        if (config == 0)
        {
            return 0;
        }
    }
    else
    {
        GLint attrib_list[3];
        EGLint count;

        attrib_list[0] = EGL_CONFIG_ID;
        attrib_list[1] = wind->info;
        attrib_list[2] = EGL_NONE;

       if (!eglChooseConfig(egl_display, attrib_list, &config, 1, &count) || count != 1)
        {
            return 0;
        }
    }

    wind->type = TK_WIND_REQUEST;
    wind->info = MakeConfigType(config);

    egl_surface = eglCreateWindowSurface(egl_display, config, 0, NULL);
    if (egl_surface == EGL_NO_SURFACE)
    {
#if 0
        FPRINT(stderr, "Can't create EGL window surface\n");
        return 0;
#else   /* Add create pBuffer Surface. */
        GLint attrib_list[5];
        EGLint count;

        attrib_list[0] = EGL_WIDTH;
        attrib_list[1] = wind->width;
        attrib_list[2] = EGL_HEIGHT;
        attrib_list[3] = wind->height;
        attrib_list[4] = EGL_NONE;

        egl_surface = eglCreatePbufferSurface(egl_display, config, attrib_list);
        if(egl_surface == EGL_NO_SURFACE)
        {
            FPRINT(stderr, "Can't create EGL window/pbuffer surface\n");
            return 0;
        }
#endif
    }

    context = eglCreateContext(egl_display, config, EGL_NO_CONTEXT, NULL);
    if (!context)
    {
        FPRINT(stderr, "Can't create a context!\n");
        return 0;
    }

    if (!eglMakeCurrent(egl_display, egl_surface, egl_surface, context))
    {
        FPRINT(stderr, "Can't make context current\n");
        return 0;
    }

	tkInitDispatch();

    return 1;
}

void tkCloseWindow(void)
{
    eglMakeCurrent(egl_display, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
    eglDestroyContext(egl_display, context);
    eglDestroySurface (egl_display, egl_surface);
}

void tkSwapBuffers(void)
{
    eglMakeCurrent(egl_display, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
    eglDestroyContext(egl_display, context);
    eglDestroySurface (egl_display, egl_surface);
    eglSwapBuffers(egl_display, egl_surface);
}

void tkQuit(void)
{
    eglMakeCurrent(egl_display, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
    eglTerminate(egl_display);
}

void tkExec( long (*Func)(TK_EventRec *pEvent) )
{
    if ( NULL != (void *)Func )
    {
        ExecFunc = Func;        /* save a pointer to the drawing function */

        event.event = TK_EVENT_EXPOSE;
        event.data[TK_WINDOWX] = 0;
        event.data[TK_WINDOWY] = 0;

        /* this will not return until finished */
        (*ExecFunc)(&event);
    }
    else
    {
        FPRINT(stderr, "NULL tkExec function!\n");
    }
}

void tkGetConfigs(TK_ConfigsRec *ptr)
{
    GLint num_config;
    EGLConfig *configs;
    EGLint configId, caveatValue;
    int i;

    ptr->count = 0;
    ptr->nonConformantCount = 0;

    egl_display = eglGetDisplay(0);

    eglInitialize(egl_display, &majorVersion, &minorVersion);

    eglGetConfigs(egl_display, NULL, 0, &num_config);
    configs = (EGLConfig *)MALLOC(num_config * sizeof(EGLConfig));
    eglGetConfigs(egl_display, configs, num_config, &num_config);

    for (i = 0; i < num_config; i++)
    {
        eglGetConfigAttrib(egl_display, configs[i], EGL_CONFIG_ID, &configId);
        eglGetConfigAttrib(egl_display, configs[i], EGL_CONFIG_CAVEAT, &caveatValue);

        if (caveatValue == EGL_NON_CONFORMANT_CONFIG)
        {
            ptr->nonConformantIDs[ptr->nonConformantCount++] = configId;
        }
        else
        {
            ptr->IDs[ptr->count++] = configId;
        }
    }
    FREE(configs);
    eglTerminate(egl_display);
}
