# extract definitions from glspec tex files.  call as
#
#	awk -f glc.awk gl.h *.tex
#
# to create a nice list of constants in the spec that are not in gl.h
#

BEGIN {
	depth = 0;
	count = 0;
}

#
# look for definitions in gl.h
#
$1 == "#define" {
	token = substr($2,4);
	IsDefined[token] = 1;
	continue;
}

#
# ignore comments in tex source
#
substr($1,1,1) == "%" {
	continue;
}

#
# extract the string in the fifth curly-bracket pair following a \doentry line
# use the depth counter to ignore nested bracket pairs
# since the fifth pair might be on a later line, keep looking if count > 0
#
(count > 0) || (substr($1,2,7) == "doentry") {
    for (i=1; i <= length($0); i++) {
	if (substr($0,i,1) == "{") {
	    depth += 1;
	    if (depth == 1) count += 1;
	    if (count == 5) {
		newtoken = "";
		for (j = i+1; 1; j++) {
		    c = substr($0,j,1);
		    if (c == "}") break;
		    if (c != "\\") {
			newtoken = sprintf("%s%s",newtoken,c);
		    }
		}
#		printf("LISTTOKEN = %s\n",newtoken);
		IsSpecified[newtoken] = 1;
		if (!IsDefined[newtoken]) {
		    printf("not in gl.h: ");
		    printf("%-20s ",FILENAME);
		    printf("%s\n",newtoken);
		}
		depth = 0;
		count = 0;
		next;
	    }
	}
	if (substr($0,i,1) == "}") {
	    depth -= 1;
	}
    }
}

#
# check each field for a glc macro, extract the string when found
#
	{
	for (i = 1; i <= NF; i++) {
	    if (index($i,"glc") == 2) {
		newtoken = "";
		start = index($i,"{") + 1;
		stop = index($i,"}") - start;
		token = substr($i,start,stop);
		for (j = 1; j <= length(token); j++) {
		    c = substr(token,j,1);
		    if (c != "\\") {
			newtoken = sprintf("%s%s",newtoken,c);
		    }
		}
#		printf("NEWTOKEN = %s\n",newtoken);
		IsSpecified[newtoken] = 1;
		if (!IsDefined[newtoken]) {
		    printf("not in gl.h: ");
		    printf("%-20s ",FILENAME);
		    printf("%s\n",newtoken);
		}
	    }
	}
}

END {
	for (s in IsDefined) {
	    if (!IsSpecified[s]) {
		printf("not in the spec: %s\n",s);
	    }
	}
}
