#!/usr/contrib/bin/tcl7.6/tcl
# pixformat - generate LaTeX2e source for a packed pixel format
#   figure

# width is the total figure width, in cm
# fieldsize is an array of bit widths, from left to right
# reverse is non-zero if the field order is reversed
proc pixformat { width fieldsize { reverse 0 } { caption "" } {label ""} } {
    # Tags for each field
    #set fieldtag [list First Second Third Fourth Fifth]
    set fieldtag [list {1st Component} 2nd 3rd 4th]

    # Number of fields
    set num [llength $fieldsize]

    # Sum up field widths to get total # bits
    set bits 0
    foreach elem $fieldsize {
        incr bits $elem
    }
    puts "% $bits bits in the packed pixel format"

    # Figure out the OpenGL enum corresponding to this format,
    #   based on the total width
    switch $bits {
        32 { set type INT   }
        16 { set type SHORT }
        8  { set type BYTE  }
        default { puts stderr "Unknown type for bits $bits" ; set type OOPS }
    }

    set tag "UNSIGNED\\_${type}"
    foreach elem $fieldsize {
        set tag "${tag}\\_$elem"
    }
    if { $reverse } {
        set tag "${tag}\\_REV"
    }

    puts "\\glc\{$tag\}:"

    # Width assigned to each bitfield
    set fieldwidth [expr double($width) / $bits]

    puts "\\begin{center}"
    puts "\\begin{tabular}\{*\{$bits\}\{@{}p\{[expr $fieldwidth]cm\}@{}\}\}"

    # Put out the bitfield labels
    for { set i [expr $bits - 1] } { $i >= 0 } { incr i -1 } {
        puts -nonewline "\{\\hspace\{[expr $fieldwidth / 4]cm\}\\tiny \\tt $i\}"
        if { $i == 0 } {
            puts "\\\\ \\hline"
        } else {
            puts -nonewline " & "
        }
    }

    # Put out the fields
    set format "|c|"
    for { set i 0  } { $i < $num } { incr i } {
        set fieldbits [lindex $fieldsize $i]
        if { $reverse } {
            set fieldlabel [lindex $fieldtag [expr $num - 1 - $i]]
        } else {
            set fieldlabel [lindex $fieldtag $i]
        }

        puts -nonewline "\\multicolumn\{$fieldbits\}\{$format\}"
        puts -nonewline "\{\{\\tiny\\hspace{0pt}\\textsf\{$fieldlabel\}\}\}"
        if { $i < $num - 1 } {
            puts " &"
        } else {
            puts " \\\\"
        }

        set format "c|" ;# This format is used for all fields except the first
    }

    puts "\\hline"
    puts "\\end{tabular}"

    if { [string length $caption] > 0 } {
        puts "\\caption\[$caption\]\{$caption\}"
        puts "\\label\{$label\}"
    } else {
        puts "\\vspace\{0.5cm\}"
    }

    puts "\\end{center}"
    puts ""
}

puts "\\begin{table}\[h\]"
    pixformat 12 { 5 6 5 }
    pixformat 12 { 4 4 4 4 }
    pixformat 12 { 5 5 5 1 }      0  "\\glc{UNSIGNED\\_SHORT} formats" "pix:packformat:short"
puts "\\end{table}"
