/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/apfunc.c,v 1.4 2005/05/11 15:21:39 artlab Exp $
*/

/*
** apfunc.c
** Alpha Plane Function Test.
**
** Description -
**    Three points are drawn at location (0.0, 0.0), (1.0, 0.0)
**    and (2.0, 0.0). The first point has a random alpha value
**    between 0.0 and 0.4, the second point has an alpha value of
**    0.5 and the third point has a random alpha value between
**    0.6 and 1.0. Each alpha function is tested by setting the
**    alpha reference value to 0.5, drawing the three points and
**    comparing the rendered result to the expected result.
**
** Technical Specification -
**    Buffer requirements:
**        Color buffer, alpha plane.
**    Color requirements:
**        Full color.
**    States requirements:
**        Disabled = GL_DITHER, GL_MULTISAMPLE.
**    Error epsilon:
**        Zero epsilon.
**    Paths:
**        Allowed = Blend, Depth, Fog, Logicop, Shade, Stencil.
**        Not allowed = Alias, Alpha, Dither.
*/

#include "conform.h"
#include "util.h"


static long errPattern[3];
static GLftype errAlphas[3];
static char errStr[320];
static char errStrFmt[] = "The behavior of the alpha function %s is not correct. The alpha value of the test points are %1.1f, %1.1f, %1.1f. The reference alpha value is 0.5. The test points pattern is %s, %s, %s, should be %s.\n";


static void ClearDrawRead(GLenum func, GLftype *buf)
{
    GLftype vertices[3][2] = {
	{ITOF(1)/2, ITOF(1)/2},
	{ITOF(3)/2, ITOF(1)/2},
	{ITOF(5)/2, ITOF(1)/2}
    };
    GLftype colors[3][4] = {
        {ITOF(1), ITOF(1), ITOF(1), ITOF(0)},
        {ITOF(1), ITOF(1), ITOF(1), ITOF(0)},
        {ITOF(1), ITOF(1), ITOF(1), ITOF(0)}
    };

    colors[0][3] = errAlphas[0] = OESRandom(ITOF(0), ATOF("0.4"));
    colors[1][3] = errAlphas[1] = ATOF("0.5");
    colors[2][3] = errAlphas[2] = OESRandom(ATOF("0.6"), ITOF(1));

    glClear(GL_COLOR_BUFFER_BIT);

    glAlphaFunct(func, ATOF("0.5"));

    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(2, GL_FTYPE, 0, &vertices[0][0]);

    glEnableClientState(GL_COLOR_ARRAY);
    glColorPointer(4, GL_FTYPE, 0, &colors[0][0]);

    glDrawArrays(GL_POINTS, 0, 3);

    glDisableClientState(GL_COLOR_ARRAY);
    glDisableClientState(GL_VERTEX_ARRAY);

    ReadScreen(0, 0, 3, 1, GL_RGB, buf);
}

static long Test(long a, long b, long c, GLftype *buf)
{
    long i;

    for (i = 0; i < 3; i++) {
	if (buf[i*3] < epsilon.zero &&
	    buf[i*3+1] < epsilon.zero &&
	    buf[i*3+2] < epsilon.zero) {     /* no color. */
	    errPattern[i] = GL_FALSE;
	} else {                             /* color. */
	    errPattern[i] = GL_TRUE;
	}
    }

    if (a == GL_TRUE) {     /* should have color. */
	if (buf[0] < epsilon.zero &&
	    buf[1] < epsilon.zero &&
	    buf[2] < epsilon.zero) {     /* no color. */
	    return ERROR;
	}
    } else {             /* should not have color. */
	if (buf[0] > epsilon.zero &&
	    buf[1] > epsilon.zero &&
	    buf[2] > epsilon.zero) {     /* color. */
	    return ERROR;
	}
    }

    if (b == GL_TRUE) {     /* should have color. */
	if (buf[3] < epsilon.zero &&
	    buf[4] < epsilon.zero &&
	    buf[5] < epsilon.zero) {     /* no color. */
	    return ERROR;
	}
    } else {             /* should not have color. */
	if (buf[3] > epsilon.zero &&
	    buf[4] > epsilon.zero &&
	    buf[5] > epsilon.zero) {     /* color. */
	    return ERROR;
	}
    }

    if (c == GL_TRUE) {     /* should have color. */
	if (buf[6] < epsilon.zero &&
	    buf[7] < epsilon.zero &&
	    buf[8] < epsilon.zero) {     /* no color. */
	    return ERROR;
	}
    } else {             /* should not have color. */
	if (buf[6] > epsilon.zero &&
	    buf[7] > epsilon.zero &&
	    buf[8] > epsilon.zero) {     /* color. */
	    return ERROR;
	}
    }

    return NO_ERROR;
}

static void MakeErrorStr(char *funcName, char *result)
{

    StrMake(errStr, errStrFmt, funcName, errAlphas[0], errAlphas[1],
	    errAlphas[2], (errPattern[0] == GL_TRUE) ? "DRAWN" : "NOT-DRAWN",
	    (errPattern[1] == GL_TRUE) ? "DRAWN" : "NOT-DRAWN",
	    (errPattern[2] == GL_TRUE) ? "DRAWN" : "NOT-DRAWN", result);
}

long APFuncExec(void)
{
    GLftype buf[9];

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glDisable(GL_DITHER);
    glDisable(GL_MULTISAMPLE);
    glEnable(GL_ALPHA_TEST);

    ClearDrawRead(GL_ALWAYS, buf);
    if (Test(GL_TRUE, GL_TRUE, GL_TRUE, buf) == ERROR) {
	MakeErrorStr("GL_ALWAYS", "DRAWN, DRAWN, DRAWN");
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }

    ClearDrawRead(GL_NEVER, buf);
    if (Test(GL_FALSE, GL_FALSE, GL_FALSE, buf) == ERROR) {
	MakeErrorStr("GL_NEVER", "NOT-DRAWN, NOT-DRAWN, NOT-DRAWN");
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }

    ClearDrawRead(GL_LESS, buf);
    if (Test(GL_TRUE, GL_FALSE, GL_FALSE, buf) == ERROR) {
	MakeErrorStr("GL_LESS", "DRAWN, NOT-DRAWN, NOT-DRAWN");
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }

    ClearDrawRead(GL_LEQUAL, buf);
    if (Test(GL_TRUE, GL_TRUE, GL_FALSE, buf) == ERROR) {
	MakeErrorStr("GL_LEQUAL", "DRAWN, DRAWN, NOT-DRAWN");
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }

    ClearDrawRead(GL_GREATER, buf);
    if (Test(GL_FALSE, GL_FALSE, GL_TRUE, buf) == ERROR) {
	MakeErrorStr("GL_GREATER", "NOT-DRAWN, NOT-DRAWN, DRAWN");
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }

    ClearDrawRead(GL_GEQUAL, buf);
    if (Test(GL_FALSE, GL_TRUE, GL_TRUE, buf) == ERROR) {
	MakeErrorStr("GL_GEQUAL", "NOT-DRAWN, DRAWN, DRAWN");
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }

    ClearDrawRead(GL_EQUAL, buf);
    if (Test(GL_FALSE, GL_TRUE, GL_FALSE, buf) == ERROR) {
	MakeErrorStr("GL_EQUAL", "NOT-DRAWN, DRAWN, NOT-DRAWN");
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }

    ClearDrawRead(GL_NOTEQUAL, buf);
    if (Test(GL_TRUE, GL_FALSE, GL_TRUE, buf) == ERROR) {
	MakeErrorStr("GL_NOTEQUAL", "DRAWN, NOT-DRAWN, DRAWN");
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }

    return NO_ERROR;
}
