/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/bclear.c,v 1.3 2005/05/11 15:21:40 artlab Exp $
*/

/*
** bclear.c
** Buffer Clear Test.
**
** Description -
**    The color buffer is cleared to both the minimum and
**    maximum color values and verified.
**    The minimum and maximum color values for the color
**    buffer are determined by the number of color bits.
**
** Error Explanation -
**    The color read back from a color buffer must match the
**    clear color (within a zero epsilon).
**
** Technical Specification -
**    Buffer requirements:
**        Color buffer.
**    Color requirements:
**        No color requirements.
**    States requirements:
**        No state restrictions.
**    Error epsilon:
**        Zero epsilons.
**    Paths:
**        Allowed = Alias, Alpha, Blend, Depth, Dither, Fog, Logicop, Shade,
**                  Stencil.
*/

#include "conform.h"
#include "util.h"

static char errStr[240];
static char errStrFmt[160] =
    "Location (%d, %d). Color is (%1.1f, %1.1f, %1.1f, %1.1f), should be (%1.1f, %1.1f, %1.1f, %1.1f).";


static long TestRGB(const GLftype *testColor, GLftype *buf)
{
    GLftype *ptr, expected[4];
    long i, j;

    glClearColort(testColor[0], testColor[1], testColor[2], testColor[3]);
    glClear(GL_COLOR_BUFFER_BIT);

    ReadScreen(0, 0, WINDSIZEX, WINDSIZEY, GL_RGBA, buf);

    expected[0] = buffer.colorBits[0] == 0 ? ITOF(0) : testColor[0];
    expected[1] = buffer.colorBits[1] == 0 ? ITOF(0) : testColor[1];
    expected[2] = buffer.colorBits[2] == 0 ? ITOF(0) : testColor[2];
    expected[3] = buffer.colorBits[3] == 0 ? ITOF(1) : testColor[3];

    ptr = buf;
    for (i = 0; i < WINDSIZEY; i++) {
	for (j = 0; j < WINDSIZEX; j++) {
	    if (ABS(ptr[0]-expected[0]) > epsilon.zero ||
		ABS(ptr[1]-expected[1]) > epsilon.zero ||
		ABS(ptr[2]-expected[2]) > epsilon.zero ||
		ABS(ptr[3]-expected[3]) > epsilon.zero) {
		StrMake(errStr, errStrFmt, j, i,
			ptr[0], ptr[1], ptr[2], ptr[3],
			expected[0], expected[1], expected[2], expected[3]);
		ErrorReport(__FILE__, __LINE__, errStr);
		return ERROR;
	    }
	    ptr += 4;
	}
    }
    return NO_ERROR;
}

static long Test(GLftype *buf)
{
    static const GLftype minRGB[] = {ITOF(0), ITOF(0), ITOF(0), ITOF(0)};
    static const GLftype maxRGB[] = {ITOF(1), ITOF(1), ITOF(1), ITOF(1)};

    if (TestRGB(minRGB, buf)) {
	return ERROR;
    }
    if (TestRGB(maxRGB, buf)) {
	return ERROR;
    }

    return NO_ERROR;
}

long BClearExec(void)
{
    GLftype *buf;

    buf = (GLftype *)MALLOC(WINDSIZEX*WINDSIZEY*4*sizeof(GLftype));

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    if (Test(buf) == ERROR) {
	FREE(buf);
	return ERROR;
    }

    FREE(buf);
    return NO_ERROR;
}
