/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/bcolor.c,v 1.3 2005/05/11 15:21:40 artlab Exp $
*/

/*
** bcolor.c
** Buffer Color Test.
**
** Description -
**    Tests the color stability of the color buffer. Since the color
**    buffer may have limited color capacity, a point drawn with
**    a requested color may render in a different color. However,
**    if this rendered color is then used to draw a second point,
**    the rendered color of the second point must match the
**    rendered color of the first point.
**
** Error Explanation -
**    Failure occurs if the rendered color from a point, when
**    used as the request color for a second point, does not
**    match the rendered color of this second point.
**
** Technical Specification -
**    Buffer requirements:
**        None.
**    Color requirements:
**        Full color.
**    States requirements:
**        Disabled = GL_DITHER, GL_MULTISAMPLE.
**    Error epsilon:
**        Color and zero epsilons.
**    Paths:
**        Allowed = Alpha, Blend, Depth, Fog, Logicop, Shade, Stencil.
**        Not allowed = Alias, Dither.
*/

#include "conform.h"
#include "util.h"
#include "utilru.h"


typedef struct _bColorRec {
    long component;
    GLftype step;
    GLftype color[4];
} bColorRec;


static char errStr[240];
static char errStrFmt[][160] = {
    "Color is (%g, %g, %g, %g). Red, green and blue components should be 0.0.",
    "Color is (%g, %g, %g, %g). Green and blue components should be 0.0.",
    "Color is (%g, %g, %g, %g). Red and blue components should be 0.0.",
    "Color is (%g, %g, %g, %g). Red and green components should be 0.0.",
    "Color is (%g, %g, %g, %g). Alpha component should 0.0.",
    "Color is (%g, %g, %g, %g). Alpha component should 1.0.",
    "Color is (%g, %g, %g, %g). Red component should be 0.0.",
    "Color is (%g, %g, %g, %g). Red component should be 1.0.",
    "Color is (%g, %g, %g, %g). Green component should be 0.0.",
    "Color is (%g, %g, %g, %g). Green component should be 1.0.",
    "Color is (%g, %g, %g, %g). Blue component should be 0.0.",
    "Color is (%g, %g, %g, %g). Blue component should be 1.0.",
    "Color is (%g, %g, %g, %g), should be %g, %g, %g, %g.",
};


/*****************************************************************************/

static void Init(void *data)
{
    bColorRec *ptr = (bColorRec *)data;
    GLftype step;

    step = (machine.pathLevel == 0) ? ITOF(16) : ITOF(2);
    if (buffer.colorBits[ptr->component] > 0) {
        ptr->step = DIV(DIV(ITOF(1),
            LDEXP(ITOF(1), buffer.colorBits[ptr->component]) - ITOF(1)), step);
    } else {
	ptr->step = DIV(ITOF(1), ITOF(7));
    }

    ptr->color[0] = ITOF(0);
    ptr->color[1] = ITOF(0);
    ptr->color[2] = ITOF(0);
    ptr->color[3] = ITOF(0);
}

static void Set(void *data, ftype *ref)
{
    bColorRec *ptr = (bColorRec *)data;

    ptr->color[ptr->component] = *ref;

    glColor4t(ptr->color[0], ptr->color[1], ptr->color[2], ptr->color[3]);
    DrawPoint2f(ITOF(1)/2, ITOF(1)/2);

    *ref += ptr->step;
}

static long Test(void *data, ftype ref)
{
    bColorRec *ptr = (bColorRec *)data;
    GLftype buf1[4], buf2[4];

    ReadScreen(0, 0, 1, 1, GL_RGBA, buf1);

    glColor4t(buf1[0], buf1[1], buf1[2], buf1[3]);
    DrawPoint2f(ITOF(1)/2, ITOF(1)/2);

    ReadScreen(0, 0, 1, 1, GL_RGBA, buf2);

    switch (ptr->component) {
      case 0:
	if (buf2[1] != ITOF(0) || buf2[2] != ITOF(0)) {
	    StrMake(errStr, errStrFmt[1], buf2[0], buf2[1], buf2[2],
		    buf2[3]);
	    ErrorReport(__FILE__, __LINE__, errStr);
	    return ERROR;
	}
	if (buffer.colorBits[3] > 0) {
	    if (buf2[3] != ITOF(0)) {
		StrMake(errStr, errStrFmt[4], buf2[0], buf2[1], buf2[2],
			buf2[3]);
		ErrorReport(__FILE__, __LINE__, errStr);
		return ERROR;
	    }
	} else  {
	    if (ABS(buf2[3]-ITOF(1)) > epsilon.zero) {
		StrMake(errStr, errStrFmt[5], buf2[0], buf2[1], buf2[2],
			buf2[3]);
		ErrorReport(__FILE__, __LINE__, errStr);
		return ERROR;
	    }
	}
	if (ref == ITOF(0)) {
	    if (buf2[0] != ITOF(0)) {
		StrMake(errStr, errStrFmt[6], buf2[0], buf2[1], buf2[2],
			buf2[3]);
		ErrorReport(__FILE__, __LINE__, errStr);
		return ERROR;
	    }
	} else if (ref == ITOF(1)) {
	    if (buffer.colorBits[0] > 0) {
		if (ABS(buf2[0]-ITOF(1)) > epsilon.zero) {
		    StrMake(errStr, errStrFmt[7], buf2[0], buf2[1], buf2[2],
			    buf2[3]);
		    ErrorReport(__FILE__, __LINE__, errStr);
		    return ERROR;
		}
	    } else {
		if (buf2[0] != ITOF(0)) {
		    StrMake(errStr, errStrFmt[6], buf2[0], buf2[1], buf2[2],
			    buf2[3]);
		    ErrorReport(__FILE__, __LINE__, errStr);
		    return ERROR;
		}
	    }
	} else {
	    if (ABS(buf1[0]-buf2[0]) > epsilon.color[0]) {
		StrMake(errStr, errStrFmt[12], buf2[0], buf2[1],
			buf2[2], buf2[3], buf1[0], buf1[1], buf1[2], buf1[3]);
		ErrorReport(__FILE__, __LINE__, errStr);
		return ERROR;
	    }
	}
	break;
      case 1:
	if (buf2[0] != ITOF(0) || buf2[2] != ITOF(0)) {
	    StrMake(errStr, errStrFmt[2], buf2[0], buf2[1], buf2[2],
		    buf2[3]);
	    ErrorReport(__FILE__, __LINE__, errStr);
	    return ERROR;
	}
	if (buffer.colorBits[3] > 0) {
	    if (buf2[3] != ITOF(0)) {
		StrMake(errStr, errStrFmt[4], buf2[0], buf2[1], buf2[2],
			buf2[3]);
		ErrorReport(__FILE__, __LINE__, errStr);
		return ERROR;
	    }
	} else  {
	    if (ABS(buf2[3]-ITOF(1)) > epsilon.zero) {
		StrMake(errStr, errStrFmt[5], buf2[0], buf2[1], buf2[2],
			buf2[3]);
		ErrorReport(__FILE__, __LINE__, errStr);
		return ERROR;
	    }
	}
	if (ref == ITOF(0)) {
	    if (buf2[1] != ITOF(0)) {
		StrMake(errStr, errStrFmt[8], buf2[0], buf2[1], buf2[2],
			buf2[3]);
		ErrorReport(__FILE__, __LINE__, errStr);
		return ERROR;
	    }
	} else if (ref == ITOF(1)) {
	    if (buffer.colorBits[1] > 0) {
		if (ABS(buf2[1]-ITOF(1)) > epsilon.zero) {
		    StrMake(errStr, errStrFmt[9], buf2[0], buf2[1], buf2[2],
			    buf2[3]);
		    ErrorReport(__FILE__, __LINE__, errStr);
		    return ERROR;
		}
	    } else {
		if (buf2[1] != ITOF(0)) {
		    StrMake(errStr, errStrFmt[8], buf2[0], buf2[1], buf2[2],
			    buf2[3]);
		    ErrorReport(__FILE__, __LINE__, errStr);
		    return ERROR;
		}
	    }
	} else {
	    if (ABS(buf1[1]-buf2[1]) > epsilon.color[1]) {
		StrMake(errStr, errStrFmt[12], buf2[0], buf2[1],
			buf2[2], buf2[3], buf1[0], buf1[1], buf1[2], buf1[3]);
		ErrorReport(__FILE__, __LINE__, errStr);
		return ERROR;
	    }
	}
	break;
      case 2:
	if (buf2[0] != ITOF(0) || buf2[1] != ITOF(0)) {
	    StrMake(errStr, errStrFmt[3], buf2[0], buf2[1], buf2[2],
		    buf2[3]);
	    ErrorReport(__FILE__, __LINE__, errStr);
	    return ERROR;
	}
	if (buffer.colorBits[3] > 0) {
	    if (buf2[3] != ITOF(0)) {
		StrMake(errStr, errStrFmt[4], buf2[0], buf2[1], buf2[2],
			buf2[3]);
		ErrorReport(__FILE__, __LINE__, errStr);
		return ERROR;
	    }
	} else  {
	    if (ABS(buf2[3]-ITOF(1)) > epsilon.zero) {
		StrMake(errStr, errStrFmt[5], buf2[0], buf2[1], buf2[2],
		        buf2[3]);
		ErrorReport(__FILE__, __LINE__, errStr);
		return ERROR;
	    }
	}
	if (ref == ITOF(0)) {
	    if (buf2[2] != ITOF(0)) {
		StrMake(errStr, errStrFmt[10], buf2[0], buf2[1],
			buf2[2], buf2[3]);
		ErrorReport(__FILE__, __LINE__, errStr);
		return ERROR;
	    }
	} else if (ref == ITOF(1)) {
	    if (buffer.colorBits[2] > 0) {
		if (ABS(buf2[2]-ITOF(1)) > epsilon.zero) {
		    StrMake(errStr, errStrFmt[11], buf2[0], buf2[1],
			    buf2[2], buf2[3]);
		    ErrorReport(__FILE__, __LINE__, errStr);
		    return ERROR;
		}
	    } else {
		if (buf2[2] != ITOF(0)) {
		    StrMake(errStr, errStrFmt[10], buf2[0], buf2[1],
			    buf2[2], buf2[3]);
		    ErrorReport(__FILE__, __LINE__, errStr);
		    return ERROR;
		}
	    }
	} else {
	    if (ABS(buf1[2]-buf2[2]) > epsilon.color[2]) {
		StrMake(errStr, errStrFmt[12], buf2[0], buf2[1],
			buf2[2], buf2[3], buf1[0], buf1[1], buf1[2], buf1[3]);
		ErrorReport(__FILE__, __LINE__, errStr);
		return ERROR;
	    }
	}
	break;
      case 3:
	if (buf2[0] != ITOF(0) || buf2[1] != ITOF(0) || buf2[2] != ITOF(0)) {
	    StrMake(errStr, errStrFmt[0], buf2[0], buf2[1], buf2[2],
		    buf2[3]);
	    ErrorReport(__FILE__, __LINE__, errStr);
	    return ERROR;
	}
	if (buffer.colorBits[3] > 0) {
	    if (ref == ITOF(0)) {
		if (buf2[3] != ITOF(0)) {
		    StrMake(errStr, errStrFmt[4], buf2[0], buf2[1],
			    buf2[2], buf2[3]);
		    ErrorReport(__FILE__, __LINE__, errStr);
		    return ERROR;
		}
	    } else if (ref == ITOF(1)) {
		if (ABS(buf2[3]-ITOF(1)) > epsilon.zero) {
		    StrMake(errStr, errStrFmt[5], buf2[0], buf2[1],
			    buf2[2], buf2[3]);
		    ErrorReport(__FILE__, __LINE__, errStr);
		    return ERROR;
		}
	    } else {
		if (ABS(buf1[3]-buf2[3]) > epsilon.color[3]) {
		    StrMake(errStr, errStrFmt[12], buf2[0], buf2[1],
			    buf2[2], buf2[3], buf1[0], buf1[1], buf1[2],
			    buf1[3]);
		    ErrorReport(__FILE__, __LINE__, errStr);
		    return ERROR;
		}
	    }
	} else {
	    if (ABS(buf2[3]-ITOF(1)) > epsilon.zero) {
		StrMake(errStr, errStrFmt[5], buf2[0], buf2[1], buf2[2],
			buf2[3]);
		ErrorReport(__FILE__, __LINE__, errStr);
		return ERROR;
	    }
	}
	break;
    }

    return NO_ERROR;
}

long BColorExec(void)
{
    bColorRec data;
    GLint max; 

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(1));
    glDisable(GL_DITHER);
    glDisable(GL_MULTISAMPLE);

    max = (machine.pathLevel == 0) ? 4 : 1;
    for (data.component = 0; data.component < max; data.component++) {
	if (RampUtil(ITOF(0), ITOF(1), Init, Set, Test, &data) == ERROR) {
	    return ERROR;
	}
    }

    return NO_ERROR;
}
