/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/bcorner.c,v 1.3 2005/05/11 15:21:40 artlab Exp $
*/

/*
** bcorner.c
** Buffer Corner Test.
**
** Description -
**    A point is drawn at each corner. The color buffer is read
**    and examined to verify that the corner points are at the
**    correct location.
**
** Error Explanation -
**    Failure occurs if any of the corner points cannot be read
**    correctly (implying that the color buffer's width and
**    height or the rendering coordinates are not correct).
**
** Technical Specification -
**    Buffer requirements:
**        Color buffer.
**    Color requirements:
**        2 colors.
**    States requirements:
**        Disabled = GL_DITHER, GL_MULTISAMPLE.
**    Error epsilon:
**        Color epsilon.
**    Paths:
**        Allowed = Alpha, Blend, Depth, Fog, Logicop, Shade, Stencil.
**        Not allowed = Alias, Dither.
*/

#include "conform.h"
#include "util.h"


static char errStr[160];
static char errStrFmt[] = "Mismatch at the %s corner in the buffer.";


static long Test(GLftype *buf)
{
    static const GLftype vertices[4][2] = {
	{ITOF(1)/2, ITOF(1)/2},
	{ITOF(WINDSIZEX)-ITOF(1)/2, ITOF(1)/2},
	{ITOF(1)/2, ITOF(WINDSIZEY)-ITOF(1)/2},
	{ITOF(WINDSIZEX)-ITOF(1)/2, ITOF(WINDSIZEY)-ITOF(1)/2}
    };

    glClear(GL_COLOR_BUFFER_BIT);

    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(2, GL_FTYPE, 0, &vertices[0][0]);
    glDrawArrays(GL_POINTS, 0, 4);
    glDisableClientState(GL_VERTEX_ARRAY);

    ReadScreen(0, 0, WINDSIZEX, WINDSIZEY, GL_AUTO_COLOR, buf);

    /*
    ** Lower left corner.
    */
    if (AutoColorCompare(buf[0], COLOR_ON) == GL_FALSE) {
	StrMake(errStr, errStrFmt, "lower left");
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }

    /*
    ** Lower right corner.
    */
    if (AutoColorCompare(buf[WINDSIZEX-1], COLOR_ON) == GL_FALSE) {
	StrMake(errStr, errStrFmt, "lower right");
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }

    /*
    ** Upper left corner.
    */
    if (AutoColorCompare(buf[WINDSIZEX*(WINDSIZEY-1)], COLOR_ON) == GL_FALSE) {
	StrMake(errStr, errStrFmt, "upper left");
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }

    /*
    ** Upper right corner.
    */
    if (AutoColorCompare(buf[WINDSIZEX*WINDSIZEY-1], COLOR_ON) == GL_FALSE) {
	StrMake(errStr, errStrFmt, "upper right");
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }

    return NO_ERROR;
}

long BCornerExec(void)
{
    GLftype *buf;

    buf = (GLftype *)MALLOC(WINDSIZEX*WINDSIZEY*sizeof(GLftype));

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    AutoClearColor(COLOR_OFF);
    AutoColor(COLOR_ON);
    glDisable(GL_DITHER);
    glDisable(GL_MULTISAMPLE);

    if (Test(buf) == ERROR) {
	FREE(buf);
	return ERROR;
    }

    FREE(buf);
    return NO_ERROR;
}
