/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/dither.c,v 1.3 2005/05/11 15:21:40 artlab Exp $
*/

/*
** dither.c
** Dither Test.
**
** Description -
**    Tests dithering by drawing rectangles of various shades
**    with both dithering enabled and disabled, and comparing
**    results to ensure that corresponding fragments differ by no
**    more than one shade. Since dithering may depend upon x and
**    y window coordinates, fragments compared must come from the
**    same window location. For configs with up to 256 colors,
**    the tests uses each color. For deeper visuals, it skips
**    over some to test up to 256 shades.
**
** Error Explanation -
**    Failure occurs if the comparison between the original
**    fragment color and the dithered fragment color varies by
**    more then one shade of color.
**
** Technical Specification -
**    Buffer requirements:
**        Color buffer.
**    Color requirements:
**        4 colors.
**    States requirements:
**        Disabled = GL_MULTISAMPLE.
**    Error epsilon:
**        color epsilon.
**    Paths:
**        Allowed = Alias, Alpha, Blend, Depth, Dither, Fog, Logicop, Shade,
**                  Stencil.
*/

#include "conform.h"
#include "util.h"


static char errStr[240];
static char errStrFmt[160] =
    "Undithered color is (%1.1f, %1.1f, %1.1f). Dithered color is (%1.1f, %1.1f, %1.1f).";

long DitherExec(void)
{
    GLftype *buf, *ref, deltaX, deltaY;
    GLftype deltaR, deltaG, deltaB;
    long j, maxShades, steps, maxSteps;
    int r, g, b;

    buf = (GLftype *)MALLOC(3*WINDSIZEX*WINDSIZEY*sizeof(GLftype));
    ref = (GLftype *)MALLOC(3*WINDSIZEX*WINDSIZEY*sizeof(GLftype));

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glDisable(GL_MULTISAMPLE);

    maxShades = 1 << buffer.maxRGBBit;
    maxSteps = (machine.pathLevel == 0) ? 8 : 2;
    steps = (maxShades > maxSteps) ? maxSteps : maxShades;
    deltaR = DIV(ITOF(1), ITOF(steps));
    deltaG = DIV(ITOF(1), ITOF(steps));
    deltaB = DIV(ITOF(1), ITOF(steps));
    deltaX = DIV(ITOF(WINDSIZEX), ITOF(steps));
    deltaY = DIV(ITOF(WINDSIZEY), ITOF(steps));

    for (r = 0; r <= steps; r++) {
	glDisable(GL_DITHER);
	for (g = 0; g <= steps; g++) {
	    for (b = 0; b <= steps; b++) {
		glColor4t(r*deltaR, g*deltaG, b*deltaB, ITOF(1));
		DrawQuad2f(g*deltaX, b*deltaY, (g+1)*deltaX, (b+1)*deltaY);
	    }
	}
	ReadScreen(0, 0, WINDSIZEX, WINDSIZEY, GL_RGB, ref);

	glEnable(GL_DITHER);
	for (g = 0; g <= steps; g++) {
	    for (b = 0; b <= steps; b++) {
		glColor4t(r*deltaR, g*deltaG, b*deltaB, ITOF(1));
		DrawQuad2f(g*deltaX, b*deltaY, (g+1)*deltaX, (b+1)*deltaY);
	    }
	}
	ReadScreen(0, 0, WINDSIZEX, WINDSIZEY, GL_RGB, buf);

	for (j = 0; j < WINDSIZEX*WINDSIZEY; j += 3) {
	    if (ABS(ref[j]-buf[j]) > epsilon.color[0] ||
		ABS(ref[j+1]-buf[j+1]) > epsilon.color[1] ||
		ABS(ref[j+2]-buf[j+2]) > epsilon.color[2]) {

		StrMake(errStr, errStrFmt, ref[j], ref[j+1], ref[j+2],
			buf[j], buf[j+1], buf[j+2]);
		ErrorReport(__FILE__, __LINE__, errStr);
		FREE(buf);
		FREE(ref);
		return ERROR;
	    }
	}
    }

    FREE(buf);
    FREE(ref);
    return NO_ERROR;
}
