/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/divzero.c,v 1.3 2005/05/11 15:21:40 artlab Exp $
*/

/*
** divzero.c
** Divide By Zero Test.
**
** Description -
**    This tests many situations where an implementation might
**    divide by zero. We try to assure, that division by zero
**    does "not lead to GL interruption or termination" as
**    specified by the OpenGL ES Specificaton. In two cases we test
**    that a proper error is generated. Error generation is
**    checked in the case of a viewport of zero width, and in the
**    case of evaluators with a trivial domain. (The evaluator
**    test is currently unfinished.) We currently test 24
**    different situations and there are more coming. These
**    situations are summarized in the comments at the bottom of
**    this file.
**
** Error Explanation -
**    Failure occurs if an implementation interrupts or
**    terminates during this test. Failure also occurs if a
**    parameter value which could cause a divide by zero should
**    generate an GL_INVALID_VALUE error and does not.
**
** Technical Specification -
**    Buffer requirements:
**        No buffer requirements.
**    Color requirements:
**        No color requirements.
**    States requirements:
**        No state restrictions.
**    Error epsilon:
**    Paths:
**        Allowed = Alias, Alpha, Blend, Depth, Dither, Fog, Logicop, Shade,
**                  Stencil.
*/

#include "conform.h"
#include "util.h"


static const GLftype safeColors[] = {
    ITOF(1), ITOF(0), ITOF(0), ITOF(1),
    ITOF(0), ITOF(1), ITOF(0), ITOF(1),
    ITOF(0), ITOF(0), ITOF(1), ITOF(1),
    ITOF(1), ITOF(1), ITOF(0), ITOF(1),
    ITOF(1), ITOF(1), ITOF(1), ITOF(1)
};

static const GLftype safeVertices[] = {
    ITOF(10), ITOF(10), ITOF(0), ITOF(1),
    ITOF(10), ITOF(1),  ITOF(0), ITOF(1),
    ITOF(1),  ITOF(1),  ITOF(0), ITOF(1),
    ITOF(1),  ITOF(10), ITOF(0), ITOF(1),
    ITOF(5),  ITOF(15), ITOF(0), ITOF(1)
};

static const GLftype safeLitVertices[] = {
    ITOF(10), ITOF(10), ITOF(1)/2, ITOF(1),
    ITOF(10), ITOF(1), ITOF(1)/2, ITOF(1),
    ITOF(1), ITOF(1), ITOF(1)/2, ITOF(1),
    ITOF(1), ITOF(10), ITOF(1)/2, ITOF(1),
    ITOF(5), ITOF(15), ITOF(1)/2, ITOF(1)
};

static const GLftype safeTexture[] = {
    ITOF(0), ITOF(0), ITOF(0), ITOF(1),
    ITOF(1), ITOF(0), ITOF(0), ITOF(1),
    ITOF(1), ITOF(1), ITOF(0), ITOF(1),
    ITOF(0), ITOF(1), ITOF(0), ITOF(1),
    ITOF(1)/2, ITOF(1)/2, ITOF(0), ITOF(1)
};

static const GLftype safeNormals[] = {
    ITOF(1), ITOF(0), ITOF(0), ITOF(1),
    ITOF(1), ITOF(1), ITOF(0), ITOF(1),
    ITOF(1), ITOF(0), ITOF(1), ITOF(1),
    ITOF(1), ITOF(1), ITOF(1), ITOF(1),
    ITOF(1), ITOF(1), ITOF(0), ITOF(1)
};

static const GLftype singularMatrix[] = {
    ITOF(1), ITOF(1), ITOF(1), ITOF(0),
    ITOF(1), ITOF(1), ITOF(1), ITOF(0),
    ITOF(0), ITOF(0), ITOF(1), ITOF(0),
    ITOF(0), ITOF(0), ITOF(0), ITOF(1)
};

static const GLftype identMatrix[] = {
    ITOF(1), ITOF(0), ITOF(0), ITOF(0),
    ITOF(0), ITOF(1), ITOF(0), ITOF(0),
    ITOF(0), ITOF(0), ITOF(1), ITOF(0),
    ITOF(0), ITOF(0), ITOF(0), ITOF(1)
};

static GLubyte texBuf[300];

static void RenderArrays(GLenum array1, GLenum array2, GLftype const *ntc, GLftype const *v)
{
    if (array2 == GL_VERTEX_ARRAY) {
	glEnableClientState(array2);
	glVertexPointer(4, GL_FTYPE, 0, v);
    } else {
	glVertexPointer(4, GL_FTYPE, 0, NULL);
    }

    switch (array1) {
    case GL_NORMAL_ARRAY:
        glEnableClientState(array1);
	glNormalPointer(GL_FTYPE, 4*sizeof(GLftype), ntc);
	break;

    case GL_TEXTURE_COORD_ARRAY:
        glEnableClientState(array1);
	glTexCoordPointer(3, GL_FTYPE, 4*sizeof(GLftype), ntc);
	break;

    case GL_COLOR_ARRAY:
        glEnableClientState(array1);
	glColorPointer(4, GL_FTYPE, 0, ntc);
	break;
    }

    glDrawArrays(GL_POINTS, 0, 2);
    glDrawArrays(GL_LINES, 0, 2);
    glDrawArrays(GL_LINE_LOOP, 0, 3);
    glDrawArrays(GL_LINE_STRIP, 0, 3);
    glDrawArrays(GL_TRIANGLES, 0, 3);
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 3);
    glDrawArrays(GL_TRIANGLE_FAN, 0, 3);
    glDrawArrays(GL_TRIANGLE_FAN, 0, 5);

    switch (array1) {
    case GL_NORMAL_ARRAY:
    case GL_TEXTURE_COORD_ARRAY:
    case GL_COLOR_ARRAY:
        glDisableClientState(array1);
	break;
    }

    if (array2 == GL_VERTEX_ARRAY) {
	glDisableClientState(array2);
    }

    /*
    ** Restore default values.
    */
    switch (array1) {
    case GL_NORMAL_ARRAY:
	glNormal3t(ITOF(0), ITOF(0), ITOF(1));
	break;

    case GL_COLOR_ARRAY:
	glColor4t(ITOF(1), ITOF(1), ITOF(1), ITOF(1));
	break;
    }
}

static void Render2DArrays(GLenum array1, GLenum array2, GLftype const *ntc, GLftype const *v)
{
    if (array2 == GL_VERTEX_ARRAY) {
	glEnableClientState(array2);
	glVertexPointer(4, GL_FTYPE, 0, v);
    } else {
	glVertexPointer(4, GL_FTYPE, 0, NULL);
    }

    switch (array1) {
    case GL_NORMAL_ARRAY:
	glEnableClientState(array1);
	glNormalPointer(GL_FTYPE, 4*sizeof(GLftype), ntc);
	break;

    case GL_TEXTURE_COORD_ARRAY:
	glEnableClientState(array1);
	glTexCoordPointer(2, GL_FTYPE, 4*sizeof(GLftype), ntc);
	break;

    case GL_COLOR_ARRAY:
	glEnableClientState(array1);
	glColorPointer(4, GL_FTYPE, 0, ntc);
	break;

    default:
	glNormalPointer(GL_FTYPE, 0, NULL);
	glTexCoordPointer(4, GL_FTYPE, 0, NULL);
	glColorPointer(4, GL_FTYPE, 0, NULL);
	break;
    }

    glDrawArrays(GL_TRIANGLES, 0, 3);
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 3);
    glDrawArrays(GL_TRIANGLE_FAN, 0, 3);
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
    glDrawArrays(GL_TRIANGLE_FAN, 0, 5);

    switch (array1) {
    case GL_NORMAL_ARRAY:
    case GL_TEXTURE_COORD_ARRAY:
    case GL_COLOR_ARRAY:
	glDisableClientState(array1);
	break;
    }

    if (array2 == GL_VERTEX_ARRAY) {
	glDisableClientState(array2);
    }

    /*
    ** Restore default values.
    */
    switch (array1) {
    case GL_NORMAL_ARRAY:
	glNormal3t(ITOF(0), ITOF(0), ITOF(1));
	break;

    case GL_COLOR_ARRAY:
	glColor4t(ITOF(1), ITOF(1), ITOF(1), ITOF(1));
	break;
    }
}

static void InitTexture(GLubyte *t)
{
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 8, 8, 0, GL_RGB, GL_UNSIGNED_BYTE, t);
    glTexImage2D(GL_TEXTURE_2D, 1, GL_RGB, 4, 4, 0, GL_RGB, GL_UNSIGNED_BYTE, t);
    glTexImage2D(GL_TEXTURE_2D, 2, GL_RGB, 2, 2, 0, GL_RGB, GL_UNSIGNED_BYTE, t);
    glTexImage2D(GL_TEXTURE_2D, 3, GL_RGB, 1, 1, 0, GL_RGB, GL_UNSIGNED_BYTE, t);
    glTexImage2D(GL_TEXTURE_2D, 4, GL_RGB, 0, 0, 0, GL_RGB, GL_UNSIGNED_BYTE, t);
}

static void ResetTextureToDefault(void)
{
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 0, 0, 0, GL_RGB, GL_UNSIGNED_BYTE, NULL);
}

static void ExerciseNormals(GLftype const *n, GLftype const *v)
{

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    RenderArrays(GL_NORMAL_ARRAY, GL_VERTEX_ARRAY, n, v);

    /*
    ** Restore default values.
    */
    glDisable(GL_LIGHTING);
    glDisable(GL_LIGHT0);
}

static void SetMatrix(GLenum mode, GLftype const *m)
{

    glMatrixMode(mode);
    glLoadMatrixt(m);
    glMatrixMode(GL_MODELVIEW);
}

/*
** This is not actually used but it makes debugging easier.
*/
#if 0
static void Blank(void)
{
     glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(1));
     glClear(GL_COLOR_BUFFER_BIT);
}
#endif

/*
** This just calls glDrawArrays() with no vertex array.
*/
static long PrimNoVertex(void)
{

    RenderArrays(0, 0, NULL, NULL);

    return NO_ERROR;
}

static long PrimWEqualsZero(void)
{
    static const GLftype v[] = {
	ITOF(10), ITOF(10), ITOF(0), ITOF(0),
	ITOF(10), ITOF(1),  ITOF(0), ITOF(1),
	ITOF(1),  ITOF(1),  ITOF(0), ITOF(1),
	ITOF(1),  ITOF(10), ITOF(0), ITOF(1),
	ITOF(5),  ITOF(15), ITOF(0), ITOF(1)
    };

    /*
    ** To test this routine, use safeVertices and call Blank() between draws.
    */

    RenderArrays(0, GL_VERTEX_ARRAY, NULL, v);

    /*
    ** The homogeneous coordinate is used as a divisor to form one
    ** component of the eye-normal.
    */
    ExerciseNormals(safeNormals, v);

    /*
    ** It is very likely that a singular modelview matrix
    ** will demand that a different computational path be used
    ** to deal with normals, so we test the interaction of a singular
    ** modelview matrix with a zero homogeneous coordinate.
    */
    SetMatrix(GL_MODELVIEW, singularMatrix);

    /*
    ** Previous call to ExerciseNormals() tests this (with correct v[0][3]).
    */
    ExerciseNormals(safeNormals, v);

    /*
    ** Reset default values.
    */
    SetMatrix(GL_MODELVIEW, identMatrix);

    return NO_ERROR;
}

/*
** This checks that none of the primitives has any difficulty if it is
** rendered as a point. It checks if the 2D primitives can render as a line.
*/
static long TrivialPrims(void)
{
    static const GLftype v1[] = {
	ITOF(10), ITOF(10), ITOF(0), ITOF(1),
	ITOF(10), ITOF(10), ITOF(0), ITOF(1),
	ITOF(10), ITOF(10), ITOF(0), ITOF(1),
	ITOF(10), ITOF(10), ITOF(0), ITOF(1),
	ITOF(10), ITOF(10), ITOF(0), ITOF(1)
    };
    static const GLftype v2[] = {
	ITOF(1), ITOF(1), ITOF(0), ITOF(1),
	ITOF(1), ITOF(1), ITOF(0), ITOF(1),
	ITOF(1), ITOF(3), ITOF(0), ITOF(1),
	ITOF(1), ITOF(3), ITOF(0), ITOF(1),
	ITOF(3), ITOF(3), ITOF(0), ITOF(1)
    };

    /*
    ** Test this with safeVertices instead of v1.
    */
    RenderArrays(0, GL_VERTEX_ARRAY, NULL, v1);

    /*
    ** Trivial 2d objects still could have two vertices.
    */
    Render2DArrays(0, GL_VERTEX_ARRAY, NULL, v2);

    return NO_ERROR;
}

/*
** With the texture coordinate's q set equal to zero, the texture
** coordinate interpolation could divide by zero.
*/
static long TexQOrWEqualsZero(void)
{
    static const GLftype texCoord[] = {
	ITOF(0), ITOF(0), ITOF(0), ITOF(0),
	ITOF(1), ITOF(0), ITOF(0), ITOF(0),
	ITOF(1), ITOF(1), ITOF(0), ITOF(0),
	ITOF(0), ITOF(1), ITOF(0), ITOF(0),
	ITOF(1)/2, ITOF(1)/2, ITOF(0), ITOF(0)
    };
    static const GLftype v[] = {
	ITOF(10), ITOF(10), ITOF(0), ITOF(0),
	ITOF(10), ITOF(1), ITOF(0), ITOF(0),
	ITOF(1), ITOF(1), ITOF(0), ITOF(0),
	ITOF(1), ITOF(10), ITOF(0), ITOF(0),
	ITOF(5), ITOF(15), ITOF(0), ITOF(0)
    };

    /*
    ** To test this, use change texCoord to have 1.0 for q coordinate
    ** and v to have 1.0 in w coordinate. Call Blank().
    */
    InitTexture(texBuf);
    glEnable(GL_TEXTURE_2D);

    RenderArrays(GL_TEXTURE_COORD_ARRAY, GL_VERTEX_ARRAY, texCoord, safeVertices);
    RenderArrays(GL_TEXTURE_COORD_ARRAY, GL_VERTEX_ARRAY, safeTexture, v);

    /*
    ** Restore default values.
    */
    ResetTextureToDefault();
    glDisable(GL_TEXTURE_2D);

    return NO_ERROR;
}

/*
** A primitive with equal vertices would theoretically divide by its length
** when interpolating the texture coordinates.
*/
static long TexTrivialPrim(void)
{
    static const GLftype v[] = {
	ITOF(10), ITOF(10), ITOF(0), ITOF(1),
	ITOF(10), ITOF(10), ITOF(0), ITOF(1),
	ITOF(10), ITOF(10), ITOF(0), ITOF(1),
	ITOF(10), ITOF(10), ITOF(0), ITOF(1),
	ITOF(10), ITOF(10), ITOF(0), ITOF(1)
    };

    /*
    ** Test this using safeVertices.
    */
    InitTexture(texBuf);
    glEnable(GL_TEXTURE_2D);
    RenderArrays(GL_TEXTURE_COORD_ARRAY, GL_VERTEX_ARRAY, safeTexture, v);

    /*
    ** Restore default values.
    */
    glDisable(GL_TEXTURE_2D);
    ResetTextureToDefault();

    return NO_ERROR;
}

/*
** A primitive with equal vertices would theoretically divide by zero length
** when interpolating the shade.
*/
static long ShadedTrivialPrim(void)
{
    static const GLftype v[] = {
	ITOF(10), ITOF(10), ITOF(0), ITOF(1),
	ITOF(10), ITOF(10), ITOF(0), ITOF(1),
	ITOF(10), ITOF(10), ITOF(0), ITOF(1),
	ITOF(10), ITOF(10), ITOF(0), ITOF(1),
	ITOF(10), ITOF(10), ITOF(0), ITOF(1)
    };

    /*
    ** To test this use safeVertices.
    */
    RenderArrays(GL_COLOR_ARRAY, GL_VERTEX_ARRAY, safeColors, v);

    return NO_ERROR;
}

/*
** A primitive with equal vertices would theoretically divide by zero length
** when interpolating the depth.
*/
static long ZTrivialPrim(void)
{
    static const GLftype v[] = {
	ITOF(10), ITOF(10), ITOF(0), ITOF(1),
	ITOF(10), ITOF(10), ITOF(0), ITOF(1),
	ITOF(10), ITOF(10), ITOF(0), ITOF(1),
	ITOF(10), ITOF(10), ITOF(0), ITOF(1),
	ITOF(10), ITOF(10), ITOF(0), ITOF(1)
    };

    /*
    ** To test this, use safeVertices.
    */

    glClearDeptht(ITOF(0));
    glClear(GL_DEPTH_BUFFER_BIT);
    glEnable(GL_DEPTH_TEST);
    RenderArrays(0, GL_VERTEX_ARRAY, NULL, v);

    /*
    ** Restore default values.
    */
    glDisable(GL_DEPTH_TEST);
    glClearDeptht(ITOF(1));

    return NO_ERROR;
}

static long ZFarEqualsZNear(void)
{
    ftype maxZ;

    /*
    ** To test this, remove glDepthRange(ITOF(0), ITOF(0)).
    */
    glDepthRanget(ITOF(0), ITOF(0));

    glClearDeptht(ITOF(1));
    glClear(GL_DEPTH_BUFFER_BIT);
    glEnable(GL_DEPTH_TEST);
    RenderArrays(0, GL_VERTEX_ARRAY, NULL, safeVertices);

    maxZ = ITOF((1 << buffer.zBits) - 1);
    glDepthRanget(ITOF(0), maxZ);
    glDisable(GL_DEPTH_TEST);

    return NO_ERROR;
}

/*
** A linear fog function with end equal to start might divide by (end - start).
*/
static long FogEndEqualsFogStart(void)
{
    static const GLftype defaultFogColor[] = {
	ITOF(0), ITOF(0), ITOF(0), ITOF(0)
    };
    static ftype end = ITOF(0), linear = GL_LINEAR;

    /*
    ** Test by not calling glFog(GL_FOG_END, end);
    */
    glFogt(GL_FOG_MODE, linear);
    glFogt(GL_FOG_END, end);

    /*
    ** Default start is 0.0.
    */

    glFogtv(GL_FOG_COLOR, &safeColors[16]);
    glEnable(GL_FOG);
    RenderArrays(GL_COLOR_ARRAY, GL_VERTEX_ARRAY, safeColors, safeVertices);

    /*
    ** Restore default values.
    */
    glFogt(GL_FOG_MODE, GL_EXP);
    glFogt(GL_FOG_END, ITOF(1));
    glFogtv(GL_FOG_COLOR, defaultFogColor);
    glDisable(GL_FOG);
    return NO_ERROR;
}

static long ViewPortIsTrivial(void)
{

    glViewport(0, 0, 0, 0);
    RenderArrays(0, GL_VERTEX_ARRAY, NULL, safeVertices);

    /*
    ** Restore default values.
    */
    glViewport(0, 0, WINDSIZEX, WINDSIZEY);

    return NO_ERROR;
}

static long SingularModelViewMatrix(void)
{

    /*
    ** To test this, take out the following call.
    */
    SetMatrix(GL_MODELVIEW, singularMatrix);
    RenderArrays(0, GL_VERTEX_ARRAY, NULL, safeVertices);

    ExerciseNormals(safeNormals, safeVertices);

    SetMatrix(GL_MODELVIEW, identMatrix);
    return NO_ERROR;
}

static long SingularScaleMatrix(void)
{
    static const GLftype badScale[] = {ITOF(1), ITOF(0), ITOF(1)};

    /*
    ** To test, leave out the following line.
    */
    glScalet(badScale[0], badScale[1], badScale[2]);

    RenderArrays(0, GL_VERTEX_ARRAY, NULL, safeVertices);
    ExerciseNormals(safeNormals, safeVertices);

    SetMatrix(GL_MODELVIEW, identMatrix);
    return NO_ERROR;
}

static long SingularTextureMatrix(void)
{

    SetMatrix(GL_TEXTURE, singularMatrix);
    glEnable(GL_TEXTURE_2D);
    InitTexture(texBuf);
    RenderArrays(GL_TEXTURE_COORD_ARRAY, GL_VERTEX_ARRAY, safeTexture, safeVertices);

    /*
    ** Restore default values.
    */
    ResetTextureToDefault();
    glDisable(GL_TEXTURE_2D);
    SetMatrix(GL_TEXTURE, identMatrix);
    return NO_ERROR;
}

static long SingularFrustum(void)
{

    glMatrixMode(GL_PROJECTION);
    glPushMatrix();

    /*
    ** Remove the following line to test this code.
    */
    /*
    ** XXX
    ** This call generates an GL_INVALID_VALUE error, awaiting spec
    ** clarification.
    */
    glFrustumt(ITOF(1), ITOF(1), ITOF(0), ITOF(1), ITOF(1), ITOF(2));

    RenderArrays(0, GL_VERTEX_ARRAY, NULL, safeVertices);
  
    /*
    ** Remove the following line to test this code.
    */
    glFrustumt(ITOF(0), ITOF(1), ITOF(0), ITOF(1), ITOF(0), ITOF(1));
    RenderArrays(0, GL_VERTEX_ARRAY, NULL, safeVertices);

    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
    return NO_ERROR;
}

static long TrivialNormal(void)
{

    glEnable(GL_NORMALIZE);

    /*
    ** Test this with the default normal (0.0, 0.0, 0.0, 1.0),
    ** or use the following line.
    ** ExerciseNormals(safeNormals, safeVertices);
    */

    ExerciseNormals(safeNormals, safeVertices);
    glDisable(GL_NORMALIZE);
    ExerciseNormals(safeNormals, safeVertices);
    /*
    ** Disable(GL_NORMALIZE) is the default.
    */
    return NO_ERROR;
}

static long ScissorNothing(void)
{

    /*
    ** To test this, remove the following line.
    */
    glScissor(WINDSIZEX/2, WINDSIZEY/2, 0, 0);
    glEnable(GL_SCISSOR_TEST);
    RenderArrays(0, GL_VERTEX_ARRAY, NULL, safeVertices);

    /*
    ** Reset default scissor.
    */
    glDisable(GL_SCISSOR_TEST);
    glScissor(0, 0, WINDSIZEX, WINDSIZEY);
    return NO_ERROR;
}

/*
** All three Attenuation Constants are set to zero.
*/
static long AttenuationConstants(void)
{
    GLftype tmp[4];

    glLightt(GL_LIGHT0, GL_CONSTANT_ATTENUATION, ITOF(0));

    /*
    ** The following are default values.
    */
    glLightt(GL_LIGHT0, GL_LINEAR_ATTENUATION, ITOF(0));
    glLightt(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, ITOF(0));
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);

    tmp[0] = ITOF(1);
    tmp[1] = ITOF(1);
    tmp[2] = ITOF(1);
    tmp[3] = ITOF(1);
    glMaterialtv(GL_FRONT_AND_BACK, GL_AMBIENT, tmp);
    glLighttv(GL_LIGHT0, GL_AMBIENT, tmp);

    /*
    **  To verify test, include this. It should render a white pentagon.
    **  glLightt(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 1.0);
    **  glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(1));
    **  glClear(GL_COLOR_BUFFER_BIT);
    */

    RenderArrays(0, GL_VERTEX_ARRAY, NULL, safeVertices);

    /*
    ** Restore default ambient values.
    */
    tmp[0] = ATOF("0.2");
    tmp[1] = ATOF("0.2");
    tmp[2] = ATOF("0.2");
    tmp[3] = ITOF(1);
    glMaterialtv(GL_FRONT_AND_BACK, GL_AMBIENT, tmp);
    
    tmp[0] = ITOF(0);
    tmp[1] = ITOF(0);
    tmp[2] = ITOF(0);
    glLighttv(GL_LIGHT0, GL_AMBIENT, tmp);

    /*
    ** Second test uses default normal and diffuse.
    */

    tmp[0] = ITOF(1);
    tmp[1] = ITOF(1);
    tmp[2] = ITOF(1);
    glMaterialtv(GL_FRONT_AND_BACK, GL_DIFFUSE, tmp);

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(1));
    glClear(GL_COLOR_BUFFER_BIT);
    glColor4t(ITOF(0), ITOF(0), ITOF(0), ITOF(1));
    RenderArrays(0, GL_VERTEX_ARRAY, NULL, safeLitVertices);

    /*
    ** Restore default diffuse parameters.
    */
    tmp[0] = ATOF("0.8");
    tmp[1] = ATOF("0.8");
    tmp[2] = ATOF("0.8");
    glMaterialtv(GL_FRONT_AND_BACK, GL_DIFFUSE, tmp);

    /*
    ** Restore default values.
    */
    glColor4t(ITOF(1), ITOF(1), ITOF(1), ITOF(1));
    glLightt(GL_LIGHT0, GL_CONSTANT_ATTENUATION, ITOF(1));
    glDisable(GL_LIGHTING);
    glDisable(GL_LIGHT0);
    return NO_ERROR;
}

/*
** Normalization of light position may bust this.
*/
static long LightPositionEqualsZero(void)
{

    return NO_ERROR;
}

/*
** Normalization of spot direction may bust this.
*/
static long SpotDirectionEqualsZero(void)
{

    return NO_ERROR;
}

/*
** Normalization of the half angle code may bust this.
*/
static long HalfAngleEqualsZero(void)
{

    return NO_ERROR;
}

/*
** XXX
** The vertex is at the eye position, division by zero occurs when
** normalizing for use in half angle computation.
*/
static long LightVertexIsAtEye(void)
{

    return NO_ERROR;
}

long DivZeroExec(void)
{
    long i;

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    for (i = 0; i < 300; i++) {
	texBuf[i] = 255;
    }

    /*
    ** Does any interruption occur if a glDrawArrays occurs without an array.
    */
    PrimNoVertex();
    glGetError();

    /*
    ** Does any interruption occur if a primitive is rendered with a vertex
    ** whose fourth coordinate is 0.0.
    */
    PrimWEqualsZero();
    glGetError();

    /*
    ** Does any interruption occur if any primitive type is rendered flat, with
    ** two coinciding vertices.
    */
    TrivialPrims();
    glGetError();

    /*
    ** Does any interruption occur if any primitive type is rendered smooth, 
    **  with two coinciding vertices.
    */
    ShadedTrivialPrim();
    glGetError();

    /*
    ** Does any interruption occur if any primitive type is rendered with 
    ** two coinciding vertices, and with depth test is enabled.
    */
    ZTrivialPrim();
    glGetError();

    /*
    ** Does any interruption occur if any primitive type is rendered with 
    ** the depth range set so that near = far.
    */
    ZFarEqualsZNear();
    glGetError();

    /*
    ** Does any interruption occur if any primitive type is rendered with 
    ** the linear fog parameters set so that fog_end = fog_start.
    */
    FogEndEqualsFogStart();
    glGetError();
  
    /*
    ** Does any interruption occur if any primitive type is rendered with 
    ** the viewport set to zero width or zero height.
    */
    ViewPortIsTrivial();
    glGetError();

    /*
    ** Does any interruption occur if any textured primitive is rendered with
    ** a singular modelview matrix.
    */
    SingularModelViewMatrix();
    glGetError();

    /*
    ** Does any interruption occur if any textured primitive is rendered with
    ** a singular modelview matrix, set by glScale.
    */
    SingularScaleMatrix();
    glGetError();

    /*
    ** Does any interruption occur if any textured primitive is rendered with
    ** a singular texture matrix.
    */
    SingularTextureMatrix();
    glGetError();

    /*
    ** Does any interruption occur if any primitive is rendered with
    ** the singular perspective matrix set by glFrustum.
    */
    SingularFrustum();
    glGetError();

    /*
    ** Does any interruption occur if any primitive is rendered with
    ** scissor defining a window of 0 width or 0 height.
    */
    ScissorNothing();
    glGetError();

    /*
    ** Does any interruption occur if any primitive is rendered lit or 
    ** spheremapped with a normal vector = (0.0,0.0,0.0):
    */
    TrivialNormal();
    glGetError();

    /*
    ** Does any interruption occur if any primitive is rendered lit with
    ** attenuationa constants = (0.0,0.0,0.0);
    */

    AttenuationConstants();
    glGetError();

    /*
    ** Does any interruption occur if any primitive is rendered lit at 
    ** with a light position vector = (0.0,0.0,0.0,1.0);
    */
    LightPositionEqualsZero();
    glGetError();

    /*
    ** Does any interruption occur if any primitive is rendered lit with a 
    ** spot Direction vector which can't be normalized because it is zero
    ** length.
    */
    SpotDirectionEqualsZero();
    glGetError();

    /*
    ** Does any interruption occur if any primitive is rendered lit at 
    ** eye-coordinates = (0.0,0.0,0.0,1.0);
    */
    LightVertexIsAtEye();
    glGetError();

    /*
    ** Does any interruption occur if any primitive is rendered lit with a 
    ** half angle vector which has zero length. (can't be normalized)
    */
    HalfAngleEqualsZero();
    glGetError();

    /*
    ** Does any interruption occur if any primitive type is rendered 
    ** textured with a vertex whose 4th coordinate is 0.0;
    */
    TexQOrWEqualsZero();
    glGetError();

    /*
    ** Does any interruption occur if any primitive type is rendered 
    ** textured with two coinciding vertices.
    */
    TexTrivialPrim();
    glGetError();

    return NO_ERROR;
}
