/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/mip.c,v 1.4 2005/05/11 15:21:40 artlab Exp $
*/

/*
** mip.c
** Automatic Mipmap Generation Test.
**
** Description -
**    Automatic mipmap generation test. A quad is first drawn with 
**    AMMG off, the color buffer is read back and the values are compared.
**    Then we turn AMMG on, supply a checkerboard texture, draw, read back
**    and compare again, expecting grey pixels.
**
** Technical Specification -
**    Buffer requirements:
**        Color buffer.
**    Color requirements:
**        Full color.
**    States requirements:
**        Disabled = GL_DITHER, GL_MULTISAMPLE.
**    Error epsilon:
**    Paths:
**        Allowed = Alpha, Blend, Depth, Fog, Logicop, Shade, Stencil.
**        Not allowed = Alias, Dither.
*/

#include "conform.h"
#include "util.h"

#define SIZE        8
#define MIP_SIZE    4

static char     errStr[240];

static GLubyte  cbTex[SIZE * SIZE * 3];
static GLuint   texObj;
static GLuint   mipObj;
static GLubyte  colors[3][3] = {{255, 0, 0}, {0, 255, 0}, {0, 0, 255}};
static GLftype  fcolors[3][3] = {{ITOF(1), 0, 0}, {0, ITOF(1), 0}, {0, 0, ITOF(1)}};
#if GL_FTYPE == GL_FIXED
#define HALF 0x8000
#else
#define HALF 0.5f
#endif
static GLftype  expectedMipColor[3] = {HALF, HALF, HALF};

static void Make(void)
{
    int i, j, x, y, level;
 
    glPixelStorei(GL_PACK_ALIGNMENT, 1);
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

    glTexEnvt(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);

    glGenTextures(1, &texObj);
    glBindTexture(GL_TEXTURE_2D, texObj);

    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_NEAREST);

    glTexParametert(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_FALSE);

    x = y = SIZE;
    
    level = 0;
    while(x >= 1)
    {
        for(i = 0; i < x * y; i++)
        {
            cbTex[i * 3 + 0] = colors[level % 3][0];
            cbTex[i * 3 + 1] = colors[level % 3][1];
            cbTex[i * 3 + 2] = colors[level % 3][2];
        }
        
        glTexImage2D(GL_TEXTURE_2D, level, GL_RGB, x, y, 0, GL_RGB, GL_UNSIGNED_BYTE, (GLvoid *)cbTex);
      
        x /= 2;
        y /= 2;
        level++;
    }

    glGenTextures(1, &mipObj);
    glBindTexture(GL_TEXTURE_2D, mipObj);

    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_NEAREST);

    glTexParametert(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE);

    for(i = 0; i < SIZE; i++)
    {
        int c = i % 2;
        
        for (j = 0; j < SIZE; j++)
        {
            cbTex[(i * SIZE + j) * 3 + 0] = c * 255;
            cbTex[(i * SIZE + j) * 3 + 1] = c * 255;
            cbTex[(i * SIZE + j) * 3 + 2] = c * 255;
            
            c = 1 - c;
        }
    }
    
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, SIZE, SIZE, 0, GL_RGB, GL_UNSIGNED_BYTE, (GLvoid *)cbTex);
}

void DrawOff(void)
{
    int     x, y;
    
    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(1));
    glClear(GL_COLOR_BUFFER_BIT);

    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, texObj);
    
    x = SIZE;
    y = SIZE;
    
    while(x >= 1)
    {
        DrawTexQuad2i2f(0, 0, x, y, ITOF(0), ITOF(0), ITOF(1), ITOF(1));
    
        x /= 2;
        y /= 2;
    }
}

static long CompareOff(void)
{
    GLftype buf[SIZE * SIZE * 3];
    GLftype expected[SIZE * SIZE * 3];
    int     i, j, x, y, level;

    x = SIZE;
    y = SIZE;
    level = 0;
    
    while(x >= 1)
    {
        for(i = 0; i < x; i++)
            for(j = 0; j < y; j++)
            {
                expected[(i * SIZE + j) * 3 + 0] = fcolors[level % 3][0];
                expected[(i * SIZE + j) * 3 + 1] = fcolors[level % 3][1];
                expected[(i * SIZE + j) * 3 + 2] = fcolors[level % 3][2];
            }
    
        x /= 2;
        y /= 2;
        level++;
    }    

    ReadScreen(0, 0, SIZE, SIZE, GL_RGB, buf);
    
    for(i = 0; i < SIZE * SIZE; i++)
        for(j = 0; j < 3; j++)
            if(ABS(buf[i * 3 + j] - expected[i * 3 + j]) > epsilon.color[j])
            {
                StrMake(errStr, "Automatic mipmap generation OFF, color is (%f, %f, %f), should be (%f, %f, %f).",
                        buf[i * 3 + 0], buf[i * 3 + 1], buf[i * 3 + 2], 
                        expected[i * 3 + 0], expected[i * 3 + 1], expected[i * 3 + 2]);
                
                ErrorReport(__FILE__, __LINE__, errStr);
                return ERROR;
            }
            
    return NO_ERROR;
}

void DrawOn(void)
{
    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(1));
    glClear(GL_COLOR_BUFFER_BIT);

    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, mipObj);

    DrawTexQuad2i2f(0, 0, MIP_SIZE, MIP_SIZE, ITOF(0), ITOF(0), ITOF(1), ITOF(1));
}

static long CompareOn(void)
{
    int     i, j;
    GLftype buf[MIP_SIZE * MIP_SIZE * 3];
	epsilonRec mipGenEpsilon;


	/* See bugzilla 4505. Hacky relax of the error epsilon due to the complete lack of specification of what kind of
	 * filter the mipmap autogen uses.
	 */

	for(i = 0; i < 4; i++)
	{
		mipGenEpsilon.color[i] = MUL(epsilon.color[i], ITOF(2));
 	}

    ReadScreen(0, 0, MIP_SIZE, MIP_SIZE, GL_RGB, buf);

    for(i = 0; i < MIP_SIZE * MIP_SIZE; i++)
        for(j = 0; j < 3; j++)
            if(ABS(buf[i * 3 + j] - expectedMipColor[j]) > mipGenEpsilon.color[j])
            {
                StrMake(errStr, "Automatic mipmap generation ON, color is (%f, %f, %f), should be (%f, %f, %f).",
                        buf[i * 3 + 0], buf[i * 3 + 1], buf[i * 3 + 2], 
                        expectedMipColor[0], expectedMipColor[1], expectedMipColor[2]);
                
                ErrorReport(__FILE__, __LINE__, errStr);
                return ERROR;
            }
            
    return NO_ERROR;
}

long MipExec(void)
{
    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));
    glDisable(GL_DITHER);
    glDisable(GL_MULTISAMPLE);

    glColor4t(ITOF(0), ITOF(1), ITOF(1), ITOF(1));

    Make();
    DrawOff();
    
    if(CompareOff() == ERROR)
        return ERROR;
    
    DrawOn();
    
    if(CompareOn() == ERROR)
        return ERROR;

    glBindTexture(GL_TEXTURE_2D, 0);
    glDeleteTextures(1, &texObj);
    glDeleteTextures(1, &mipObj);
    
    return NO_ERROR;
}

