/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/miplin.c,v 1.3 2005/05/11 15:21:40 artlab Exp $
*/

/*
** miplin.c
** Mipmap Linear Test.
**
** Description -
**    Tests the interpolation between mipmap levels. An eight
**    level deep mipmap is set up, each level distinguished by a
**    different color. A polygon is drawn with texture
**    coordinates set so that the level used for each fragment
**    will ramp from a starting level to the next consecutive
**    level. The starting and ending color is checked to see that
**    the correct levels were used. Monotonicity between each
**    fragment texel is also checked.
**
** Technical Specification -
**    Buffer requirements:
**        Color buffer.
**    Color requirements:
**        Full color.
**    States requirements:
**        Disabled = GL_DITHER, GL_MULTISAMPLE.
**    Error epsilon:
**    Paths:
**        Allowed = Alpha, Blend, Depth, Fog, Logicop, Shade, Stencil.
**        Not allowed = Alias, Dither.
*/

#include "conform.h"
#include "util.h"


#define STEP 20


static long errSizeX, errSizeY;
static char errFilter[40];
static char errStr[240];
static char errStrFmt[][160] = {
    "Map size is %d by %d. Minification filter is %s. Color is (%g, %g, %g), should be (%g, %g, %g).",
    "Map size is %d by %d. Minification filter is %s. Color is not monotonic. Current color is (%g, %g, %g), last color was (%g, %g, %g)."
};


static void Make(GLsizei sizeX, GLsizei sizeY, GLubyte *buf)
{
    GLubyte *ptr;
    GLint level, i;

    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

    level = 0;
    while (sizeX > 0 || sizeY > 0) {
	ptr = buf;
	for (i = 0; i < 32*32; i++) {
            *ptr++ = FTOI(colorMap[level+1][0] * 255);
            *ptr++ = FTOI(colorMap[level+1][1] * 255);
            *ptr++ = FTOI(colorMap[level+1][2] * 255);
	}
	glTexImage2D(GL_TEXTURE_2D, level++, GL_RGB, (sizeX==0)?1:sizeX,
		     (sizeY==0)?1:sizeY, 0,
		     GL_RGB, GL_UNSIGNED_BYTE, buf);
	sizeX /= 2;
	sizeY /= 2;
    }
}

static long Draw(GLftype sizeX, GLftype sizeY, GLftype baseSizeX,
		 GLftype baseSizeY)
{
    GLftype x, y, i, dvShrink, dudx, dvdy;
    long drawnCount;

    dvShrink = 16 * DIV(baseSizeY, sizeY); 
    drawnCount = 0;
    for (i = ITOF(0); i <= ITOF(STEP); i += ITOF(1)) {
	GLftype istep = DIV(i, ITOF(STEP));
	x = MUL((ITOF(1) - istep), DIV(ITOF(1), sizeX)) +
            MUL(istep, DIV(ITOF(2), sizeX));

	y = DIV(
            MUL((ITOF(1) - istep), DIV(ITOF(1), sizeY)) +
            MUL(istep, DIV(ITOF(2), sizeY))
            , dvShrink);
	dudx = MUL(baseSizeX, x);
	dvdy = MUL(baseSizeY, y);

	if (dudx+dvdy <= DIV(MUL(ATOF("1.99"), baseSizeX), sizeX)) {
	    GLftype vertices[4][2];
	    GLftype tex_coords[4][2];

	    vertices[0][0] = i+ITOF(0); vertices[0][1] = ITOF(0);
	    vertices[1][0] = i+ITOF(1); vertices[1][1] = ITOF(0);
	    vertices[2][0] = i+ITOF(0); vertices[2][1] = ITOF(1);
	    vertices[3][0] = i+ITOF(1); vertices[3][1] = ITOF(1);

	    tex_coords[0][0] = ITOF(0); tex_coords[0][1] = ITOF(0);
	    tex_coords[1][0] =       x; tex_coords[1][1] = ITOF(0);
	    tex_coords[2][0] = ITOF(0); tex_coords[2][1] =       y;
	    tex_coords[3][0] =       x; tex_coords[3][1] =       y;

	    glEnableClientState(GL_VERTEX_ARRAY);
	    glVertexPointer(2, GL_FTYPE, 0, &vertices[0][0]);

	    glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	    glTexCoordPointer(2, GL_FTYPE, 0, &tex_coords[0][0]);

	    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

	    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	    glDisableClientState(GL_VERTEX_ARRAY);

	    drawnCount++;
	}
    }
    return drawnCount;
}

static long Compare(GLftype *buf, long index, long drawCount)
{
    GLftype last[3], rhoNormError;
    long i, x;

    ReadScreen(0, 0, STEP+1, 1, GL_RGB, buf);

    x = index - 1;
    rhoNormError = LOG2(LDEXP(ITOF(1), x) + ATOF("0.0625")) - ITOF(x); 

    if (ABS(buf[0]-colorMap[index][0]) > rhoNormError+epsilon.color[0]*3/2 ||
        ABS(buf[1]-colorMap[index][1]) > rhoNormError+epsilon.color[1]*3/2 ||
        ABS(buf[2]-colorMap[index][2]) > rhoNormError+epsilon.color[2]*3/2) {
	StrMake(errStr, errStrFmt[0], errSizeX, errSizeY, errFilter,
		buf[0], buf[1], buf[2], colorMap[index][0], colorMap[index][1],
		colorMap[index][2]);
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }
    last[0] = buf[0];
    last[1] = buf[1];
    last[2] = buf[2];
    for (i = 3; i < drawCount*3; i += 3) {
	if (colorMap[index][0] > colorMap[index+1][0]) {
	    if (buf[i]-last[0] > epsilon.color[0]) {
		StrMake(errStr, errStrFmt[1], errSizeX, errSizeY, errFilter,
			buf[0], buf[1], buf[2], last[0], last[1], last[2]);
		ErrorReport(__FILE__, __LINE__, errStr);
		return ERROR;
	    }
	} else if (colorMap[index][0] < colorMap[index+1][0]) {
	    if (buf[i]-last[0] < -epsilon.color[0]) {
		StrMake(errStr, errStrFmt[1], errSizeX, errSizeY, errFilter,
			buf[0], buf[1], buf[2], last[0], last[1], last[2]);
		ErrorReport(__FILE__, __LINE__, errStr);
		return ERROR;
	    }
	}
	if (colorMap[index][1] > colorMap[index+1][1]) {
	    if (buf[i+1]-last[1] > epsilon.color[1]) {
		StrMake(errStr, errStrFmt[1], errSizeX, errSizeY, errFilter,
			buf[0], buf[1], buf[2], last[0], last[1], last[2]);
		ErrorReport(__FILE__, __LINE__, errStr);
		return ERROR;
	    }
	} else if (colorMap[index][1] < colorMap[index+1][1]) {
	    if (buf[i+1]-last[1] < -epsilon.color[1]) {
		StrMake(errStr, errStrFmt[1], errSizeX, errSizeY, errFilter,
			buf[0], buf[1], buf[2], last[0], last[1], last[2]);
		ErrorReport(__FILE__, __LINE__, errStr);
		return ERROR;
	    }
	}
	if (colorMap[index][2] > colorMap[index+1][2]) {
	    if (buf[i+2]-last[2] > epsilon.color[2]) {
		StrMake(errStr, errStrFmt[1], errSizeX, errSizeY, errFilter,
			buf[0], buf[1], buf[2], last[0], last[1], last[2]);
		ErrorReport(__FILE__, __LINE__, errStr);
		return ERROR;
	    }
	} else if (colorMap[index][2] < colorMap[index+1][2]) {
	    if (buf[i+2]-last[2] < -epsilon.color[2]) {
		StrMake(errStr, errStrFmt[1], errSizeX, errSizeY, errFilter,
			buf[0], buf[1], buf[2], last[0], last[1], last[2]);
		ErrorReport(__FILE__, __LINE__, errStr);
		return ERROR;
	    }
	}
	last[0] = buf[i];
	last[1] = buf[i+1];
	last[2] = buf[i+2];
    }

    return NO_ERROR;
}

static long Test(GLenum filter, long sizeX, long sizeY, GLftype *buf)
{
    GLftype baseSizeX, baseSizeY;
    long i, drawCount;

    GetEnumName(filter, errFilter);

    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, (GLftype)filter);

    baseSizeX = ITOF(sizeX);
    baseSizeY = ITOF(sizeY);
    i = 1;
    while (sizeX > 1 && sizeY > 1) {
	errSizeX = sizeX;
	errSizeY = sizeY;

	glClear(GL_COLOR_BUFFER_BIT);

	drawCount = Draw(ITOF(sizeX), ITOF(sizeY), baseSizeX, baseSizeY);

	if (Compare(buf, i++, drawCount) == ERROR) {
	    return ERROR;
	}

	sizeX /= 2;
	sizeY /= 2;
    }

    return NO_ERROR;
}

long MipLinExec(void)
{
    GLftype *buf, x;
    GLubyte *texBuf;
    long max; 
    GLsizei i, j;

    buf = (GLftype *)MALLOC(WINDSIZEX*WINDSIZEY*3*sizeof(GLftype));
    texBuf = (GLubyte *)MALLOC(WINDSIZEX*WINDSIZEY*3*sizeof(GLubyte));

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));
    glDisable(GL_DITHER);
    glDisable(GL_MULTISAMPLE);
    glEnable(GL_TEXTURE_2D);
    x = GL_DECAL;
    glTexEnvtv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, &x);

    max = (machine.pathLevel == 0) ? 2 : 16;
    for (i = 32; i > 2; i /= 2) {
	for (j = 32; j > max; j /= 2) {
	    Make(i, j, texBuf);
	    if (Test(GL_NEAREST_MIPMAP_LINEAR, i, j, buf) == ERROR) {
		FREE(buf);
		return ERROR;
	    }
	    if (Test(GL_LINEAR_MIPMAP_LINEAR, i, j, buf) == ERROR) {
		FREE(buf);
		return ERROR;
	    }
	}
    }

    FREE(buf);
    FREE(texBuf);
    return NO_ERROR;
}
