/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/mstack.c,v 1.4 2005/05/11 15:21:40 artlab Exp $
*/

/*
** mstack.c
** Matrix Stack Test.
**
** Description -
**    Test the functionality of glPushMatrix() and glPopMatrix()
**    on the three matrix modes (GL_MODELVIEW_MATRIX,
**    GL_PROJECTION_MATRIX and GL_TEXTURE_MATRIX). Each matrix
**    stack is pushed and popped to their maximum depth with
**    unique matrices. Each matrix popped off the stack is
**    checked to verify stack ordering.
**
** Technical Specification -
**    Buffer requirements:
**        Color buffer.
**    Color requirements:
**        Full color.
**    States requirements:
**        Disabled = GL_DITHER, GL_MULTISAMPLE.
**    Error epsilon:
**    Paths:
**       Allowed = Alias, Alpha, Blend, Depth, Fog, Logicop, Shade,
**                 Stencil.
**       Not allowed = Dither.
*/

#include "conform.h"
#include "util.h"


static char errStr[320];
static char errStrFmt[] = "Matrix mode is %s.\nScale is %1.1f";

static void DrawTexQuad(GLint x1, GLint y1, GLint x2, GLint y2,
    GLftype s1, GLftype t1, GLftype s2, GLftype t2, GLftype r, GLftype q)
{
    GLshort vertices[4][2];
    GLftype tex_coords[4][4];

    vertices[0][0] = (GLshort)x1; vertices[0][1] = (GLshort)y1;
    vertices[1][0] = (GLshort)x2; vertices[1][1] = (GLshort)y1;
    vertices[2][0] = (GLshort)x1; vertices[2][1] = (GLshort)y2;
    vertices[3][0] = (GLshort)x2; vertices[3][1] = (GLshort)y2;

    tex_coords[0][0] = s1; tex_coords[0][1] = t1; tex_coords[0][2] = r; tex_coords[0][3] = q;
    tex_coords[1][0] = s2; tex_coords[1][1] = t1; tex_coords[1][2] = r; tex_coords[1][3] = q;
    tex_coords[2][0] = s1; tex_coords[2][1] = t2; tex_coords[2][2] = r; tex_coords[2][3] = q;
    tex_coords[3][0] = s2; tex_coords[3][1] = t2; tex_coords[3][2] = r; tex_coords[3][3] = q;

    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(2, GL_SHORT, 0, &vertices[0][0]);

    glEnableClientState(GL_TEXTURE_COORD_ARRAY);
    glTexCoordPointer(4, GL_FTYPE, 0, &tex_coords[0][0]);

    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
    glDisableClientState(GL_VERTEX_ARRAY);
}

static void MakeMatrix(GLftype value, GLftype *buf)
{
    long i;

    for (i = 0; i < 16; i++) {
	buf[i] = ITOF(0);
    }
    for (i = 0; i < 4; i++) {
	buf[i*4+i] = value;
    }
}

static long TestIdentity(GLftype s_max, GLftype t_max, GLftype r_max, GLftype q_max)
{
    static const GLsizei width = 8;
    static const GLsizei height = 8;
    GLubyte *textureBuf;
    GLubyte *workBuf;
    GLsizei textureBits[4];
    GLftype textureEpsilon[4];
    GLuint texture;
    long i, j, k, l;

    textureBuf = (GLubyte *)MALLOC(width*height*3*sizeof(GLubyte));
    workBuf = (GLubyte *)MALLOC(width*height*4*sizeof(GLubyte));

    for (i = 0; i < height; i++) {
	for (j = 0; j < width; j++) {
	    k = (i * width + j) * 3;
	    textureBuf[k] = (GLubyte)FTOI(OESRandom(ITOF(0), ITOF(255)));
	    textureBuf[k+1] = (GLubyte)FTOI(OESRandom(ITOF(0), ITOF(255)));
	    textureBuf[k+2] = (GLubyte)FTOI(OESRandom(ITOF(0), ITOF(255)));
	}
    }

    GetComponentBits(GL_RGB, GL_UNSIGNED_BYTE, textureBits);
    CalcTextureEpsilon(textureBits, textureEpsilon);

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glClear(GL_COLOR_BUFFER_BIT);
    glDisable(GL_DITHER);
    glDisable(GL_MULTISAMPLE);

    glGenTextures(1, &texture);
    glBindTexture(GL_TEXTURE_2D, texture);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, width, height, 0, GL_RGB, GL_UNSIGNED_BYTE, textureBuf);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);

    glEnable(GL_TEXTURE_2D);
    glColor4t(ITOF(1), ITOF(1), ITOF(1), ITOF(1));
    DrawTexQuad(0, 0, width, height, ITOF(0), ITOF(0), s_max, t_max, r_max, q_max);
    glDisable(GL_TEXTURE_2D);

    glDeleteTextures(1, &texture);

    glReadPixels(0, 0, width, height, GL_RGBA, GL_UNSIGNED_BYTE, workBuf);

    for (i = 0; i < height; i++) {
	for (j = 0; j < width; j++) {
	    k = (i * width + j) * 4;
	    l = (i * width + j) * 3;
	    if (DIV(ABS(ITOF(workBuf[k]-textureBuf[l])), ITOF(255)) > 2 * textureEpsilon[0] ||
	        DIV(ABS(ITOF(workBuf[k+1]-textureBuf[l+1])), ITOF(255)) > 2 * textureEpsilon[1] ||
	        DIV(ABS(ITOF(workBuf[k+2]-textureBuf[l+2])), ITOF(255)) > 2 * textureEpsilon[2]) {
		FREE(textureBuf);
		FREE(workBuf);
		return ERROR;
            }
        }
    }

    FREE(textureBuf);
    FREE(workBuf);
    return NO_ERROR;
}

static long CompareModelview(GLftype value)
{
    GLftype projection[16];

    MakeMatrix(DIV(ITOF(1), value), projection);

    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glMultMatrixt(projection);

    if (TestIdentity(ITOF(1), ITOF(1), ITOF(1), ITOF(1)) == ERROR) {
        glPopMatrix();
        glMatrixMode(GL_MODELVIEW);
	return ERROR;
    }

    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
    return NO_ERROR;
}

static long CompareProjection(GLftype value)
{
    GLftype modelview[16];

    MakeMatrix(DIV(ITOF(1), value), modelview);

    glMatrixMode(GL_MODELVIEW);
    glLoadMatrixt(modelview);

    if (TestIdentity(ITOF(1), ITOF(1), ITOF(1), ITOF(1)) == ERROR) {
        glMatrixMode(GL_PROJECTION);
	return ERROR;
    }

    glMatrixMode(GL_PROJECTION);
    return NO_ERROR;
}

static long CompareTexture(GLftype value)
{
    GLftype x;

    x = DIV(ITOF(1), value);

    if (TestIdentity(x, x, x, x) == ERROR) {
	return ERROR;
    }

    return NO_ERROR;
}

static long Compare(GLenum mode, GLftype value)
{
    switch (mode) {
    case GL_MODELVIEW:
	return CompareModelview(value);

    case GL_PROJECTION:
	return CompareProjection(value);

    case GL_TEXTURE:
	return CompareTexture(value);

    default:
	return ERROR;
    }
}

static long Test(GLenum mode, GLint max)
{
    char tmp[40];
    GLftype buf[16];
    GLint i;

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glMatrixMode(mode);
    GetEnumName(mode, tmp);

    for (i = 1; i < max; i++) {
	MakeMatrix(ITOF(i), buf);
        if (mode == GL_PROJECTION) {
	    glLoadIdentity();
	    glOrthot(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY), ITOF(-1), ITOF(1));
	    glMultMatrixt(buf);
	} else {
	    glLoadMatrixt(buf);
	}
	if (Compare(mode, ITOF(i)) == ERROR) {
	    StrMake(errStr, errStrFmt, tmp, ITOF(i));
	    ErrorReport(__FILE__, __LINE__, errStr);
	    return ERROR;
	}
	glPushMatrix();
    }

    if (Compare(mode, ITOF(max-1)) == ERROR) {
	StrMake(errStr, errStrFmt, tmp, ITOF(max-1));
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }

    for (i = max-1; i >= 1; i--) {
	glPopMatrix();
	if (Compare(mode, ITOF(i)) == ERROR) {
	    StrMake(errStr, errStrFmt, tmp, ITOF(i));
	    ErrorReport(__FILE__, __LINE__, errStr);
	    return ERROR;
	}
    }

    glLoadIdentity();
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    return NO_ERROR;
}

long MatrixStackExec(void)
{
    GLint max;

    if (machine.pathLevel == 0) {
	glGetIntegerv(GL_MAX_MODELVIEW_STACK_DEPTH, &max);
    } else {
	max = 2;
    }
    if (Test(GL_MODELVIEW, max) == ERROR) {
	return ERROR;
    }

    if (machine.pathLevel == 0) {
	glGetIntegerv(GL_MAX_PROJECTION_STACK_DEPTH, &max);
    } else {
	max = 2;
    }
    if (Test(GL_PROJECTION, max) == ERROR) {
	return ERROR;
    }

    if (machine.pathLevel == 0) {
	glGetIntegerv(GL_MAX_TEXTURE_STACK_DEPTH, &max);
    } else {
	max = 2;
    }
    if (Test(GL_TEXTURE, max) == ERROR) {
	return ERROR;
    }

    return NO_ERROR;
}
