/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/multitex.c,v 1.2 2005/05/11 15:21:40 artlab Exp $
*/

/*
** multitex.c
** Multi Texture Test.
**
** Description -
**    Tests multitexture. Generate a unique texture for each texture
**    unit.  Each texture unit is seperately enabled and the
**    environment mode is set to DECAL.  A polygon is drawn with
**    texture enabled.  The texture coordinates are set so that the
**    texture should appear on the polygon with no distortion. The
**    rendered polygon is read back and compared to the corresponding
**    texture pattern.  All texture units are enabled and environment
**    mode is set to MODULATE.  A polygon is drawn as above.  The
**    rendered polygon is read back and compared with the product of
**    all individual textures.
**
** Technical Specification -
**    Buffer requirements:
**        Color buffer.
**    Color requirements:
**        Full color.
**    States requirements:
**        Disabled = GL_DITHER, GL_MULTISAMPLE.
**    Error epsilon:
**    Paths:
**        Allowed = Alpha, Blend, Depth, Fog, Logicop, Shade, Stencil.
**        Not allowed = Alias, Dither.
*/

#include "conform.h"
#include "util.h"

static char errEnv[40], errMagFilter[40], errMinFilter[40];
static char errStr[320];
static char errStrFmt[] = "Location (%d, %d). Texture environment mode is %s. Magnification filter is %s, minification filter is %s. Color is (%1.1f, %1.1f, %1.1f), should be (%1.1f, %1.1f, %1.1f.)";

#define MAX_TEXTURES 32
#define SIZE 32

static GLint numTextures;
static GLuint textureNames[MAX_TEXTURES];
static GLftype *textureBuf[MAX_TEXTURES], *workBuf;

static void Cleanup(void);

static void Make(GLsizei size, GLftype *buf, long texnum)
{
    GLint level, i, j, k;
    GLubyte *tex_buf;

    tex_buf = (GLubyte *)MALLOC(SIZE*SIZE*3*sizeof(GLubyte));

    for (i = 0; i < SIZE; i++) {
        for (j = 0; j < SIZE; j++) {
            k = (i * SIZE + j) * 3;
            buf[k+0] = (((i+j)%SIZE) == ((SIZE/2-1)+texnum)) ? ITOF(0) : ITOF(1);
            buf[k+1] = (((i+j)%SIZE) == ((SIZE/2-1)+texnum)) ? ITOF(1) : ITOF(0);
            buf[k+2] = ITOF(1);
            tex_buf[k+0] = (GLubyte)FTOI(buf[k + 0] * 255);
            tex_buf[k+1] = (GLubyte)FTOI(buf[k + 1] * 255);
            tex_buf[k+2] = (GLubyte)FTOI(buf[k + 2] * 255);
        }
    }

    level = 0;
    while (size > 0) {
	glTexImage2D(GL_TEXTURE_2D, level++, GL_RGB, size, size, 0, GL_RGB,
		     GL_UNSIGNED_BYTE, tex_buf);
	size /= 2;
    }

    FREE(tex_buf);
}

static long DecalTest(long n)
{
    long i, j, k;
    GLftype *texBuf;

    glClientActiveTexture(GL_TEXTURE0+n);
    DrawTexQuad2i2f(0, 0, SIZE, SIZE, ITOF(0), ITOF(0), ITOF(1), ITOF(1));

    ReadScreen(0, 0, SIZE, SIZE, GL_RGB, workBuf);

    texBuf = textureBuf[n];

    for (i = 0; i < SIZE; i++) {
        for (j = 0; j < SIZE; j++) {
            k = (i * SIZE + j) * 3;
            if (ABS(workBuf[k]-texBuf[k]) >     epsilon.color[0]*2 ||
                ABS(workBuf[k+1]-texBuf[k+1]) > epsilon.color[1]*2 ||
                ABS(workBuf[k+2]-texBuf[k+2]) > epsilon.color[2]*2) {
                StrMake(errStr, errStrFmt, i, j, errEnv, errMagFilter,
                        errMinFilter, workBuf[k], workBuf[k+1],
                        workBuf[k+2], texBuf[k], texBuf[k+1],
                        texBuf[k+2]);
                ErrorReport(__FILE__, __LINE__, errStr);
                return ERROR;
            }
        }
    }

    return NO_ERROR;
}

static long ModulateAllTest(void)
{
    static const GLshort vertices[4][2] = {
       	{   0,    0},
       	{SIZE,    0},
	{   0, SIZE},
	{SIZE, SIZE}
    };
    static const GLshort tex_coords[4][2] = {
	{0, 0},
        {1, 0},
        {0, 1},
        {1, 1}
    };
    long i, j, k, n;
    GLftype expected[3];

    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(2, GL_SHORT, 0, &vertices[0][0]);

    for (n = 0; n < numTextures; n++) {
        glClientActiveTexture(GL_TEXTURE0+n);
        glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glTexCoordPointer(2, GL_SHORT, 0, &tex_coords[0][0]);
    }

    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

    glDisableClientState(GL_VERTEX_ARRAY);
    for (n = 0; n < numTextures; n++) {
        glClientActiveTexture(GL_TEXTURE0+n);
        glDisableClientState(GL_TEXTURE_COORD_ARRAY);
    }

    ReadScreen(0, 0, SIZE, SIZE, GL_RGB, workBuf);

    for (i = 0; i < SIZE; i++) {
        for (j = 0; j < SIZE; j++) {

            k = (i * SIZE + j) * 3;

            expected[0] = expected[1] = expected[2] = ITOF(1);
            for (n = 0; n < numTextures; n++) {
                expected[0] = MUL(expected[0], textureBuf[n][k]);
                expected[1] = MUL(expected[1], textureBuf[n][k+1]);
                expected[2] = MUL(expected[2], textureBuf[n][k+2]);
            }

            if (ABS(workBuf[k+0]-expected[0]) > epsilon.color[0]+epsilon.zero ||
                ABS(workBuf[k+1]-expected[1]) > epsilon.color[1]+epsilon.zero ||
                ABS(workBuf[k+2]-expected[2]) > epsilon.color[2]+epsilon.zero) {
                StrMake(errStr, errStrFmt, i, j, errEnv, errMagFilter,
                        errMinFilter, workBuf[k], workBuf[k+1],
                        workBuf[k+2], expected[0], expected[1],
                        expected[2]);
                ErrorReport(__FILE__, __LINE__, errStr);
                return ERROR;
            }
        }
    }

    return NO_ERROR;
}

static void Cleanup()
{
    long i;

    for (i = 0; i < numTextures; i++) {
        glClientActiveTexture(GL_TEXTURE0+i);
#ifdef COMMON_LITE
	/* should be GL_FLOAT, but does not exist */
	glTexCoordPointer(4, GL_FIXED, 0, NULL);
#endif
#ifdef COMMON
	glTexCoordPointer(4, GL_FLOAT, 0, NULL);
#endif
        glActiveTexture(GL_TEXTURE0+i);
        glDisable(GL_TEXTURE_2D);
        glBindTexture(GL_TEXTURE_2D, 0);
        glTexEnvt(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
        FREE(textureBuf[i]);
    }
    glClientActiveTexture(GL_TEXTURE0);
    glActiveTexture(GL_TEXTURE0);
    glDeleteTextures(numTextures, textureNames);
    FREE(workBuf);
}

long MultiTexExec(void)
{
    long i;
    GLenum envMode, minFilter, magFilter;

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glClear(GL_COLOR_BUFFER_BIT);
    glDisable(GL_DITHER);
    glDisable(GL_MULTISAMPLE);
    glColor4t(ITOF(1), ITOF(1), ITOF(1), ITOF(1));

    minFilter = GL_NEAREST;
    magFilter = GL_NEAREST;
    GetEnumName(minFilter, errMinFilter);
    GetEnumName(magFilter, errMagFilter);

    glGetIntegerv(GL_MAX_TEXTURE_UNITS, &numTextures);
    if (numTextures < 1) {
        char format[] =
            "GL_MAX_TEXTURE_UNITS is %d, should be at least 1.";
        StrMake(errStr, format, numTextures);
        ErrorReport(__FILE__, __LINE__, errStr);
        return ERROR;
    }
    if (numTextures == 1) {
	return NOT_SUPPORTED;
    }
    if (numTextures > MAX_TEXTURES) {
        /* We're only prepared to test up to 32 textures at this time.
         * The test will need to be modified to handle more, since the
         * enums may not be contiguous.
         */
        numTextures = MAX_TEXTURES;
    }

    glGenTextures(numTextures, textureNames);

    for (i = 0; i < numTextures; i++) {
        glActiveTexture(GL_TEXTURE0+i);
        glBindTexture(GL_TEXTURE_2D, textureNames[i]);
        glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, minFilter);
        glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, magFilter);
        textureBuf[i] = (GLftype *)MALLOC(SIZE*SIZE*3*sizeof(GLftype));
        Make(SIZE, textureBuf[i], i);
    }
    workBuf = (GLftype *)MALLOC(WINDSIZEX*WINDSIZEY*3*sizeof(GLftype));

    envMode = GL_DECAL;
    GetEnumName(envMode, errEnv);
    for (i = 0; i < numTextures; i++) {
        glActiveTexture(GL_TEXTURE0+i);
        glEnable(GL_TEXTURE_2D);
        glTexEnvt(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, envMode);

        if (DecalTest(i) == ERROR) {
            Cleanup();
            return ERROR;
        }

        glDisable(GL_TEXTURE_2D);
    }

    envMode = GL_MODULATE;
    GetEnumName(envMode, errEnv);
    for (i = 0; i < numTextures; i++) {
        glActiveTexture(GL_TEXTURE0+i);
        glEnable(GL_TEXTURE_2D);
        glTexEnvt(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, envMode);
    }

    if (ModulateAllTest() == ERROR) {
        Cleanup();
        return ERROR;
    }

    Cleanup();
    return NO_ERROR;
}
