/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/mustpass.c,v 1.15 2005/06/29 14:43:24 artlab Exp $
*/

/*
** mustpass.c
** Must Pass Test.
**
** Description -
**     Minimum OpenGL ES conformance tests:
**	  Alpha plane test.
**	  Blend test.
**	  Color buffer clear test.
**	  Color buffer color test.
**	  Copy texture test.
**	  Copy sub texture test.
**	  Depth offset test.
**	  Depth range test.
**	  Draw elements test.
**	  Fog test.
**	  Homogeneous coordinate test.
**	  Lighting test.
**	  Logic op test.
**	  Packed pixels test.
**	  Rescale normal test.
**	  Scissor test.
**	  Stencil test.
**	  SubTexture test.
**	  Specular color test.
**	  Texture test.
**	  Texture object test.
**	  Texture LOD control test.
**	  Vertex array test.
**	  Write mask test.
**
** Technical Specification -
**    Buffer requirements:
**	  Color buffer, depth buffer, stencil plane, alpha buffer.
**    Color requirements:
**	  No color requirements.
**    States requirements:
**	  Disabled = GL_DITHER, GL_MULTISAMPLE.
**    Error epsilon:
**    Paths:
**	  Not allowed = Alias, Alpha, Blend, Depth, Dither, Fog,
**			Shade, Stencil.
*/

#include "conform.h"
#include "util.h"
#include "utilt.h"
#include "utilp.h"

static long TestBuffer(GLint, GLint, GLftype, GLftype, GLftype);
static long TestBufferAbsolute(GLint, GLint, GLftype, GLftype, GLftype);
static long Alpha(void);
static long Blend(void);
static long Clear(void);
static long Color(void);
static long Depth(void);
static long DepthRange(void);
static long Fog(void);
static long Homogeneous(void);
static long Light(void);
static long Scissor(void);
static long Stencil(void);
static long Texture(void);
static long WriteMask(void);
static long CopyTexture(void);
static long CopySubTexture(void);
static long DepthOffset(void);
static long TextureObject(void);
static long SubTexture(void);
static long VertexArray(void);
static long LogicOp(void);
static long DrawElements(void);
static void SpecularReset(void);
static long SpecularColor(void);
static long RescaleNormal(void);
static long PackedPixels(void);
#ifdef GL_OES_VERSION_1_1
static long PointParameter(void);
static long PointSizeArray(void);
static long PointSprite(void);
static long UserClip(void);
static long TexCombine(void);
#endif

long MustPassExec(void)
{
    glDisable(GL_DITHER);
    glDisable(GL_MULTISAMPLE);

    if (Alpha() == ERROR) {
	return ERROR;
    }
    if (Blend() == ERROR) {
	return ERROR;
    }
    if (Clear() == ERROR) {
	return ERROR;
    }
    if (Color() == ERROR) {
	return ERROR;
    }
    if (buffer.zBits > 0) {
	if (Depth() == ERROR) {
	    return ERROR;
	}
	if (DepthRange() == ERROR) {
	    return ERROR;
	}
    }
    if (Fog() == ERROR) {
	return ERROR;
    }
    if (Homogeneous() == ERROR) {
	return ERROR;
    }
    if (Light() == ERROR) {
	return ERROR;
    }
    if (Scissor() == ERROR) {
	return ERROR;
    }
    if (buffer.stencilBits > 0) {
	if (Stencil() == ERROR) {
	    return ERROR;
	}
    }
    if (Texture() == ERROR) {
	return ERROR;
    }
    if (WriteMask() == ERROR) {
	return ERROR;
    }
    if (TextureObject() == ERROR) {
	return ERROR;
    }
    if (CopyTexture() == ERROR) {
	return ERROR;
    }
    if (SubTexture() == ERROR) {
	return ERROR;
    }
    if (CopySubTexture() == ERROR) {
	return ERROR;
    }
    if (LogicOp() == ERROR) {
	return ERROR;
    }
    if (VertexArray() == ERROR) {
	return ERROR;
    }
    if (buffer.zBits > 8) {  /* 8 should have enough units */
	if (DepthOffset() == ERROR) {
	    return ERROR;
	}
    }
    if (DrawElements() == ERROR) {
	return ERROR;
    }
    if (PackedPixels() == ERROR ) {
	return ERROR;
    }
    if (RescaleNormal() == ERROR) {
	return ERROR;
    }
    if (SpecularColor() == ERROR) {
	return ERROR;
    }
#ifdef GL_OES_VERSION_1_1
    if (PointParameter() == ERROR) {
	return ERROR;
    }
    if (PointSizeArray() == ERROR) {
	return ERROR;
    }
    if (PointSprite() == ERROR) {
	return ERROR;
    }
	if (UserClip() == ERROR) {
	return ERROR;
	}
    if (TexCombine() == ERROR) {
        return ERROR;
    }
#endif

    return NO_ERROR;
}

static long TestBuffer(GLint x, GLint y, GLftype r, GLftype g, GLftype b)
{
    GLftype buf[3];
    GLftype errorRGB[3];

    r = buffer.colorBits[0] == 0 ? 0 : r;
    g = buffer.colorBits[1] == 0 ? 0 : g;
    b = buffer.colorBits[2] == 0 ? 0 : b;
    errorRGB[0] = buffer.colorBits[0] == 0 ? ITOF(0) : ATOF("0.49");
    errorRGB[1] = buffer.colorBits[1] == 0 ? ITOF(0) : ATOF("0.49");
    errorRGB[2] = buffer.colorBits[2] == 0 ? ITOF(0) : ATOF("0.49");
    ReadScreen(WINDSIZEX*x/100, WINDSIZEY*y/100, 1, 1, GL_RGB, buf);

    if (ABS(buf[0]-r) > errorRGB[0] ||
	ABS(buf[1]-g) > errorRGB[1] ||
	ABS(buf[2]-b) > errorRGB[2]) {
	return ERROR;
    }
    return NO_ERROR;
}

static long TestBufferAbsolute(GLint x, GLint y, GLftype r, GLftype g, GLftype b)
{
    GLftype buf[3];
    GLftype errorRGB[3];

    r = buffer.colorBits[0] == 0 ? 0 : r;
    g = buffer.colorBits[1] == 0 ? 0 : g;
    b = buffer.colorBits[2] == 0 ? 0 : b;
    errorRGB[0] = buffer.colorBits[0] == 0 ? ITOF(0) : ATOF("0.49");
    errorRGB[1] = buffer.colorBits[1] == 0 ? ITOF(0) : ATOF("0.49");
    errorRGB[2] = buffer.colorBits[2] == 0 ? ITOF(0) : ATOF("0.49");
    ReadScreen(x, y, 1, 1, GL_RGB, buf);
    if (ABS(buf[0]-r) > errorRGB[0] ||
	ABS(buf[1]-g) > errorRGB[1] ||
	ABS(buf[2]-b) > errorRGB[2]) {
	return ERROR;
    }
    return NO_ERROR;
}

/*
** Alpha Test.
** Set up alpha function and alpha reference value. Draw primitive
** with alpha values such that the alpha function should not pass.
** Draw primitive again with alpha values such that the alpha
** function will pass.	*/
static long Alpha(void)
{

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    /*
    ** Clear screen.
    */
    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glClear(GL_COLOR_BUFFER_BIT);

    /*
    ** Set up alpha function.
    */
    glEnable(GL_ALPHA_TEST);
    glAlphaFunct(GL_NOTEQUAL, ITOF(0));

    /*
    ** Draw a polygon with a alpha value of 0.0.
    ** Polygon should not render.
    */
    glColor4t(ITOF(1), ITOF(1), ITOF(1), ITOF(0));
    DrawQuad2i(WINDSIZEX/4, WINDSIZEY/4, WINDSIZEX*3/4, WINDSIZEY*3/4);

    /*
    ** Check if polygon did not render.
    */
    if (TestBuffer(50, 50, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Alpha test.");
	return ERROR;
    }

    /*
    ** Draw a polygon with a alpha value of 1.0.
    ** Polygon should render.
    */
    glColor4t(ITOF(1), ITOF(1), ITOF(1), ITOF(1));
    DrawQuad2i(WINDSIZEX/4, WINDSIZEY/4, WINDSIZEX*3/4, WINDSIZEY*3/4);

    /*
    ** Check if polygon rendered.
    */
    if (TestBuffer(50, 50, ITOF(1), ITOF(1), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Alpha test.");
	return ERROR;
    }

    glDisable(GL_ALPHA_TEST);
    return NO_ERROR;
}

/*
** Blend Test.
** Draw primitive with known color (setting blend destination
** color). Enable blending. Draw same primitive in another
** known color (setting blend source color). Compare rendered
** color to calculated results.
*/
static long Blend(void)
{

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    /*
    ** Set destination color to zeros.
    */
    glDisable(GL_BLEND);
    glColor4t(ITOF(0), ITOF(0), ITOF(0), ITOF(1));
    DrawPoint2f(ITOF(1)/2, ITOF(1)/2);

    /*
    ** Set blend function.
    */
    glBlendFunc(GL_ZERO, GL_ONE);

    /*
    ** Set source color to ones.
    */
    glEnable(GL_BLEND);
    glColor4t(ITOF(1), ITOF(1), ITOF(1), ITOF(1));
    DrawPoint2f(ITOF(1)/2, ITOF(1)/2);

    /*
    ** Check that the blended color is the destination color.
    */
    if (TestBuffer(0, 0, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Blend test.");
	return ERROR;
    }

    /*
    ** Set destination color to zeros.
    */
    glDisable(GL_BLEND);
    glColor4t(ITOF(0), ITOF(0), ITOF(0), ITOF(1));
    DrawPoint2f(ITOF(1)/2, ITOF(1)/2);

    /*
    ** Set blend function.
    */
    glBlendFunc(GL_ONE, GL_ZERO);

    /*
    ** Set source color to ones.
    */
    glEnable(GL_BLEND);
    glColor4t(ITOF(1), ITOF(1), ITOF(1), ITOF(1));
    DrawPoint2f(ITOF(1)/2, ITOF(1)/2);

    /*
    ** Check that the blended color is the source color.
    */
    if (TestBuffer(0, 0, ITOF(1), ITOF(1), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Blend test.");
	return ERROR;
    }

    glDisable(GL_BLEND);
    return NO_ERROR;
}

/*
** Clear Test.
** Set the clear value. Check that the screen cleared to the
** correct value. Repeat test with a different clear value.
*/
static long Clear(void)
{

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(1));
    glClear(GL_COLOR_BUFFER_BIT);
    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Clear test.");
	return ERROR;
    }

    glClearColort(ITOF(1), ITOF(1), ITOF(1), ITOF(1));
    glClear(GL_COLOR_BUFFER_BIT);
    if (TestBuffer(30, 30, ITOF(1), ITOF(1), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Clear test.");
	return ERROR;
    }

    return NO_ERROR;
}

/*
** Draw primitive with know color. Check that the primitive
** rendered in the correct color.
*/
static long Color(void)
{
    static const GLftype vertices[3][2] = {
	{ITOF(WINDSIZEX)/4, ITOF(WINDSIZEY)/4},
	{ITOF(WINDSIZEX)/4, ITOF(WINDSIZEY)*3/4},
	{ITOF(WINDSIZEX)*3/4, ITOF(WINDSIZEY)/4}
    };

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(1));
    glClear(GL_COLOR_BUFFER_BIT);

    glColor4t(ITOF(0), ITOF(1), ITOF(0), ITOF(1));

    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(2, GL_FTYPE, 0, &vertices[0][0]);
    glDrawArrays(GL_TRIANGLES, 0, 3);
    glDisableClientState(GL_VERTEX_ARRAY);

    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Color test.");
	return ERROR;
    }
    if (TestBuffer(30, 30, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Color test.");
	return ERROR;
    }

    return NO_ERROR;
}

/*
** Depth Test.
** Draw a green polygon. Draw an intersecting red
** polygon. Check that the whole image is red (since the red
** polygon came second and depth is not enabled.) Clear,
** enable the depth-test, and then redraw the green polygon
** and check two points are green. Draw the intersecting red
** polygon, Check that one of the above two points is now red,
** and the other is still green.
*/
static long Depth(void)
{
    static const GLftype vertices1[4][3] = {
	{ITOF(WINDSIZEX)/4,   ITOF(WINDSIZEY)*3/4, ITOF(-1)},
	{ITOF(WINDSIZEX)/4,   ITOF(WINDSIZEY)/4,   ITOF(-1)},
	{ITOF(WINDSIZEX)*3/4, ITOF(WINDSIZEY)*3/4, ITOF(0)},
	{ITOF(WINDSIZEX)*3/4, ITOF(WINDSIZEY)/4,   ITOF(0)}
    };
    static const GLftype vertices2[4][3] = {
	{ITOF(WINDSIZEX)/4,   ITOF(WINDSIZEY)*3/4, ITOF(0)},
	{ITOF(WINDSIZEX)/4,   ITOF(WINDSIZEY)/4,   ITOF(0)},
	{ITOF(WINDSIZEX)*3/4, ITOF(WINDSIZEY)*3/4, ITOF(-1)},
	{ITOF(WINDSIZEX)*3/4, ITOF(WINDSIZEY)/4,   ITOF(-1)}
    };
    static const GLftype identity[16] = {
	ITOF(1), ITOF(0), ITOF(0), ITOF(0),
	ITOF(0), ITOF(1), ITOF(0), ITOF(0),
	ITOF(0), ITOF(0), ITOF(1), ITOF(0),
	ITOF(0), ITOF(0), ITOF(0), ITOF(1)
    };

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(1));
    glClear(GL_COLOR_BUFFER_BIT);

    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadMatrixt(identity);
    glOrthot(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY), ITOF(1), ITOF(-1));
    glMatrixMode(GL_MODELVIEW);

    glEnableClientState(GL_VERTEX_ARRAY);

    /*
    ** Check that second object drawn is visible if depth test off.
    */
    glColor4t(ITOF(0), ITOF(1), ITOF(0), ITOF(1));
    glVertexPointer(3, GL_FTYPE, 0, &vertices1[0][0]);
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

    glColor4t(ITOF(1), ITOF(0), ITOF(0), ITOF(1));
    glVertexPointer(3, GL_FTYPE, 0, &vertices2[0][0]);
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

    glDisableClientState(GL_VERTEX_ARRAY);

    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Depth test.");
	return ERROR;
    }
    if (TestBuffer(30, 50, ITOF(1), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Depth test.");
	return ERROR;
    }
    if (TestBuffer(65, 50, ITOF(1), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Depth test.");
	return ERROR;
    }

    /*
    ** Check that green object is visible when depth test first turned on.
    */
    glDepthRanget(ITOF(-1), ITOF(1));
    glEnable(GL_DEPTH_TEST);
    glClear(GL_DEPTH_BUFFER_BIT);

    glColor4t(ITOF(0), ITOF(1), ITOF(0), ITOF(1));
    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(3, GL_FTYPE, 0, &vertices1[0][0]);
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
    glDisableClientState(GL_VERTEX_ARRAY);

    if (TestBuffer(30, 50, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Depth test.");
	return ERROR;
    }
    if (TestBuffer(65, 50, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Depth test.");
	return ERROR;
    }

    /*
    ** Check that half red and half green object is visible when depth test on
    ** after first drawing.
    */
    glColor4t(ITOF(1), ITOF(0), ITOF(0), ITOF(1));
    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(3, GL_FTYPE, 0, &vertices2[0][0]);
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
    glDisableClientState(GL_VERTEX_ARRAY);

    if (TestBuffer(30, 50, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Depth test.");
	return ERROR;
    }
    if (TestBuffer(65, 50, ITOF(1), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Depth test.");
	return ERROR;
    }

    glDisable(GL_DEPTH_TEST);
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);

    return NO_ERROR;
}

static void DrawQuad(GLint x1, GLint y1, GLint x2, GLint y2, GLint z)
{
    GLshort vertices[4][3];
    vertices[0][0] = (GLshort)x1; vertices[0][1] = (GLshort)y1;
    vertices[1][0] = (GLshort)x2; vertices[1][1] = (GLshort)y1;
    vertices[2][0] = (GLshort)x1; vertices[2][1] = (GLshort)y2;
    vertices[3][0] = (GLshort)x2; vertices[3][1] = (GLshort)y2;

    vertices[0][2] =
    vertices[1][2] =
    vertices[2][2] =
    vertices[3][2] = z;

    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(3, GL_SHORT, 0, &vertices[0][0]);
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
    glDisableClientState(GL_VERTEX_ARRAY);
}

/*
** Depth Range Test.
** Setup a reversed depth range.
** Draw a red polygon closer to the far plane.
** Draw a greed polygon closer to the near plane.
** Verify that the red polygon is visible.
** Repeat with default depth range and verify
** that the green polygon is visible.
*/
static long DepthRange(void)
{
    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrthot(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY), ITOF(-2), ITOF(2));
    glMatrixMode(GL_MODELVIEW);

    glEnable(GL_DEPTH_TEST);

    /*
    ** Test reversed depth range.
    */

    glDepthRanget(ITOF(1), ITOF(0));

    glColor4t(ITOF(1), ITOF(0), ITOF(0), ITOF(1));
    DrawQuad(0, 0, WINDSIZEX, WINDSIZEY, -1);

    glColor4t(ITOF(0), ITOF(1), ITOF(0), ITOF(1));
    DrawQuad(0, 0, WINDSIZEX, WINDSIZEY, +1);

    if (TestBuffer(10, 10, ITOF(1), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Depth Range test.");
	return ERROR;
    }

    /*
    ** Test default depth range.
    */

    glDepthRanget(ITOF(0), ITOF(1));
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glColor4t(ITOF(1), ITOF(0), ITOF(0), ITOF(1));
    DrawQuad(0, 0, WINDSIZEX, WINDSIZEY, -1);

    glColor4t(ITOF(0), ITOF(1), ITOF(0), ITOF(1));
    DrawQuad(0, 0, WINDSIZEX, WINDSIZEY, +1);

    if (TestBuffer(10, 10, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Depth Range test.");
	return ERROR;
    }

    glDisable(GL_DEPTH_TEST);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glMatrixMode(GL_MODELVIEW);

    return NO_ERROR;
}

/*
** Fog Test.
** Check that a primitive which is well beyond the
** linear fog end value is entirely the current color when fog
** is disabled, and is entirely the fog color when fog is
** enabled.
*/
static long Fog(void)
{
    static const GLftype red[4] = {
	ITOF(1), ITOF(0), ITOF(0), ITOF(1)
    };
    static const GLftype vertices[4][3] = {
	{ITOF(WINDSIZEX)/4,   ITOF(WINDSIZEY)/4,   ITOF(-3)/4},
	{ITOF(WINDSIZEX)/4,   ITOF(WINDSIZEY)*3/4, ITOF(-3)/4},
	{ITOF(WINDSIZEX)*3/4, ITOF(WINDSIZEY)*3/4, ITOF(-3)/4},
	{ITOF(WINDSIZEX)*3/4, ITOF(WINDSIZEY)/4,   ITOF(-3)/4}
    };
    static const GLftype identity[16] = {
	ITOF(1), ITOF(0), ITOF(0), ITOF(0),
	ITOF(0), ITOF(1), ITOF(0), ITOF(0),
	ITOF(0), ITOF(0), ITOF(1), ITOF(0),
	ITOF(0), ITOF(0), ITOF(0), ITOF(1)
    };

    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadMatrixt(identity);
    glOrthot(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY), ITOF(1), ITOF(-1));
    glMatrixMode(GL_MODELVIEW);

    /*
    ** Choose z, fogEnd and fogStart values so that fog factor will
    ** clamp to zero for all known fog algorithms.
    ** (z, z-interpolation from vertex z, distance, distance-interpolation)
    */
    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(1));
    glClear(GL_COLOR_BUFFER_BIT);

    glColor4t(ITOF(0), ITOF(1), ITOF(0), ITOF(1));

    glFogtv(GL_FOG_COLOR, red);
    glFogt(GL_FOG_START, ITOF(0));
    glFogt(GL_FOG_END, ITOF(1)/2);
    glFogt(GL_FOG_MODE, GL_LINEAR);

    /*
    ** Check that object drawn is colored since fog is disabled.
    */
    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(3, GL_FTYPE, 0, &vertices[0][0]);
    glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
    glDisableClientState(GL_VERTEX_ARRAY);

    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Fog test.");
	return ERROR;
    }
    if (TestBuffer(30, 30, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Fog test.");
	return ERROR;
    }
    if (TestBuffer(65, 65, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Fog test.");
	return ERROR;
    }

    /*
    ** Check that object drawn is fogged now that fog enabled.
    */
    glEnable(GL_FOG);
    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(3, GL_FTYPE, 0, &vertices[0][0]);
    glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
    glDisableClientState(GL_VERTEX_ARRAY);

    /*
    ** The object should be only the fog color.
    */
    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Fog test.");
	return ERROR;
    }
    if (TestBuffer(30, 30, ITOF(1), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Fog test.");
	return ERROR;
    }
    if (TestBuffer(65, 65, ITOF(1), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Fog test.");
	return ERROR;
    }

    glDisable(GL_FOG);
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);

    return NO_ERROR;
}

static void DrawQuad4f(GLftype x1, GLftype y1, GLftype x2, GLftype y2, GLftype q)
{
    GLftype v[4][4];
    v[0][0] = x1; v[0][1] = y1; v[0][2] = ITOF(0); v[0][3] = q;
    v[1][0] = x2; v[1][1] = y1; v[1][2] = ITOF(0); v[1][3] = q;
    v[2][0] = x1; v[2][1] = y2; v[2][2] = ITOF(0); v[2][3] = q;
    v[3][0] = x2; v[3][1] = y2; v[3][2] = ITOF(0); v[3][3] = q;

    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(4, GL_FTYPE, 0, &v[0][0]);
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
    glDisableClientState(GL_VERTEX_ARRAY);
}

/*
** Homogeneous Coordinate Test.
** Draw a polygon with 4th coordinates clamped to 1.0.
** Check that a point outside of the polygon is clear and that
** points near the vertices of the polygon are the current
** color/index. Multiply all of the vertex coordinates by
** 10.0 (including the 4th coordinate). Examine the same points
** checked previously. There should be no observable effect.
** Multiply all of the vertex coordinates by 1/10 (including
** the 4th coordinate). Repeat the above tests.
*/
static long Homogeneous(void)
{

    Ortho3D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY), ITOF(10), ITOF(-10));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(1));
    glClear(GL_COLOR_BUFFER_BIT);

    glColor4t(ITOF(0), ITOF(1), ITOF(0), ITOF(1));
    DrawQuad4f(ITOF(WINDSIZEX)/4, ITOF(WINDSIZEY)/4, ITOF(WINDSIZEX)*3/4, ITOF(WINDSIZEY)*3/4, ITOF(1));

    /*
    ** Outside.
    */
    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Homogeneous test.");
	return ERROR;
    }

    /*
    ** Inside.
    */
    if (TestBuffer(30, 30, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Homogeneous test.");
	return ERROR;
    }
    if (TestBuffer(30, 70, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Homogeneous test.");
	return ERROR;
    }
    if (TestBuffer(70, 70, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Homogeneous test.");
	return ERROR;
    }
    if (TestBuffer(70, 30, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Homogeneous test.");
	return ERROR;
    }

    glClear(GL_COLOR_BUFFER_BIT);

    glColor4t(ITOF(1), ITOF(0), ITOF(0), ITOF(1));
    DrawQuad4f(ITOF(WINDSIZEX)*5/2, ITOF(WINDSIZEY)*5/2, ITOF(WINDSIZEX)*15/2, ITOF(WINDSIZEY)*15/2, ITOF(10));

    /*
    ** Outside.
    */
    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Homogeneous test.");
	return ERROR;
    }

    /*
    ** Inside.
    */
    if (TestBuffer(30, 30, ITOF(1), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Homogeneous test.");
	return ERROR;
    }
    if (TestBuffer(30, 70, ITOF(1), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Homogeneous test.");
	return ERROR;
    }
    if (TestBuffer(70, 70, ITOF(1), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Homogeneous test.");
	return ERROR;
    }
    if (TestBuffer(70, 30, ITOF(1), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Homogeneous test.");
	return ERROR;
    }

    glClear(GL_COLOR_BUFFER_BIT);

    glColor4t(ITOF(0), ITOF(1), ITOF(0), ITOF(1));
    DrawQuad4f(ITOF(5)/2, ITOF(5)/2, ITOF(15)/2, ITOF(15)/2, ATOF("0.1"));
    DrawQuad4f(
        MUL(ITOF(WINDSIZEX), ATOF("0.025")),
        MUL(ITOF(WINDSIZEY), ATOF("0.025")),
        MUL(ITOF(WINDSIZEX), ATOF("0.075")),
        MUL(ITOF(WINDSIZEY), ATOF("0.075")),
        ATOF("0.1"));

    /*
    ** Outside.
    */
    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Homogeneous test.");
	return ERROR;
    }

    /*
    ** Inside.
    */
    if (TestBuffer(30, 30, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Homogeneous test.");
	return ERROR;
    }
    if (TestBuffer(30, 70, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Homogeneous test.");
	return ERROR;
    }
    if (TestBuffer(70, 70, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Homogeneous test.");
	return ERROR;
    }
    if (TestBuffer(70, 30, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Homogeneous test.");
	return ERROR;
    }

    return NO_ERROR;
}

/*
** Lighting Test.
** Initialize lighting parameters to green ambient light. Set
** current color to red. Draw with light disabled. Check
** that the primitive is red. Enable lighting and redraw
** the same primitive. Check that the primitive is green.
*/
static long Light(void)
{
    GLftype buf[4];

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(1));
    glClear(GL_COLOR_BUFFER_BIT);

    /*
    ** Set ambient to red.
    */
    buf[0] = ITOF(1);
    buf[1] = ITOF(0);
    buf[2] = ITOF(0);
    buf[3] = ITOF(1);
    glLightModeltv(GL_LIGHT_MODEL_AMBIENT, buf);
    glMaterialtv(GL_FRONT_AND_BACK, GL_AMBIENT, buf);

    glColor4t(ITOF(0), ITOF(1), ITOF(0), ITOF(1));

    /*
    ** Check that object drawn is colored green since light is disabled.
    */
    DrawQuad2f(ITOF(WINDSIZEX)/4, ITOF(WINDSIZEY)/4, ITOF(WINDSIZEX)*3/4, ITOF(WINDSIZEY)*3/4);

    if (TestBuffer(30, 30, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Light test.");
	return ERROR;
    }

    /*
    ** Check that object drawn is lit, now that lighting is enabled.
    */
    glEnable(GL_LIGHTING);
    DrawQuad2f(ITOF(WINDSIZEX)/4, ITOF(WINDSIZEY)/4, ITOF(WINDSIZEX)*3/4, ITOF(WINDSIZEY)*3/4);
    glDisable(GL_LIGHTING);

    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Light test.");
	return ERROR;
    }
    if (TestBuffer(30, 30, ITOF(1), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Light test.");
	return ERROR;
    }

    return NO_ERROR;
}

/*
** Scissor Test.
** Set up scissor box. Draw primitive. Check that primitive only
** rendered in scissor box.
*/
static long Scissor(void)
{

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glColor4t(ITOF(1), ITOF(1), ITOF(1), ITOF(1));

    glScissor(WINDSIZEX/10, WINDSIZEY/10, WINDSIZEX/10, WINDSIZEY/10);

    /*
    ** Clear screen.
    */
    glDisable(GL_SCISSOR_TEST);
    glClear(GL_COLOR_BUFFER_BIT);

    /*
    ** Draw polygon over entire screen.
    */
    glEnable(GL_SCISSOR_TEST);
    DrawQuad2f(ITOF(0), ITOF(0), ITOF(WINDSIZEX), ITOF(WINDSIZEY));

    /*
    ** Check if polygon rendered in scissor area.
    */
    if (TestBuffer(15, 15, ITOF(1), ITOF(1), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Scissor test.");
	return ERROR;
    }

    /*
    ** Check if polygon did not rendered outside scissor area.
    */
    if (TestBuffer(5, 5, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Scissor test.");
	return ERROR;
    }

    glDisable(GL_SCISSOR_TEST);
    return NO_ERROR;
}

/*
** Stencil Test.
** Set the stencil function so that a primitive should fail
** the stencil test. Clear the screen, draw the primitive,
** and check that it did not render. Change the function so
** that the primitive should pass. Draw the primitive and
** check that it did render.
*/
static long Stencil(void)
{
    static const GLftype vertices[4][2] = {
	{(ITOF(20) * WINDSIZEX)/48, (ITOF(20) * WINDSIZEY)/48},
	{(ITOF(30) * WINDSIZEX)/48, (ITOF(80) * WINDSIZEY)/48},
	{(ITOF(80) * WINDSIZEX)/48, (ITOF(30) * WINDSIZEY)/48},
	{(ITOF(70) * WINDSIZEX)/48, (ITOF(80) * WINDSIZEY)/48}
    };

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glClearStencil(1);
    glClear(GL_COLOR_BUFFER_BIT|GL_STENCIL_BUFFER_BIT);

    glColor4t(ITOF(0), ITOF(0), ITOF(1), ITOF(1));

    glEnable(GL_STENCIL_TEST);
    glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);

    glStencilFunc(GL_GREATER, 0, ~0);

    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(2, GL_FTYPE, 0, &vertices[0][0]);
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
    glDisableClientState(GL_VERTEX_ARRAY);

    if (TestBuffer(50, 50, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Stencil test.");
	return ERROR;
    }

    glStencilFunc(GL_LEQUAL, 0, ~0);

    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(2, GL_FTYPE, 0, &vertices[0][0]);
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
    glDisableClientState(GL_VERTEX_ARRAY);

    if (TestBuffer(50, 50, ITOF(0), ITOF(0), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Stencil test.");
	return ERROR;
    }

    glDisable(GL_STENCIL_TEST);
    return NO_ERROR;
}

static void
DrawTexQuad2s2b(GLshort x1, GLshort y1, GLshort x2, GLshort y2,
    GLbyte s1, GLbyte t1, GLbyte s2, GLbyte t2)
{
    GLshort vertices[4][2];
    GLbyte texcoords[4][2];

    vertices[0][0] = x1; vertices[0][1] = y1;
    vertices[1][0] = x2; vertices[1][1] = y1;
    vertices[2][0] = x1; vertices[2][1] = y2;
    vertices[3][0] = x2; vertices[3][1] = y2;

    texcoords[0][0] = s1; texcoords[0][1] = t1;
    texcoords[1][0] = s2; texcoords[1][1] = t1;
    texcoords[2][0] = s1; texcoords[2][1] = t2;
    texcoords[3][0] = s2; texcoords[3][1] = t2;

    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(2, GL_SHORT, 0, &vertices[0][0]);
    glEnableClientState(GL_TEXTURE_COORD_ARRAY);
    glTexCoordPointer(2, GL_BYTE, 0, &texcoords[0][0]);
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
    glDisableClientState(GL_VERTEX_ARRAY);
}

/*
** Texture Test.
** Initialize a texture 1/2 green and 1/2 magenta. Draw an
** object with texture disabled. Check that it is rendered the
** current color, white.  Enable texture. Draw the same
** object. Check that the object is green on the left and
** magenta on the right.
*/
static long Texture(void)
{
    static const GLubyte t[] = {
	0, 255, 0,
	255, 0, 255
    };

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glClear(GL_COLOR_BUFFER_BIT);

    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 2, 1, 0, GL_RGB, GL_UNSIGNED_BYTE, t);
    glTexEnvt(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

    glColor4t(ITOF(1), ITOF(1), ITOF(1), ITOF(1));

    glMatrixMode(GL_TEXTURE);
    glLoadIdentity();
    glScalet(ITOF(1)/4, ITOF(1)/4, ITOF(1)/4);
    glMatrixMode(GL_MODELVIEW);

    DrawTexQuad2s2b(WINDSIZEX/4, WINDSIZEY/4, WINDSIZEX*3/4, WINDSIZEY*3/4, 1, 1, 3, 3);

    glMatrixMode(GL_TEXTURE);
    glLoadIdentity();
    glMatrixMode(GL_MODELVIEW);

    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Texture test.");
	return ERROR;
    }
    if (TestBuffer(35, 50, ITOF(1), ITOF(1), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Texture test.");
	return ERROR;
    }
    if (TestBuffer(65, 50, ITOF(1), ITOF(1), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Texture test.");
	return ERROR;
    }

    glEnable(GL_TEXTURE_2D);

    glMatrixMode(GL_TEXTURE);
    glLoadIdentity();
    glScalet(ITOF(1)/4, ITOF(1)/4, ITOF(1)/4);
    glMatrixMode(GL_MODELVIEW);

    DrawTexQuad2s2b(WINDSIZEX/4, WINDSIZEY/4, WINDSIZEX*3/4, WINDSIZEY*3/4, 1, 1, 3, 3);

    glMatrixMode(GL_TEXTURE);
    glLoadIdentity();
    glMatrixMode(GL_MODELVIEW);

    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Texture test.");
	return ERROR;
    }
    if (TestBuffer(35, 50, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Texture test.");
	return ERROR;
    }
    if (TestBuffer(65, 50, ITOF(1), ITOF(0), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Texture test.");
	return ERROR;
    }

    glTexEnvt(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glDisable(GL_TEXTURE_2D);

    return NO_ERROR;
}

/*
** WriteMask Test.
** Disable all writing, make sure nothing gets written.
** Enable all writing, make sure everything gets written.
*/
static long WriteMask(void)
{
    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glClear(GL_COLOR_BUFFER_BIT);

    glColor4t(ITOF(1), ITOF(1), ITOF(1), ITOF(1));
    glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
    DrawQuad2i(WINDSIZEX/5, WINDSIZEY/5, WINDSIZEX*4/5, WINDSIZEY*4/5);
    if (TestBuffer(50, 50, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
        ErrorReport(__FILE__, __LINE__, "Write mask test.");
        return ERROR;
    }

    glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
    DrawQuad2i(WINDSIZEX/5, WINDSIZEY/5, WINDSIZEX*4/5, WINDSIZEY*4/5);
    if (TestBuffer(50, 50, ITOF(1), ITOF(1), ITOF(1)) == ERROR) {
        ErrorReport(__FILE__, __LINE__, "Write mask test.");
        return ERROR;
    }

    return NO_ERROR;
}

static long TextureObject(void)
{
    static const GLubyte t0[] = {
	0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255, 0,
	0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255, 0,
	0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255, 0,
	0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255, 0,
	0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255, 0,
	0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255, 0,
	0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255, 0,
	0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255, 0,
	0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255, 0,
	0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255, 0,
	0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255, 0,
	0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255, 0,
	0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255, 0,
	0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255, 0,
	0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255, 0,
	0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255, 0
    };
    static const GLubyte t1[] = {
	0, 0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255,
	0, 0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255,
	0, 0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255,
	0, 0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255,
	0, 0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255,
	0, 0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255,
	0, 0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255,
	0, 0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255,
	0, 0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255,
	0, 0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255,
	0, 0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255,
	0, 0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255,
	0, 0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255,
	0, 0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255,
	0, 0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255,
	0, 0, 255, 0, 0, 255, 0, 0, 255, 0, 0, 255
    };
    GLuint tName[2];

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glClear(GL_COLOR_BUFFER_BIT);

    tName[0] = 1;
    tName[1] = 1;

    glGenTextures(2, tName);

    /*
    ** Bind first texture tName[0]
    */

    glBindTexture(GL_TEXTURE_2D, tName[0]);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 8, 8, 0, GL_RGB, GL_UNSIGNED_BYTE, t0);
    glTexEnvt(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

    /*
    ** Bind next texture tName[1]
    */

    glBindTexture(GL_TEXTURE_2D, tName[1]);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 8, 8, 0, GL_RGB, GL_UNSIGNED_BYTE, t1);
    glTexEnvt(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

    /*
    ** draw untextured
    */
    glColor4t(ITOF(1), ITOF(1), ITOF(1), ITOF(1));
    DrawTexQuad2i2f(WINDSIZEX/4, WINDSIZEY/4, WINDSIZEX*3/4, WINDSIZEY*3/4, ITOF(1)/4, ITOF(1)/4, ITOF(3)/4, ITOF(3)/4);


    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Texture Object test.");
	return ERROR;
    }
    if (TestBuffer(35, 50, ITOF(1), ITOF(1), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Texture Object test.");
	return ERROR;
    }
    if (TestBuffer(65, 50, ITOF(1), ITOF(1), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Texture Object test.");
	return ERROR;
    }

    /*
    **	draw with second texture Object
    */
    glEnable(GL_TEXTURE_2D);
    DrawTexQuad2i2f(WINDSIZEX/4, WINDSIZEY/4, WINDSIZEX*3/4, WINDSIZEY*3/4, ITOF(1)/4, ITOF(1)/4, ITOF(3)/4, ITOF(3)/4);

    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) { /* outside */
	ErrorReport(__FILE__, __LINE__, "Texture Object test.");
	return ERROR;
    }
    if (TestBuffer(50, 50, ITOF(0), ITOF(0), ITOF(1)) == ERROR) { /* inside */
	ErrorReport(__FILE__, __LINE__, "Texture Object test.");
	return ERROR;
    }

    /*
    **	draw with first texture Object
    **	should still be enabled
    */

    glBindTexture(GL_TEXTURE_2D, tName[0]);
    DrawTexQuad2i2f(WINDSIZEX/4, WINDSIZEY/4, WINDSIZEX*3/4, WINDSIZEY*3/4, ITOF(1)/4, ITOF(1)/4, ITOF(3)/4, ITOF(3)/4);

    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) { /* outside */
	ErrorReport(__FILE__, __LINE__, "Texture Object test.");
	return ERROR;
    }
    if (TestBuffer(50, 50, ITOF(0), ITOF(1), ITOF(0)) == ERROR) { /* inside */
	ErrorReport(__FILE__, __LINE__, "Texture Object test.");
	return ERROR;
    }

    /*
    ** Clean up
    */
    glDisable(GL_TEXTURE_2D);

    glBindTexture(GL_TEXTURE_2D, 0);
    glDeleteTextures(2, tName);

    return NO_ERROR;
}

static long CopyTexture(void)
{
    static const GLubyte t[] = {
	0, 255, 0,	0, 255, 0,	0, 255, 0,	0, 255, 0,
	255, 0, 255,	255, 0, 255,	255, 0, 255,	255, 0, 255,

	0, 255, 0,	0, 255, 0,	0, 255, 0,	0, 255, 0,
	255, 0, 255,	255, 0, 255,	255, 0, 255,	255, 0, 255,

	0, 255, 0,	0, 255, 0,	0, 255, 0,	0, 255, 0,
	255, 0, 255,	255, 0, 255,	255, 0, 255,	255, 0, 255,

	0, 255, 0,	0, 255, 0,	0, 255, 0,	0, 255, 0,
	255, 0, 255,	255, 0, 255,	255, 0, 255,	255, 0, 255,
    };
    GLenum colorBufferFormat, textureFormats[5];

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glClear(GL_COLOR_BUFFER_BIT);

    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 8, 4, 0, GL_RGB, GL_UNSIGNED_BYTE, t);
    glTexEnvt(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

    glColor4t(ITOF(0), ITOF(0), ITOF(1), ITOF(1));
    DrawTexQuad2i2f(WINDSIZEX/4, WINDSIZEY/4, WINDSIZEX*3/4, WINDSIZEY*3/4, ITOF(1)/4, ITOF(1)/4, ITOF(3)/4, ITOF(3)/4);

    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Texture test.");
	return ERROR;
    }
    if (TestBuffer(35, 50, ITOF(0), ITOF(0), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Texture test.");
	return ERROR;
    }
    if (TestBuffer(65, 50, ITOF(0), ITOF(0), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Texture test.");
	return ERROR;
    }

    glEnable(GL_TEXTURE_2D);
    DrawTexQuad2i2f(WINDSIZEX/4, WINDSIZEY/4, WINDSIZEX*3/4, WINDSIZEY*3/4, ITOF(1)/4, ITOF(1)/4, ITOF(3)/4, ITOF(3)/4);

    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Texture test.");
	return ERROR;
    }
    if (TestBuffer(35, 50, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Texture test.");
	return ERROR;
    }
    if (TestBuffer(65, 50, ITOF(1), ITOF(0), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Texture test.");
	return ERROR;
    }

    glDisable(GL_TEXTURE_2D);
    glColor4t(ITOF(0), ITOF(0), ITOF(1), ITOF(1));
    DrawTexQuad2i2f(WINDSIZEX*10/100, WINDSIZEY*10/100, WINDSIZEX*60/100, WINDSIZEY*60/100, ITOF(1)/4, ITOF(1)/4, ITOF(3)/4, ITOF(3)/4);

    if (TestBuffer(45, 45, ITOF(0), ITOF(0), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Texture test.");
	return ERROR;
    }

    colorBufferFormat = GuessColorBufferFormat();
    (void) CalcCopyTextureTargetFormats(colorBufferFormat, textureFormats);
    /* textureFormat[0] is closest matching format */

    glCopyTexImage2D(GL_TEXTURE_2D, 0, textureFormats[0], 20, 20, 8, 4, 0);
    glClear(GL_COLOR_BUFFER_BIT);

    glColor4t(ITOF(1), ITOF(1), ITOF(1), ITOF(1));
    DrawTexQuad2i2f(WINDSIZEX/4, WINDSIZEY/4, WINDSIZEX*3/4, WINDSIZEY*3/4, ITOF(1)/4, ITOF(1)/4, ITOF(3)/4, ITOF(3)/4);

    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Texture test.");
	return ERROR;
    }
    if (TestBuffer(35, 50, ITOF(1), ITOF(1), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Texture test.");
	return ERROR;
    }
    if (TestBuffer(65, 50, ITOF(1), ITOF(1), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Texture test.");
	return ERROR;
    }

    /* Should be the color of the quad in the lower left*/
    glEnable(GL_TEXTURE_2D);

    glColor4t(ITOF(1), ITOF(1), ITOF(1), ITOF(1));
    DrawTexQuad2i2f(WINDSIZEX/4, WINDSIZEY/4, WINDSIZEX*3/4, WINDSIZEY*3/4, ITOF(1)/4, ITOF(1)/4, ITOF(3)/4, ITOF(3)/4);

    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Texture test.");
	return ERROR;
    }
    if (TestBuffer(35, 50, ITOF(0), ITOF(0), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Texture test.");
	return ERROR;
    }
    if (TestBuffer(65, 50, ITOF(0), ITOF(0), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Texture test.");
	return ERROR;
    }

    glTexEnvt(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glDisable(GL_TEXTURE_2D);

    return NO_ERROR;
}

static long SubTexture(void)
{
    static const GLubyte t[] = {
	  0, 255,   0,   0, 255,   0,	  0, 255,   0,   0, 255,   0,
	255,   0, 255, 255,   0, 255,	255,   0, 255, 255,   0, 255,
	  0, 255,   0,   0, 255,   0,	  0, 255,   0,   0, 255,   0,
	255,   0, 255, 255,   0, 255,	255,   0, 255, 255,   0, 255,
	  0, 255,   0,   0, 255,   0,	  0, 255,   0,   0, 255,   0,
	255,   0, 255, 255,   0, 255,	255,   0, 255, 255,   0, 255,
	  0, 255,   0,   0, 255,   0,	  0, 255,   0,   0, 255,   0,
	255,   0, 255, 255,   0, 255,	255,   0, 255, 255,   0, 255,
	  0, 255,   0,   0, 255,   0,	  0, 255,   0,   0, 255,   0,
	255,   0, 255, 255,   0, 255,	255,   0, 255, 255,   0, 255,
	  0, 255,   0,   0, 255,   0,	  0, 255,   0,   0, 255,   0,
	255,   0, 255, 255,   0, 255,	255,   0, 255, 255,   0, 255,
	  0, 255,   0,   0, 255,   0,	  0, 255,   0,   0, 255,   0,
	255,   0, 255, 255,   0, 255,	255,   0, 255, 255,   0, 255,
	  0, 255,   0,   0, 255,   0,	  0, 255,   0,   0, 255,   0,
	255,   0, 255, 255,   0, 255,	255,   0, 255, 255,   0, 255,
    };
    static const GLubyte tSub[] = {
	  0, 255, 255,   0, 255, 255,   0, 255, 255,   0, 255, 255,
	  0, 255, 255,   0, 255, 255,   0, 255, 255,   0, 255, 255,
	  0, 255, 255,   0, 255, 255,   0, 255, 255,   0, 255, 255,
	  0, 255, 255,   0, 255, 255,   0, 255, 255,   0, 255, 255,
	  0, 255, 255,   0, 255, 255,   0, 255, 255,   0, 255, 255,
	  0, 255, 255,   0, 255, 255,   0, 255, 255,   0, 255, 255,
	  0, 255, 255,   0, 255, 255,   0, 255, 255,   0, 255, 255,
	  0, 255, 255,   0, 255, 255,   0, 255, 255,   0, 255, 255,
    };

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glClear(GL_COLOR_BUFFER_BIT);

    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 8, 8, 0, GL_RGB, GL_UNSIGNED_BYTE, t);
    glTexEnvt(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

    glColor4t(ITOF(1), ITOF(1), ITOF(1), ITOF(1));
    DrawTexQuad2i2f(WINDSIZEX/4, WINDSIZEY/4, WINDSIZEX*3/4, WINDSIZEY*3/4, ITOF(1)/4, ITOF(1)/4, ITOF(3)/4, ITOF(3)/4);

    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Subtexture test.");
	return ERROR;
    }
    if (TestBuffer(35, 50, ITOF(1), ITOF(1), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Subtexture test.");
	return ERROR;
    }
    if (TestBuffer(65, 50, ITOF(1), ITOF(1), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Subtexture test.");
	return ERROR;
    }

    glEnable(GL_TEXTURE_2D);
    DrawTexQuad2i2f(WINDSIZEX/4, WINDSIZEY/4, WINDSIZEX*3/4, WINDSIZEY*3/4, ITOF(1)/4, ITOF(1)/4, ITOF(3)/4, ITOF(3)/4);

    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Subtexture test.");
	return ERROR;
    }
    if (TestBuffer(35, 50, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Subtexture test.");
	return ERROR;
    }
    if (TestBuffer(65, 50, ITOF(1), ITOF(0), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Subtexture test.");
	return ERROR;
    }

    glTexSubImage2D(GL_TEXTURE_2D, 0, 4, 0, 4, 8,
		    GL_RGB, GL_UNSIGNED_BYTE, tSub);

    DrawTexQuad2i2f(WINDSIZEX/4, WINDSIZEY/4, WINDSIZEX*3/4, WINDSIZEY*3/4, ITOF(1)/4, ITOF(1)/4, ITOF(3)/4, ITOF(3)/4);

    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Subtexture test.");
	return ERROR;
    }
    if (TestBuffer(35, 50, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Subtexture test.");
	return ERROR;
    }
    if (TestBuffer(65, 50, ITOF(0), ITOF(1), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Subtexture test.");
	return ERROR;
    }

    glTexEnvt(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glDisable(GL_TEXTURE_2D);

    return NO_ERROR;
}

static long CopySubTexture(void)
{
    static const GLubyte t[] = {
	  0, 255,   0,   0, 255,   0,	  0, 255,   0,   0, 255,   0,
	255,   0, 255, 255,   0, 255,	255,   0, 255, 255,   0, 255,
	  0, 255,   0,   0, 255,   0,	  0, 255,   0,   0, 255,   0,
	255,   0, 255, 255,   0, 255,	255,   0, 255, 255,   0, 255,
	  0, 255,   0,   0, 255,   0,	  0, 255,   0,   0, 255,   0,
	255,   0, 255, 255,   0, 255,	255,   0, 255, 255,   0, 255,
	  0, 255,   0,   0, 255,   0,	  0, 255,   0,   0, 255,   0,
	255,   0, 255, 255,   0, 255,	255,   0, 255, 255,   0, 255,
	  0, 255,   0,   0, 255,   0,	  0, 255,   0,   0, 255,   0,
	255,   0, 255, 255,   0, 255,	255,   0, 255, 255,   0, 255,
	  0, 255,   0,   0, 255,   0,	  0, 255,   0,   0, 255,   0,
	255,   0, 255, 255,   0, 255,	255,   0, 255, 255,   0, 255,
	  0, 255,   0,   0, 255,   0,	  0, 255,   0,   0, 255,   0,
	255,   0, 255, 255,   0, 255,	255,   0, 255, 255,   0, 255,
	  0, 255,   0,   0, 255,   0,	  0, 255,   0,   0, 255,   0,
	255,   0, 255, 255,   0, 255,	255,   0, 255, 255,   0, 255,
    };

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glClear(GL_COLOR_BUFFER_BIT);

    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 8, 8, 0, GL_RGB, GL_UNSIGNED_BYTE, t);
    glTexEnvt(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

    glColor4t(ITOF(1), ITOF(1), ITOF(1), ITOF(1));
    DrawTexQuad2i2f(WINDSIZEX/4, WINDSIZEY/4, WINDSIZEX*3/4, WINDSIZEY*3/4, ITOF(1)/4, ITOF(1)/4, ITOF(3)/4, ITOF(3)/4);

    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Subtexture test.");
	return ERROR;
    }
    if (TestBuffer(35, 50, ITOF(1), ITOF(1), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Subtexture test.");
	return ERROR;
    }
    if (TestBuffer(65, 50, ITOF(1), ITOF(1), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Subtexture test.");
	return ERROR;
    }

    glEnable(GL_TEXTURE_2D);
    DrawTexQuad2i2f(WINDSIZEX/4, WINDSIZEY/4, WINDSIZEX*3/4, WINDSIZEY*3/4, ITOF(1)/4, ITOF(1)/4, ITOF(3)/4, ITOF(3)/4);

    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Subtexture test.");
	return ERROR;
    }
    if (TestBuffer(35, 50, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Subtexture test.");
	return ERROR;
    }
    if (TestBuffer(65, 50, ITOF(1), ITOF(0), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Subtexture test.");
	return ERROR;
    }

    glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 4, 0, 6, 6, 4, 8);

    glClearColort(ITOF(1), ITOF(1), ITOF(1), ITOF(1));
    glClear(GL_COLOR_BUFFER_BIT);

    DrawTexQuad2i2f(WINDSIZEX/4, WINDSIZEY/4, WINDSIZEX*3/4, WINDSIZEY*3/4, ITOF(1)/4, ITOF(1)/4, ITOF(3)/4, ITOF(3)/4);

    if (TestBuffer(10, 10, ITOF(1), ITOF(1), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Subtexture test.");
	return ERROR;
    }
    if (TestBuffer(35, 50, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Subtexture test.");
	return ERROR;
    }
    if (TestBuffer(65, 50, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Copy Subtexture test.");
	return ERROR;
    }

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glTexEnvt(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glDisable(GL_TEXTURE_2D);

    return NO_ERROR;
}

static long LogicOp(void)
{

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    /*
    ** Draw destination color.
    */

    glColor4t(ITOF(0), ITOF(1), ITOF(0), ITOF(1));
    DrawPoint2f(ITOF(1)/2, ITOF(1)/2);

    if (TestBuffer(0, 0, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Logic op test.");
	return ERROR;
    }

    /*
    ** Set logic op.
    */
    glLogicOp(GL_XOR);

    /*
    ** Draw source color.
    */
    glEnable(GL_COLOR_LOGIC_OP);
    glColor4t(ITOF(0), ITOF(0), ITOF(1), ITOF(1));
    DrawQuad2f(ITOF(0), ITOF(0), ITOF(5), ITOF(5));

    /*
    ** 0, 0, 1 (source color) XOR 0, 1, 0 (destination color ) ->  0, 1, 1
    */
    if (TestBufferAbsolute(0, 0, ITOF(0), ITOF(1), ITOF(1)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Logic op test.");
	return ERROR;
    }

    /*
    ** Draw destination color.
    */
    glDisable(GL_COLOR_LOGIC_OP);
    glColor4t(ITOF(1), ITOF(0), ITOF(0), ITOF(1));
    DrawQuad2f(ITOF(0), ITOF(0), ITOF(5), ITOF(5));

    if (TestBufferAbsolute(0, 0, ITOF(1), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Logic op test.");
	return ERROR;
    }

    /*
    ** Draw source color
    */
    glEnable(GL_COLOR_LOGIC_OP);
    glColor4t(ITOF(1), ITOF(1), ITOF(0), ITOF(1));
    DrawQuad2f(ITOF(0), ITOF(0), ITOF(5), ITOF(5));

    /*
    ** 1, 1, 0	(source color) XOR 1, 0, 0 (destination color) ->  0, 1, 0
    */
    if (TestBufferAbsolute(0, 0, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Logic op test.");
	return ERROR;
    }

    glDisable(GL_COLOR_LOGIC_OP);
    return NO_ERROR;
}

static long VertexArray(void)
{
    static const GLftype vArray[] = {
	ITOF(WINDSIZEX)/4,   ITOF(WINDSIZEY)/4,
	ITOF(WINDSIZEX)*3/4, ITOF(WINDSIZEY)/4,
	ITOF(WINDSIZEX)*3/4, ITOF(WINDSIZEY)*3/4,
	ITOF(WINDSIZEX)/4,   ITOF(WINDSIZEY)*3/4
    };

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glClear(GL_COLOR_BUFFER_BIT);

    glVertexPointer(2, GL_FTYPE, 0, (void *)vArray);

    glColor4t(ITOF(0), ITOF(1), ITOF(0), ITOF(1));

    if (TestBuffer(50, 50, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Vertex Array test.");
	return ERROR;
    }

    glEnableClientState(GL_VERTEX_ARRAY);
    glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
    glDisableClientState(GL_VERTEX_ARRAY);

    if (TestBuffer(50, 50, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Vertex Array test.");
	return ERROR;
    }

    return NO_ERROR;
}

static long DepthOffset(void)
{
    static const GLftype identity[16] = {
	ITOF(1), ITOF(0), ITOF(0), ITOF(0),
	ITOF(0), ITOF(1), ITOF(0), ITOF(0),
	ITOF(0), ITOF(0), ITOF(1), ITOF(0),
	ITOF(0), ITOF(0), ITOF(0), ITOF(1)
    };
    static const GLftype vertices[4][3] = {
	{ITOF(WINDSIZEX)/4,   ITOF(WINDSIZEY)*3/4, ITOF(0)},  /* z_w goes from 0.5 to 0.75 */
	{ITOF(WINDSIZEX)/4,   ITOF(WINDSIZEY)/4,   ITOF(0)},
	{ITOF(WINDSIZEX)*3/4, ITOF(WINDSIZEY)*3/4, ITOF(1)/2},
	{ITOF(WINDSIZEX)*3/4, ITOF(WINDSIZEY)/4,   ITOF(1)/2}
    };

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(1));
    glClearDeptht(ITOF(0));
				    /* GL_LESS is depth Function */
    glClear(GL_COLOR_BUFFER_BIT);

    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadMatrixt(identity);
    glOrthot(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY), ITOF(1), ITOF(-1));    /* Id in Z */
    glMatrixMode(GL_MODELVIEW);
    glLoadMatrixt(identity);

    glEnable(GL_POLYGON_OFFSET_FILL);
    glPolygonOffsett(ITOF(0), ITOF(-4));		/* Window Coords in [0,1] */

    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(3, GL_FTYPE, 0, &vertices[0][0]);

    /*
    ** Check that second object drawn is visible if depth test off.
    */
    glColor4t(ITOF(0), ITOF(1), ITOF(0), ITOF(1));
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

    glColor4t(ITOF(1), ITOF(0), ITOF(0), ITOF(1));
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

    glDisableClientState(GL_VERTEX_ARRAY);

    if (TestBuffer(10, 10, ITOF(0), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Depth Offset test.");
	return ERROR;
    }
    if (TestBuffer(50, 50, ITOF(1), ITOF(0), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Depth Offset test.");
	return ERROR;
    }

    /*
    ** Check that object is 'Green'
    */
    glEnable(GL_DEPTH_TEST);
    glPolygonOffsett(ITOF(0), ITOF(0));
    glClearDeptht(ITOF(1));
    glClear(GL_DEPTH_BUFFER_BIT);

    glColor4t(ITOF(0), ITOF(1), ITOF(0), ITOF(1));
    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(3, GL_FTYPE, 0, &vertices[0][0]);
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
    glDisableClientState(GL_VERTEX_ARRAY);

    if (TestBuffer(50, 50, ITOF(0), ITOF(1), ITOF(0)) == ERROR) {
	ErrorReport(__FILE__, __LINE__, "Depth Offset test.");
	return ERROR;
    }

    glPolygonOffsett(ITOF(0), ITOF(-4));   /*dz/dx = 0.25/50 use Scale of -100.0 */

    /*
    ** Check that red draws
    */
    glColor4t(ITOF(1), ITOF(0), ITOF(0), ITOF(1));
    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(3, GL_FTYPE, 0, &vertices[0][0]);
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
    glDisableClientState(GL_VERTEX_ARRAY);

    if (TestBuffer(50, 50, ITOF(1), ITOF(0), ITOF(0)) == ERROR) {
        ErrorReport(__FILE__, __LINE__, "Depth Offset test.");
        return ERROR;
    }

    glPolygonOffsett(ITOF(0), ITOF(0));
    glDisable(GL_POLYGON_OFFSET_FILL);
    glDisable(GL_DEPTH_TEST);
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);

    return NO_ERROR;
}

/*  DrawElements test
**  Tests DrawElements.
**  Clear screen.
**  Draw green quad using that function.
**  Verify that a green pixel was drawn in
**  the center of the quad.
*/
static long DrawElements(void)
{
    static const GLshort vertices[6][2] = {
	{WINDSIZEX/4,    WINDSIZEY/4},
	{WINDSIZEX*3/4,  WINDSIZEY/4},
	{WINDSIZEX/4,    WINDSIZEY*3/4},
        {WINDSIZEX*3/4,  WINDSIZEY*3/4},
	{WINDSIZEX/2,    WINDSIZEY/4},
        {WINDSIZEX*45/100, WINDSIZEY*45/100}
    };
    static const GLubyte quad[4] = {0, 1, 2, 3};

    glEnableClientState(GL_VERTEX_ARRAY);

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(1));

    glClear(GL_COLOR_BUFFER_BIT);

    glColor4t(ITOF(0), ITOF(0), ITOF(1), ITOF(1));
    glVertexPointer(2, GL_SHORT, 0, &vertices[0][0]);
    glDrawElements(GL_TRIANGLE_STRIP, 4, GL_UNSIGNED_BYTE, quad);

    if (TestBuffer(50, 50, ITOF(0), ITOF(0), ITOF(1))) {
	ErrorReport(__FILE__, __LINE__, " Draw Elements test.");
	glDisableClientState(GL_VERTEX_ARRAY);
	return ERROR;
    }

    glDisableClientState(GL_VERTEX_ARRAY);

    return NO_ERROR;
}

/* Separate Specular Color test */
typedef enum {
    Black, Red, Green, Yellow, Blue, Magenta, Cyan, White
} Cname;

static const GLftype rgb[][4] = {
    { ITOF(0), ITOF(0), ITOF(0), ITOF(1) },
    { ITOF(1), ITOF(0), ITOF(0), ITOF(1) },
    { ITOF(0), ITOF(1), ITOF(0), ITOF(1) },
    { ITOF(1), ITOF(1), ITOF(0), ITOF(1) },
    { ITOF(0), ITOF(0), ITOF(1), ITOF(1) },
    { ITOF(1), ITOF(0), ITOF(1), ITOF(1) },
    { ITOF(0), ITOF(1), ITOF(1), ITOF(1) },
    { ITOF(1), ITOF(1), ITOF(1), ITOF(1) }
};

/* The test cases for separate specular color */
typedef struct {
    GLint   paramset;				    /* Parameter set ID */
    GLint   lit, textured, flat;		    /* GL state */
    Cname   ambient, diffuse, emission, specular;   /* Material parameters */
    Cname   unlitResult, litResult;		    /* Expected results */
} SpecularTestCase;
SpecularTestCase SpecularTestParam[] = {
    /*Param Lit Tex Flat	Amb.   Diff.   Emis.   Spec.  Unlit   Lit */
    {	0,   0,  0,   0,	Red,  Black,  Black,  Green,  White,  White },
    {	1,   0,  0,   0,	Black,	Red,  Black,  Green,  White,  White },
    {	2,   0,  0,   0,	Black,  Black,	Red,  Green,  White,  White },
    {	3,   0,  0,   1,	Red,  Black,  Black,  Green,  White,  White },
    {	4,   0,  0,   1,	Black,	Red,  Black,  Green,  White,  White },
    {	5,   0,  0,   1,	Black,  Black,	Red,  Green,  White,  White },
    {	6,   0,  1,   0,	Red,  Black,  Black,  Green,   Blue,   Blue },
    {	7,   0,  1,   0,	Black,	Red,  Black,  Green,   Blue,   Blue },
    {	8,   0,  1,   0,	Black,  Black,	Red,  Green,   Blue,   Blue },
    {	9,   0,  1,   1,	Red,  Black,  Black,  Green,   Blue,   Blue },
    {	10,  0,  1,   1,	Black,	Red,  Black,  Green,   Blue,   Blue },
    {	11,  0,  1,   1,	Black,  Black,	Red,  Green,   Blue,   Blue },
    {	12,  1,  0,   0,	Red,  Black,  Black,  Green,	Red, Yellow },
    {	13,  1,  0,   0,	Black,	Red,  Black,  Green,  Black, Yellow },
    {	14,  1,  0,   0,	Black,  Black,	Red,  Green,	Red, Yellow },
    {	15,  1,  0,   1,	Red,  Black,  Black,  Green,	Red, Yellow },
    {	16,  1,  0,   1,	Black,	Red,  Black,  Green,  Black, Yellow },
    {	17,  1,  0,   1,	Black,  Black,	Red,  Green,	Red, Yellow },
    {	18,  1,  1,   0,	Red,  Black,  Black,  Green,   Blue,   Blue },
    {	19,  1,  1,   0,	Black,	Red,  Black,  Green,   Blue,   Blue },
    {	20,  1,  1,   0,	Black,  Black,	Red,  Green,   Blue,   Blue },
    {	21,  1,  1,   1,	Red,  Black,  Black,  Green,   Blue,   Blue },
    {	22,  1,  1,   1,	Black,	Red,  Black,  Green,   Blue,   Blue },
    {	23,  1,  1,   1,	Black,  Black,	Red,  Green,   Blue,   Blue }
};
static const int specularTestParamCount = sizeof(SpecularTestParam) / sizeof(SpecularTestParam[0]);

/* Draw point, line, and tstrip primitives to test separate specular color */
static void SpecularDrawPrimitives(void)
{
    /* All primitives uniformly sample texture at the center */
    static const GLftype tex_coords[4][2] = {
        {ITOF(1)/2, ITOF(1)/2},
        {ITOF(1)/2, ITOF(1)/2},
        {ITOF(1)/2, ITOF(1)/2},
        {ITOF(1)/2, ITOF(1)/2}
    };
    static const GLftype point_vertices[2][2] = {
        {ITOF(WINDSIZEX*10/100) + ITOF(1)/2, ITOF(WINDSIZEY*80/100) + ITOF(1)/2},
        {ITOF(WINDSIZEX*40/100) + ITOF(1)/2, ITOF(WINDSIZEY*80/100) + ITOF(1)/2}
    };
    static const GLftype point_normals[2][3] = {
        {ITOF(1), ITOF(0), ITOF(0)},
        {ITOF(0), ITOF(0), ITOF(1)}
    };
    static const GLftype line_vertices[2][2] = {
        {ITOF(WINDSIZEX*10/100) + ITOF(4)/10, ITOF(WINDSIZEY*60/100) + ITOF(1)/2},
        {ITOF(WINDSIZEX*40/100) + ITOF(6)/10, ITOF(WINDSIZEY*60/100) + ITOF(1)/2}
    };
    static const GLftype line_normals[2][3] = {
        {ITOF(1), ITOF(0), ITOF(0)},
        {ITOF(0), ITOF(0), ITOF(1)}
    };
    static const GLftype triangle_vertices[4][3] = {
	{ITOF(WINDSIZEX*10/100) + ITOF(4)/10, ITOF(WINDSIZEY*40/100) + ITOF(6)/10, ITOF(0)},
	{ITOF(WINDSIZEX*10/100) + ITOF(4)/10, ITOF(WINDSIZEY*10/100) + ITOF(4)/10, ITOF(0)},
	{ITOF(WINDSIZEX*40/100) + ITOF(6)/10, ITOF(WINDSIZEY*40/100) + ITOF(6)/10, ITOF(0)},
	{ITOF(WINDSIZEX*40/100) + ITOF(6)/10, ITOF(WINDSIZEY*10/100) + ITOF(4)/10, ITOF(0)}
    };
    static const GLftype triangle_normals[4][3] = {
        {ITOF(1), ITOF(0), ITOF(0)},
        {ITOF(1), ITOF(0), ITOF(0)},
        {ITOF(0), ITOF(0), ITOF(1)},
        {ITOF(0), ITOF(0), ITOF(1)}
    };

    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_NORMAL_ARRAY);
    glEnableClientState(GL_TEXTURE_COORD_ARRAY);

    glTexCoordPointer(2, GL_FTYPE, 0, &tex_coords[0][0]);

    glVertexPointer(2, GL_FTYPE, 0, &point_vertices[0][0]);
    glNormalPointer(GL_FTYPE, 0, &point_normals[0][0]);
    glDrawArrays(GL_POINTS, 0, 2);

    glVertexPointer(2, GL_FTYPE, 0, &line_vertices[0][0]);
    glNormalPointer(GL_FTYPE, 0, &line_normals[0][0]);
    glDrawArrays(GL_LINES, 0, 2);

    glVertexPointer(3, GL_FTYPE, 0, &triangle_vertices[0][0]);
    glNormalPointer(GL_FTYPE, 0, &triangle_normals[0][0]);
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
    glDisableClientState(GL_NORMAL_ARRAY);
    glDisableClientState(GL_VERTEX_ARRAY);
}

/* Test if RGB components of test are equal to ref within
** specified tolerance 'err'.
*/
static long SpecularColorMatch(GLftype const *test, GLftype const *ref, GLftype err)
{
    GLftype refcolor[3], error[3];

    refcolor[0] = buffer.colorBits[0] == 0 ? ITOF(0) : ref[0];
    refcolor[1] = buffer.colorBits[1] == 0 ? ITOF(0) : ref[1];
    refcolor[2] = buffer.colorBits[2] == 0 ? ITOF(0) : ref[2];
    error[0] = buffer.colorBits[0] == 0 ? ITOF(0) : err;
    error[1] = buffer.colorBits[1] == 0 ? ITOF(0) : err;
    error[2] = buffer.colorBits[2] == 0 ? ITOF(0) : err;

    if (ABS(test[0] - refcolor[0]) > error[0] ||
	ABS(test[1] - refcolor[1]) > error[1] ||
	ABS(test[2] - refcolor[2]) > error[2]) {
	return ERROR;
    } else
	return NO_ERROR;
}

/* Test if RGB components of test lie in the range defined by
** ref1 and ref2.
*/
static long SpecularColorRange(GLftype const *test, GLftype const *ref1, GLftype const *ref2)
{
    GLint i;
    GLftype low, high;
    GLftype refcolor1[3], refcolor2[3];

    refcolor1[0] = buffer.colorBits[0] == 0 ? ITOF(0) : ref1[0];
    refcolor1[1] = buffer.colorBits[1] == 0 ? ITOF(0) : ref1[1];
    refcolor1[2] = buffer.colorBits[2] == 0 ? ITOF(0) : ref1[2];

    refcolor2[0] = buffer.colorBits[0] == 0 ? ITOF(0) : ref2[0];
    refcolor2[1] = buffer.colorBits[1] == 0 ? ITOF(0) : ref2[1];
    refcolor2[2] = buffer.colorBits[2] == 0 ? ITOF(0) : ref2[2];

    for (i = 0; i < 3; i++) {
	if (refcolor1[i] < refcolor2[i]) {
	    low = refcolor1[i];
	    high = refcolor2[i];
	} else {
	    low = refcolor2[i];
	    high = refcolor1[i];
	}

	if (test[i] < low || test[i] > high)
	    return ERROR;
    }

    return NO_ERROR;
}

/* Test rendered colors, which should be either the unlit (N.L=0)
** color for this test case, the lit color, or lie between the
** unlit and lit colors, as specified below.
*/
static long SpecularTestResult(SpecularTestCase *testcase)
{
    GLftype buf[3];
    GLftype const *unlitColor, *litColor;
    GLint i;
    typedef enum { Unlit, Lit, Blended } Result;
    static struct Test {
	GLint x, y;
	Result colorSmooth, colorFlat;
    } test[] = {
	/* x                      y                       SMOOTH   FLAT */
	{ WINDSIZEX*10/100, WINDSIZEY*80/100, Unlit,   Unlit }, /* Point with N.L = 0 */
	{ WINDSIZEX*40/100, WINDSIZEY*80/100, Lit,	   Lit	 }, /* Point with N.L = 1 */
	{ WINDSIZEX*10/100, WINDSIZEY*60/100, Unlit,   Lit	 }, /* First vertex of line, N.L = 0 */
	{ WINDSIZEX*30/100, WINDSIZEY*60/100, Blended, Lit	 }, /* Between line endpoints */
	{ WINDSIZEX*10/100, WINDSIZEY*25/100, Blended, Lit	 }, /* Left edge midpoint of first tstrip */
	{ WINDSIZEX*40/100, WINDSIZEY*25/100, Blended, Lit	 }, /* Right edge midpoint of first tstrip */
/* Might also check (40,60) (second vertex of line, N.L=1) */
    };
    static const int testCount = sizeof(test) / sizeof(test[0]);

    unlitColor = rgb[testcase->unlitResult];
    litColor = rgb[testcase->litResult];

    /* Compute expected colors and verify the results */
    for (i = 0; i < testCount; i++) {
	struct Test *t = &test[i];
	Result result;
	long correct;

	ReadScreen(t->x, t->y, 1, 1, GL_RGB, buf);

	result = testcase->flat ? t->colorFlat : t->colorSmooth;
	switch (result) {
	    default:
		correct = -1;
		break;
	    case Unlit:
		correct = SpecularColorMatch(buf, unlitColor, ATOF("0.49"));
		break;
	    case Lit:
		correct = SpecularColorMatch(buf, litColor, ATOF("0.49"));
		break;
	    case Blended:
		correct = SpecularColorRange(buf, unlitColor, litColor);
		break;
	}

	if (correct == ERROR) {
	    if (result != Blended) {
		GLftype const *right = (result == Lit) ? litColor : unlitColor;
		ErrorPrintf(__FILE__, __LINE__, "Separate Specular Color test "
		    "parameter set %d failed at %d, %d: "
		    "expected %g %g %g, got %g %g %g",
		    testcase->paramset,
		    t->x, t->y,
		    right[0], right[1], right[2],
		    buf[0], buf[1], buf[2]);
	    } else {
		ErrorPrintf(__FILE__, __LINE__, "Separate Specular Color test "
		    "parameter set %d failed at %d, %d: "
		    "expected %g %g %g...%g %g %g, got %g %g %g",
		    testcase->paramset,
		    t->x, t->y,
		    unlitColor[0], unlitColor[1], unlitColor[2],
		    litColor[0], litColor[1], litColor[2],
		    buf[0], buf[1], buf[2]);
	    }
	    return ERROR;
	}
    }

    return NO_ERROR;
}

/* Test lighting for a specific combination of lit/unlit,
** textured/untextured, smooth/flat shaded, single/separate color,
** and choice of material properties.
*/
static long SpecularTest(SpecularTestCase *t)
{
    GLftype const
	*ambient = rgb[t->ambient],
	*diffuse = rgb[t->diffuse],
	*emission = rgb[t->emission],
	*specular = rgb[t->specular];

    if (t->lit) {
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
    } else {
	glDisable(GL_LIGHTING);
	glDisable(GL_LIGHT0);
    }

    if (t->textured)
	glEnable(GL_TEXTURE_2D);
    else
	glDisable(GL_TEXTURE_2D);

    if (t->flat)
	glShadeModel(GL_FLAT);
    else
	glShadeModel(GL_SMOOTH);

    glMaterialtv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
    glMaterialtv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
    glMaterialtv(GL_FRONT_AND_BACK, GL_EMISSION, emission);
    glMaterialtv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);

    /* Clear buffer and draw the test primitives */
    glClear(GL_COLOR_BUFFER_BIT);
    SpecularDrawPrimitives();

    /* Compute expected colors and verify the results */
    if (SpecularTestResult(t) == ERROR)
	return ERROR;

    return NO_ERROR;
}

/* Reset GL state which may have been modified by a specular test */
static void SpecularReset() {
    glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
    glShadeModel(GL_SMOOTH);
    glDisable(GL_LIGHTING);
    glDisable(GL_LIGHT0);
    glDisable(GL_TEXTURE_2D);
}

/* SpecularColor test
**  Test all combinations of lighting, texturing, shading model.
*/
static long SpecularColor(void)
{
    static GLftype lightPosition[] = { ITOF(0), ITOF(0), ITOF(1), ITOF(0) };
    static GLubyte texture[2][2][3];
    GLint i, j;

    /* initialization */
    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(1));

    glColor4t(rgb[White][0], rgb[White][1], rgb[White][2], rgb[White][3]);
    glMaterialtv(GL_FRONT_AND_BACK, GL_SPECULAR, rgb[Green]);

    /* Blue 2x2 texture */
    for (i = 0; i < 2; i++) {
	for (j = 0; j < 2; j++) {
	    texture[i][j][0] = 0;
	    texture[i][j][1] = 0;
	    texture[i][j][2] = 255;
	}
    }
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    glTexEnvt(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 2, 2, 0, GL_RGB, GL_UNSIGNED_BYTE, texture);

    /* White light at infinity */
    glLighttv(GL_LIGHT0, GL_POSITION, lightPosition);
    glLighttv(GL_LIGHT0, GL_DIFFUSE, rgb[White]);
    glLighttv(GL_LIGHT0, GL_AMBIENT, rgb[White]);
    glLighttv(GL_LIGHT0, GL_SPECULAR, rgb[White]);
    glLightModelt(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);

    /* Loop over all test cases */
    for (i = 0; i < specularTestParamCount; i++) {
	if (SpecularTest(&SpecularTestParam[i]) == ERROR) {
	    SpecularReset();
	    return ERROR;
	}
    }

    SpecularReset();
    return NO_ERROR;
}

/*
** Rescale Normal Test.
*/
static long RescaleNormal(void)
{
    static const GLftype
	scale[] = { ITOF(1)/4, ITOF(1), ITOF(4) },
	black[] = { ITOF(0), ITOF(0), ITOF(0), ITOF(1) },
	white[] = { ITOF(1), ITOF(1), ITOF(1), ITOF(1) };
    static GLftype vertices[3][2];
    GLftype lightpos[4];
    GLint i, j;

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glEnable(GL_RESCALE_NORMAL);

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));

    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    glOrthot(ITOF(-1), ITOF(1), ITOF(-1), ITOF(1), ITOF(1), ITOF(-1));

    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();

    glMaterialtv(GL_FRONT_AND_BACK, GL_AMBIENT, black);
    glMaterialtv(GL_FRONT_AND_BACK, GL_EMISSION, black);
    glMaterialt(GL_FRONT_AND_BACK, GL_SHININESS, ITOF(1));

    glLighttv(GL_LIGHT0, GL_AMBIENT, black);
    glLighttv(GL_LIGHT0, GL_DIFFUSE, white);
    glLighttv(GL_LIGHT0, GL_SPECULAR, white);

    /* Test diffuse lighting */
    glMaterialtv(GL_FRONT_AND_BACK, GL_SPECULAR, black);
    glMaterialtv(GL_FRONT_AND_BACK, GL_DIFFUSE, white);

    /* Loop over normals in each of XYZ axes */
    for (j = 0; j < 3; j++) {
	/* Set light position and normal to this axis */
	for (i = 0; i < 4; i++)
	    lightpos[i] = (i == j) ? ITOF(1) : ITOF(0);

	glLighttv(GL_LIGHT0, GL_POSITION, lightpos);
	glNormal3t(lightpos[0], lightpos[1], lightpos[2]);

	for (i = 0; i < 3; i++) {
	    glLoadIdentity();
	    glScalet(scale[i], scale[i], scale[i]);

	    glClear(GL_COLOR_BUFFER_BIT);

	    vertices[0][0] = DIV(ATOF("0.7"), scale[i]);  vertices[0][1] =  DIV(ITOF(0), scale[i]);
	    vertices[1][0] = DIV(ATOF("-0.2"), scale[i]); vertices[1][1] =  DIV(ATOF("0.35"), scale[i]);
	    vertices[2][0] = DIV(ATOF("-0.2"), scale[i]); vertices[2][1] =  DIV(ATOF("-0.35"), scale[i]);

	    glEnableClientState(GL_VERTEX_ARRAY);
            glVertexPointer(2, GL_FTYPE, 0, &vertices[0][0]);
	    glDrawArrays(GL_TRIANGLES, 0, 3);
	    glDisableClientState(GL_VERTEX_ARRAY);

	    if (TestBuffer(50, 50, ITOF(1), ITOF(1), ITOF(1)) == ERROR) {
		ErrorReport(__FILE__, __LINE__, "RescaleNormal test.");
		return ERROR;
	    }
	}
    }

    /* Test specular lighting */
    glMaterialtv(GL_FRONT_AND_BACK, GL_SPECULAR, white);
    glMaterialtv(GL_FRONT_AND_BACK, GL_DIFFUSE, black);

    /* Z axis only */
    lightpos[0] = lightpos[1] = lightpos[3] = ITOF(0);
    lightpos[2] = ITOF(1);

    glLighttv(GL_LIGHT0, GL_POSITION, lightpos);
    glNormal3t(lightpos[0], lightpos[1], lightpos[2]);

    for (i = 0; i < 3; i++) {
	glLoadIdentity();
	glScalet(scale[i], scale[i], scale[i]);

	glNormal3t(ITOF(0), ITOF(0), ITOF(1));

	glClear(GL_COLOR_BUFFER_BIT);

	vertices[0][0] = DIV(ATOF("0.7"), scale[i]);  vertices[0][1] =  DIV(ITOF(0), scale[i]);
	vertices[1][0] = DIV(ATOF("-0.2"), scale[i]); vertices[1][1] =  DIV(ATOF("0.35"), scale[i]);
	vertices[2][0] = DIV(ATOF("-0.2"), scale[i]); vertices[2][1] =  DIV(ATOF("-0.35"), scale[i]);

	glEnableClientState(GL_VERTEX_ARRAY);
        glVertexPointer(2, GL_FTYPE, 0, &vertices[0][0]);
	glDrawArrays(GL_TRIANGLES, 0, 3);
	glDisableClientState(GL_VERTEX_ARRAY);

	if (TestBuffer(50, 50, ITOF(1), ITOF(1), ITOF(1)) == ERROR) {
	    ErrorReport(__FILE__, __LINE__, "RescaleNormal test.");
	    return ERROR;
	}
    }

    glDisable(GL_LIGHTING);
    glDisable(GL_LIGHT0);
    glDisable(GL_RESCALE_NORMAL);
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
    glPopMatrix();

    return NO_ERROR;
}

/* Packed Pixels Test
**  For each packed format, define an RGBA image whose left
**  pixel is green and right pixel is magenta.
**  Convert the image to packed RGB or RGBA format and draw it.
**  Verify that the resulting image is green on the left and
**  magenta on the right.
*/
static long PackedPixels(void)
{
    static const GLenum types[] = {
	GL_UNSIGNED_SHORT_5_6_5,
	GL_UNSIGNED_SHORT_4_4_4_4,
	GL_UNSIGNED_SHORT_5_5_5_1
    };
    static const GLsizei type_count = sizeof(types) / sizeof(types[0]);
    static const GLftype dataBuf[8] = {
	ITOF(0), ITOF(1), ITOF(0), ITOF(0),		    /* green pixel on left */
	ITOF(1), ITOF(0), ITOF(1), ITOF(0)		    /* magenta pixel on right */
    };
    GLubyte convertBuf[8 * sizeof(GLftype)];
    long i;

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));
    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glColor4t(ITOF(0), ITOF(0), ITOF(0), ITOF(1));

    for (i = 0; i != type_count; i++) {
	packPixRec const *format = packPixInfo(types[i]);

	ConvertRGBAtoPackedImage(dataBuf, convertBuf, format, 2);
	glClear(GL_COLOR_BUFFER_BIT);

	if (format->components == 3) {
	    DrawPixels(0, 0, 2, 1, GL_RGB, format->type, convertBuf);
	} else {
	    DrawPixels(0, 0, 2, 1, GL_RGBA, format->type, convertBuf);
	}

	if (TestBufferAbsolute(0, 0, ITOF(0), ITOF(1), ITOF(0))) {
	    ErrorReport(__FILE__, __LINE__, " PackedPixels test.");
	    return ERROR;
	}

	if (TestBufferAbsolute(1, 0, ITOF(1), ITOF(0), ITOF(1))) {
	    ErrorReport(__FILE__, __LINE__, " PackedPixels test.");
	    return ERROR;
	}
    }

    return NO_ERROR;
}

#ifdef GL_OES_VERSION_1_1

/* Point Parameter Test:
** Tests that point parameters control the size of rendered points
**
** Clear the screen to black and render a blue point in the bottom left
** corner at (1/2, 1/2, 1) using point parameter settings to define the
** point size. The parameters are chosen (a = 4, b = 2, c = 1) so that
** a distance of 0 will produce an attenuation of 1/2. This, combined
** with a point size of 2 should produce a unit point.
**
** Test (0,0) to ensure that the point was rendered.
**/

static long PointParameter(void)
{
    static const GLftype vertices[1][3] = {{ITOF(1)/2, ITOF(1)/2, ITOF(1)}};
    static const GLftype point_atten[] = {ITOF(2), ITOF(1), ITOF(1)};

    glVertexPointer(2, GL_FTYPE, 0, vertices);

    glEnableClientState(GL_VERTEX_ARRAY);

    Ortho3D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY),
	    ITOF(0), ITOF(2));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(1));
    glClear(GL_COLOR_BUFFER_BIT);

    glPointSizet(ITOF(2)); 
    glPointParametertv(GL_POINT_DISTANCE_ATTENUATION, point_atten); 

    glColor4t(ITOF(0), ITOF(0), ITOF(1), ITOF(1));
    glDrawArrays(GL_POINTS, 0, 1);

    if (TestBuffer(0, 0, ITOF(0), ITOF(0), ITOF(1)))
    {
	ErrorReport(__FILE__, __LINE__, " Point Parameter test.");
	glDisableClientState(GL_VERTEX_ARRAY);
	return ERROR;
    }
    glDisableClientState(GL_VERTEX_ARRAY);

    return NO_ERROR;
}

/*  Point Size Array test:
**
**  Tests the use of using the pixel size array to control the size
**  of rendered points.
**
**  Clear the screen and render a point in the bottom left corner
**  using the point-size array to define the point size. Then test
**  to ensure that the point was rendered.
*/

static long PointSizeArray(void)
{
    static const GLftype vertices[1][2] = {{ITOF(1)/2, ITOF(1)/2}};
    static const GLftype pointSizes[1]  = {ITOF(1)};

    glVertexPointer(2, GL_FTYPE, 0, vertices);
    glPointSizePointerOES(GL_FTYPE, 0, pointSizes);

    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_POINT_SIZE_ARRAY_OES);

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(1));
    glClear(GL_COLOR_BUFFER_BIT);

    glColor4t(ITOF(0), ITOF(0), ITOF(1), ITOF(1));
    glDrawArrays(GL_POINTS, 0, 1);

    if (TestBuffer(0, 0, ITOF(0), ITOF(0), ITOF(1)))
	{
		ErrorReport(__FILE__, __LINE__, " Point Size Array test.");
		glDisableClientState(GL_VERTEX_ARRAY);
		glDisableClientState(GL_POINT_SIZE_ARRAY_OES);
		return ERROR;
    }
    glDisableClientState(GL_VERTEX_ARRAY);
    glDisableClientState(GL_POINT_SIZE_ARRAY_OES);

    return NO_ERROR;
}

/*  Point Sprite test:
**
**  Tests that a point-sprite can be drawn in the simplest case.
**
*/

static long PointSprite(void)
{
	GLint red   = 0x55;
	GLint green = 0xff;
	GLint blue  = 0xaa;

	GLubyte	tMap[3];
	GLftype x;

	tMap[0] = red;
	tMap[1] = green;
	tMap[2] = blue;

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(1));
    glClear(GL_COLOR_BUFFER_BIT);

    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 1, 1, 0, GL_RGB, GL_UNSIGNED_BYTE, tMap);

    x = GL_DECAL;
    glTexEnvtv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, &x);
    x = GL_NEAREST;
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, x);
    x = GL_NEAREST;
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, x);
	{
		GLftype	vertexArray[1][2] = {{ITOF(1)/2, ITOF(1)/2}};

		glVertexPointer(2, GL_FTYPE, 0, vertexArray);
		glEnableClientState(GL_VERTEX_ARRAY);
		glEnable(GL_POINT_SPRITE_OES);
		glEnable(GL_TEXTURE_2D);

		x = GL_TRUE;
	    glTexEnvtv(GL_POINT_SPRITE_OES, GL_COORD_REPLACE_OES, &x);

		glPointSizet(ITOF(1));
		glDrawArrays(GL_POINTS, 0, 1);

		x = GL_FALSE;
	    glTexEnvtv(GL_POINT_SPRITE_OES, GL_COORD_REPLACE_OES, &x);
		glDisable(GL_POINT_SPRITE_OES);
		glDisableClientState(GL_VERTEX_ARRAY);
		glDisable(GL_TEXTURE_2D);
	}

    if (TestBuffer(0, 0, ITOF(red)/255, ITOF(green)/255, ITOF(blue)/255))
	{
		ErrorReport(__FILE__, __LINE__, " Point Sprite test.");
		return ERROR;
    }
    return NO_ERROR;
}

/* User clip plane test:
**
** Draws a point without any clip planes. Then with a plane that shouldn't
** discard it and then with a plane that should discard it.
**
*/

static long UserClip (void)
{
	static const GLftype vertices[2] = { ITOF(1)/2, ITOF(1)/2 };
	GLftype plane[4] = { ITOF(1), ITOF(0), ITOF(0), ITOF(0) };

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glEnableClientState(GL_VERTEX_ARRAY);
	glVertexPointer(2, GL_FTYPE, 0, vertices);

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
	glClear(GL_COLOR_BUFFER_BIT);

    glColor4t(ITOF(1), ITOF(1), ITOF(1), ITOF(1));

	/*
	** Draw the point without clip planes.
	*/
	glDrawArrays(GL_POINTS, 0, 1);
    if (TestBuffer(0, 0, ITOF(1), ITOF(1), ITOF(1)))
	{
		ErrorReport(__FILE__, __LINE__, " User Clip Plane test.");
		glDisableClientState(GL_VERTEX_ARRAY);
		return ERROR;
    }
	glClear(GL_COLOR_BUFFER_BIT);

	/*
	** Set the plane which shouldn't clip the point.
	*/
	glEnable(GL_CLIP_PLANE0);
	glClipPlanet(GL_CLIP_PLANE0, plane);
	glDrawArrays(GL_POINTS, 0, 1);
    if (TestBuffer(0, 0, ITOF(1), ITOF(1), ITOF(1)))
	{
		ErrorReport(__FILE__, __LINE__, " User Clip Plane test.");
		glDisable(GL_CLIP_PLANE0);
		glDisableClientState(GL_VERTEX_ARRAY);
		return ERROR;
    }
	glClear(GL_COLOR_BUFFER_BIT);


	/*
	** Set the plane which should clip the point.
	*/
	plane[0] = -plane[0];
	glClipPlanet(GL_CLIP_PLANE0, plane);
	glDrawArrays(GL_POINTS, 0, 1);
    if (!TestBuffer(0, 0, ITOF(1), ITOF(1), ITOF(1)))
	{
		ErrorReport(__FILE__, __LINE__, " User Clip Plane test.");
		glDisable(GL_CLIP_PLANE0);
		glDisableClientState(GL_VERTEX_ARRAY);
		return ERROR;
    }

	glDisable(GL_CLIP_PLANE0);
    glDisableClientState(GL_VERTEX_ARRAY);

    return NO_ERROR;
}

static long TestTexCombine(GLftype r, GLftype g, GLftype b)
{
    GLftype buf[3];
    GLftype errorRGB[3];

    r = buffer.colorBits[0] == 0 ? 0 : r;
    g = buffer.colorBits[1] == 0 ? 0 : g;
    b = buffer.colorBits[2] == 0 ? 0 : b;
    errorRGB[0] = buffer.colorBits[0] == 0 ? ITOF(0) : ATOF("0.124");
    errorRGB[1] = buffer.colorBits[1] == 0 ? ITOF(0) : ATOF("0.124");
    errorRGB[2] = buffer.colorBits[2] == 0 ? ITOF(0) : ATOF("0.124");
    ReadScreen(0, 0, 1, 1, GL_RGB, buf);
    if (ABS(buf[0]-r) > errorRGB[0] ||
        ABS(buf[1]-g) > errorRGB[1] ||
        ABS(buf[2]-b) > errorRGB[2]) {
            return ERROR;
    }
    return NO_ERROR;
}

static long TexCombine(void)
{
    GLftype array[4][2]  = { {ITOF(0),ITOF(0)},  {ITOF(1),ITOF(0)}, {ITOF(1),ITOF(1)}, {ITOF(0),ITOF(1)} };

    GLftype polyColor[4] = { ITOF(3)/4, ITOF(2)/4, ITOF(1)/4, ITOF(2)/4 };
    GLftype envColor[4]  = { ITOF(2)/4, ITOF(1)/4, ITOF(3)/4, ITOF(1)/4 };
    GLubyte texData[4]   = { 0x40, 0xC0, 0x80, 0xC0 };
    GLenum  combinemode[8]  = { GL_REPLACE, GL_MODULATE, GL_ADD, GL_ADD_SIGNED,
				GL_INTERPOLATE, GL_SUBTRACT, GL_DOT3_RGB,
				GL_DOT3_RGBA};
    GLftype expectrgb[8][3] = {
                                { ITOF(3)/4,  ITOF(2)/4,  ITOF(1)/4  },
                                { ITOF(6)/16, ITOF(3)/16, ITOF(2)/16 },
                                { ITOF(1),    ITOF(3)/4,  ITOF(1)    },
                                { ITOF(3)/4,  ITOF(1)/4,  ITOF(2)/4  },
                                { ITOF(9)/16, ITOF(7)/16, ITOF(8)/16 },
                                { ITOF(1)/4,  ITOF(1)/4,  ITOF(0)/4  },
                                { ITOF(0)/16, ITOF(0)/16, ITOF(0)/16 },
                                { ITOF(0)/16, ITOF(0)/16, ITOF(0)/16 },
                              };

    GLint   numtexunits;
    GLint   i, t;
    GLuint  tName = 1;
   
	Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glColor4t(polyColor[0], polyColor[1], polyColor[2], polyColor[3]);

    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(2, GL_FTYPE, 0, array);

    glGetIntegerv(GL_MAX_TEXTURE_UNITS, &numtexunits);
    glBindTexture(GL_TEXTURE_2D, 1);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 1, 1, 0, GL_RGBA, GL_UNSIGNED_BYTE, texData);
	glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,     (GLftype)GL_REPEAT);
	glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,     (GLftype)GL_REPEAT);
	glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, (GLftype)GL_NEAREST);
	glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, (GLftype)GL_NEAREST);

    /* Check each texture unit individually */
    for (t=0; t<numtexunits; ++t)
    {
        for (i=0; i<numtexunits; ++i)
        {
            glActiveTexture(GL_TEXTURE0+i);
            glClientActiveTexture(GL_TEXTURE0+i);
            if (i==t)
            {
                glEnable(GL_TEXTURE_2D);
                glEnableClientState(GL_TEXTURE_COORD_ARRAY);
            }
            else
            {
                glDisable(GL_TEXTURE_2D);
                glDisableClientState(GL_TEXTURE_COORD_ARRAY);
            }
        }

        glActiveTexture(GL_TEXTURE0+t);
        glClientActiveTexture(GL_TEXTURE0+t);
        glBindTexture(GL_TEXTURE_2D, 1);
        glTexCoordPointer(2, GL_FTYPE, 0, array);
        glTexEnvt(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE,   (GLftype)GL_COMBINE);
        
        /* Test each rgb combine mode */
        glTexEnvt(GL_TEXTURE_ENV, GL_SRC0_RGB,           (GLftype)GL_PRIMARY_COLOR);
        glTexEnvt(GL_TEXTURE_ENV, GL_SRC1_RGB,           (GLftype)GL_CONSTANT);
        glTexEnvt(GL_TEXTURE_ENV, GL_SRC2_RGB,           (GLftype)GL_TEXTURE);
        glTexEnvt(GL_TEXTURE_ENV, GL_OPERAND0_RGB,       (GLftype)GL_SRC_COLOR);
        glTexEnvt(GL_TEXTURE_ENV, GL_OPERAND1_RGB,       (GLftype)GL_SRC_COLOR);
        glTexEnvt(GL_TEXTURE_ENV, GL_OPERAND2_RGB,       (GLftype)GL_SRC_COLOR);
        glTexEnvtv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, envColor);
        glTexEnvt(GL_TEXTURE_ENV, GL_RGB_SCALE,          ITOF(1));
        for (i=0; i<8; ++i)
        {
            glTexEnvt(GL_TEXTURE_ENV, GL_COMBINE_RGB,       (GLftype)combinemode[i]);
            glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
            if (TestTexCombine(expectrgb[i][0], expectrgb[i][1], expectrgb[i][2]) == ERROR)
            {
		        ErrorReport(__FILE__, __LINE__, " Texture Combine test.");
                glDisable(GL_TEXTURE_2D);
                glDisableClientState(GL_TEXTURE_COORD_ARRAY);
                glDisableClientState(GL_VERTEX_ARRAY);
                return ERROR;
            }
        }
        glDisable(GL_TEXTURE_2D);
        glDisableClientState(GL_TEXTURE_COORD_ARRAY);
    }

    glDisableClientState(GL_VERTEX_ARRAY);
    
    glBindTexture(GL_TEXTURE_2D, 0);
    glDeleteTextures(1, &tName);
    
    return NO_ERROR;
}

#endif /* GL_OES_VERSION_1_1 */
