/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/pntrast.c,v 1.3 2005/05/11 15:21:40 artlab Exp $
*/

/*
** pntrast.c
** Point Rasterization Test.
**
** Description -
**    Points of increasing size (both odd and even sizes) are
**    drawn then checked for correctness.
**    Point size tested are limited to half the window size, in this case 24.
**
** Technical Specification -
**    Buffer requirements:
**        Color buffer.
**    Color requirements:
**        2 colors.
**    States requirements:
**        Disabled = GL_DITHER, GL_MULTISAMPLE.
**    Error epsilon:
**    Paths:
**        Allowed = Alpha, Blend, Depth, Fog, Logicop, Shade, Stencil.
**        Not allowed = Alias, Dither.
*/

#include "conform.h"
#include "util.h"


static char errStr[160];
static char errStrFmt[][80] = {
    "Point did not draw correctly along the %s edge. Size = %d.",
    "Point did not draw correctly. Size = %d.",
};


long PointRasterExec(void)
{
    GLftype *buf, *ptr;
    GLint size, i, j;
    GLint count, pntRange[2], maxSize;

    buf = (GLftype *)MALLOC(WINDSIZEX*WINDSIZEY*sizeof(GLftype));

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    AutoClearColor(COLOR_OFF);
    AutoColor(COLOR_ON);
    glDisable(GL_DITHER);
    glDisable(GL_MULTISAMPLE);

    glGetIntegerv(GL_ALIASED_POINT_SIZE_RANGE, pntRange);

    /* 
    ** Limit point sizes tested to half the window size.
    */
    maxSize = (pntRange[1] > WINDSIZEX/2) ? WINDSIZEX/2 : pntRange[1];

    for (count = 1; count <= maxSize; count++) {
	size = count;

	glClear(GL_COLOR_BUFFER_BIT);

        glPointSizet(ITOF(size));

	{
	    GLftype vertex[2];
	    vertex[0] = ITOF(size)/2 + ITOF(1);
	    vertex[1] = ITOF(size)/2 + ITOF(1);

	    glEnableClientState(GL_VERTEX_ARRAY);
	    glVertexPointer(2, GL_FTYPE, 0, &vertex[0]);
	    glDrawArrays(GL_POINTS, 0, 1);
	    glDisableClientState(GL_VERTEX_ARRAY);
	}

	ReadScreen(0, 0, size+2, size+2, GL_AUTO_COLOR, buf);

	ptr = buf;
	for (i = 0; i < size+2; i++) {
	    if (AutoColorCompare(*ptr++, COLOR_OFF) == GL_FALSE) {
		StrMake(errStr, errStrFmt[0], "bottom", size);
		ErrorReport(__FILE__, __LINE__, errStr);
		FREE(buf);
		return ERROR;
	    }
	}
	for (i = 0; i < size; i++) {
	    if (AutoColorCompare(*ptr++, COLOR_OFF) == GL_FALSE) {
		StrMake(errStr, errStrFmt[0], "left", size);
		ErrorReport(__FILE__, __LINE__, errStr);
		FREE(buf);
		return ERROR;
	    }

	    for (j = 0; j < size; j++) {
		if (AutoColorCompare(*ptr++, COLOR_ON) == GL_FALSE) {
		    StrMake(errStr, errStrFmt[1], size);
		    ErrorReport(__FILE__, __LINE__, errStr);
		    FREE(buf);
		    return ERROR;
		}
	    }

	    if (AutoColorCompare(*ptr++, COLOR_OFF) == GL_FALSE) {
		StrMake(errStr, errStrFmt[0], "rigth", size);
		ErrorReport(__FILE__, __LINE__, errStr);
		FREE(buf);
		return ERROR;
	    }
	}
	for (i = 0; i < size+2; i++) {
	    if (AutoColorCompare(*ptr++, COLOR_OFF) == GL_FALSE) {
		StrMake(errStr, errStrFmt[0], "top", size);
		ErrorReport(__FILE__, __LINE__, errStr);
		FREE(buf);
		return ERROR;
	    }
	}
    }

    FREE(buf);
    return NO_ERROR;
}
