/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/polycull.c,v 1.3 2005/05/11 15:21:40 artlab Exp $
*/

/*
** polycull.c
** Polygon Culling Test.
**
** Description -
**    First, with culling disabled, a polygon is drawn both
**    clockwise and counter-clockwise. The polygon should render
**    in both cases. Culling is then enabled. With culling
**    direction set to GL_FRONT, a polygon is drawn both
**    clockwise and counter-clockwise. Only the polygon drawn in
**    the clockwise direction should render. With culling
**    direction is set to GL_BACK, a polygon is drawn both
**    clockwise and counter-clockwise. Only the polygon drawn in
**    the counter-clockwise direction should render.
**
** Technical Specification -
**    Buffer requirements:
**        Color buffer.
**    Color requirements:
**        2 colors.
**    States requirements:
**        Disabled = GL_DITHER, GL_MULTISAMPLE.
**    Error epsilon:
**    Paths:
**        Allowed = Alpha, Blend, Depth, Fog, Logicop, Shade, Stencil.
**        Not allowed = Alias, Dither.
*/

#include "conform.h"
#include "util.h"


static char errStr[][160] = {
    "GL_CULL_FACE is disabled. Polygon was drawn counter-clockwise. Polygon was not rendered, should have rendered.",
    "GL_CULL_FACE is disabled. Polygon was drawn clockwise. Polygon was not rendered, should have rendered.",
    "GL_CULL_FACE is enabled. glCullFace is GL_FRONT. Polygon was drawn clockwise. Polygon was not rendered, should have rendered.",
    "GL_CULL_FACE is enabled. glCullFace is GL_FRONT. Polygon was drawn counter-clockwise. Polygon was rendered, should not have rendered.",
    "GL_CULL_FACE is enabled. glCullFace is GL_BACK. Polygon was drawn clockwise. Polygon was rendered, should not have rendered.",
    "GL_CULL_FACE is enabled. glCullFace is GL_BACK. Polygon was drawn counter-clockwise. Polygon was not rendered, should have rendered.",
};


static long CheckZero(GLftype *buf)
{
    long i;

    ReadScreen(0, 0, WINDSIZEX, WINDSIZEY, GL_AUTO_COLOR, buf);
    for (i = 0; i < WINDSIZEX*WINDSIZEY; i++) {
	if (AutoColorCompare(buf[i], COLOR_OFF) == GL_FALSE) {
	    return ERROR;
	}
    }
    return NO_ERROR;
}

static long CheckNonZero(GLftype *buf)
{
    long i;

    ReadScreen(0, 0, WINDSIZEX, WINDSIZEY, GL_AUTO_COLOR, buf);
    for (i = 0; i < WINDSIZEX*WINDSIZEY; i++) {
	if (AutoColorCompare(buf[i], COLOR_ON) == GL_FALSE) {
	    return ERROR;
	}
    }
    return NO_ERROR;
}

static void DrawTriangles(long dir)
{

    glClear(GL_COLOR_BUFFER_BIT);

    glEnableClientState(GL_VERTEX_ARRAY);

    if (dir == GL_CCW) {
	const GLshort vertices[6][2] = {
	    {0, 0},
	    {WINDSIZEX, 0},
	    {WINDSIZEX, WINDSIZEY},
	    {WINDSIZEX, WINDSIZEY},
	    {0, WINDSIZEY},
	    {0, 0}
	};
	glVertexPointer(2, GL_SHORT, 0, &vertices[0][0]);
	glDrawArrays(GL_TRIANGLES, 0, 6);
    } else {
	const GLshort vertices[6][2] = {
	    {0, 0},
	    {0, WINDSIZEY},
	    {WINDSIZEX, WINDSIZEY},
	    {WINDSIZEX, WINDSIZEY},
	    {WINDSIZEX, 0},
	    {0, 0}
	};
	glVertexPointer(2, GL_SHORT, 0, &vertices[0][0]);
	glDrawArrays(GL_TRIANGLES, 0, 6);
    }

    glDisableClientState(GL_VERTEX_ARRAY);
}

static void DrawTriangleFan(long dir)
{

    glClear(GL_COLOR_BUFFER_BIT);

    glEnableClientState(GL_VERTEX_ARRAY);

    if (dir == GL_CCW) {
	const GLshort vertices[4][2] = {
	    {0, 0},
	    {WINDSIZEX, 0},
	    {WINDSIZEX, WINDSIZEY},
	    {0, WINDSIZEY}
	};
	glVertexPointer(2, GL_SHORT, 0, &vertices[0][0]);
	glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
    } else {
	const GLshort vertices[4][2] = {
	    {0, 0},
	    {0, WINDSIZEY},
	    {WINDSIZEX, WINDSIZEY},
	    {WINDSIZEX, 0}
	};
	glVertexPointer(2, GL_SHORT, 0, &vertices[0][0]);
	glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
    }

    glDisableClientState(GL_VERTEX_ARRAY);
}

static void DrawTriangleStrip(long dir)
{

    glClear(GL_COLOR_BUFFER_BIT);

    glEnableClientState(GL_VERTEX_ARRAY);

    if (dir == GL_CCW) {
	const GLshort vertices[4][2] = {
	    {0, 0},
	    {WINDSIZEX, 0},
	    {0, WINDSIZEY},
	    {WINDSIZEX, WINDSIZEY}
	};
	glVertexPointer(2, GL_SHORT, 0, &vertices[0][0]);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
    } else {
	const GLshort vertices[4][2] = {
	    {WINDSIZEX, 0},
	    {0, 0},
	    {WINDSIZEX, WINDSIZEY},
	    {0, WINDSIZEY}
	};
	glVertexPointer(2, GL_SHORT, 0, &vertices[0][0]);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
    }

    glDisableClientState(GL_VERTEX_ARRAY);
}

long PolyCullExec(void)
{
    GLftype *buf;

    buf = (GLftype *)MALLOC(WINDSIZEX*WINDSIZEY*sizeof(GLftype));

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    AutoClearColor(COLOR_OFF);
    AutoColor(COLOR_ON);
    glDisable(GL_DITHER);
    glDisable(GL_MULTISAMPLE);

    glDisable(GL_CULL_FACE);

    DrawTriangles(GL_CCW);
    if (CheckNonZero(buf) == ERROR) {
	ErrorReport(__FILE__, __LINE__, errStr[0]);
	FREE(buf);
	return ERROR;
    }
    DrawTriangles(GL_CW);
    if (CheckNonZero(buf) == ERROR) {
	ErrorReport(__FILE__, __LINE__, errStr[1]);
	FREE(buf);
	return ERROR;
    }
    DrawTriangleStrip(GL_CCW);
    if (CheckNonZero(buf) == ERROR) {
	ErrorReport(__FILE__, __LINE__, errStr[0]);
	FREE(buf);
	return ERROR;
    }
    DrawTriangleStrip(GL_CW);
    if (CheckNonZero(buf) == ERROR) {
	ErrorReport(__FILE__, __LINE__, errStr[1]);
	FREE(buf);
	return ERROR;
    }
    DrawTriangleFan(GL_CCW);
    if (CheckNonZero(buf) == ERROR) {
	ErrorReport(__FILE__, __LINE__, errStr[0]);
	FREE(buf);
	return ERROR;
    }
    DrawTriangleFan(GL_CW);
    if (CheckNonZero(buf) == ERROR) {
	ErrorReport(__FILE__, __LINE__, errStr[1]);
	FREE(buf);
	return ERROR;
    }

    glEnable(GL_CULL_FACE);
    glCullFace(GL_FRONT);

    DrawTriangles(GL_CW);
    if (CheckNonZero(buf) == ERROR) {
	ErrorReport(__FILE__, __LINE__, errStr[2]);
	FREE(buf);
	return ERROR;
    }
    DrawTriangles(GL_CCW);
    if (CheckZero(buf) == ERROR) {
	ErrorReport(__FILE__, __LINE__, errStr[3]);
	FREE(buf);
	return ERROR;
    }
    DrawTriangleStrip(GL_CW);
    if (CheckNonZero(buf) == ERROR) {
	ErrorReport(__FILE__, __LINE__, errStr[2]);
	FREE(buf);
	return ERROR;
    }
    DrawTriangleStrip(GL_CCW);
    if (CheckZero(buf) == ERROR) {
	ErrorReport(__FILE__, __LINE__, errStr[3]);
	FREE(buf);
	return ERROR;
    }
    DrawTriangleFan(GL_CW);
    if (CheckNonZero(buf) == ERROR) {
	ErrorReport(__FILE__, __LINE__, errStr[2]);
	FREE(buf);
	return ERROR;
    }
    DrawTriangleFan(GL_CCW);
    if (CheckZero(buf) == ERROR) {
	ErrorReport(__FILE__, __LINE__, errStr[3]);
	FREE(buf);
	return ERROR;
    }

    glCullFace(GL_BACK);

    DrawTriangles(GL_CW);
    if (CheckZero(buf) == ERROR) {
	ErrorReport(__FILE__, __LINE__, errStr[4]);
	FREE(buf);
	return ERROR;
    }
    DrawTriangles(GL_CCW);
    if (CheckNonZero(buf) == ERROR) {
	ErrorReport(__FILE__, __LINE__, errStr[5]);
	FREE(buf);
	return ERROR;
    }
    DrawTriangleStrip(GL_CW);
    if (CheckZero(buf) == ERROR) {
	ErrorReport(__FILE__, __LINE__, errStr[4]);
	FREE(buf);
	return ERROR;
    }
    DrawTriangleStrip(GL_CCW);
    if (CheckNonZero(buf) == ERROR) {
	ErrorReport(__FILE__, __LINE__, errStr[5]);
	FREE(buf);
	return ERROR;
    }
    DrawTriangleFan(GL_CW);
    if (CheckZero(buf) == ERROR) {
	ErrorReport(__FILE__, __LINE__, errStr[4]);
	FREE(buf);
	return ERROR;
    }
    DrawTriangleFan(GL_CCW);
    if (CheckNonZero(buf) == ERROR) {
	ErrorReport(__FILE__, __LINE__, errStr[5]);
	FREE(buf);
	return ERROR;
    }

    FREE(buf);
    return NO_ERROR;
}
