/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/readfmt.c,v 1.4 2005/05/11 15:21:40 artlab Exp $
*/

/*
** readfmt.c
** Read Format Test.
**
** Description -
**    Draws a randomly textured rectangle int he lower left hand corner.
**    The queried read format is used to read back the rectangle. The
**    result is unpacked and compared against the original texture data.
**
** Technical Specification -
**    Buffer requirements:
**	  Color buffer.
**    Color requirements:
**	  Full color.
**    States requirements:
**	  Disabled = GL_DITHER, GL_MULTISAMPLE.
**    Error epsilon:
**	  Color epsilon. Zero epsilons.
**    Paths:
**	  Allowed = Alpha, Blend, Depth, Fog, Logicop, Shade, Stencil.
**	  Not allowed = Alias, Dither.
*/

#include "conform.h"
#include "util.h"
#include "utilp.h"

#define W 32
#define H 32
#define FREE_AND_RETURN(err) {			    \
    FREE(drawBuf);				    \
    FREE(drawBuf2);				    \
    FREE(readBuf);					    \
    FREE(readBuf2);					    \
    return(err);				    \
}
#define MAKE_ERROR_STRING(t, f) {			\
    GetEnumName(t, typeStr);				\
    GetEnumName(f, formatStr);				\
    StrMake(errStr, errStrFmt, typeStr, formatStr);	\
}

static char errStr[160];
static char errStrFmt[] = "ReadPixels type is %s, format is %s.";


static void MakeBuf(GLsizei width, GLsizei height, GLubyte *buf, GLftype *fBuf)
{
    long i;

    for (i = 0; i < width*height*4; i++) {
        buf[i] = (GLubyte)(255*OESRandom(ITOF(0), ITOF(1)));
        fBuf[i] = DIV(ITOF(buf[i]), ITOF(255));
    }
}

static GLsizei GetComponents(GLenum format)
{
    switch (format) {
    default:
	return 0;

    case GL_ALPHA:
    case GL_LUMINANCE:
	return 1;

    case GL_LUMINANCE_ALPHA:
	return 2;

    case GL_RGB:
	return 3;

    case GL_RGBA:
    case GL_BGRA_EXT:
	return 4;
    }
}

static GLsizei GetPixelSize(GLenum format, GLenum type)
{
    switch (type) {
    default:
	return 0;

    case GL_UNSIGNED_BYTE:
	return GetComponents(format);

    case GL_UNSIGNED_SHORT_5_6_5:
    case GL_UNSIGNED_SHORT_4_4_4_4:
    case GL_UNSIGNED_SHORT_5_5_5_1:
    case GL_UNSIGNED_SHORT_4_4_4_4_REV_EXT:
    case GL_UNSIGNED_SHORT_1_5_5_5_REV_EXT:
	return 2;
    }
}

static void
ConvertToRGBA(GLenum format, GLenum type, GLubyte const *packed_color, GLftype rgba[4])
{
    if (type == GL_UNSIGNED_BYTE) {

	switch (format) {
	case GL_ALPHA:
	    rgba[0] =
	    rgba[1] =
	    rgba[2] = ITOF(0);
	    rgba[3] = DIV(ITOF(packed_color[0]), ITOF(255));
	    break;

	/* problematic */
	case GL_LUMINANCE:
	    rgba[0] =
	    rgba[1] =
	    rgba[2] = DIV(ITOF(packed_color[0]), ITOF(255)*3);
	    rgba[3] = ITOF(1);
	    break;

	/* problematic */
	case GL_LUMINANCE_ALPHA:
	    rgba[0] =
	    rgba[1] =
	    rgba[2] = DIV(ITOF(packed_color[0]), ITOF(255)*3);
	    rgba[3] = DIV(ITOF(packed_color[1]), ITOF(255));
	    break;

	case GL_RGB:
	    rgba[0] = DIV(ITOF(packed_color[0]), ITOF(255));
	    rgba[1] = DIV(ITOF(packed_color[1]), ITOF(255));
	    rgba[2] = DIV(ITOF(packed_color[2]), ITOF(255));
	    rgba[3] = ITOF(1);
	    break;

	case GL_RGBA:
	    rgba[0] = DIV(ITOF(packed_color[0]), ITOF(255));
	    rgba[1] = DIV(ITOF(packed_color[1]), ITOF(255));
	    rgba[2] = DIV(ITOF(packed_color[2]), ITOF(255));
	    rgba[3] = DIV(ITOF(packed_color[3]), ITOF(255));
	    break;

	case GL_BGRA_EXT:
	    rgba[0] = DIV(ITOF(packed_color[2]), ITOF(255));
	    rgba[1] = DIV(ITOF(packed_color[1]), ITOF(255));
	    rgba[2] = DIV(ITOF(packed_color[0]), ITOF(255));
	    rgba[3] = DIV(ITOF(packed_color[3]), ITOF(255));
	    break;
	}

    } else {
        packPixRec const *packed_format = packPixInfo(type);
        ConvertPackedtoRGBA(packed_color, rgba, packed_format);
    }
}

static void ConvertBuf(GLsizei width, GLsizei height, GLenum format, GLenum type, GLvoid const *buf, GLftype *fBuf)
{
    GLubyte const *ubBuf;
    GLsizei pixel_size;
    long i;

    ubBuf = (GLubyte const *) buf;
    pixel_size = GetPixelSize(format, type);

    for (i = 0; i < width*height; ++i) {
	ConvertToRGBA(format, type, &ubBuf[i * pixel_size], &fBuf[i * 4]);
    }
}

static void GetEpsilon(GLenum format, GLenum type, GLftype pack_epsilon[4])
{
    GLsizei bits[4];
    long i;

    GetComponentBits(format, type, bits);

    /* texture epsilon */
    for (i = 0; i < 4; ++i) {
	if (bits[i] > 4) {
	    bits[i] = 4;
	}
    }

    for (i = 0; i < 4; ++i) {
	if (bits[i] == 0 || buffer.colorBits[i] == 0) {
	    pack_epsilon[i] = ITOF(2);
	} else {
	    pack_epsilon[i] = CalcEpsilon(bits[i]);
	    if (pack_epsilon[i] < epsilon.color[i]) {
		pack_epsilon[i] = epsilon.color[i];
	    }
	}
    }
}

static long TestBuf(GLftype const *refBuf, GLftype const *testBuf, GLsizei width, GLsizei height, GLenum format, GLenum type)
{
    GLftype epsilon[4];
    long i, j, k;

    /* cannot use texture epsilon */
    GetEpsilon(format, type, epsilon);

    k = 0;
    for (i = 0; i < width * height; ++i) {
	for (j = 0; j < 4; ++j) {
	    if (ABS(testBuf[k]-refBuf[k]) >= epsilon[j]) {
		return ERROR;
	    }
	    k++;
	}
    }

    return NO_ERROR;
}

long ReadFormatExec(void)
{
    GLubyte *drawBuf;
    GLvoid *readBuf;
    GLftype *drawBuf2, *readBuf2;
    GLint format, type;
    char typeStr[40], formatStr[40];
    GLboolean bgraAvailable = GL_FALSE;
    GLboolean bgra1555Available = GL_FALSE;

	if(tkCheckExtension("GL_EXT_read_format_bgra") || tkCheckExtension("GL_IMG_read_format"))
		bgraAvailable = GL_TRUE;

	bgra1555Available = tkCheckExtension("GL_EXT_read_format_bgra");

    drawBuf = (GLubyte *)MALLOC(W*H*4*sizeof(GLubyte));
    drawBuf2 = (GLftype *)MALLOC(W*H*4*sizeof(GLftype));
    readBuf = (GLvoid *)MALLOC(W*H*4*sizeof(GLftype)); /* unknown type */
    readBuf2 = (GLftype *)MALLOC(W*H*4*sizeof(GLftype));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glColor4t(ITOF(1), ITOF(1), ITOF(1), ITOF(1));
    glDisable(GL_DITHER);
    glDisable(GL_MULTISAMPLE);

    glGetIntegerv(GL_IMPLEMENTATION_COLOR_READ_FORMAT_OES, &format);
    glGetIntegerv(GL_IMPLEMENTATION_COLOR_READ_TYPE_OES, &type);

    switch (format) {
 	case GL_BGRA_EXT :
		if (bgraAvailable) break;
    default:
	MAKE_ERROR_STRING(type, format);
	ErrorReport(__FILE__, __LINE__, errStr);
	FREE_AND_RETURN(NOT_SUPPORTED);

    case GL_ALPHA:
    case GL_LUMINANCE:
    case GL_LUMINANCE_ALPHA:
    case GL_RGB:
    case GL_RGBA:
        break;
    }

    switch (type) {
    default:
	MAKE_ERROR_STRING(type, format);
	ErrorReport(__FILE__, __LINE__, errStr);
	FREE_AND_RETURN(NOT_SUPPORTED);

    case GL_UNSIGNED_SHORT_5_6_5:
	if (format == GL_RGB)
	    break;

	MAKE_ERROR_STRING(type, format);
	ErrorReport(__FILE__, __LINE__, errStr);
	FREE_AND_RETURN(NOT_SUPPORTED);

    case GL_UNSIGNED_SHORT_4_4_4_4:
    case GL_UNSIGNED_SHORT_5_5_5_1:
	if (format == GL_RGBA)
	    break;

	MAKE_ERROR_STRING(type, format);
	ErrorReport(__FILE__, __LINE__, errStr);
	FREE_AND_RETURN(NOT_SUPPORTED);
	case GL_UNSIGNED_SHORT_4_4_4_4_REV_EXT :
	if ((format == GL_BGRA_EXT) && bgraAvailable)
		break;
	MAKE_ERROR_STRING(type, format);
	ErrorReport(__FILE__, __LINE__, errStr);
	FREE_AND_RETURN(NOT_SUPPORTED);

	case GL_UNSIGNED_SHORT_1_5_5_5_REV_EXT :
	if ((format == GL_BGRA_EXT) && bgra1555Available)
		break;
	MAKE_ERROR_STRING(type, format);
	ErrorReport(__FILE__, __LINE__, errStr);
	FREE_AND_RETURN(NOT_SUPPORTED);

    case GL_UNSIGNED_BYTE:
	break;
    }

    MakeBuf(W, H, drawBuf, drawBuf2);

    glClear(GL_COLOR_BUFFER_BIT);

    DrawPixels(0, 0, W, H, GL_RGBA, GL_UNSIGNED_BYTE, drawBuf);
    glReadPixels(0, 0, W, H, format, type, readBuf);

    ConvertBuf(W, H, format, type, readBuf, readBuf2);

    if (TestBuf(drawBuf2, readBuf2, W, H, format, type) == ERROR) {
	MAKE_ERROR_STRING(type, format);
	ErrorReport(__FILE__, __LINE__, errStr);
	FREE_AND_RETURN(ERROR);
    }

    FREE_AND_RETURN(NO_ERROR);
}
