/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/rescalen.c,v 1.4 2005/05/11 15:21:40 artlab Exp $
*/

/*
** rescalen.c
** Rescale Normal Test.
**
** Description -
**  Tests RESCALE_NORMAL and NORMALIZE modes. Based on the xformn.c
**  Transform Normal test with added normal scaling tests.
**
** Technical Specification -
**    Buffer requirements:
**	  Color buffer.
**    Color requirements:
**	  2 colors.
**    States requirements:
**	  Disabled = GL_DITHER, GL_MULTISAMPLE.
**    Error epsilon:
**	  Color epsilon.
**    Paths:
**	  Allowed = Alpha, Blend, Depth, Logicop, Stencil.
**	  Not allowed = Alias, Dither, Fog, Shade.
*/

#include "conform.h"
#include "util.h"
#include "utils.h"

/* Range of angles to run the tests over */
#define ROTANGLE 87

static char errStr[512], errMode[40];
static char errStrFmtRGB[] =
    "Rescale mode %s. Scale factor is %g. "
    "Angle between normal and light is %4.2f. "
    "Expected color is (%1.3f, %1.3f, %1.3f). "
    "Actual color is (%1.3f, %1.3f, %1.3f)";

static GLftype MakeScale(void);

static GLint TestNormalRGB(GLftype angle, GLftype *buf, GLenum mode,
			   GLftype scale)
{
    GLftype cosine;
    GLint midX, midY, i;

    midX = WINDSIZEX / 2;
    midY = WINDSIZEY / 2;

    i = WINDSIZEX * midY * 3 + midX * 3;
    cosine = COS(MUL(angle, PI)/180);
    cosine = (cosine < ITOF(0)) ? ITOF(0) : cosine;

    if (ABS(buf[i]-cosine) < epsilon.color[0] &&
	ABS(buf[i+1]-cosine) < epsilon.color[1] &&
	ABS(buf[i+2]-cosine) < epsilon.color[2]) {
	return NO_ERROR;
    }

    GetEnumName(mode, errMode);
    StrMake(errStr, errStrFmtRGB, errMode, scale, angle,
	    cosine, cosine, cosine, buf[i], buf[i+1], buf[i+2]);
    return ERROR;
}

static GLftype MakeScale() {
    return EXP(OESRandom(ITOF(-2), ITOF(2)));
}

static long Test(GLenum mode)
{
    GLftype front_mat_ambient[] = { ITOF(0), ITOF(0), ITOF(0), ITOF(1) };
    GLftype front_mat_diffuse[] = { ITOF(1), ITOF(1), ITOF(1), ITOF(1) };
    GLftype front_mat_specular[] = { ITOF(0), ITOF(0), ITOF(0), ITOF(1) };
    GLftype position[] = { ITOF(0), ITOF(0), ITOF(1), ITOF(0) };
    GLftype vertices[4][2];
    GLftype vertices2[4][3];
    GLftype *buf;
    GLftype polyAngle, lightAngle, step;
    GLint i;
    GLftype scale;

    buf = (GLftype *)MALLOC(WINDSIZEX*WINDSIZEY*3*sizeof(GLftype));

    glViewport(0, 0, WINDSIZEX, WINDSIZEY);

    AutoClearColor(BLACK);
    glDisable(GL_DITHER);
    glDisable(GL_MULTISAMPLE);

    /*
    ** To test value of normal, set lighting equation parameters so that
    ** most values drop out.  The equation reduces to
    **	    c = max(0, n dot VP)
    **	    and (n dot VP) reduces to cos(theta),
    ** where theta is the angle  between the normal and the vector from
    ** the vertex to the light.
    */
    glMaterialtv(GL_FRONT_AND_BACK, GL_AMBIENT, front_mat_ambient);
    glMaterialtv(GL_FRONT_AND_BACK, GL_SPECULAR, front_mat_specular);
    glMaterialtv(GL_FRONT_AND_BACK, GL_DIFFUSE, front_mat_diffuse);

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);

    glLighttv(GL_LIGHT0, GL_POSITION, position);
    glNormal3t(position[0], position[1], position[2]);

    /*
    ** Test the normal after rotation by rotating the polygon about the
    ** y-axis and examining the resulting color of at a point on the y-axis.
    */
    step = (machine.pathLevel == 0) ? ITOF(12) : ITOF(45);
    for (polyAngle = -ITOF(ROTANGLE); polyAngle < ITOF(ROTANGLE); polyAngle += step) {
	glClear(GL_COLOR_BUFFER_BIT);

	glPushMatrix();

	scale = MakeScale();
	glRotatet(polyAngle, ITOF(0), ITOF(1), ITOF(0));
	glScalet(scale, scale, scale);

	vertices[0][0] = DIV(ATOF("-0.7"), scale); vertices[0][1] = DIV(ATOF("+0.6"), scale);
	vertices[1][0] = DIV(ATOF("-0.7"), scale); vertices[1][1] = DIV(ATOF("-0.6"), scale);
	vertices[2][0] = DIV(ATOF("+0.7"), scale); vertices[2][1] = ITOF(0);

	glEnableClientState(GL_VERTEX_ARRAY);
	glVertexPointer(2, GL_FTYPE, 0, &vertices[0][0]);
	glDrawArrays(GL_TRIANGLES, 0, 3);
	glDisableClientState(GL_VERTEX_ARRAY);

	glPopMatrix();

	ReadScreen(0, 0, WINDSIZEX, WINDSIZEY, GL_RGB, buf);
	if (TestNormalRGB(polyAngle, buf, mode, scale) == ERROR) {
	    ErrorReport(__FILE__, __LINE__, errStr);
	    FREE(buf);
	    return ERROR;
	}
    }

    /*
    ** Move the light source to locations on a circle of radius one in the xz
    ** plane. Rotate the polygon around the y-axis.
    */
    for (i = 0; i < 4; i++) {
	lightAngle = OESRandom(-ITOF(ROTANGLE), ITOF(ROTANGLE));
	position[0] = SIN(MUL(lightAngle, PI)/180);
	position[1] = ITOF(0);
	position[2] = COS(MUL(lightAngle, PI)/180);
	glLighttv(GL_LIGHT0, GL_POSITION, position);

	step = (machine.pathLevel == 0) ? ITOF(12) : ITOF(45);
	for (polyAngle = -ITOF(ROTANGLE); polyAngle < ITOF(ROTANGLE); polyAngle += step) {
	    glClear(GL_COLOR_BUFFER_BIT);

	    glPushMatrix();

	    scale = MakeScale();
	    glRotatet(polyAngle, ITOF(0), ITOF(1), ITOF(0));
	    glScalet(scale, scale, scale);

	    vertices[0][0] = DIV(ATOF("-0.7"), scale); vertices[0][1] = DIV(ATOF("+0.6"), scale);
	    vertices[1][0] = DIV(ATOF("-0.7"), scale); vertices[1][1] = DIV(ATOF("-0.6"), scale);
	    vertices[2][0] = DIV(ATOF("+0.7"), scale); vertices[2][1] = ITOF(0);

	    glEnableClientState(GL_VERTEX_ARRAY);
	    glVertexPointer(2, GL_FTYPE, 0, &vertices[0][0]);
	    glDrawArrays(GL_TRIANGLES, 0, 3);
	    glDisableClientState(GL_VERTEX_ARRAY);

	    glPopMatrix();

	    ReadScreen(0, 0, WINDSIZEX, WINDSIZEY, GL_RGB, buf);
	    if (TestNormalRGB(lightAngle-polyAngle, buf, mode, scale) == ERROR) {
		ErrorReport(__FILE__, __LINE__, errStr);
		FREE(buf);
		return ERROR;
	    }
	}
    }

    /*
    ** Move the light source to locations on a circle of radius one in the yz
    ** plane. Rotate the polygon around the x-axis.
    */
    for (i = 0; i < 4; i++) {
	lightAngle = OESRandom(-ITOF(ROTANGLE), ITOF(ROTANGLE));
	position[0] = ITOF(0);
	position[1] = -SIN(MUL(lightAngle, PI)/180);
	position[2] = COS(MUL(lightAngle, PI)/180);
	glLighttv(GL_LIGHT0, GL_POSITION, position);

	step = (machine.pathLevel == 0) ? ITOF(12) : ITOF(45);
	for (polyAngle = -ITOF(ROTANGLE); polyAngle < ITOF(ROTANGLE); polyAngle += step) {
	    glClear(GL_COLOR_BUFFER_BIT);

	    glPushMatrix();

	    scale = MakeScale();
	    glRotatet(polyAngle, ITOF(1), ITOF(0), ITOF(0));
	    glScalet(scale, scale, scale);

	    vertices[0][0] = DIV(ATOF("-0.7"), scale); vertices[0][1] = DIV(ATOF("+0.7"), scale);
	    vertices[1][0] = DIV(ATOF("+0.7"), scale); vertices[1][1] = DIV(ATOF("+0.7"), scale);
	    vertices[2][0] = DIV(ATOF("+0.7"), scale); vertices[2][1] = DIV(ATOF("-0.7"), scale);
	    vertices[3][0] = DIV(ATOF("-0.7"), scale); vertices[3][1] = DIV(ATOF("-0.7"), scale);

	    glEnableClientState(GL_VERTEX_ARRAY);
	    glVertexPointer(2, GL_FTYPE, 0, &vertices[0][0]);
	    glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
	    glDisableClientState(GL_VERTEX_ARRAY);

	    glPopMatrix();

	    ReadScreen(0, 0, WINDSIZEX, WINDSIZEY, GL_RGB, buf);
	    if (TestNormalRGB(lightAngle-polyAngle, buf, mode, scale) == ERROR) {
		ErrorReport(__FILE__, __LINE__, errStr);
		FREE(buf);
		return ERROR;
	    }
	}
    }

    /*
    ** Repeat the test on a 45 degree axis.
    ** The light is fixed at a distance 1 from the origin, tilted 45 degrees
    ** down in the yz plane. The polygon is a rectangle tilted 45 degrees about
    ** the x-axis, rotating about the (0, 1, 1) axis.
    */
    position[0] = ITOF(0);
    position[1] = -SIN(MUL(ITOF(45), PI)/180);
    position[2] = COS(MUL(ITOF(45), PI)/180);
    glLighttv(GL_LIGHT0, GL_POSITION, position);

    step = (machine.pathLevel == 0) ? ITOF(12) : ITOF(45);
    for (polyAngle = -ITOF(60); polyAngle < ITOF(60); polyAngle += step) {
	glClear(GL_COLOR_BUFFER_BIT);

	glPushMatrix();

	scale = MakeScale();
	glRotatet(polyAngle, ITOF(0), ITOF(1), ITOF(1));
	glScalet(scale, scale, scale);
	glNormal3t(position[0], position[1], position[2]);

	vertices2[0][0] = DIV(ATOF("+0.5"), scale); vertices2[0][1] = DIV(ATOF("+0.3"), scale); vertices2[0][2] = DIV(ATOF("+0.5"), scale);
	vertices2[1][0] = DIV(ATOF("-0.5"), scale); vertices2[1][1] = DIV(ATOF("+0.3"), scale); vertices2[1][2] = DIV(ATOF("+0.5"), scale);
	vertices2[2][0] = DIV(ATOF("-0.5"), scale); vertices2[2][1] = DIV(ATOF("-0.3"), scale); vertices2[2][2] = DIV(ATOF("-0.5"), scale);
	vertices2[3][0] = DIV(ATOF("+0.5"), scale); vertices2[3][1] = DIV(ATOF("-0.3"), scale); vertices2[3][2] = DIV(ATOF("-0.5"), scale);

	glEnableClientState(GL_VERTEX_ARRAY);
	glVertexPointer(3, GL_FTYPE, 0, &vertices2[0][0]);
	glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
	glDisableClientState(GL_VERTEX_ARRAY);

	glPopMatrix();

	ReadScreen(0, 0, WINDSIZEX, WINDSIZEY, GL_RGB, buf);
	if (TestNormalRGB(polyAngle, buf, mode, scale) == ERROR) {
	    ErrorReport(__FILE__, __LINE__, errStr);
	    FREE(buf);
	    return ERROR;
	}
    }

    FREE(buf);

    return NO_ERROR;
}

long RescaleNormalExec(void)
{
    static GLenum modes[2] = { GL_NORMALIZE, GL_RESCALE_NORMAL };
    GLint i;

    for (i = 0; i < 2; i++) {
	glEnable(modes[i]);
	if (Test(modes[i]) == ERROR) {
	    glDisable(modes[i]);
	    StateReset();
	    return ERROR;
	}
	glDisable(modes[i]);
	StateReset();
    }

    return NO_ERROR;
}
