/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/spclear.c,v 1.3 2005/05/11 15:21:40 artlab Exp $
*/

/*
** spclear.c
** Stencil Plane Clear Test.
**
** Description -
**    A clear to zero and a clear to one is performed on the
**    stencil plane and verified. If stencil plane is not
**    available, a clear to one should behave as a clear to
**    zero.
**
** Technical Specification -
**    Buffer requirements:
**        No buffer requirements.
**    Color requirements:
**        No color requirements.
**    States requirements:
**        Disabled = GL_DITHER, GL_MULTISAMPLE.
**    Error epsilon:
**    Paths:
**        Allowed = Alias, Alpha, Blend, Depth, Fog, Logicop, Shade.
**        Not allowed = Dither, Stencil.
*/

#include "conform.h"
#include "util.h"


static char errStr[240];
static char errStrFmt[] = "Location (%d, %d). Color value is (%1.1f, %1.1f, %1.1f, %1.1f), should be (%1.1f, %1.1f, %1.1f, %1.1f).";


static long Test(GLftype value, GLftype *buf)
{
    GLftype expected[4];
    long i, j, k, l;

    expected[0] = (buffer.colorBits[0] == 0 ? ITOF(0) : value);
    expected[1] = (buffer.colorBits[1] == 0 ? ITOF(0) : value);
    expected[2] = (buffer.colorBits[2] == 0 ? ITOF(0) : value);
    expected[3] = (buffer.colorBits[3] == 0 ? ITOF(1) : value);

    ReadScreen(0, 0, WINDSIZEX, WINDSIZEY, GL_RGBA, buf);

    if (glGetError() != GL_NO_ERROR) {
	return ERROR;
    }

    l = 0;
    for (i = 0; i < WINDSIZEY; i++) {
	for (j = 0; j < WINDSIZEX; j++) {
	    for (k = 0; k < 4; k++) {
	        if (ABS(buf[l * 4 + k]-expected[k]) > epsilon.zero) {
		    StrMake(errStr, errStrFmt, j, i,
			    buf[l*4 + 0], buf[l*4 + 1], buf[l*4 + 2], buf[l* + 3],
			    expected[0], expected[1], expected[2], expected[3]);
		    ErrorReport(__FILE__, __LINE__, errStr);
		    return ERROR;
	        }
	    }
	    l++;
	}
    }
    return NO_ERROR;
}

long SPClearExec(void)
{
    GLftype *buf;
    GLint i;

    buf = (GLftype *)MALLOC(WINDSIZEX*WINDSIZEY*4*sizeof(GLftype));

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glDisable(GL_DITHER);
    glDisable(GL_MULTISAMPLE);

    glEnable(GL_STENCIL_TEST);
    glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);

    glClearStencil(0);
    glClear(GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
    glStencilFunc(GL_NOTEQUAL, 0, ~0);
    DrawPlane(ITOF(1), ITOF(0));
    if (buffer.stencilBits == 0) {
       	if (Test(ITOF(1), buf) == ERROR) {
	    FREE(buf);
	    return ERROR;
	}
    } else {
       	if (Test(ITOF(0), buf) == ERROR) {
	    FREE(buf);
	    return ERROR;
	}
    }

    glStencilFunc(GL_EQUAL, 0, ~0);
    DrawPlane(ITOF(1), ITOF(0));
    if (Test(ITOF(1), buf) == ERROR) {
	FREE(buf);
	return ERROR;
    }

    i = FTOI(LDEXP(ITOF(1), buffer.stencilBits)) - 1;
    glClearStencil(i);
    glClear(GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
    glStencilFunc(GL_NOTEQUAL, i, ~0);
    DrawPlane(ITOF(1), ITOF(0));
    if (buffer.stencilBits == 0) {
       	if (Test(ITOF(1), buf) == ERROR) {
	    FREE(buf);
	    return ERROR;
	}
    } else {
       	if (Test(ITOF(0), buf) == ERROR) {
	    FREE(buf);
	    return ERROR;
	}
    }
    glStencilFunc(GL_EQUAL, i, ~0);
    DrawPlane(ITOF(1), ITOF(0));
    if (Test(ITOF(1), buf) == ERROR) {
	FREE(buf);
	return ERROR;
    }

    FREE(buf);
    return NO_ERROR;
}
