/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/spcorner.c,v 1.3 2005/05/11 15:21:40 artlab Exp $
*/

/*
** spcorner.c
** Stencil Plane Corner Test.
**
** Description -
**    The stencil function is set to GL_NEVER, the stencil
**    operation is set to GL_INCR and the stencil plane is
**    cleared to zero. A point is drawn at each corner of the
**    color buffer. Because of the stencil settings, the corners
**    of the stencil plane should be set to one. The stencil
**    plane is examined to verify that the corner points
**    are at the correct location.
**
** Error Explanation -
**    Failure occurs if any of the corner points cannot be read
**    correctly (implying that the stencil plane's width and
**    height or the rendering coordinates are not correct).
**
** Technical Specification -
**    Buffer requirements:
**        Color buffer, stencil plane.
**    Color requirements:
**        2 colors.
**    States requirements:
**        Disabled = GL_DITHER, GL_MULTISAMPLE.
**    Error epsilon:
**    Paths:
**        Allowed = Alpha, Blend, Depth, Fog, Logicop, Shade.
**        Not allowed = Alias, Dither, Stencil.
*/

#include "conform.h"
#include "util.h"


static char errStr[160];
static char errStrFmt[] = "Mismatch at the %s corner.";

long SPCornerExec(void)
{
    static const GLftype vertices[4][4] = {
        {ITOF(1)/2, ITOF(1)/2, ITOF(0), ITOF(1)},
        {ITOF(WINDSIZEX)-ITOF(1)/2, ITOF(1)/2, ITOF(0), ITOF(1)},
        {ITOF(1)/2, ITOF(WINDSIZEY)-ITOF(1)/2, ITOF(0), ITOF(1)},
        {ITOF(WINDSIZEX)-ITOF(1)/2, ITOF(WINDSIZEY)-ITOF(1)/2, ITOF(0), ITOF(1)}
    };
    GLftype *buf;

    if (buffer.stencilBits == 0) {
	return NO_ERROR;
    }

    buf = (GLftype *)MALLOC(WINDSIZEX*WINDSIZEY*sizeof(GLftype));

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glDisable(GL_DITHER);
    glDisable(GL_MULTISAMPLE);

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));

    glEnable(GL_STENCIL_TEST);
    glClearStencil(0);
    glStencilFunc(GL_NEVER, 0, ~0);
    glStencilOp(GL_INCR, GL_KEEP, GL_KEEP);

    glClear(GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(4, GL_FTYPE, 0, &vertices[0][0]);
    glDrawArrays(GL_POINTS, 0, 4);
    glDisableClientState(GL_VERTEX_ARRAY);

    glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);

    glStencilFunc(GL_NOTEQUAL, 1, ~0);
    DrawPlane(ITOF(1), ITOF(0));
    ReadScreen(0, 0, WINDSIZEX, WINDSIZEY, GL_LUMINANCE, buf);

    if (buffer.stencilBits == 0) {
	if (buf[0] <= ITOF(1)/2) {
	    StrMake(errStr, errStrFmt, "lower left");
	    ErrorReport(__FILE__, __LINE__, errStr);
	    FREE(buf);
	    return ERROR;
	}
	if (buf[WINDSIZEX-1] <= ITOF(1)/2) {
	    StrMake(errStr, errStrFmt, "lower right");
	    ErrorReport(__FILE__, __LINE__, errStr);
	    FREE(buf);
	    return ERROR;
	}
	if (buf[WINDSIZEX*(WINDSIZEY-1)] <= ITOF(1)/2) {
	    StrMake(errStr, errStrFmt, "upper left");
	    ErrorReport(__FILE__, __LINE__, errStr);
	    FREE(buf);
	    return ERROR;
	}
	if (buf[WINDSIZEX*WINDSIZEY-1] <= ITOF(1)/2) {
	    StrMake(errStr, errStrFmt, "upper right");
	    ErrorReport(__FILE__, __LINE__, errStr);
	    FREE(buf);
	    return ERROR;
	}
    } else {    
	if (buf[0] >= ITOF(1)/2) {
	    StrMake(errStr, errStrFmt, "lower left");
	    ErrorReport(__FILE__, __LINE__, errStr);
	    FREE(buf);
	    return ERROR;
	}
	if (buf[WINDSIZEX-1] >= ITOF(1)/2) {
	    StrMake(errStr, errStrFmt, "lower right");
	    ErrorReport(__FILE__, __LINE__, errStr);
	    FREE(buf);
	    return ERROR;
	}
	if (buf[WINDSIZEX*(WINDSIZEY-1)] >= ITOF(1)/2) {
	    StrMake(errStr, errStrFmt, "upper left");
	    ErrorReport(__FILE__, __LINE__, errStr);
	    FREE(buf);
	    return ERROR;
	}
	if (buf[WINDSIZEX*WINDSIZEY-1] >= ITOF(1)/2) {
	    StrMake(errStr, errStrFmt, "upper right");
	    ErrorReport(__FILE__, __LINE__, errStr);
	    FREE(buf);
	    return ERROR;
	}
    }

    glStencilFunc(GL_EQUAL, 1, ~0);
    DrawPlane(ITOF(1), ITOF(0));
    ReadScreen(0, 0, WINDSIZEX, WINDSIZEY, GL_LUMINANCE, buf);

    if (buf[0] <= ITOF(1)/2) {
	StrMake(errStr, errStrFmt, "lower left");
	ErrorReport(__FILE__, __LINE__, errStr);
	FREE(buf);
	return ERROR;
    }
    if (buf[WINDSIZEX-1] <= ITOF(1)/2) {
	StrMake(errStr, errStrFmt, "lower right");
	ErrorReport(__FILE__, __LINE__, errStr);
	FREE(buf);
	return ERROR;
    }
    if (buf[WINDSIZEX*(WINDSIZEY-1)] <= ITOF(1)/2) {
	StrMake(errStr, errStrFmt, "upper left");
	ErrorReport(__FILE__, __LINE__, errStr);
	FREE(buf);
	return ERROR;
    }
    if (buf[WINDSIZEX*WINDSIZEY-1] <= ITOF(1)/2) {
	StrMake(errStr, errStrFmt, "upper right");
	ErrorReport(__FILE__, __LINE__, errStr);
	FREE(buf);
	return ERROR;
    }

    FREE(buf);
    return NO_ERROR;
}
