/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/spop.c,v 1.4 2005/05/11 15:21:40 artlab Exp $
*/

/*
** spop.c
** Stencil Plane Operation Test.
**
** Description -
**    Tests the conformance of glStencilOp(). Each stencil
**    operation (GL_KEEP, GL_REPLACE, GL_ZERO, GL_INCR, GL_DECR,
**    GL_INVERT) is tested by initializing the stencil plane to a
**    known value, performing the stencil operation (by setting
**    up the stencil and depth functions and rendering a point)
**    and comparing the stencil plane result to an expected
**    value. The sfail operation is tested by setting the
**    stencil function to GL_NEVER. The dpfail operation is
**    tested by setting the stencil function to GL_ALWAYS and the
**    depth function to GL_NEVER. The dppass operation is tested
**    by setting both the stencil function and depth function to
**    GL_ALWAYS.
**
** Technical Specification -
**    Buffer requirements:
**        Color buffer, stencil plane.
**    Color requirements:
**        2 colors.
**    States requirements:
**        No state requirements.
**    Error epsilon:
**    Paths:
**        Allowed = Alias, Alpha, Blend, Depth, Dither, Fog, Logicop,
**                  Shade.
**        Not allowed = Dither, Stencil.
*/

#include "conform.h"
#include "util.h"


static char errStr[160];
static char errStrFmt[] = "glStencilOp(%s). Stencil value should be %d.";


static GLuint Setup(GLenum func, GLenum op)
{
    GLuint minValue, maxValue, refValue, testValue, value = 0;

    minValue = 0;
    maxValue = (1 << buffer.stencilBits) - 1;

    refValue = minValue + 1;
    testValue = maxValue - 1;

    glStencilFunc(func, refValue, ~0);
    glClearStencil(testValue);

    switch (op) {
      case GL_KEEP:
	value = testValue;
	break;
      case GL_ZERO:
	value = minValue;
	break;
      case GL_REPLACE:
	value = refValue;
	break;
      case GL_INCR:
	value = testValue + 1;
	break;
      case GL_DECR:
	value = (testValue >= 1) ? (testValue - 1) : 0;
	break;
      case GL_INVERT:
	value = ~testValue;
	break;
    }

    value &= maxValue;

    if (value < minValue) {
	value = minValue;
    } else if (value > maxValue) {
	value = maxValue;
    }

    return value;
}

static long Test(GLenum op1, GLenum op2, GLenum op3, GLuint value,
    GLboolean depthTest)
{
    GLftype vertex[4];
    GLftype buf;
    GLint x, y;

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));

    glClear(GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
    glStencilOp(op1, op2, op3);

    x = FTOI(OESRandom(ITOF(0), ITOF(WINDSIZEX - 1)));
    y = FTOI(OESRandom(ITOF(0), ITOF(WINDSIZEY - 1)));

    vertex[0] = ITOF(x) + ITOF(1)/2;
    vertex[1] = ITOF(y) + ITOF(1)/2;
    vertex[2] = ITOF(0);
    vertex[3] = ITOF(1);

    glColor4t(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(4, GL_FTYPE, 0, &vertex[0]);
    glDrawArrays(GL_POINTS, 0, 1);
    glDisableClientState(GL_VERTEX_ARRAY);

    if (depthTest) {
	glDisable(GL_DEPTH_TEST);
    }

    glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
    glStencilFunc(GL_NOTEQUAL, value, ~0);
    DrawPlane(ITOF(1), ITOF(0));
    ReadScreen(x, y, 1, 1, GL_LUMINANCE, &buf);
    if (buf >= ITOF(1)/2) {
	return ERROR;
    }

    glStencilFunc(GL_EQUAL, value, ~0);
    DrawPlane(ITOF(1), ITOF(0));
    ReadScreen(x, y, 1, 1, GL_LUMINANCE, &buf);
    if (buf <= ITOF(1)/2) {
	return ERROR;
    }

    if (depthTest) {
	glEnable(GL_DEPTH_TEST);
    }

    return NO_ERROR;
}

long SPOpExec(void)
{
    GLboolean depthTest;
    GLuint trueValue;

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glDisable(GL_DITHER);
    glDisable(GL_MULTISAMPLE);

    glEnable(GL_STENCIL_TEST);

    depthTest = GL_FALSE;

    trueValue = Setup(GL_NEVER, GL_KEEP);
    if (buffer.stencilBits == 0) {
	return NO_ERROR;
    }

    if (Test(GL_KEEP, GL_KEEP, GL_KEEP, trueValue, depthTest) == ERROR) {
        StrMake(errStr, errStrFmt, "GL_KEEP, GL_KEEP, GL_KEEP", trueValue);
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }
    trueValue = Setup(GL_NEVER, GL_ZERO);
    if (Test(GL_ZERO, GL_KEEP, GL_KEEP, trueValue, depthTest) == ERROR) {
        StrMake(errStr, errStrFmt, "GL_ZERO, GL_KEEP, GL_KEEP", trueValue);
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }
    trueValue = Setup(GL_NEVER, GL_REPLACE);
    if (Test(GL_REPLACE, GL_KEEP, GL_KEEP, trueValue, depthTest) == ERROR) {
        StrMake(errStr, errStrFmt, "GL_REPLACE, GL_KEEP, GL_KEEP", trueValue);
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }
    trueValue = Setup(GL_NEVER, GL_INCR);
    if (Test(GL_INCR, GL_KEEP, GL_KEEP, trueValue, depthTest) == ERROR) {
        StrMake(errStr, errStrFmt, "GL_INCR, GL_KEEP, GL_KEEP", trueValue);
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }
    trueValue = Setup(GL_NEVER, GL_DECR);
    if (Test(GL_DECR, GL_KEEP, GL_KEEP, trueValue, depthTest) == ERROR) {
        StrMake(errStr, errStrFmt, "GL_DECR, GL_KEEP, GL_KEEP", trueValue);
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }
    trueValue = Setup(GL_NEVER, GL_INVERT);
    if (Test(GL_INVERT, GL_KEEP, GL_KEEP, trueValue, depthTest) == ERROR) {
        StrMake(errStr, errStrFmt, "GL_INVERT, GL_KEEP, GL_KEEP", trueValue);
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }

    glEnable(GL_DEPTH_TEST);
    depthTest = GL_TRUE;

    if (buffer.zBits > 0) {
	glDepthFunc(GL_NEVER);

	trueValue = Setup(GL_ALWAYS, GL_KEEP);
	if (Test(GL_KEEP, GL_KEEP, GL_KEEP, trueValue, depthTest) == ERROR) {
            StrMake(errStr, errStrFmt, "GL_KEEP, GL_KEEP, GL_KEEP", trueValue);
	    ErrorReport(__FILE__, __LINE__, errStr);
	    return ERROR;
	}
	trueValue = Setup(GL_ALWAYS, GL_ZERO);
	if (Test(GL_KEEP, GL_ZERO, GL_KEEP, trueValue, depthTest) == ERROR) {
            StrMake(errStr, errStrFmt, "GL_KEEP, GL_ZERO, GL_KEEP", trueValue);
	    ErrorReport(__FILE__, __LINE__, errStr);
	    return ERROR;
	}
	trueValue = Setup(GL_ALWAYS, GL_REPLACE);
	if (Test(GL_KEEP, GL_REPLACE, GL_KEEP, trueValue, depthTest) == ERROR) {
            StrMake(errStr, errStrFmt, "GL_KEEP, GL_REPLACE, GL_KEEP", trueValue);
	    ErrorReport(__FILE__, __LINE__, errStr);
	    return ERROR;
	}
	trueValue = Setup(GL_ALWAYS, GL_INCR);
	if (Test(GL_KEEP, GL_INCR, GL_KEEP, trueValue, depthTest) == ERROR) {
            StrMake(errStr, errStrFmt, "GL_KEEP, GL_INCR, GL_KEEP", trueValue);
	    ErrorReport(__FILE__, __LINE__, errStr);
	    return ERROR;
	}
	trueValue = Setup(GL_ALWAYS, GL_DECR);
	if (Test(GL_KEEP, GL_DECR, GL_KEEP, trueValue, depthTest) == ERROR) {
            StrMake(errStr, errStrFmt, "GL_KEEP, GL_DECR, GL_KEEP", trueValue);
	    ErrorReport(__FILE__, __LINE__, errStr);
	    return ERROR;
	}
	trueValue = Setup(GL_ALWAYS, GL_INVERT);
	if (Test(GL_KEEP, GL_INVERT, GL_KEEP, trueValue, depthTest) == ERROR) {
            StrMake(errStr, errStrFmt, "GL_KEEP, GL_INVERT, GL_KEEP", trueValue);
	    ErrorReport(__FILE__, __LINE__, errStr);
	    return ERROR;
        }
    }

    glDepthFunc(GL_ALWAYS);

    trueValue = Setup(GL_ALWAYS, GL_KEEP);
    if (Test(GL_KEEP, GL_KEEP, GL_KEEP, trueValue, depthTest) == ERROR) {
        StrMake(errStr, errStrFmt, "GL_KEEP, GL_KEEP, GL_KEEP", trueValue);
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }
    trueValue = Setup(GL_ALWAYS, GL_ZERO);
    if (Test(GL_KEEP, GL_KEEP, GL_ZERO, trueValue, depthTest) == ERROR) {
        StrMake(errStr, errStrFmt, "GL_KEEP, GL_KEEP, GL_ZERO", trueValue);
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }
    trueValue = Setup(GL_ALWAYS, GL_REPLACE);
    if (Test(GL_KEEP, GL_KEEP, GL_REPLACE, trueValue, depthTest) == ERROR) {
        StrMake(errStr, errStrFmt, "GL_KEEP, GL_KEEP, GL_REPLACE", trueValue);
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }
    trueValue = Setup(GL_ALWAYS, GL_INCR);
    if (Test(GL_KEEP, GL_KEEP, GL_INCR, trueValue, depthTest) == ERROR) {
        StrMake(errStr, errStrFmt, "GL_KEEP, GL_KEEP, GL_INCR", trueValue);
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }
    trueValue = Setup(GL_ALWAYS, GL_DECR);
    if (Test(GL_KEEP, GL_KEEP, GL_DECR, trueValue, depthTest) == ERROR) {
        StrMake(errStr, errStrFmt, "GL_KEEP, GL_KEEP, GL_DECR", trueValue);
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }
    trueValue = Setup(GL_ALWAYS, GL_INVERT);
    if (Test(GL_KEEP, GL_KEEP, GL_INVERT, trueValue, depthTest) == ERROR) {
        StrMake(errStr, errStrFmt, "GL_KEEP, GL_KEEP, GL_INVERT", trueValue);
	ErrorReport(__FILE__, __LINE__, errStr);
	return ERROR;
    }

    return NO_ERROR;
}
