/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
**
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/texcombine.c,v 1.5 2005/05/17 13:12:33 artlab Exp $
*/

#include "conform.h"
#include "util.h"

#ifdef GL_OES_VERSION_1_1

/******************************************************************************/

static GLint OESRandomInt(GLint lo, GLint hi)
{
    GLint   x = FTOI(OESRandom(ITOF(lo),ITOF(hi+1)));
    if (x > hi) x = hi;
    return x;
}

/******************************************************************************/

#define NUM_TEST_ITERATIONS_LONG    1024
#define NUM_TEST_ITERATIONS_SHORT     32

#define NUM_FORMATS  6
#define TEX_SIZE    16
#define NUM_COLOR_MODES    8
#define NUM_ALPHA_MODES    6
#define NUM_SOURCES        4
#define NUM_COLOR_OPERANDS 4
#define NUM_ALPHA_OPERANDS 2

typedef struct {
    GLenum  type;
    GLenum  format;
    GLint   rbits;
    GLint   gbits;
    GLint   bbits;
    GLint   abits;
    GLint   rshift;
    GLint   gshift;
    GLint   bshift;
    GLint   ashift;
} texfmt_t;

static texfmt_t formats[NUM_FORMATS] = {
    { GL_UNSIGNED_SHORT_5_6_5,   GL_RGB,             5, 6, 5, 0, 11,  5,  0,  0 },
    { GL_UNSIGNED_SHORT_4_4_4_4, GL_RGBA,            4, 4, 4, 4, 12,  8,  4,  0 },
    { GL_UNSIGNED_SHORT_5_5_5_1, GL_RGBA,            5, 5, 5, 1, 11,  6,  1,  0 },
    { GL_UNSIGNED_BYTE,          GL_LUMINANCE_ALPHA, 8, 0, 0, 8,  0,  0,  0,  8 }, 
    { GL_UNSIGNED_BYTE,          GL_LUMINANCE,       8, 0, 0, 0,  0,  0,  0,  0 },
    { GL_UNSIGNED_BYTE,          GL_ALPHA,           0, 0, 0, 8,  0,  0,  0,  0 }
};

static const GLenum colormodes[NUM_COLOR_MODES] = 
{
    GL_REPLACE,
    GL_MODULATE,
    GL_ADD,
    GL_ADD_SIGNED,
    GL_INTERPOLATE,
    GL_SUBTRACT,
    GL_DOT3_RGB,
    GL_DOT3_RGBA
};

static const GLenum alphamodes[NUM_ALPHA_MODES] = 
{
    GL_REPLACE,
    GL_MODULATE,
    GL_ADD,
    GL_ADD_SIGNED,
    GL_INTERPOLATE,
    GL_SUBTRACT
};

static const GLenum sources[NUM_SOURCES] =
{
    GL_TEXTURE,
    GL_CONSTANT,
    GL_PRIMARY_COLOR,
    GL_PREVIOUS
};

static const GLenum coloroperands[NUM_COLOR_OPERANDS] =
{
    GL_SRC_COLOR,
    GL_ONE_MINUS_SRC_COLOR,
    GL_SRC_ALPHA,
    GL_ONE_MINUS_SRC_ALPHA
};

static const GLenum alphaoperands[NUM_ALPHA_OPERANDS] =
{
    GL_SRC_ALPHA,
    GL_ONE_MINUS_SRC_ALPHA
};

/******************************************************************************/

static GLftype texture[NUM_FORMATS][TEX_SIZE][TEX_SIZE][4];

static GLint   numtexunits;

/******************************************************************************/

static GLint    texenable[16];
static GLint    texbind[16];
static GLenum   combinergb[16];
static GLenum   combinealpha[16];
static GLenum   srcrgb[16][3];
static GLenum   srcalpha[16][3];
static GLenum   operandrgb[16][3];
static GLenum   operandalpha[16][3];
static GLftype  texenvcolor[16][4];
static GLftype  rgbscale[16];
static GLftype  alphascale[16];

static GLint    texel[16][2];

static GLftype  polycolor[4];

static GLftype  posArray[4][2];
static GLftype  texArray[16][4][2];

/******************************************************************************/

static void
InitCombineTest(void)
{
    GLubyte rawtexture[4*TEX_SIZE*TEX_SIZE];
    GLint   i, j, k;

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    for (k=0; k<NUM_FORMATS; ++k)
    {
        GLubyte     *dst8  = rawtexture;
        GLushort    *dst16 = (GLushort*)rawtexture;
        GLint   bits = formats[k].rbits + formats[k].gbits + formats[k].bbits + formats[k].abits;
        GLint   rmax = (1<<formats[k].rbits)-1;
        GLint   gmax = (1<<formats[k].gbits)-1;
        GLint   bmax = (1<<formats[k].bbits)-1;
        GLint   amax = (1<<formats[k].abits)-1;

        for (j=0; j<TEX_SIZE; ++j)
        {
            for (i=0; i<TEX_SIZE; ++i)
            {
                GLint   r, g, b, a;

                r = OESRandomInt(0, 15);
                g = OESRandomInt(0, 15);
                b = OESRandomInt(0, 15);
                a = OESRandomInt(0, 15);

                r |= (r << 4);
                g |= (g << 4);
                b |= (b << 4);
                a |= (a << 4);

                r >>= (8-formats[k].rbits);
                g >>= (8-formats[k].gbits);
                b >>= (8-formats[k].bbits);
                a >>= (8-formats[k].abits);

                if (rmax)
                    texture[k][j][i][0] = DIV(ITOF(r),ITOF(rmax));
                else
                    texture[k][j][i][0] = ITOF(0);

                if (gmax)
                    texture[k][j][i][1] = DIV(ITOF(g),ITOF(gmax));
                else
                    texture[k][j][i][1] = texture[k][j][i][0];

                if (bmax)
                    texture[k][j][i][2] = DIV(ITOF(b),ITOF(bmax));
                else
                    texture[k][j][i][2] = texture[k][j][i][0];

                if (amax)
                    texture[k][j][i][3] = DIV(ITOF(a),ITOF(amax));
                else
                    texture[k][j][i][3] = ITOF(1);

                /* In case of big-endian architecture, Luminance-Alpha shift values need to be swapped.
                 * The current shift values expect little-endian architecture.
                 */
                if (GL_LUMINANCE_ALPHA == formats[k].format)
                {
                    *dst8++ = (GLubyte)r;
                    *dst8++ = (GLubyte)a;
                }
                else if (bits == 8)
                {
                    *dst8++ = (GLubyte)(r << formats[k].rshift)
                            | (GLubyte)(g << formats[k].gshift)
                            | (GLubyte)(b << formats[k].bshift)
                            | (GLubyte)(a << formats[k].ashift);
                }
                else
                {
                    *dst16++ = (GLushort)(r << formats[k].rshift)
                             | (GLushort)(g << formats[k].gshift)
                             | (GLushort)(b << formats[k].bshift)
                             | (GLushort)(a << formats[k].ashift);
                }
            }
        }

        glBindTexture(GL_TEXTURE_2D, k);
        glTexImage2D(GL_TEXTURE_2D, 0, formats[k].format, TEX_SIZE, TEX_SIZE, 0, formats[k].format, formats[k].type, rawtexture);
        glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,     (GLftype)GL_REPEAT);
        glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,     (GLftype)GL_REPEAT);
        glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, (GLftype)GL_NEAREST);
        glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, (GLftype)GL_NEAREST);
    }

    glGetIntegerv(GL_MAX_TEXTURE_UNITS, &numtexunits);

    glVertexPointer(2, GL_FTYPE, 0, posArray);
    glEnableClientState(GL_VERTEX_ARRAY);
    for (k=0; k<numtexunits; ++k)
    {
        glClientActiveTexture(GL_TEXTURE0+k);
        glTexCoordPointer(2, GL_FTYPE, 0, texArray[k]);
        glEnableClientState(GL_TEXTURE_COORD_ARRAY);
    }

    posArray[0][0] = ITOF(0);
    posArray[0][1] = ITOF(0);
    posArray[1][0] = ITOF(1);
    posArray[1][1] = ITOF(0);
    posArray[2][0] = ITOF(1);
    posArray[2][1] = ITOF(1);
    posArray[3][0] = ITOF(0);
    posArray[3][1] = ITOF(1);
}

static void
CleanupCombineTest(void)
{
    GLuint i;
    
    glBindTexture(GL_TEXTURE_2D, 0);
    for (i=1; i<NUM_FORMATS; i++) {
        glDeleteTextures(1, &i);
    }
}

/******************************************************************************/

static void RandomizeCombineTest(void)
{
    GLint   i, k;

    for (i=0; i<4; ++i)
        polycolor[i]  = OESRandom(ITOF(0), ITOF(1));
    for (k=0; k<numtexunits; ++k)
    {
        texenable[k]    = OESRandomInt(0,4) ? 1 : 0;
        texbind[k]      = OESRandomInt(0,NUM_FORMATS-1);
        combinergb[k]   = colormodes[OESRandomInt(0,NUM_COLOR_MODES-1)];
        combinealpha[k] = alphamodes[OESRandomInt(0,NUM_ALPHA_MODES-1)];
        for (i=0; i<3; ++i)
        {
            srcrgb[k][i]       = sources[OESRandomInt(0,NUM_SOURCES-1)];
            srcalpha[k][i]     = sources[OESRandomInt(0,NUM_SOURCES-1)];
            operandrgb[k][i]   = coloroperands[OESRandomInt(0,NUM_COLOR_OPERANDS-1)];
            operandalpha[k][i] = alphaoperands[OESRandomInt(0, NUM_ALPHA_OPERANDS-1)];
        }
        for (i=0; i<4; ++i)
            texenvcolor[k][i]  = OESRandom(ITOF(0), ITOF(1));
        rgbscale[k] = ITOF(1 << OESRandomInt(0,2));
        alphascale[k] = ITOF(1 << OESRandomInt(0,2));
        texel[k][0] = OESRandomInt(0,TEX_SIZE-1);
        texel[k][1] = OESRandomInt(0,TEX_SIZE-1);

        if (combinergb[k] == GL_DOT3_RGBA)
            alphascale[k] = rgbscale[k];
    }

}

/******************************************************************************/

static void RenderCombineTest(void)
{
    GLint   i, k;

    glColor4t(polycolor[0], polycolor[1], polycolor[2], polycolor[3]);

    for (k=0; k<numtexunits; ++k)
    {
        glActiveTexture(GL_TEXTURE0+k);
        glClientActiveTexture(GL_TEXTURE0+k);

        if (texenable[k])
            glEnable(GL_TEXTURE_2D);
        else
            glDisable(GL_TEXTURE_2D);
        glBindTexture(GL_TEXTURE_2D, texbind[k]);

        glTexEnvt(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE,  (GLftype)GL_COMBINE);
        glTexEnvt(GL_TEXTURE_ENV, GL_COMBINE_RGB,       (GLftype)combinergb[k]);
        glTexEnvt(GL_TEXTURE_ENV, GL_COMBINE_ALPHA,     (GLftype)combinealpha[k]);
        glTexEnvt(GL_TEXTURE_ENV, GL_SRC0_RGB,          (GLftype)srcrgb[k][0]);
        glTexEnvt(GL_TEXTURE_ENV, GL_SRC1_RGB,          (GLftype)srcrgb[k][1]);
        glTexEnvt(GL_TEXTURE_ENV, GL_SRC2_RGB,          (GLftype)srcrgb[k][2]);
        glTexEnvt(GL_TEXTURE_ENV, GL_SRC0_ALPHA,        (GLftype)srcalpha[k][0]);
        glTexEnvt(GL_TEXTURE_ENV, GL_SRC1_ALPHA,        (GLftype)srcalpha[k][1]);
        glTexEnvt(GL_TEXTURE_ENV, GL_SRC2_ALPHA,        (GLftype)srcalpha[k][2]);
        glTexEnvt(GL_TEXTURE_ENV, GL_OPERAND0_RGB,      (GLftype)operandrgb[k][0]);
        glTexEnvt(GL_TEXTURE_ENV, GL_OPERAND1_RGB,      (GLftype)operandrgb[k][1]);
        glTexEnvt(GL_TEXTURE_ENV, GL_OPERAND2_RGB,      (GLftype)operandrgb[k][2]);
        glTexEnvt(GL_TEXTURE_ENV, GL_OPERAND0_ALPHA,    (GLftype)operandalpha[k][0]);
        glTexEnvt(GL_TEXTURE_ENV, GL_OPERAND1_ALPHA,    (GLftype)operandalpha[k][1]);
        glTexEnvt(GL_TEXTURE_ENV, GL_OPERAND2_ALPHA,    (GLftype)operandalpha[k][2]);
        glTexEnvtv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, texenvcolor[k]);
        glTexEnvt(GL_TEXTURE_ENV, GL_RGB_SCALE,         (GLftype)rgbscale[k]);
        glTexEnvt(GL_TEXTURE_ENV, GL_ALPHA_SCALE,       (GLftype)alphascale[k]);

        for (i=0; i<4; ++i)
        {
            texArray[k][i][0] = DIV(ITOF(2*texel[k][0]+1),ITOF(2*TEX_SIZE));
            texArray[k][i][1] = DIV(ITOF(2*texel[k][1]+1),ITOF(2*TEX_SIZE));
        }
    }

    glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
}

/******************************************************************************/

static void PredictCombineTest(GLftype *result)
{
    GLftype stage[17][4];
    GLint   i, j, k;

    for (i=0; i<4; ++i)
        stage[0][i] = polycolor[i];

    for (k=0; k<numtexunits; ++k)
    {
        if (!texenable[k])
        {
            for (i=0; i<4; ++i)
                stage[k+1][i] = stage[k][i];
        }

        else
        {
            GLftype *srcptrrgb;
            GLftype *srcptralpha;
            GLftype arg[3][4];
            
            for (j=0; j<3; ++j)
            {
                switch (srcrgb[k][j])
                {
                    case GL_TEXTURE:
                        srcptrrgb = texture[texbind[k]][texel[k][1]][texel[k][0]];
                        break;
                    case GL_CONSTANT:
                        srcptrrgb = texenvcolor[k];
                        break;
                    case GL_PRIMARY_COLOR:
                        srcptrrgb = polycolor;
                        break;
                    case GL_PREVIOUS:
                        srcptrrgb = stage[k];
                        break;
                    default: /* Error case */
                        ErrorReport(__FILE__, __LINE__, "illegal srcrgb val");
                        srcptrrgb = NULL;
                }

                switch (srcalpha[k][j])
                {
                    case GL_TEXTURE:
                        srcptralpha = texture[texbind[k]][texel[k][1]][texel[k][0]];
                        break;
                    case GL_CONSTANT:
                        srcptralpha = texenvcolor[k];
                        break;
                    case GL_PRIMARY_COLOR:
                        srcptralpha = polycolor;
                        break;
                    case GL_PREVIOUS:
                        srcptralpha = stage[k];
                        break;
                    default: /* Error case */
                        ErrorReport(__FILE__, __LINE__,"illegal srcalpha val");
                        srcptralpha = NULL;
               }

                switch(operandrgb[k][j])
                {
                    case GL_SRC_COLOR:
                        arg[j][0] = srcptrrgb[0];
                        arg[j][1] = srcptrrgb[1];
                        arg[j][2] = srcptrrgb[2];
                        break;
                    case GL_ONE_MINUS_SRC_COLOR:
                        arg[j][0] = ITOF(1) - srcptrrgb[0];
                        arg[j][1] = ITOF(1) - srcptrrgb[1];
                        arg[j][2] = ITOF(1) - srcptrrgb[2];
                        break;
                    case GL_SRC_ALPHA:
                        arg[j][0] = srcptrrgb[3];
                        arg[j][1] = srcptrrgb[3];
                        arg[j][2] = srcptrrgb[3];
                        break;
                    case GL_ONE_MINUS_SRC_ALPHA:
                        arg[j][0] = ITOF(1) - srcptrrgb[3];
                        arg[j][1] = ITOF(1) - srcptrrgb[3];
                        arg[j][2] = ITOF(1) - srcptrrgb[3];
                        break;
                }

                switch(operandalpha[k][j])
                {
                    case GL_SRC_ALPHA:
                        arg[j][3] = srcptralpha[3];
                        break;
                    case GL_ONE_MINUS_SRC_ALPHA:
                        arg[j][3] = ITOF(1) - srcptralpha[3];
                        break;
                }
            }

            switch (combinergb[k])
            {
                case GL_REPLACE:
                    stage[k+1][0] = arg[0][0];
                    stage[k+1][1] = arg[0][1];
                    stage[k+1][2] = arg[0][2];
                    break;
                case GL_MODULATE:
                    stage[k+1][0] = MUL(arg[0][0], arg[1][0]);
                    stage[k+1][1] = MUL(arg[0][1], arg[1][1]);
                    stage[k+1][2] = MUL(arg[0][2], arg[1][2]);
                    break;
                case GL_ADD:
                    stage[k+1][0] = ADD(arg[0][0], arg[1][0]);
                    stage[k+1][1] = ADD(arg[0][1], arg[1][1]);
                    stage[k+1][2] = ADD(arg[0][2], arg[1][2]);
                    break;
                case GL_ADD_SIGNED:
                    stage[k+1][0] = SUB(ADD(arg[0][0], arg[1][0]), DIV(ITOF(1),ITOF(2)));
                    stage[k+1][1] = SUB(ADD(arg[0][1], arg[1][1]), DIV(ITOF(1),ITOF(2)));
                    stage[k+1][2] = SUB(ADD(arg[0][2], arg[1][2]), DIV(ITOF(1),ITOF(2)));
                    break;
                case GL_INTERPOLATE:
                    stage[k+1][0] = ADD(MUL(arg[0][0], arg[2][0]), MUL(arg[1][0], SUB(ITOF(1), arg[2][0])));
                    stage[k+1][1] = ADD(MUL(arg[0][1], arg[2][1]), MUL(arg[1][1], SUB(ITOF(1), arg[2][1])));
                    stage[k+1][2] = ADD(MUL(arg[0][2], arg[2][2]), MUL(arg[1][2], SUB(ITOF(1), arg[2][2])));
                    break;
                case GL_SUBTRACT:
                    stage[k+1][0] = SUB(arg[0][0], arg[1][0]);
                    stage[k+1][1] = SUB(arg[0][1], arg[1][1]);
                    stage[k+1][2] = SUB(arg[0][2], arg[1][2]);
                    break;
               case GL_DOT3_RGB:
               case GL_DOT3_RGBA:
                    arg[0][0] = SUB(arg[0][0], DIV(ITOF(1),ITOF(2)));
                    arg[0][1] = SUB(arg[0][1], DIV(ITOF(1),ITOF(2)));
                    arg[0][2] = SUB(arg[0][2], DIV(ITOF(1),ITOF(2)));
                    arg[1][0] = SUB(arg[1][0], DIV(ITOF(1),ITOF(2)));
                    arg[1][1] = SUB(arg[1][1], DIV(ITOF(1),ITOF(2)));
                    arg[1][2] = SUB(arg[1][2], DIV(ITOF(1),ITOF(2)));
                    arg[0][0] = MUL(arg[0][0], arg[1][0]);
                    arg[0][1] = MUL(arg[0][1], arg[1][1]);
                    arg[0][2] = MUL(arg[0][2], arg[1][2]);
                    stage[k+1][0] = MUL(ITOF(4), ADD(ADD(arg[0][0], arg[0][1]), arg[0][2]));
                    stage[k+1][1] = stage[k+1][0];
                    stage[k+1][2] = stage[k+1][0];
                    break;
            }

            switch (combinealpha[k])
            {
                case GL_REPLACE:
                    stage[k+1][3] = arg[0][3];
                    break;
                case GL_MODULATE:
                    stage[k+1][3] = MUL(arg[0][3], arg[1][3]);
                    break;
                case GL_ADD:
                    stage[k+1][3] = ADD(arg[0][3], arg[1][3]);
                    break;
                case GL_ADD_SIGNED:
                    stage[k+1][3] = SUB(ADD(arg[0][3], arg[1][3]), DIV(ITOF(1),ITOF(2)));
                    break;
                case GL_INTERPOLATE:
                    stage[k+1][3] = ADD(MUL(arg[0][3], arg[2][3]), MUL(arg[1][3], SUB(ITOF(1), arg[2][3])));
                    break;
                case GL_SUBTRACT:
                    stage[k+1][3] = SUB(arg[0][3], arg[1][3]);
                    break;
            }
            if (combinergb[k] == GL_DOT3_RGBA)
                stage[k+1][3] = stage[k+1][0];

            stage[k+1][0] = MUL(stage[k+1][0], rgbscale[k]);
            stage[k+1][1] = MUL(stage[k+1][1], rgbscale[k]);
            stage[k+1][2] = MUL(stage[k+1][2], rgbscale[k]);
            stage[k+1][3] = MUL(stage[k+1][3], alphascale[k]);

            for (i=0; i<4; ++i)
            {
                if (stage[k+1][i] < ITOF(0))
                    stage[k+1][i] = ITOF(0);
                if (stage[k+1][i] > ITOF(1))
                    stage[k+1][i] = ITOF(1);
            }
        }
    }

    result[0] = stage[numtexunits][0];
    result[1] = stage[numtexunits][1];
    result[2] = stage[numtexunits][2];
    result[3] = stage[numtexunits][3];
}

/******************************************************************************/

long TexCombineExec(void)
{
    GLint   iterations;
    GLsizei textureBits[4] = { 4, 4, 4, 4 };
    GLftype textureEpsilon[4];
    char    errStr[1024];
    GLint   i;

    if (machine.pathLevel == 0) {
        iterations = NUM_TEST_ITERATIONS_LONG;
    } else {
        iterations = NUM_TEST_ITERATIONS_SHORT;
    }

    InitCombineTest();
    CalcTextureEpsilon(textureBits, textureEpsilon);
    for (i=0; i<iterations; ++i)
    {
        GLftype prediction[4];
        GLftype buf[3]; 

        RandomizeCombineTest();
        RenderCombineTest();
        PredictCombineTest(prediction);
        ReadScreen(0, 0, 1, 1, GL_RGB, buf);

        if (  (ABS(prediction[0]-buf[0]) > 4*textureEpsilon[0]) 
           || (ABS(prediction[1]-buf[1]) > 4*textureEpsilon[1])
           || (ABS(prediction[2]-buf[2]) > 4*textureEpsilon[2]) )
        {
            StrMake(
                errStr, "Expected color is (%g %g %g).  Returned color is (%g %g %g).",
                prediction[0], prediction[1], prediction[2], 
                buf[0], buf[1], buf[2]);
            ErrorReport(__FILE__, __LINE__, errStr);
            return ERROR;
        }
    }
    CleanupCombineTest();

    return NO_ERROR;
}

/******************************************************************************/

#endif
