/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/texdecal.c,v 1.4 2005/05/11 15:21:40 artlab Exp $
*/

/*
** texdecal.c
** Texture Decal Test.
**
** Description -
**    Simple texture test. A polygon is drawn with a texture. The
**    texture coordinates are set so that the texture should
**    appear on the polygon with no distortion. The rendered
**    polygon is read back and compared to the texture pattern.
**
** Technical Specification -
**    Buffer requirements:
**        Color buffer.
**    Color requirements:
**        Full color.
**    States requirements:
**        Disabled = GL_DITHER, GL_MULTISAMPLE.
**    Error epsilon:
**    Paths:
**        Allowed = Alpha, Blend, Depth, Fog, Logicop, Shade, Stencil.
**        Not allowed = Alias, Dither.
*/

#include "conform.h"
#include "util.h"

#define SIZE 32

static char errEnv[40], errMagFilter[40], errMinFilter[40];
static char errStr[320];
static char errStrFmt[] = "Location (%d, %d). Texture environment mode is %s. Magnification filter is %s, minification filter is %s. Color is (%d, %d, %d), should be (%d, %d, %d.)";


static void Make(GLsizei size, GLubyte *buf)
{
    GLint level, i;

    for (i = 0; i < SIZE*SIZE*3; i++) {
	buf[i] = (GLubyte)(255*OESRandom(ITOF(0), ITOF(1)));
    }

    level = 0;
    while (size > 0) {
	glTexImage2D(GL_TEXTURE_2D, level++, GL_RGB, size, size, 0, GL_RGB,
		     GL_UNSIGNED_BYTE, (GLvoid *)buf);
	size /= 2;
    }
}

long TexDecalExec(void)
{
    GLubyte *textureBuf, *workBuf;
    GLftype x;
    GLsizei textureBits[4];
    GLftype textureEpsilon[4];
    long i, j, k, l;

    textureBuf = (GLubyte *)MALLOC(SIZE*SIZE*3*sizeof(GLubyte));
    workBuf = (GLubyte *)MALLOC(WINDSIZEX*WINDSIZEY*4*sizeof(GLubyte));

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glClear(GL_COLOR_BUFFER_BIT);
    glDisable(GL_DITHER);
    glDisable(GL_MULTISAMPLE);

    glEnable(GL_TEXTURE_2D);

    x = GL_DECAL;
    glTexEnvtv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, &x);
    GetEnumName((GLenum)x, errEnv);

    x = GL_NEAREST;
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, x);
    GetEnumName((GLenum)x, errMinFilter);

    x = GL_NEAREST;
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, x);
    GetEnumName((GLenum)x, errMagFilter);

    Make(SIZE, textureBuf);

    GetComponentBits(GL_RGB, GL_UNSIGNED_BYTE, textureBits);
    CalcTextureEpsilon(textureBits, textureEpsilon);

    DrawTexQuad2i2f(0, 0, SIZE, SIZE, ITOF(0), ITOF(0), ITOF(1), ITOF(1));

    glReadPixels(0, 0, SIZE, SIZE, GL_RGBA, GL_UNSIGNED_BYTE, workBuf);

    for (i = 0; i < SIZE; i++) {
	for (j = 0; j < SIZE; j++) {
	    k = (i * SIZE + j) * 4;
	    l = (i * SIZE + j) * 3;
	    if (ABS(ITOF(workBuf[k]-textureBuf[l]))/255 > 2 * textureEpsilon[0] ||
		ABS(ITOF(workBuf[k+1]-textureBuf[l+1]))/255 > 2 * textureEpsilon[1] ||
		ABS(ITOF(workBuf[k+2]-textureBuf[l+2]))/255 > 2 * textureEpsilon[2]) {
		StrMake(errStr, errStrFmt, i, j, errEnv, errMagFilter,
			errMinFilter, workBuf[k], workBuf[k+1],
			workBuf[k+2], textureBuf[l], textureBuf[l+1],
			textureBuf[l+2]);
		ErrorReport(__FILE__, __LINE__, errStr);
		FREE(textureBuf);
		FREE(workBuf);
		return ERROR;
	    }
	}
    }

    FREE(textureBuf);
    FREE(workBuf);
    return NO_ERROR;
}
