/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/texedge.c,v 1.3 2005/05/11 15:21:40 artlab Exp $
*/

/*
** texedge.c
** Texture Edge Clamp Test.
**
** Description -
**    Setup a 2D texture with a green border & white body.
**    Set up texture parameters for edge clamping.
**    Draw quad with texture coords from -0.5 to +1.5.
**    Check that no white was drawn on any edge.
**
** Technical Specification -
**    Buffer requirements:
**	  Color buffer.
**    Color requirements:
**	  Full color.
**    States requirements:
**	  Disabled = GL_DITHER, GL_MULTISAMPLE.
**    Error epsilon:
**	  Color epsilon. Zero epsilons.
**    Paths:
**	  Allowed = Alpha, Blend, Depth, Fog, Logicop, Shade, Stencil.
**	  Not allowed = Alias, Dither.
*/

#include "conform.h"
#include "util.h"

static long TestPixRGB(GLint x, GLint y, GLftype r, GLftype g, GLftype b)
{
    GLftype buf[3];

    ReadScreen(x, y, 1, 1, GL_RGB, buf);

    if (ABS(buf[0]-r) > epsilon.color[0] ||
	ABS(buf[1]-g) > epsilon.color[1] ||
	ABS(buf[2]-b) > epsilon.color[2])
	return ERROR;

    return NO_ERROR;
}

static long Check2D(void)
{
    static const GLubyte texture[] = {
	  0, 255,   0,	  0, 255,   0,	  0, 255,   0,	  0, 255,   0,
	  0, 255,   0,  255, 255, 255,  255, 255, 255,	  0, 255,   0,
	  0, 255,   0,  255, 255, 255,  255, 255, 255,	  0, 255,   0,
	  0, 255,   0,	  0, 255,   0,	  0, 255,   0,	  0, 255,   0 };
    static GLuint texName;

    glGenTextures(1, &texName);
    glBindTexture(GL_TEXTURE_2D, texName);

    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 4, 4, 0, GL_RGB, GL_UNSIGNED_BYTE,
		 texture);

    glTexEnvt(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);

    glEnable(GL_TEXTURE_2D);

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(1));
    glClear(GL_COLOR_BUFFER_BIT);

    DrawTexQuad2i2f(WINDSIZEX/4, WINDSIZEY/4, WINDSIZEX*3/4 + 1, WINDSIZEY*3/4 + 1, ATOF("-0.5"), ATOF("-0.5"), ATOF("1.5"), ATOF("1.5"));

    glDisable(GL_TEXTURE_2D);

    glBindTexture(GL_TEXTURE_2D, 0);
    glDeleteTextures(1, &texName);

    if (TestPixRGB(WINDSIZEX/4, WINDSIZEY/2, ITOF(0), ITOF(1), ITOF(0))) {
	ErrorReport(__FILE__, __LINE__, " Texture Edge Clamp test.");
	return ERROR;
    }

    if (TestPixRGB(WINDSIZEX/2, WINDSIZEY*3/4, ITOF(0), ITOF(1), ITOF(0))) {
	ErrorReport(__FILE__, __LINE__, " Texture Edge Clamp test.");
	return ERROR;
    }

    if (TestPixRGB(WINDSIZEX/2, WINDSIZEY/4, ITOF(0), ITOF(1), ITOF(0))) {
	ErrorReport(__FILE__, __LINE__, " Texture Edge Clamp test.");
	return ERROR;
    }

    if (TestPixRGB(WINDSIZEX*3/4, WINDSIZEY/2, ITOF(0), ITOF(1), ITOF(0))) {
	ErrorReport(__FILE__, __LINE__, " Texture Edge Clamp test.");
	return ERROR;
    }

    return NO_ERROR;
}

long TextureEdgeClampExec(void)
{
    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));
    glDisable(GL_DITHER);
    glDisable(GL_MULTISAMPLE);

    glColor4t(ITOF(0), ITOF(1), ITOF(1), ITOF(1));
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

    if (Check2D() == ERROR) {
        glEnable(GL_DITHER);
	return ERROR;
    }

    glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
    glEnable(GL_DITHER);

    return NO_ERROR;
}
