/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/texpalet.c,v 1.4 2005/05/11 15:21:40 artlab Exp $
*/

/*
** texpalet.c
** Compressed Paletted Texture Test.
**
** Description -
**    The supported compressed texture formats are queried. The following
**    formats are required to be supported:
**
**        GL_PALETTE4_RGB8_OES,
**        GL_PALETTE4_RGBA8_OES,
**        GL_PALETTE4_R5_G6_B5_OES,
**        GL_PALETTE4_RGBA4_OES,
**        GL_PALETTE4_RGB5_A1_OES,
**        GL_PALETTE8_RGB8_OES,
**        GL_PALETTE8_RGBA8_OES,
**        GL_PALETTE8_R5_G6_B5_OES,
**        GL_PALETTE8_RGBA4_OES, and
**        GL_PALETTE8_RGB5_A1_OES
**
**    For each compressed paletted texture from the above list, a
**    compressed paletted texture is created by generating random palette
**    and texture data (palette indices). This texture is then loaded with
**    glCompressedTexImage2D() (level = 0). A polygon is drawn with this
**    texture. The texture coordinates are set so that the texture should
**    appear on the polygon with no distortion. The rendered polygon is
**    read back and compared to the uncompressed texture pattern.
**
** Technical Specification -
**    Buffer requirements:
**        Color buffer.
**    Color requirements:
**        Full color.
**    States requirements:
**        Disabled = GL_DITHER, GL_MULTISAMPLE.
**    Error epsilon:
**        Color epsilon.
**    Paths:
**        Allowed = Alpha, Blend, Depth, Fog, Logicop, Shade, Stencil.
**        Not allowed = Alias, Dither.
*/ 

#include "conform.h"
#include "util.h"
#include "utilp.h"


#ifdef GL_OES_compressed_paletted_texture

#define SIZE 32

static char formatName[40];
static char errEnv[40], errMagFilter[40], errMinFilter[40];
static char errStr[320];
static const char *errStrFmt[2] = {
    "Unsupported format %s.\n",
    "Format is %s. Location (%d, %d). Texture environment mode is %s. Magnification filter is %s, minification filter is %s. Color is (%.3g, %.3g, %.3g), should be (%.3g, %.3g, %.3g)."
};

static GLenum GetPackedType(GLenum format)
{
    switch (format) {
    default:
	return (GLenum)-1;

    case GL_PALETTE4_R5_G6_B5_OES:
    case GL_PALETTE8_R5_G6_B5_OES:
	return GL_UNSIGNED_SHORT_5_6_5;

    case GL_PALETTE4_RGBA4_OES:
    case GL_PALETTE8_RGBA4_OES:
	return GL_UNSIGNED_SHORT_4_4_4_4;

    case GL_PALETTE4_RGB5_A1_OES:
    case GL_PALETTE8_RGB5_A1_OES:
	return GL_UNSIGNED_SHORT_5_5_5_1;

    case GL_PALETTE4_RGB8_OES:
    case GL_PALETTE8_RGB8_OES:
    case GL_PALETTE4_RGBA8_OES:
    case GL_PALETTE8_RGBA8_OES:
	/* no packed type */
	return GL_UNSIGNED_BYTE;
    }
}

static GLsizei GetPaletteCount(GLenum format)
{
    switch (format) {
    default:
        return 0;

    case GL_PALETTE4_R5_G6_B5_OES:
    case GL_PALETTE4_RGB8_OES:
    case GL_PALETTE4_RGBA4_OES:
    case GL_PALETTE4_RGB5_A1_OES:
    case GL_PALETTE4_RGBA8_OES:
	return 16;

    case GL_PALETTE8_R5_G6_B5_OES:
    case GL_PALETTE8_RGB8_OES:
    case GL_PALETTE8_RGBA4_OES:
    case GL_PALETTE8_RGB5_A1_OES:
    case GL_PALETTE8_RGBA8_OES:
	return 256;
    }
}

static GLsizei GetPixelSize(GLenum format)
{
    switch (format) {
    default:
        return 0;

    case GL_PALETTE4_R5_G6_B5_OES:
    case GL_PALETTE4_RGBA4_OES:
    case GL_PALETTE4_RGB5_A1_OES:
    case GL_PALETTE8_R5_G6_B5_OES:
    case GL_PALETTE8_RGBA4_OES:
    case GL_PALETTE8_RGB5_A1_OES:
	return 2;

    case GL_PALETTE4_RGB8_OES:
    case GL_PALETTE8_RGB8_OES:
	return 3;

    case GL_PALETTE4_RGBA8_OES:
    case GL_PALETTE8_RGBA8_OES:
	return 4;
    }
}

static GLsizei GetPaletteSize(GLenum format)
{
    return GetPixelSize(format) * GetPaletteCount(format);
}

static void Make(GLenum format, GLsizei size, GLubyte *buf)
{
    GLsizei palette_size, indices_size, compressed_size;
    long i;

    palette_size = GetPaletteSize(format);

    switch (GetPaletteCount(format)) {
    default:
	indices_size = 0;
	break;

    case 16:
	indices_size = size * size / 2;
	break;

    case 256:
	indices_size = size * size;
	break;
    }

    compressed_size = palette_size + indices_size;

    for (i = 0; i < compressed_size; i++) {
	buf[i] = (GLubyte)(255*OESRandom(ITOF(0), ITOF(1)));
    }

    glCompressedTexImage2D(GL_TEXTURE_2D, 0, format, size, size, 0,
	compressed_size, buf);
}

static GLsizei GetNibble(GLubyte byte, long index)
{
    switch (index) {
    default:
	return 0;

    case 0:
	return byte >> 4;

    case 1:
	return byte & 0x0F;
    }
}

static void ToRGBA(GLenum format, GLubyte const *packed_color, GLftype rgba[4])
{
    GLenum packed_type;
    GLsizei pixel_size;
    long i;

    packed_type = GetPackedType(format);
    pixel_size = GetPixelSize(format);

    if (packed_type == GL_UNSIGNED_BYTE) {
        for (i = 0; i < pixel_size; i++) {
	    rgba[i] = DIV(ITOF(packed_color[i]), ITOF(255));
        }
        for (; i < 3; i++) {
	    rgba[i] = ITOF(0);
        }
        for (; i < 4; i++) {
	    rgba[i] = ITOF(1);
        }

    } else {
	packPixRec const *packed_format = packPixInfo(packed_type);
        ConvertPackedtoRGBA(packed_color, rgba, packed_format);
    }
}

static void Uncompress(GLenum format, const GLvoid *compressed_buf, GLftype* uncompressed_buf)
{
    GLsizei palette_entries, pixel_size, index;
    GLubyte const *palette;
    GLubyte const *indices;
    long i, j, k;

    palette_entries = GetPaletteCount(format);
    pixel_size = GetPixelSize(format);

    palette = (GLubyte const *)compressed_buf;
    indices = palette + GetPaletteSize(format);

    switch (palette_entries) {
    case 16:
	k = 0;
        for (i = 0; i < SIZE; i++) {
            for (j = 0; j < SIZE; j++) {
		index = GetNibble(indices[k/2], k % 2);
		ToRGBA(format, &palette[index * pixel_size],
		    &uncompressed_buf[k * 4]);
		k++;
            }
        }
	break;

    case 256:
	k = 0;
        for (i = 0; i < SIZE; i++) {
            for (j = 0; j < SIZE; j++) {
		index = indices[k];
		ToRGBA(format, &palette[index * pixel_size],
		    &uncompressed_buf[k * 4]);
		k++;
            }
        }
	break;
    }
}

long TexPaletExec(void)
{
    static const GLenum formats[] = {
	GL_PALETTE4_RGB8_OES,
	GL_PALETTE4_RGBA8_OES,
	GL_PALETTE4_R5_G6_B5_OES,
	GL_PALETTE4_RGBA4_OES,
	GL_PALETTE4_RGB5_A1_OES,
	GL_PALETTE8_RGB8_OES,
	GL_PALETTE8_RGBA8_OES,
	GL_PALETTE8_R5_G6_B5_OES,
	GL_PALETTE8_RGBA4_OES,
	GL_PALETTE8_RGB5_A1_OES
    };
    GLubyte *textureBuf;
    GLftype *workBuf, *unpackedBuf, x;
    GLsizei textureBits[4];
    GLftype textureEpsilon[4];
    GLint *supportedFormats, numSupportedFormats;
    long i, j, k, l, f;

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glClear(GL_COLOR_BUFFER_BIT);
    glDisable(GL_DITHER);
    glDisable(GL_MULTISAMPLE);

    glEnable(GL_TEXTURE_2D);

    glGetIntegerv(GL_NUM_COMPRESSED_TEXTURE_FORMATS, &numSupportedFormats);

    supportedFormats = (GLint *)MALLOC(numSupportedFormats*sizeof(GLint));
    glGetIntegerv(GL_COMPRESSED_TEXTURE_FORMATS, supportedFormats);

    for (i = 0; i < 10; i++) {
        for (j = 0; j < numSupportedFormats; j++) {
	    if ((GLenum)supportedFormats[j] == formats[i]) {
		break;
	    }
        }
        if (j == numSupportedFormats) {
	    GetEnumName(formats[i], formatName);
	    StrMake(errStr, errStrFmt[0], formatName);
	    ErrorReport(__FILE__, __LINE__, errStr);
	    FREE(supportedFormats);
	    return ERROR;
        }
    }

    textureBuf = (GLubyte *)MALLOC((256*4 + SIZE*SIZE)*sizeof(GLubyte));
    workBuf = (GLftype *)MALLOC(WINDSIZEX*WINDSIZEY*4*sizeof(GLftype));
    unpackedBuf = (GLftype *)MALLOC(SIZE*SIZE*4*sizeof(GLftype));

    /* need modulate to test alpha channel */
    x = GL_MODULATE;
    glTexEnvtv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, &x);
    GetEnumName((GLenum)x, errEnv);

    x = GL_NEAREST;
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, x);
    GetEnumName((GLenum)x, errMinFilter);

    x = GL_NEAREST;
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, x);
    GetEnumName((GLenum)x, errMagFilter);

    for (f = 0; f != 10; f++) {
	GLenum format = formats[f];

	GetEnumName(format, formatName);
	Make(format, SIZE, textureBuf);

        GetCompressedComponentBits(format, textureBits);
        CalcTextureEpsilon(textureBits, textureEpsilon);

	DrawTexQuad2i2f(0, 0, SIZE, SIZE, ITOF(0), ITOF(0), ITOF(1), ITOF(1));
	ReadScreen(0, 0, SIZE, SIZE, GL_RGBA, workBuf);

	Uncompress(format, textureBuf, unpackedBuf);

	for (i = 0; i < SIZE; i++) {
	    for (j = 0; j < SIZE; j++) {
		k = (i * SIZE + j) * 4;
		l = (i * SIZE + j) * 4;
		if (ABS(workBuf[k]-unpackedBuf[l]) > 2 * textureEpsilon[0] ||
		    ABS(workBuf[k+1]-unpackedBuf[l+1]) > 2 * textureEpsilon[1] ||
		    ABS(workBuf[k+2]-unpackedBuf[l+2]) > 2 * textureEpsilon[2] ||
		    ABS(workBuf[k+3]-unpackedBuf[l+3]) > 2 * textureEpsilon[3]) {
		    StrMake(errStr, errStrFmt[1], formatName, i, j, errEnv, errMagFilter,
			errMinFilter, workBuf[k], workBuf[k+1],
			workBuf[k+2], unpackedBuf[l], unpackedBuf[l+1],
			unpackedBuf[l+2]);
		    ErrorReport(__FILE__, __LINE__, errStr);
		    FREE(supportedFormats);
		    FREE(textureBuf);
		    FREE(workBuf);
		    FREE(unpackedBuf);
		    return ERROR;
		}
	    }
	}
    }

    FREE(supportedFormats);
    FREE(textureBuf);
    FREE(workBuf);
    FREE(unpackedBuf);
    return NO_ERROR;
}

#endif /* GL_OES_compressed_paletted_texture */
