/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conform/tritile.c,v 1.3 2005/05/11 15:21:40 artlab Exp $
*/

/*
** tritile.c
** Triangle Tiling Test.
**
** Description -
**    Tests the edging of triangles (GL_TRIANGLES,
**    GL_TRIANGLE_FAN, GL_TRIANGLE_STRIP). A series of triangles
**    are drawn and their edges are checked for fragment
**    duplication or dropout. Fragment duplication can be
**    detected with blending.
**
** Error Explanation -
**    Failure occurs if fragment duplication or dropout is
**    detected.
**
** Technical Specification -
**    Buffer requirements:
**        Color buffer.
**    Color requirements:
**        Full color.
**    States requirements:
**        Disabled = GL_DITHER, GL_MULTISAMPLE.
**    Error epsilon:
**    Paths:
**        Allowed = Alpha, Blend, Depth, Fog, Logicop, Shade, Stencil.
**        Not allowed = Alias, Dither.
*/

#include "conform.h"
#include "util.h"


static long errX, errY;
static char errStr[240];
static char errStrFmt[] = "Triangle primitive is %s. Incorrect triangle edge at (%d, %d).";


static long Test(GLint size, GLftype *buf)
{
    long i, j, k;

    ReadScreen(0, 0, (GLsizei)size, (GLsizei)size, GL_RGB, buf);
    for (i = 0; i < size; i++) {
	for (j = 0; j < size; j++) {
	    k = (i * size + j) * 3;
	    if (ABS(buf[k]-ITOF(1)/2) > epsilon.color[0] ||
		ABS(buf[k+1]-ITOF(1)/2) > epsilon.color[1] ||
		ABS(buf[k+2]-ITOF(1)/2) > epsilon.color[2]) {
		errX = j;
		errY = i;
		return ERROR;
	    }
	}
    }
    return NO_ERROR;
}

long TriTileExec(void)
{
    GLftype *buf;
    GLint size, i;

    buf = (GLftype *)MALLOC(WINDSIZEX*WINDSIZEY*3*sizeof(GLftype));

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));
    glDisable(GL_DITHER);
    glDisable(GL_MULTISAMPLE);

    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glColor4t(ITOF(1)/2, ITOF(1)/2, ITOF(1)/2, ITOF(1));
    glEnable(GL_BLEND);
    glBlendFunc(GL_ONE, GL_ONE);

    size = (machine.pathLevel == 0) ? WINDSIZEX : (WINDSIZEX / 8);

    glClear(GL_COLOR_BUFFER_BIT);

    for (i = 0; i < size; i++) {
        GLshort vertices[6][2];
        vertices[0][0] = 0;             vertices[0][1] = i;
        vertices[1][0] = (GLshort)size; vertices[1][1] = i;
        vertices[2][0] = (GLshort)size; vertices[2][1] = i+1;
        vertices[3][0] = 0;             vertices[3][1] = i;
        vertices[4][0] = 0;             vertices[4][1] = i+1;
        vertices[5][0] = (GLshort)size; vertices[5][1] = i+1;

        glEnableClientState(GL_VERTEX_ARRAY);
        glVertexPointer(2, GL_SHORT, 0, vertices);
        glDrawArrays(GL_TRIANGLES, 0, 6);
        glDisableClientState(GL_VERTEX_ARRAY);
    }

    if (Test(size, buf) == ERROR) {
	StrMake(errStr, errStrFmt, "GL_TRIANGLES", errX, errY);
	ErrorReport(__FILE__, __LINE__, errStr);
	FREE(buf);
	return ERROR;
    }

    glClear(GL_COLOR_BUFFER_BIT);

    {
        GLshort vertices[(WINDSIZEX+1)*2][2];
        for (i = 0; i <= size; i++) {
            vertices[i*2+0][0] = 0;             vertices[i*2+0][1] = i;
            vertices[i*2+1][0] = (GLshort)size; vertices[i*2+1][1] = i;
        }
  
        glEnableClientState(GL_VERTEX_ARRAY);
        glVertexPointer(2, GL_SHORT, 0, vertices);
        glDrawArrays(GL_TRIANGLE_STRIP, 0, (size+1)*2);
        glDisableClientState(GL_VERTEX_ARRAY);
    }

    if (Test(size, buf) == ERROR) {
	StrMake(errStr, errStrFmt, "GL_TRIANGLE_STRIP", errX, errY);
	ErrorReport(__FILE__, __LINE__, errStr);
	FREE(buf);
	return ERROR;
    }

    glClear(GL_COLOR_BUFFER_BIT);

    {
        GLshort vertices[(WINDSIZEX+1)*2][2];
        vertices[0][0] = 0; vertices[0][1] = 0;
        for (i = 0; i <= size; i++) {
            vertices[i+1][0] = size; vertices[i+1][1] = i;
        }
        for (i = 0; i < size; i++) {
            vertices[i+size+2][0] = size - i - 1; vertices[i+size+2][1] = size;
        }

        glEnableClientState(GL_VERTEX_ARRAY);
        glVertexPointer(2, GL_SHORT, 0, &vertices[0][0]);
        glDrawArrays(GL_TRIANGLE_FAN, 0, (size+1)*2);
        glDisableClientState(GL_VERTEX_ARRAY);
    }

    if (Test(size, buf) == ERROR) {
	StrMake(errStr, errStrFmt, "GL_TRIANGLE_FAN", errX, errY);
	ErrorReport(__FILE__, __LINE__, errStr);
	FREE(buf);
	return ERROR;
    }

    FREE(buf);
    return NO_ERROR;
}
