/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conformshell/driver.c,v 1.4 2005/05/11 15:21:42 artlab Exp $
*/

#include "conform.h"
#include "util.h"
#include "utils.h"
#include "pathdata.h"
#include "path.h"
#include "driver.h"


static long testList[1000];


static long FindTestIndex(const char *id)
{
    long i;

    for (i = 0; STRCMP(driver[i].name, "End of List") != 0; i++) {
	if (STRCMP(driver[i].id, id) == 0) {
	    return i;
	}
    }
    return -1;
}

void DriverInit(void)
{
    long i;

    for (i = 0; STRCMP(driver[i].name, "End of List") != 0; i++) {
	testList[i] = i;
	driver[i].errorCount = 0;   /* Initialize to no failures */
    }
    testList[i] = GL_NULL;
}

long DriverSetup1(const char *id)
{
    long index;

    index = FindTestIndex(id);
    if (index == -1) {
	return ERROR;
    } else {
	testList[0] = index;
	testList[1] = GL_NULL;
	return NO_ERROR;
    }
}

long DriverSetup2(const char *fileName)
{
    FILEHANDLE file;
    char buf[82];
    long index, i;

    file = FOPEN(fileName, "r");
    if (file != NULL) {
	i = 0;
	while (!FEOF(file)) {
	    FSCANF(file, "%[^\n]", buf);
	    if (buf[0] != '#') {
		index = FindTestIndex(buf);
		if (index == -1) {
		    Output(1, "Cannot find test '%s'.\n", buf);
		    FCLOSE(file);
		    return ERROR;
		} else {
		    testList[i++] = index;
		}
	    }
	    FSCANF(file, "%[\n]", buf);
	}
	testList[i] = GL_NULL;
	FCLOSE(file);
	return NO_ERROR;
    } else {
	return ERROR;
    }
}

static long CheckColor(long requirement)
{
    long bitsNeeded = 0;

    switch (requirement) {
      case COLOR_NONE:
	bitsNeeded = 0;
	break;
      case COLOR_AUTO:
	bitsNeeded = 1;
	break;
      case COLOR_MIN:
	bitsNeeded = 2;
	break;
      case COLOR_FULL:
	bitsNeeded = 3;
	break;
    }
    if (buffer.maxRGBBit >= bitsNeeded) {
	return GL_TRUE;
    } else {
	return GL_FALSE;
    }
}

static long DriverExec(long (*Func)(void))
{
    long error;
    GLenum x;
    char buf[40];

    error = (*Func)();

    if (error == ERROR) {
	while ((x = glGetError()) != GL_NO_ERROR) {
	    GetEnumName(x, buf);
	    Output(2, "    GL Error - %s.\n", buf);
	}
	return ERROR_TEST;
    } else if (error == NOT_SUPPORTED) {
	return TEST_NOT_SUPPORTED;
    } else {
	if (GLErrorReport() == ERROR) {
	    return ERROR_GL;
	} else {
	    return NO_ERROR;
	}
    }
}

void Driver(void)
{
    driverRec *ptr;
    long error = NO_ERROR, pass, index, i;

    if (machine.stateCheckFlag == GL_TRUE) {
	Output(1, "Default State test");
	error = StateCheck();
	if (StateInit() == ERROR) {
	    error |= ERROR_STATE;
	}

	if (error == NO_ERROR) {
	    Output(1, " passed.\n");
	} else {
	    Output(1, " failed.\n");
	    StateReport();
	    Output(1, "Cannot continue until default state test passes.\n\n");
	    Output(0, "%s failed.\n", appl.name);
	    return;
	}
    }

    pass = INDIFFERENT;
    for (ptr = &driver[0]; STRCMP(ptr->name, "End of List") != 0; ptr++) {
	ptr->error = NO_ERROR;
    }

    for (index = 0; testList[index] != GL_NULL; index++) {
	ptr = &driver[testList[index]];
	Output(1, "%s test", ptr->name);

	if (CheckColor(ptr->colorRequirement) == GL_FALSE) {
	    Output(1, " cannot run for current config.\n");
	} else if (! buffer.canUse) {
	    Output(1, " does not exist for this config.\n");
	} else {
	    SRAND(machine.randSeed);
	    switch (machine.pathLevel) {
	      case 0:
		StateSave();
		error = DriverExec(ptr->TestRGB);
		error |= StateReset();
		break;
	      case 1:
		StateSave();
		PathInit1(ptr->pathMask);
		error = DriverExec(ptr->TestRGB);
		error |= StateReset();
		break;
	      case 2:
		for (i = 0; i < PATH_TOTAL; i++) {
		    StateSave();
		    PathInit2(i, ptr->pathMask);
		    error = DriverExec(ptr->TestRGB);
		    error |= StateReset();
		    /* Exit if error or test not supported */
		    if (error != NO_ERROR) {
			break;
		    }
		}
		break;
	      case 3:
		for (i = 0; i < PATH_TOTAL; i++) {
		    StateSave();
		    PathInit3(i, ptr->pathMask);
		    error = DriverExec(ptr->TestRGB);
		    error |= StateReset();
		    /* Exit if error or test not supported */
		    if (error != NO_ERROR) {
			break;
		    }
		}
		break;
	      case 4:
		StateSave();
		PathInit4();
		error = DriverExec(ptr->TestRGB);
		error |= StateReset();
		break;
	    }
	    if (error == NO_ERROR) {
		if (pass == INDIFFERENT) {
		    pass = GL_TRUE;
		}
		Output(1, " passed.\n");
	    } else if (error == TEST_NOT_SUPPORTED) {
		Output(1, " not supported for this config (not an error).\n");
		ptr->error = TEST_NOT_SUPPORTED;
	    } else if (error & ERROR_STATE) {
		if (error == ERROR_STATE) {
		    Output(1, " passed.\n");
		} else if (error & (ERROR_TEST|ERROR_GL)) {
		    /* Either error bit may be ORed with ERROR_STATE. A
		    ** "failed" message was already printed by ErrorReport
		    ** or GLErrorReport in this case, but the test should
		    ** still be tagged as failing for the summary report.
		    */
		    ptr->error = ERROR;
		}
		Output(1, "\nState not restored to default condition.\n");
		StateReport();
		Output(1, "Cannot continue until state is reset.\n");
		pass = GL_FALSE;
		break;
	    } else {
		pass = GL_FALSE;
		ptr->error = ERROR;
	    }
	}
    }

    Output(1, "\n");

    if (pass == GL_TRUE) {
	Output(0, "%s (%s) passed.\n", appl.name, appl.type);
    } else if (pass == GL_FALSE) {
	Output(0, "%s (%s) failed.\n", appl.name, appl.type);
    } else {
	Output(0, "%s (%s) subset passed.\n", appl.name, appl.type);
    }

    for (index = 0; testList[index] != GL_NULL; index++) {
	ptr = &driver[testList[index]];
	if (ptr->error == ERROR) {
	    ptr->errorCount++;	    /* Increment total error count */
	    Output(0, "    %s test (Test number #%d) failed.\n", ptr->name,
		   ptr->number);
	} else if (ptr->error == TEST_NOT_SUPPORTED) {
	    Output(0, "    %s test (Test number #%d) not supported (not an error).\n",
		   ptr->name, ptr->number);
	}
    }
}
