/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conformshell/path.c,v 1.4 2005/05/11 15:21:42 artlab Exp $
*/

#include "conform.h"
#include "util.h"
#include "utils.h"
#include "pathdata.h"
#include "path.h"
#include "driver.h"


pathStateRec paths =  {
    {
	PathAlias,
	PathAlpha,
	PathBlend,
	PathDepth,
	PathDither,
	PathFog,
	PathLogicOp,
	PathShade,
	PathStencil
    },
    {
	{
	    {
		PATHDATA_ENABLE, PATHDATA_DISABLE
	    }
	}
    },
    {
	{
	    {
		PATHDATA_ENABLE, PATHDATA_DISABLE
	    },
	    {
		GL_EQUAL,
		GL_NOTEQUAL,
		GL_LESS,
		GL_LEQUAL,
		GL_GREATER,
		GL_GEQUAL,
		GL_ALWAYS,
		GL_NEVER,
		GL_NULL
	    },
	    {
		ITOF(0), ITOF(1)
	    }
	}
    },
    {
	{
	    {
		PATHDATA_ENABLE, PATHDATA_DISABLE
	    },
	    {
		GL_ZERO,
		GL_ONE,
		GL_DST_COLOR,
		GL_ONE_MINUS_DST_COLOR,
		GL_SRC_ALPHA,
		GL_ONE_MINUS_SRC_ALPHA,
		GL_DST_ALPHA,
		GL_ONE_MINUS_DST_ALPHA,
		GL_SRC_ALPHA_SATURATE,
		GL_NULL
	    },
	    {
		GL_ZERO,
		GL_ONE,
		GL_SRC_COLOR,
		GL_ONE_MINUS_SRC_COLOR,
		GL_SRC_ALPHA,
		GL_ONE_MINUS_SRC_ALPHA,
		GL_DST_ALPHA,
		GL_ONE_MINUS_DST_ALPHA,
		GL_NULL
	    }
	}
    },
    {
	{
	    {
		PATHDATA_ENABLE, PATHDATA_DISABLE
	    },
	    {
		ITOF(0), ITOF(1)
	    },
	    {
		ITOF(0), ITOF(1)
	    },
	    {
		ITOF(0), ITOF(1)
	    },
	    {
		GL_EQUAL,
		GL_NOTEQUAL,
		GL_LESS,
		GL_LEQUAL,
		GL_GREATER,
		GL_GEQUAL,
		GL_ALWAYS,
		GL_NEVER,
		GL_NULL
	    }
	}
    },
    {
	{
	    {
		PATHDATA_ENABLE, PATHDATA_DISABLE
	    }
        }
    },
    {
        {
	    {
		PATHDATA_ENABLE, PATHDATA_DISABLE
	    },
	    {
		{
		    ITOF(0), ITOF(1)
		},
		{
		    ITOF(0), ITOF(1)
		},
		{
		    ITOF(0), ITOF(1)
		},
		{
		    ITOF(0), ITOF(1)
		}
	    },
	    {
		ITOF(0), ITOF(1)
	    },
	    {
		ITOF(0), ITOF(1)
	    },
	    {
		ITOF(0), ITOF(1)
	    },
	    {
		ITOF(0), ITOF(1)
	    },
	    {
		GL_LINEAR,
		GL_EXP,
		GL_EXP2,
		GL_NULL
	    }
	}
    },
    {
	{
	    {
		PATHDATA_ENABLE, PATHDATA_DISABLE
	    },
	    {
		GL_NOOP,
		GL_SET,
		GL_EQUIV,
		GL_INVERT,
		GL_CLEAR,
		GL_COPY,
		GL_COPY_INVERTED,
		GL_AND,
		GL_AND_REVERSE,
		GL_AND_INVERTED,
		GL_OR,
		GL_OR_REVERSE,
		GL_OR_INVERTED,
		GL_NAND,
		GL_XOR,
		GL_NOR,
		GL_NULL
	    }
	}
    },
    {
	{
	    {
		GL_FLAT,
		GL_SMOOTH,
		GL_NULL
	    }
	}
    },
    {
	{
	    {
		PATHDATA_ENABLE, PATHDATA_DISABLE
	    },
	    {
		0, 0xFFFFFFFF
	    },
	    {
		0, 0xFFFFFFFF
	    },
	    {
		GL_EQUAL,
		GL_NOTEQUAL,
		GL_LESS,
		GL_LEQUAL,
		GL_GREATER,
		GL_GEQUAL,
		GL_ALWAYS,
		GL_NEVER,
		GL_NULL
	    },
	    {
		-10, 10
	    },
	    {
		0, 0xFFFFFFFF
	    },
	    {
		GL_KEEP,
		GL_REPLACE,
		GL_INCR,
		GL_DECR,
		GL_INVERT,
		GL_ZERO,
		GL_NULL
	    },
	    {
		GL_KEEP,
		GL_REPLACE,
		GL_INCR,
		GL_DECR,
		GL_INVERT,
		GL_ZERO,
		GL_NULL
	    },
	    {
		GL_KEEP,
		GL_REPLACE,
		GL_INCR,
		GL_DECR,
		GL_INVERT,
		GL_ZERO,
		GL_NULL
	    }
	}
    }
};


/*****************************************************************************/

static void Set(void)
{
    long list[PATH_TOTAL], x, y, tmp, i;

    for (i = 0; i < PATH_TOTAL; i++) {
	list[i] = i;
    }
    for (i = 0; i < PATH_TOTAL*6; i++) {
	x = FTOI(OESRandom(ITOF(0), ITOF(PATH_TOTAL-1)));
	do {
	    y = FTOI(OESRandom(ITOF(0), ITOF(PATH_TOTAL-1)));
	} while (x == y);
	tmp = list[x];
	list[x] = list[y];
	list[y] = tmp;
    }

    paths.op = PATHOP_SET;
    for (i = 0; i < PATH_TOTAL; i++) {
	(*paths.Func[list[i]])();
    }
}

void PathInit1(unsigned long mask)
{
    unsigned long i;

    for (i = 0; i < PATH_TOTAL; i++) {
	if (mask & (1 << i)) {
	    paths.op = PATHOP_INIT_CUSTOM;
	} else {
	    paths.op = PATHOP_INIT_DEFAULT;
	}
	(*paths.Func[i])();
    }
    Set();
}

void PathInit2(long activePath, unsigned long mask)
{
    unsigned long i;

    for (i = 0; i < PATH_TOTAL; i++) {
	if (i == (unsigned long)activePath && mask & (1 << i)) {
	    paths.op = PATHOP_INIT_CUSTOM;
	} else {
	    paths.op = PATHOP_INIT_DEFAULT;
	}
	(*paths.Func[i])();
    }
    Set();
}

void PathInit3(long activePath, unsigned long mask)
{
    unsigned long i;

    for (i = 0; i < PATH_TOTAL; i++) {
	if (i != (unsigned long)activePath && mask & (1 << i)) {
	    paths.op = PATHOP_INIT_CUSTOM;
	} else {
	    paths.op = PATHOP_INIT_DEFAULT;
	}
	(*paths.Func[i])();
    }
    Set();
}

void PathInit4(void)
{
    long i;

    for (i = 0; i < PATH_TOTAL; i++) {
	paths.op = PATHOP_INIT_DEFAULT;
	(*paths.Func[i])();
	paths.op = PATHOP_SET;
	(*paths.Func[i])();
    }
}

/*****************************************************************************/

GLenum PathGetList(GLenum *list)
{
    long total, index;

    total = 0;
    while (list[total] != GL_NULL) {
	total++;
    }
    index = FTOI(OESRandom(ITOF(0), ITOF(total)));
    return list[index];
}

GLubyte PathGetRange_uchar(GLubyte *range)
{

    return (GLubyte)FTOI(OESRandom(ITOF(range[0]), ITOF(range[1])));
}

GLbyte PathGetRange_char(GLbyte *range)
{

    return (GLbyte)FTOI(OESRandom(ITOF(range[0]), ITOF(range[1])));
}

GLushort PathGetRange_ushort(GLushort *range)
{

    return (GLushort)FTOI(OESRandom(ITOF(range[0]), ITOF(range[1])));
}

GLshort PathGetRange_short(GLshort *range)
{

    return (GLshort)FTOI(OESRandom(ITOF(range[0]), ITOF(range[1])));
}

GLuint PathGetRange_ulong(GLuint *range)
{
    ftype x;

    x = DIV(ITOF(RAND()), ITOF(RANDOM_MAX));
    return (GLuint)FTOI(MUL(x, ITOF(range[1] - range[0])) + ITOF(range[0]));
}

GLint PathGetRange_long(GLint *range)
{
    ftype x;

    x = DIV(ITOF(RAND()), ITOF(RANDOM_MAX));
    return (GLint)FTOI(MUL(x, ITOF(range[1] - range[0])) + ITOF(range[0]));
}

GLftype PathGetRange_ftype(GLftype *range)
{

    return OESRandom(range[0], range[1]);
}

long PathGetToggle(void)
{

    return (OESRandom(ITOF(0), ITOF(10)) > ITOF(5)) ? PATHDATA_ENABLE : PATHDATA_DISABLE;
}

void PathReport(void)
{
    long i;

    Output(2, "    Path Report.\n");

    paths.op = PATHOP_REPORT;
    for (i = 0; i < PATH_TOTAL; i++) {
	(*paths.Func[i])();
    }
}
