/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conformshell/paths.c,v 1.3 2005/05/11 15:21:42 artlab Exp $
*/

#include "conform.h"
#include "util.h"
#include "pathdata.h"
#include "path.h"


void PathAlias(void)
{
    aliasPathStateRec *ptr = &paths.alias;

    switch (paths.op) {
      case PATHOP_INIT_DEFAULT:
	ptr->current.data1 = PATHDATA_DISABLE;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_INIT_GARBAGE:
	ptr->current.data1 = PATHDATA_DISABLE;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_INIT_CUSTOM:
	ptr->current.data1 = (long)aliasPath2.state;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_SET:
	if (ptr->lock != PATHTEST_LOCKED) {
	    if (ptr->current.data1 == PATHDATA_ENABLE) {
		glEnable(GL_POINT_SMOOTH);
		glEnable(GL_LINE_SMOOTH);
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE);

		glDisable(GL_ALPHA_TEST);
		paths.alpha.lock = PATHTEST_LOCKED;
		glDisable(GL_BLEND);
		paths.blend.lock = PATHTEST_LOCKED;
		glDisable(GL_DEPTH_TEST);
		paths.depth.lock = PATHTEST_LOCKED;
		glDisable(GL_DITHER);
		paths.dither.lock = PATHTEST_LOCKED;
		glDisable(GL_FOG);
		paths.fog.lock = PATHTEST_LOCKED;
	    } else {
		glDisable(GL_POINT_SMOOTH);
		glDisable(GL_LINE_SMOOTH);
	    }
	}
	break;
      case PATHOP_REPORT:
	if (ptr->current.data1 == PATHDATA_ENABLE) {
	    Output(2, "        Alias Path active.\n");
	    Output(2, "            GL_POINT_SMOOTH = enabled.\n");
	    Output(2, "            GL_LINE_SMOOTH = enabled.\n");
	    Output(2, "            Blend src function = GL_SRC_ALPHA,"
		   "Blend dest function = GL_ONE.\n");
	} else {
	    Output(2, "        Alias Path inactive.\n");
	}
	break;
    }
}

void PathAlpha(void)
{
    alphaPathStateRec *ptr = &paths.alpha;
    char str[40];

    switch (paths.op) {
      case PATHOP_INIT_DEFAULT:
	ptr->current.data1 = PATHDATA_DISABLE;
	ptr->current.data2 = GL_ALWAYS;
	ptr->current.data3 = ITOF(0);
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_INIT_GARBAGE:
	ptr->current.data1 = PATHDATA_DISABLE;
	ptr->current.data2 = PathGetList(ptr->true_.data2);
	ptr->current.data3 = PathGetRange_ftype(ptr->true_.data3);
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_INIT_CUSTOM:
	ptr->current.data1 = alphaPath2.state;
	ptr->current.data2 = alphaPath2.func;
	ptr->current.data3 = alphaPath2.ref;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_SET:
	if (ptr->lock != PATHTEST_LOCKED) {
	    if (ptr->current.data1 == PATHDATA_ENABLE) {
		glEnable(GL_ALPHA_TEST);

		glDisable(GL_POINT_SMOOTH);
		glDisable(GL_LINE_SMOOTH);
		paths.alias.lock = PATHTEST_LOCKED;
	    } else {
		glDisable(GL_ALPHA_TEST);
	    }
	    glAlphaFunct(ptr->current.data2, ptr->current.data3);
	}
	break;
      case PATHOP_REPORT:
	if (ptr->current.data1 == PATHDATA_ENABLE) {
	    Output(2, "        Alpha Path active.\n");
	    GetEnumName(ptr->current.data2, str);
	    Output(2, "            Function = %s.\n", str);
	    Output(2, "            Reference = %f.\n", ptr->current.data3);
	} else {
	    Output(2, "        Alpha Path inactive.\n");
	}
	break;
    }
}

void PathBlend(void)
{
    blendPathStateRec *ptr = &paths.blend;
    char str[40];

    switch (paths.op) {
      case PATHOP_INIT_DEFAULT:
	ptr->current.data1 = PATHDATA_DISABLE;
	ptr->current.data2 = GL_ONE;
	ptr->current.data3 = GL_ZERO;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_INIT_GARBAGE:
	ptr->current.data1 = PATHDATA_DISABLE;
	ptr->current.data2 = PathGetList(ptr->true_.data2);
	ptr->current.data3 = PathGetList(ptr->true_.data3);
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_INIT_CUSTOM:
	ptr->current.data1 = blendPath2.state;
	ptr->current.data2 = blendPath2.srcFunc;
	ptr->current.data3 = blendPath2.destFunc;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_SET:
	if (ptr->lock != PATHTEST_LOCKED) {
	    if (ptr->current.data1 == PATHDATA_ENABLE) {
		glEnable(GL_BLEND);

		glDisable(GL_POINT_SMOOTH);
		glDisable(GL_LINE_SMOOTH);
		paths.alias.lock = PATHTEST_LOCKED;
		glDisable(GL_ALPHA_TEST);
		paths.alpha.lock = PATHTEST_LOCKED;
	    } else {
		glDisable(GL_BLEND);
	    }
	    glBlendFunc(ptr->current.data2, ptr->current.data3);
	}
	break;
      case PATHOP_REPORT:
	if (ptr->current.data1 == PATHDATA_ENABLE) {
	    Output(2, "        Blend Path active.\n");
	    GetEnumName(ptr->current.data2, str);
	    Output(2, "            Src function = %s, ", str);
	    GetEnumName(ptr->current.data3, str);
	    Output(2, "Dest function = %s.\n", str);
	} else {
	    Output(2, "        Blend Path inactive.\n");
	}
	break;
    }
}

void PathDepth(void)
{
    depthPathStateRec *ptr = &paths.depth;
    char str[40];

    switch (paths.op) {
      case PATHOP_INIT_DEFAULT:
	ptr->current.data1 = PATHDATA_DISABLE;
	ptr->current.data2 = ITOF(1);
	ptr->current.data3 = ITOF(0);
	ptr->current.data4 = ITOF(1);
	ptr->current.data5 = GL_LESS;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_INIT_GARBAGE:
	ptr->current.data1 = PATHDATA_DISABLE;
	ptr->current.data2 = PathGetRange_ftype(ptr->true_.data2);
	ptr->current.data3 = PathGetRange_ftype(ptr->true_.data3);
	ptr->current.data4 = PathGetRange_ftype(ptr->true_.data4);
	ptr->current.data5 = PathGetList(ptr->true_.data5);
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_INIT_CUSTOM:
	ptr->current.data1 = depthPath2.state;
	ptr->current.data2 = depthPath2.clear;
	ptr->current.data3 = depthPath2.min;
	ptr->current.data4 = depthPath2.max;
	ptr->current.data5 = depthPath2.func;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_SET:
	if (ptr->lock != PATHTEST_LOCKED) {
	    if (ptr->current.data1 == PATHDATA_ENABLE) {
		glEnable(GL_DEPTH_TEST);

		glDisable(GL_POINT_SMOOTH);
		glDisable(GL_LINE_SMOOTH);
		paths.alias.lock = PATHTEST_LOCKED;
		glDisable(GL_STENCIL_TEST);
		paths.stencil.lock = PATHTEST_LOCKED;
	    } else {
		glDisable(GL_DEPTH_TEST);
	    }
	    glDepthRanget(ptr->current.data3, ptr->current.data4);
	    glClearDeptht(ptr->current.data2);
	    glClear(GL_DEPTH_BUFFER_BIT);
	    glDepthFunc(ptr->current.data5);
	}
	break;
      case PATHOP_REPORT:
	if (ptr->current.data1 == PATHDATA_ENABLE) {
	    Output(2, "        Depth Path active.\n");
	    Output(2, "            Clear value = %f.\n", ptr->current.data2);
	    Output(2, "            Range = %f, %f.\n", ptr->current.data3,
		   ptr->current.data4);
	    GetEnumName(ptr->current.data5, str);
	    Output(2, "            Function = %s.\n", str);
	} else {
	    Output(2, "        Depth Path inactive.\n");
	}
	break;
    }
}

void PathDither(void)
{
    ditherPathStateRec *ptr = &paths.dither;

    switch (paths.op) {
      case PATHOP_INIT_DEFAULT:
	ptr->current.data1 = PATHDATA_ENABLE;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_INIT_GARBAGE:
	ptr->current.data1 = PATHDATA_DISABLE;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_INIT_CUSTOM:
	ptr->current.data1 = ditherPath2.state;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_SET:
	if (ptr->lock != PATHTEST_LOCKED) {
	    if (ptr->current.data1 == PATHDATA_ENABLE) {
		glEnable(GL_DITHER);
	    } else {
		glDisable(GL_DITHER);
	    }
	}
	break;
      case PATHOP_REPORT:
	if (ptr->current.data1 == PATHDATA_ENABLE) {
	    Output(2, "        Dither Path active.\n");
	} else {
	    Output(2, "        Dither Path inactive.\n");
	}
	break;
    }
}

void PathFog(void)
{
    fogPathStateRec *ptr = &paths.fog;
    char str[40];
    long i;

    switch (paths.op) {
      case PATHOP_INIT_DEFAULT:
	ptr->current.data1 = PATHDATA_DISABLE;
	for (i = 0; i < 4; i++) {
	    ptr->current.data2[i] = ITOF(0);
	}
	ptr->current.data3 = ITOF(0);
	ptr->current.data4 = ITOF(1);
	ptr->current.data5 = ITOF(0);
	ptr->current.data6 = ITOF(1);
	ptr->current.data7 = GL_EXP;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_INIT_GARBAGE:
	ptr->current.data1 = PATHDATA_DISABLE;
	for (i = 0; i < 4; i++) {
	    ptr->current.data2[i] = PathGetRange_ftype(ptr->true_.data2[i]);
	}
	ptr->current.data3 = PathGetRange_ftype(ptr->true_.data3);
	ptr->current.data4 = PathGetRange_ftype(ptr->true_.data4);
	ptr->current.data5 = PathGetRange_ftype(ptr->true_.data5);
	ptr->current.data6 = PathGetRange_ftype(ptr->true_.data6);
	ptr->current.data7 = PathGetList(ptr->true_.data7);
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_INIT_CUSTOM:
	ptr->current.data1 = fogPath2.state;
	for (i = 0; i < 4; i++) {
	    ptr->current.data2[i] = fogPath2.color[i];
	}
	ptr->current.data3 = fogPath2.index;
	ptr->current.data4 = fogPath2.density;
	ptr->current.data5 = fogPath2.start;
	ptr->current.data6 = fogPath2.end;
	ptr->current.data7 = fogPath2.mode;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_SET:
	if (ptr->lock != PATHTEST_LOCKED) {
	    if (ptr->current.data1 == PATHDATA_ENABLE) {
		glEnable(GL_FOG);

		glDisable(GL_DEPTH_TEST);
		paths.depth.lock = PATHTEST_LOCKED;
	    } else {
		glDisable(GL_FOG);
	    }
	    glFogtv(GL_FOG_COLOR, ptr->current.data2);
	    glFogtv(GL_FOG_DENSITY, &ptr->current.data4);
	    glFogtv(GL_FOG_START, &ptr->current.data5);
	    glFogtv(GL_FOG_END, &ptr->current.data6);
	    glFogt(GL_FOG_MODE, (GLftype)ptr->current.data7);
	}
	break;
      case PATHOP_REPORT:
	if (ptr->current.data1 == PATHDATA_ENABLE) {
	    Output(2, "        Fog Path active.\n");
	    Output(2, "            RGBA color = %f, %f, %f, %f.\n",
		   ptr->current.data2[0], ptr->current.data2[1],
		   ptr->current.data2[2], ptr->current.data2[3]);
	    Output(2, "            Density = %f.\n", ptr->current.data4);
	    Output(2, "            Start and end values = %f, %f.\n",
		   ptr->current.data5, ptr->current.data6);
	    GetEnumName(ptr->current.data7, str);
	    Output(2, "            Mode = %s.\n", str);
	} else {
	    Output(2, "        Fog Path inactive.\n");
	}
	break;
    }
}

void PathLogicOp(void)
{
    logicOpPathStateRec *ptr = &paths.logicOp;
    char str[40];

    switch (paths.op) {
      case PATHOP_INIT_DEFAULT:
	ptr->current.data1 = PATHDATA_DISABLE;
	ptr->current.data2 = GL_COPY;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_INIT_GARBAGE:
	ptr->current.data1 = PATHDATA_DISABLE;
	ptr->current.data2 = PathGetList(ptr->true_.data2);
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_INIT_CUSTOM:
	ptr->current.data1 = logicOpPath2.state;
	ptr->current.data2 = logicOpPath2.func;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_SET:
	if (ptr->lock != PATHTEST_LOCKED) {
	    if (ptr->current.data1 == PATHDATA_ENABLE) {
		glEnable(GL_COLOR_LOGIC_OP);
	    } else {
		glDisable(GL_COLOR_LOGIC_OP);
	    }
	    glLogicOp(ptr->current.data2);
	}
	break;
      case PATHOP_REPORT:
	if (ptr->current.data1 == PATHDATA_ENABLE) {
	    Output(2, "        LogicOp Path active.\n");
	    GetEnumName(ptr->current.data2, str);
	    Output(2, "            Function = %s.\n", str);
	} else {
	    Output(2, "        LogicOp Path inactive.\n");
	}
	break;
    }
}

#if 0
void PathMultisample(void)
{
    multisamplePathStateRec *ptr = &paths.multisample;

    switch (paths.op) {
      case PATHOP_INIT_DEFAULT:
	ptr->current.data1 = PATHDATA_ENABLE;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_INIT_GARBAGE:
	ptr->current.data1 = PATHDATA_DISABLE;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_INIT_CUSTOM:
	ptr->current.data1 = multisamplePath2.state;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_SET:
	if (ptr->lock != PATHTEST_LOCKED) {
	    if (ptr->current.data1 == PATHDATA_ENABLE) {
		glEnable(GL_MULTISAMPLE);
	    } else {
		glDisable(GL_MULTISAMPLE);
	    }
	}
	break;
      case PATHOP_REPORT:
	if (ptr->current.data1 == PATHDATA_ENABLE) {
	    Output(2, "        Multisample Path active.\n");
	} else {
	    Output(2, "        Multisample Path inactive.\n");
	}
	break;
    }
}
#endif

void PathShade(void)
{
    shadePathStateRec *ptr = &paths.shade;
    char str[40];

    switch (paths.op) {
      case PATHOP_INIT_DEFAULT:
	ptr->current.data1 = GL_SMOOTH;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_INIT_GARBAGE:
	ptr->current.data1 = GL_SMOOTH;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_INIT_CUSTOM:
	ptr->current.data1 = shadePath2.mode;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_SET:
	if (ptr->lock != PATHTEST_LOCKED) {
	    glShadeModel(ptr->current.data1);
	}
	break;
      case PATHOP_REPORT:
	GetEnumName(ptr->current.data1, str);
	Output(2, "        Shade model Path = %s.\n", str);
	break;
    }
}

void PathStencil(void)
{
    stencilPathStateRec *ptr = &paths.stencil;
    char str[40];

    switch (paths.op) {
      case PATHOP_INIT_DEFAULT:
	ptr->current.data1 = PATHDATA_DISABLE;
	ptr->current.data2 = 0;
	ptr->current.data3 = ~0;
	ptr->current.data4 = GL_ALWAYS;
	ptr->current.data5 = 0;
	ptr->current.data6 = ~0;
	ptr->current.data7 = GL_KEEP;
	ptr->current.data8 = GL_KEEP;
	ptr->current.data9 = GL_KEEP;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_INIT_GARBAGE:
	ptr->current.data1 = PATHDATA_DISABLE;
	ptr->current.data2 = PathGetRange_long(ptr->true_.data2);
	ptr->current.data3 = PathGetRange_ulong(ptr->true_.data3);
	ptr->current.data4 = PathGetList(ptr->true_.data4);
	ptr->current.data5 = PathGetRange_long(ptr->true_.data5);
	ptr->current.data6 = PathGetRange_ulong(ptr->true_.data6);
	ptr->current.data7 = PathGetList(ptr->true_.data7);
	ptr->current.data8 = PathGetList(ptr->true_.data8);
	ptr->current.data9 = PathGetList(ptr->true_.data9);
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_INIT_CUSTOM:
	ptr->current.data1 = stencilPath2.state;
	ptr->current.data2 = stencilPath2.clear;
	ptr->current.data3 = stencilPath2.writeMask;
	ptr->current.data4 = stencilPath2.func;
	ptr->current.data5 = stencilPath2.ref;
	ptr->current.data6 = stencilPath2.mask;
	ptr->current.data7 = stencilPath2.op1;
	ptr->current.data8 = stencilPath2.op2;
	ptr->current.data9 = stencilPath2.op3;
	ptr->lock = PATHTEST_UNLOCKED;
	break;
      case PATHOP_SET:
	if (ptr->lock != PATHTEST_LOCKED) {
	    if (ptr->current.data1 == PATHDATA_ENABLE) {
		glEnable(GL_STENCIL_TEST);

		glDisable(GL_DEPTH_TEST);
		paths.depth.lock = PATHTEST_LOCKED;
	    } else {
		glDisable(GL_STENCIL_TEST);
	    }

	    glClearStencil(ptr->current.data2);
	    glClear(GL_STENCIL_BUFFER_BIT);
	    glStencilMask(ptr->current.data3);
	    glStencilFunc(ptr->current.data4, ptr->current.data5,
			  ptr->current.data6);
	    glStencilOp(ptr->current.data7, ptr->current.data8,
			ptr->current.data9);
        }
	break;
      case PATHOP_REPORT:
	if (ptr->current.data1 == PATHDATA_ENABLE) {
	    Output(2, "        Stencil Path active.\n");
	    Output(2, "            Clear value = %d.\n", ptr->current.data2);
	    Output(2, "            Mask value = %X.\n",
		   ptr->current.data3);
	    GetEnumName(ptr->current.data4, str);
	    Output(2, "            Function = %s, reference = %d, mask = %X.\n",
		   str, ptr->current.data5, ptr->current.data6);
	    GetEnumName(ptr->current.data7, str);
	    Output(2, "            Op1 = %s, ", str);
	    GetEnumName(ptr->current.data8, str);
	    Output(2, "op2 = %s, ", str);
	    GetEnumName(ptr->current.data9, str);
	    Output(2, "op3 = %s.\n", str);
	} else {
	    Output(2, "        Stencil Path inactive.\n");
	}
	break;
    }
}
