/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conformshell/shell.c,v 1.4 2005/05/11 15:21:42 artlab Exp $
*/

#include "conform.h"
#include "util.h"
#include "utils.h"
#include "pathdata.h"
#include "driver.h"

/* Marker that no explicit config ID was specified */
#define UNSPECIFIED -99

TK_WindowRec wind;
machineRec machine;
bufferRec buffer;
epsilonRec epsilon;
const GLftype colorMap[][3] = {
    {ITOF(0), ITOF(0), ITOF(0)},
    {ITOF(1), ITOF(0), ITOF(0)},
    {ITOF(0), ITOF(1), ITOF(0)},
    {ITOF(0), ITOF(0), ITOF(1)},
    {ITOF(1), ITOF(1), ITOF(0)},
    {ITOF(1), ITOF(0), ITOF(1)},
    {ITOF(0), ITOF(1), ITOF(1)},
    {ITOF(1), ITOF(1), ITOF(1)}
};


static long Setup(int argc, char **argv)
{
    FILEHANDLE file;
    long i, j;

    machine.randSeed = 1;
    machine.pathLevel = 0;
    machine.verboseLevel = 1;
    machine.stateCheckFlag = GL_TRUE;
    machine.failMode = GL_FALSE;
    machine.logFile = 0;
    machine.logFileName[0] = 0;
    buffer.configID = UNSPECIFIED;
    DriverInit();

    for (i = 1; i < argc; i++) {
        if (STRCMP(argv[i], "-1") == 0) {  /* run single test. */
            if (i+1 >= argc || argv[i+1][0] == '-') {
                Output(0, "-1 (No test name).\n\n");
                return GL_TRUE;
            } else {
                if (DriverSetup1(argv[++i]) == ERROR) {
                    Output(0, "-1 (Bad test name).\n\n");
                    return GL_TRUE;
                }
            }
        } else if (STRCMP(argv[i], "-C") == 0) {  /* config. */
            if (i+1 >= argc || argv[i+1][0] == '-') {
                Output(0, "-C (No config ID).\n\n");
                return GL_TRUE;
            } else {
                buffer.configID = ATOI(argv[++i]);
                if (buffer.configID < 0 || buffer.configID > 1) {
                    Output(0, "-C (Bad config ID).\n\n");
                    return GL_TRUE;
                } else {
                    buffer.configID = -buffer.configID;
                }
            }
        } else if (STRCMP(argv[i], "-c") == 0) {  /* config. */
            if (i+1 >= argc || argv[i+1][0] == '-') {
                Output(0, "-c (No config ID).\n\n");
                return GL_TRUE;
            } else {
                buffer.configID = ATOI(argv[++i]);
            }
        } else if (STRCMP(argv[i], "-f") == 0) {  /* use test set. */
            if (i+1 >= argc || argv[i+1][0] == '-') {
                Output(0, "-f (No file name).\n\n");
                return GL_TRUE;
            } else {
                if (DriverSetup2(argv[++i]) == ERROR) {
                    Output(0, "-f (Bad file).\n\n");
                    return GL_TRUE;
                }
            }
        } else if (STRCMP(argv[i], "-G") == 0) {  /* generate test set. */
            if (i+1 >= argc || argv[i+1][0] == '-') {
                Output(0, "-G (No file name).\n\n");
            } else {
                file = FOPEN(argv[++i], "w");
                if (file != NULL) {
                    FPRINTF1(file, "# %s Tests.\n", appl.title);
                    for (j = 0; STRCMP(driver[j].name, "End of List") != 0; j++) {
                        FPRINTF1(file, "#%s\n", driver[j].id);
                    }
                    FCLOSE(file);
                    Output(0, "Test file %s created.\n\n", argv[i]);
                } else {
                    Output(0, "-G (Bad file name).\n\n");
                }
            }
            return GL_TRUE;
        } else if (STRCMP(argv[i], "-g") == 0) {  /* generate test set. */
            if (i+1 >= argc || argv[i+1][0] == '-') {
                Output(0, "-g (No file name).\n\n");
            } else {
                file = FOPEN(argv[++i], "w");
                if (file != NULL) {
                    FPRINTF1(file, "# %s Tests.\n", appl.title);
                    for (j = 0; STRCMP(driver[j].name, "End of List") != 0; j++) {
                        FPRINTF1(file, "%s\n", driver[j].id);
                    }
                    FCLOSE(file);
                    Output(0, "Test file %s created.\n\n", argv[i]);
                } else {
                    Output(0, "-g (Bad file name).\n\n");
                }
            }
            return GL_TRUE;
        } else if (STRCMP(argv[i], "-h") == 0) {  /* help. */
            Output(0, "Options:\n");
            Output(0, "\t-1 <test>\tSingle test using \"test\" id.\n");
            Output(0, "\t-c <id>\t\tUse config id.\n");
            Output(0, "\t-C [1-1]\t\tUse predefined config.\n");
            Output(0, "\t-f <file>\tUse test set in \"file\".\n");
            Output(0, "\t-g <file>\tGenerate test set in \"file\".\n");
            Output(0, "\t-h\t\tPrint this help screen.\n");
            Output(0, "\t-l <file>\tGenerate logfile.\n");
            Output(0, "\t-p [1-4]\tSet path level.\n");
            Output(0, "\t-r <seed>\tSet random seed.\n");
            Output(0, "\t-s\t\tSkip state check.\n");
            Output(0, "\t-v [0-2]\tVerbose level.\n");
            Output(0, "\t-x\t\tForce fail for config tests.\n");
            Output(0, "\n");
            return GL_TRUE;
        } else if (STRCMP(argv[i], "-l") == 0) {  /* logfile. */
            if (i+1 >= argc || argv[i+1][0] == '-') {
                Output(0, "-l (No logfile name).\n\n");
                return GL_TRUE;
            } else {
                machine.logFile = 1;
                STRCPY(machine.logFileName, argv[++i]);
                file = FOPEN(machine.logFileName, "w");
                if (file != NULL) {
                    FCLOSE(file);
                    Output(0, "Logfile %s created.\n\n", argv[i]);
                } else {
                    Output(0, "-l (Bad logfile name).\n\n");
                }
            }
        } else if (STRCMP(argv[i], "-p") == 0) {  /* path options. */
            if (i+1 >= argc || argv[i+1][0] == '-') {
                Output(0, "-p (No path number).\n\n");
                return GL_TRUE;
            } else {
                machine.pathLevel = ATOI(argv[++i]);
                if (machine.pathLevel > 4) {
                    Output(0, "-p (Bad path number).\n\n");
                    return GL_TRUE;
                }
            }
        } else if (STRCMP(argv[i], "-r") == 0) {  /* random seed. */
            if (i+1 >= argc || argv[i+1][0] == '-') {
                Output(0, "-r (No random seed).\n\n");
                return GL_TRUE;
            } else {
                machine.randSeed = (unsigned int)ATOI(argv[++i]);
            }
        } else if (STRCMP(argv[i], "-s") == 0) {  /* skip state check. */
            machine.stateCheckFlag = GL_FALSE;
        } else if (STRCMP(argv[i], "-x") == 0) {  /* force fail. */
            machine.failMode = GL_TRUE;
        } else if (STRCMP(argv[i], "-v") == 0) {  /* verbose. */
            if (i+1 >= argc || argv[i+1][0] == '-') {
                Output(0, "-v (No verbose level).\n\n");
                return GL_TRUE;
            } else {
                machine.verboseLevel = ATOI(argv[++i]);
                if (machine.verboseLevel < 0 || machine.verboseLevel > 2) {
                    Output(0, "-v (Bad verbose level).\n\n");
                    return GL_TRUE;
                }
            }
        } else {
            Output(0, "%s (Bad option).\n\n", argv[i]);
            return GL_TRUE;
        }
    }
    return GL_FALSE;
}

static void BufferSetup(void)
{
    long i;

    buffer.canUse = GL_TRUE;

    glGetIntegerv(GL_RED_BITS, &buffer.colorBits[0]);
    glGetIntegerv(GL_GREEN_BITS, &buffer.colorBits[1]);
    glGetIntegerv(GL_BLUE_BITS, &buffer.colorBits[2]);
    glGetIntegerv(GL_ALPHA_BITS, &buffer.colorBits[3]);
    glGetIntegerv(GL_DEPTH_BITS, &buffer.zBits);
    glGetIntegerv(GL_STENCIL_BITS, &buffer.stencilBits);

    buffer.maxRGBComponent = 0;
    if (buffer.colorBits[1] > buffer.colorBits[buffer.maxRGBComponent]) {
        buffer.maxRGBComponent = 1;
    }
    if (buffer.colorBits[2] > buffer.colorBits[buffer.maxRGBComponent]) {
        buffer.maxRGBComponent = 2;
    }
    for (i = 0; i < 3; i++) {
        buffer.minRGB[i] = ITOF(0);
        buffer.maxRGB[i] = ITOF(0);
    }
    buffer.maxRGB[buffer.maxRGBComponent] = ITOF(1);
    buffer.minRGBBit = MIN(buffer.colorBits[0], MIN(buffer.colorBits[1],
                           buffer.colorBits[2]));
    buffer.maxRGBBit = MAX(buffer.colorBits[0], MAX(buffer.colorBits[1],
                           buffer.colorBits[2]));
}

static void EpsilonSetup(void)
{
    long i;

    epsilon.zero = LDEXP(ITOF(1), -13);

    for (i = 0; i < 4; i++) {
        epsilon.color[i] = CalcEpsilon(MIN(buffer.colorBits[i], 8));
    }
}

static void MachineReport(void)
{

    Output(0, "Setup Report.\n");

    if (machine.verboseLevel > 0) {
        Output(0, "    Verbose level = %d.\n", machine.verboseLevel);
    }

    Output(0, "    Random number seed = %d.\n", machine.randSeed);

    if (machine.pathLevel == 0) {
        Output(0, "    Path inactive.\n");
    } else if (machine.pathLevel == 99) {
        Output(0, "    Fast inert path.\n");
    } else {
        Output(0, "    Path level = %d.\n", machine.pathLevel);
    }

    if (machine.failMode == GL_TRUE) {
        Output(0, "    Fail mode on.\n");
    }

    Output(0, "\n");
}

static void ConfigReport(void)
{

    Output(0, "Config Report.\n");

    if (buffer.configID != UNSPECIFIED) {
        Output(0, "    Config ID = %d.\n", buffer.configID);
    }

    Output(0, "    RGBA (%d, %d, %d, %d).\n", buffer.colorBits[0],
        buffer.colorBits[1], buffer.colorBits[2], buffer.colorBits[3]);

    if (buffer.stencilBits) {
        Output(0, "    Stencil (%d).\n", buffer.stencilBits);
    }

    if (buffer.zBits) {
        Output(0, "    Depth (%d).\n", buffer.zBits);
    }

    Output(0, "\n");
}

static void EpsilonReport(void)
{

    Output(2, "Epsilon Report.\n");
    Output(2, "    zero error epsilon = %.3g.\n", epsilon.zero);
    Output(2, "    RGBA error epsilon = %.3g, %.3g, %.3g, %.3g.\n",
        epsilon.color[0], epsilon.color[1], epsilon.color[2],
        epsilon.color[3]);
    Output(2, "\n");
}

long Exec(TK_EventRec *ptr)
{

    if (ptr->event == TK_EVENT_EXPOSE) {
        BufferSetup();
        EpsilonSetup();
        StateSetup();

        MachineReport();
        ConfigReport();
        EpsilonReport();

        Driver();
        return 0;
    }
    return 1;
}

long NonConformantExec(TK_EventRec *ptr)
{
    if (ptr->event == TK_EVENT_EXPOSE) {
        ConfigReport();
        Output(0, "  Config is identified as NON_CONFORMANT.\n");
        Output(0, "  It may be rendered to using an OpenGL ES context.\n\n");
        return 0;
    }
    return 1;
}

static TK_ConfigsRec list;

#if defined(CONFORM_ALTERNATE_MAIN)
 int conformain(int argc, char **argv)
#else
 int main(int argc, char **argv)
#endif

{
    long i, testsFailed;
    driverRec *ptr;
    /* Flag if list of configs has been initialized */
    GLboolean listValid = GL_FALSE;

    if (Setup(argc, argv) == GL_TRUE) {
        tkQuit();
        return EXIT_FAILURE;
    }

    if (machine.logFile) {
        machine.logFileHandle = FOPEN(machine.logFileName, "a");
    }

    Output(0, "%s Test\n", appl.title);
    Output(0, "Version %s\n", appl.version);
    Output(0, "\n");

    StrMake(wind.name, "%s Test", appl.name);
    wind.x = 0;
    wind.y = 0;
    wind.width = WINDSIZEX;
    wind.height = WINDSIZEY;

    if (buffer.configID == UNSPECIFIED) {
        tkGetConfigs(&list);
        listValid = GL_TRUE;

        for (i = 0; i < list.count; i++) {
            wind.type = TK_WIND_CONFIG;
            buffer.configID = (GLint)list.IDs[i];
            wind.info = buffer.configID;
            if (tkNewWindow(&wind)) {
                tkExec(Exec);
                tkCloseWindow();
            } else {
                Output(0, "Config ID not found.\n");
            }
        }
        for (i = 0; i < list.nonConformantCount; i++) {
            wind.type = TK_WIND_CONFIG;
            buffer.configID = (GLint)list.nonConformantIDs[i];
            wind.info = buffer.configID;
            if (tkNewWindow(&wind)) {
                tkExec(NonConformantExec);
                tkCloseWindow();
            } else {
                Output(0, "Unable to open window with config identified"
                "as NON_CONFORMANT.\n");
            }
        }

    } else {
        switch (buffer.configID) {
        case -1:
            buffer.configID = UNSPECIFIED;
            wind.type = TK_WIND_REQUEST;
            wind.info = TK_WIND_STENCIL | TK_WIND_Z;
            if (tkNewWindow(&wind)) {
                tkExec(Exec);
                tkCloseWindow();
                Output(0, "\n");
            } else {
                Output(0, "Config requested not found.\n\n");
            }
            break;
        default:
            wind.type = TK_WIND_CONFIG;
            wind.info = buffer.configID;
            if (tkNewWindow(&wind)) {
                tkExec(Exec);
                tkCloseWindow();
                Output(0, "\n");
            } else {
                Output(0, "Display requested not found.\n\n");
            }
            break;
        }
    }

    /* Summarize all failed tests. This loops over tests that may not
     * be run (not in the test list), but that's OK because their
     * errorCount will always be 0.
     */
    Output(0, "******************** SUMMARY OF 'conform' RESULTS ********************\n");
    testsFailed = 0;
    for (i = 0; STRCMP(driver[i].name, "End of List") != 0; i++) {
        ptr = &driver[i];
        if (ptr->errorCount > 0) {
            Output(0, "SUMMARY: %s test (#%d) failed %d %s at path level %d.\n",
                ptr->name, ptr->number, ptr->errorCount,
                (ptr->errorCount == 1) ? "time" : "times",
                machine.pathLevel);
            testsFailed++;
        }
    }

    if (testsFailed > 0) {
        Output(0, "SUMMARY: %d %s failed across all configs at path level %d.\n",
            testsFailed,
            (testsFailed == 1) ? "test" : "tests",
            machine.pathLevel);
    } else {
        Output(0, "SUMMARY: NO tests failed at any path level.\n");
    }

    /* Summarize conformant/nonconformant config count */
    if (listValid == GL_TRUE) {
        Output(0, "SUMMARY: %d conformant configs, %d non-conformant configs.\n",
            list.count, list.nonConformantCount);
    } else {
        Output(0, "SUMMARY: did not query conformant/non-conformant config count.\n");
    }
    Output(0, "****************** END SUMMARY OF 'conform' RESULTS ******************\n\n");

    tkQuit();

    if (machine.logFileHandle)
        FCLOSE(machine.logFileHandle);

    return EXIT_SUCCESS;
}
