/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conformshell/util.c,v 1.4 2005/05/11 15:21:42 artlab Exp $
*/

#include <assert.h>
#include "conform.h"
#include "pathdata.h"
#include "path.h"
#include "util.h"
#include "utils.h"


struct _enumInfoRec {
    char name[40];
    long value;
} enumInfo[] = {
    {
	"GL_ADD", GL_ADD
    },
    {
        "GL_ALIASED_LINE_WIDTH_RANGE", GL_ALIASED_LINE_WIDTH_RANGE
    },
    {
        "GL_ALIASED_POINT_SIZE_RANGE", GL_ALIASED_POINT_SIZE_RANGE
    },
    {
	"GL_ALPHA", GL_ALPHA
    },
    {
	"GL_ALPHA_BITS", GL_ALPHA_BITS
    },
    {
	"GL_ALPHA_TEST", GL_ALPHA_TEST
    },
    {
	"GL_ALWAYS", GL_ALWAYS
    },
    {
	"GL_AMBIENT", GL_AMBIENT
    },
    {
	"GL_AMBIENT_AND_DIFFUSE", GL_AMBIENT_AND_DIFFUSE
    },
    {
	"GL_AND", GL_AND
    },
    {
	"GL_AND_INVERTED", GL_AND_INVERTED
    },
    {
	"GL_AND_REVERSE", GL_AND_REVERSE
    },
    {
	"GL_BACK", GL_BACK
    },
    {
	"GL_BLEND", GL_BLEND
    },
    {
	"GL_BLUE_BITS", GL_BLUE_BITS
    },
    {
	"GL_BYTE", GL_BYTE
    },
    {
	"GL_CCW", GL_CCW
    },
    {
	"GL_CLAMP_TO_EDGE", GL_CLAMP_TO_EDGE
    },
    {
	"GL_CLEAR", GL_CLEAR
    },
    {
        "GL_COLOR_ARRAY", GL_COLOR_ARRAY
    },
    {
        "GL_COLOR_LOGIC_OP", GL_COLOR_LOGIC_OP
    },
    {
	"GL_COLOR_MATERIAL", GL_COLOR_MATERIAL
    },
    {
        "GL_COMPRESSED_TEXTURE_FORMATS", GL_COMPRESSED_TEXTURE_FORMATS
    },
    {
	"GL_CONSTANT_ATTENUATION", GL_CONSTANT_ATTENUATION
    },
    {
	"GL_COPY", GL_COPY
    },
    {
	"GL_COPY_INVERTED", GL_COPY_INVERTED
    },
    {
	"GL_CULL_FACE", GL_CULL_FACE
    },
    {
	"GL_CW", GL_CW
    },
    {
	"GL_DECAL", GL_DECAL
    },
    {
	"GL_DECR", GL_DECR
    },
    {
	"GL_DEPTH_BITS", GL_DEPTH_BITS
    },
    {
	"GL_DEPTH_TEST", GL_DEPTH_TEST
    },
    {
	"GL_DIFFUSE", GL_DIFFUSE
    },
    {
	"GL_DITHER", GL_DITHER
    },
    {
	"GL_DONT_CARE", GL_DONT_CARE
    },
    {
	"GL_DST_ALPHA", GL_DST_ALPHA
    },
    {
	"GL_DST_COLOR", GL_DST_COLOR
    },
    {
	"GL_EMISSION", GL_EMISSION
    },
    {
	"GL_EQUAL", GL_EQUAL
    },
    {
	"GL_EQUIV", GL_EQUIV
    },
    {
	"GL_EXP", GL_EXP
    },
    {
	"GL_EXP2", GL_EXP2
    },
    {
	"GL_EXTENSIONS", GL_EXTENSIONS
    },
    {
	"GL_FASTEST", GL_FASTEST
    },
    {
	"GL_FIXED", GL_FIXED
    },
    {
	"GL_FLAT", GL_FLAT
    },
#ifdef COMMON
    {
	"GL_FLOAT", GL_FLOAT
    },
#endif
    {
	"GL_FOG", GL_FOG
    },
    {
	"GL_FOG_COLOR", GL_FOG_COLOR
    },
    {
	"GL_FOG_DENSITY", GL_FOG_DENSITY
    },
    {
	"GL_FOG_END", GL_FOG_END
    },
    {
	"GL_FOG_HINT", GL_FOG_HINT
    },
    {
	"GL_FOG_MODE", GL_FOG_MODE
    },
    {
	"GL_FOG_START", GL_FOG_START
    },
    {
	"GL_FRONT", GL_FRONT
    },
    {
	"GL_FRONT_AND_BACK", GL_FRONT_AND_BACK
    },
    {
	"GL_GEQUAL", GL_GEQUAL
    },
    {
	"GL_GREATER", GL_GREATER
    },
    {
	"GL_GREEN_BITS", GL_GREEN_BITS
    },
#ifdef GL_OES_read_format
    {
        "GL_IMPLEMENTATION_COLOR_READ_FORMAT_OES", GL_IMPLEMENTATION_COLOR_READ_FORMAT_OES
    },
    {
        "GL_IMPLEMENTATION_COLOR_READ_TYPE_OES", GL_IMPLEMENTATION_COLOR_READ_TYPE_OES
    },
#endif
    {
	"GL_INCR", GL_INCR
    },
    {
	"GL_INVALID_ENUM", GL_INVALID_ENUM
    },
    {
	"GL_INVALID_OPERATION", GL_INVALID_OPERATION
    },
    {
	"GL_INVALID_VALUE", GL_INVALID_VALUE
    },
    {
	"GL_INVERT", GL_INVERT
    },
    {
	"GL_KEEP", GL_KEEP
    },
    {
	"GL_LEQUAL", GL_LEQUAL
    },
    {
	"GL_LESS", GL_LESS
    },
    {
	"GL_LIGHT0", GL_LIGHT0
    },
    {
	"GL_LIGHT1", GL_LIGHT1
    },
    {
	"GL_LIGHT2", GL_LIGHT2
    },
    {
	"GL_LIGHT3", GL_LIGHT3
    },
    {
	"GL_LIGHT4", GL_LIGHT4
    },
    {
	"GL_LIGHT5", GL_LIGHT5
    },
    {
	"GL_LIGHT6", GL_LIGHT6
    },
    {
	"GL_LIGHT7", GL_LIGHT7
    },
    {
	"GL_LIGHTING", GL_LIGHTING
    },
    {
	"GL_LIGHT_MODEL_AMBIENT", GL_LIGHT_MODEL_AMBIENT
    },
    {
	"GL_LIGHT_MODEL_TWO_SIDE", GL_LIGHT_MODEL_TWO_SIDE
    },
    {
	"GL_LINEAR", GL_LINEAR
    },
    {
	"GL_LINEAR_ATTENUATION", GL_LINEAR_ATTENUATION
    },
    {
	"GL_LINEAR_MIPMAP_LINEAR", GL_LINEAR_MIPMAP_LINEAR
    },
    {
	"GL_LINEAR_MIPMAP_NEAREST", GL_LINEAR_MIPMAP_NEAREST
    },
    {
	"GL_LINES", GL_LINES
    },
    {
	"GL_LINE_LOOP", GL_LINE_LOOP
    },
    {
	"GL_LINE_SMOOTH", GL_LINE_SMOOTH
    },
    {
	"GL_LINE_SMOOTH_HINT", GL_LINE_SMOOTH_HINT
    },
    {
	"GL_LINE_STRIP", GL_LINE_STRIP
    },
    {
	"GL_LUMINANCE", GL_LUMINANCE
    },
    {
	"GL_LUMINANCE_ALPHA", GL_LUMINANCE_ALPHA
    },
    {
	"GL_MAX_LIGHTS", GL_MAX_LIGHTS
    },
    {
	"GL_MAX_MODELVIEW_STACK_DEPTH", GL_MAX_MODELVIEW_STACK_DEPTH
    },
    {
	"GL_MAX_PROJECTION_STACK_DEPTH", GL_MAX_PROJECTION_STACK_DEPTH
    },
    {
	"GL_MAX_TEXTURE_SIZE", GL_MAX_TEXTURE_SIZE
    },
    {
	"GL_MAX_TEXTURE_STACK_DEPTH", GL_MAX_TEXTURE_STACK_DEPTH
    },
    {
        "GL_MAX_TEXTURE_UNITS", GL_MAX_TEXTURE_UNITS
    },
    {
	"GL_MAX_VIEWPORT_DIMS", GL_MAX_VIEWPORT_DIMS
    },
    {
	"GL_MODELVIEW", GL_MODELVIEW
    },
    {
	"GL_MODULATE", GL_MODULATE
    },
    {
	"GL_NAND", GL_NAND
    },
    {
	"GL_NEAREST", GL_NEAREST
    },
    {
	"GL_NEAREST_MIPMAP_LINEAR", GL_NEAREST_MIPMAP_LINEAR
    },
    {
	"GL_NEAREST_MIPMAP_NEAREST", GL_NEAREST_MIPMAP_NEAREST
    },
    {
	"GL_NEVER", GL_NEVER
    },
    {
	"GL_NICEST", GL_NICEST
    },
    {
	"GL_NOOP", GL_NOOP
    },
    {
	"GL_NOR", GL_NOR
    },
    {
	"GL_NORMALIZE", GL_NORMALIZE
    },
    {
        "GL_NORMAL_ARRAY", GL_NORMAL_ARRAY
    },
    {
	"GL_NOTEQUAL", GL_NOTEQUAL
    },
    {
        "GL_NUM_COMPRESSED_TEXTURE_FORMATS", GL_NUM_COMPRESSED_TEXTURE_FORMATS
    },
    {
	"GL_ONE", GL_ONE
    },
    {
	"GL_ONE_MINUS_DST_ALPHA", GL_ONE_MINUS_DST_ALPHA
    },
    {
	"GL_ONE_MINUS_DST_COLOR", GL_ONE_MINUS_DST_COLOR
    },
    {
	"GL_ONE_MINUS_SRC_ALPHA", GL_ONE_MINUS_SRC_ALPHA
    },
    {
	"GL_ONE_MINUS_SRC_COLOR", GL_ONE_MINUS_SRC_COLOR
    },
    {
	"GL_OR", GL_OR
    },
    {
	"GL_OR_INVERTED", GL_OR_INVERTED
    },
    {
	"GL_OR_REVERSE", GL_OR_REVERSE
    },
    {
	"GL_OUT_OF_MEMORY", GL_OUT_OF_MEMORY
    },
    {
	"GL_PACK_ALIGNMENT", GL_PACK_ALIGNMENT
    },
#ifdef GL_OES_compressed_paletted_texture
    {
	"GL_PALETTE4_RGB8_OES", GL_PALETTE4_RGB8_OES
    },
    {
	"GL_PALETTE4_RGBA8_OES", GL_PALETTE4_RGBA8_OES
    },
    {
	"GL_PALETTE4_R5_G6_B5_OES", GL_PALETTE4_R5_G6_B5_OES
    },
    {
	"GL_PALETTE4_RGBA4_OES", GL_PALETTE4_RGBA4_OES
    },
    {
	"GL_PALETTE4_RGB5_A1_OES", GL_PALETTE4_RGB5_A1_OES
    },
    {
	"GL_PALETTE8_RGB8_OES", GL_PALETTE8_RGB8_OES
    },
    {
	"GL_PALETTE8_RGBA8_OES", GL_PALETTE8_RGBA8_OES
    },
    {
	"GL_PALETTE8_R5_G6_B5_OES", GL_PALETTE8_R5_G6_B5_OES
    },
    {
	"GL_PALETTE8_RGBA4_OES", GL_PALETTE8_RGBA4_OES
    },
    {
	"GL_PALETTE8_RGB5_A1_OES", GL_PALETTE8_RGB5_A1_OES
    },
#endif /* GL_OES_compressed_paletted_texture */
    {
	"GL_PERSPECTIVE_CORRECTION_HINT", GL_PERSPECTIVE_CORRECTION_HINT
    },
    {
	"GL_POINTS", GL_POINTS
    },
    {
	"GL_POINT_SMOOTH", GL_POINT_SMOOTH
    },
    {
	"GL_POINT_SMOOTH_HINT", GL_POINT_SMOOTH_HINT
    },
    {
	"GL_POSITION", GL_POSITION
    },
    {
	"GL_PROJECTION", GL_PROJECTION
    },
    {
	"GL_QUADRATIC_ATTENUATION", GL_QUADRATIC_ATTENUATION
    },
    {
	"GL_RED_BITS", GL_RED_BITS
    },
    {
	"GL_RENDERER", GL_RENDERER
    },
    {
	"GL_REPEAT", GL_REPEAT
    },
    {
	"GL_REPLACE", GL_REPLACE
    },
    {
        "GL_RESCALE_NORMAL", GL_RESCALE_NORMAL
    },
    {
	"GL_RGB", GL_RGB
    },
    {
	"GL_RGBA", GL_RGBA
    },
    {
        "GL_SAMPLE_ALPHA_TO_COVERAGE", GL_SAMPLE_ALPHA_TO_COVERAGE
    },
    {
        "GL_SAMPLE_ALPHA_TO_ONE", GL_SAMPLE_ALPHA_TO_ONE
    },
    {
        "GL_SAMPLE_COVERAGE", GL_SAMPLE_COVERAGE
    },
    {
	"GL_SCISSOR_TEST", GL_SCISSOR_TEST
    },
    {
	"GL_SET", GL_SET
    },
    {
	"GL_SHININESS", GL_SHININESS
    },
    {
	"GL_SHORT", GL_SHORT
    },
    {
	"GL_SMOOTH", GL_SMOOTH
    },
    {
	"GL_SMOOTH_LINE_WIDTH_RANGE", GL_SMOOTH_LINE_WIDTH_RANGE
    },
    {
	"GL_SMOOTH_POINT_SIZE_RANGE", GL_SMOOTH_POINT_SIZE_RANGE
    },
    {
	"GL_SPECULAR", GL_SPECULAR
    },
    {
	"GL_SPOT_CUTOFF", GL_SPOT_CUTOFF
    },
    {
	"GL_SPOT_DIRECTION", GL_SPOT_DIRECTION
    },
    {
	"GL_SPOT_EXPONENT", GL_SPOT_EXPONENT
    },
    {
	"GL_SRC_ALPHA", GL_SRC_ALPHA
    },
    {
	"GL_SRC_ALPHA_SATURATE", GL_SRC_ALPHA_SATURATE
    },
    {
	"GL_SRC_COLOR", GL_SRC_COLOR
    },
    {
	"GL_STACK_OVERFLOW", GL_STACK_OVERFLOW
    },
    {
	"GL_STACK_UNDERFLOW", GL_STACK_UNDERFLOW
    },
    {
	"GL_STENCIL_BITS", GL_STENCIL_BITS
    },
    {
	"GL_STENCIL_TEST", GL_STENCIL_TEST
    },
    {
	"GL_SUBPIXEL_BITS", GL_SUBPIXEL_BITS
    },
    {
	"GL_TEXTURE", GL_TEXTURE
    },
    {
        "GL_TEXTURE0", GL_TEXTURE0
    },
    {
        "GL_TEXTURE1", GL_TEXTURE1
    },
    {
        "GL_TEXTURE2", GL_TEXTURE2
    },
    {
        "GL_TEXTURE3", GL_TEXTURE3
    },
    {
        "GL_TEXTURE4", GL_TEXTURE4
    },
    {
        "GL_TEXTURE5", GL_TEXTURE5
    },
    {
        "GL_TEXTURE6", GL_TEXTURE6
    },
    {
        "GL_TEXTURE7", GL_TEXTURE7
    },
    {
        "GL_TEXTURE8", GL_TEXTURE8
    },
    {
        "GL_TEXTURE9", GL_TEXTURE9
    },
    {
        "GL_TEXTURE10", GL_TEXTURE10
    },
    {
        "GL_TEXTURE11", GL_TEXTURE11
    },
    {
        "GL_TEXTURE12", GL_TEXTURE12
    },
    {
        "GL_TEXTURE13", GL_TEXTURE13
    },
    {
        "GL_TEXTURE14", GL_TEXTURE14
    },
    {
        "GL_TEXTURE15", GL_TEXTURE15
    },
    {
        "GL_TEXTURE16", GL_TEXTURE16
    },
    {
        "GL_TEXTURE17", GL_TEXTURE17
    },
    {
        "GL_TEXTURE18", GL_TEXTURE18
    },
    {
        "GL_TEXTURE19", GL_TEXTURE19
    },
    {
        "GL_TEXTURE20", GL_TEXTURE20
    },
    {
        "GL_TEXTURE21", GL_TEXTURE21
    },
    {
        "GL_TEXTURE22", GL_TEXTURE22
    },
    {
        "GL_TEXTURE23", GL_TEXTURE23
    },
    {
        "GL_TEXTURE24", GL_TEXTURE24
    },
    {
        "GL_TEXTURE25", GL_TEXTURE25
    },
    {
        "GL_TEXTURE26", GL_TEXTURE26
    },
    {
        "GL_TEXTURE27", GL_TEXTURE27
    },
    {
        "GL_TEXTURE28", GL_TEXTURE28
    },
    {
        "GL_TEXTURE29", GL_TEXTURE29
    },
    {
        "GL_TEXTURE30", GL_TEXTURE30
    },
    {
        "GL_TEXTURE31", GL_TEXTURE31
    },
    {
	"GL_TEXTURE_2D", GL_TEXTURE_2D
    },
    {
        "GL_TEXTURE_COORD_ARRAY", GL_TEXTURE_COORD_ARRAY
    },
    {
	"GL_TEXTURE_ENV", GL_TEXTURE_ENV
    },
    {
	"GL_TEXTURE_ENV_COLOR", GL_TEXTURE_ENV_COLOR
    },
    {
	"GL_TEXTURE_ENV_MODE", GL_TEXTURE_ENV_MODE
    },
    {
	"GL_TEXTURE_MAG_FILTER", GL_TEXTURE_MAG_FILTER
    },
    {
	"GL_TEXTURE_MIN_FILTER", GL_TEXTURE_MIN_FILTER
    },
    {
	"GL_TEXTURE_WRAP_S", GL_TEXTURE_WRAP_S
    },
    {
	"GL_TEXTURE_WRAP_T", GL_TEXTURE_WRAP_T
    },
    {
	"GL_TRIANGLES", GL_TRIANGLES
    },
    {
	"GL_TRIANGLE_FAN", GL_TRIANGLE_FAN
    },
    {
	"GL_TRIANGLE_STRIP", GL_TRIANGLE_STRIP
    },
    {
	"GL_UNPACK_ALIGNMENT", GL_UNPACK_ALIGNMENT
    },
    {
	"GL_UNSIGNED_BYTE", GL_UNSIGNED_BYTE
    },
    {
	"GL_UNSIGNED_SHORT", GL_UNSIGNED_SHORT
    },
    {
	"GL_UNSIGNED_SHORT_4_4_4_4", GL_UNSIGNED_SHORT_4_4_4_4
    },
    {
	"GL_UNSIGNED_SHORT_5_5_5_1", GL_UNSIGNED_SHORT_5_5_5_1
    },
    {
	"GL_UNSIGNED_SHORT_5_6_5", GL_UNSIGNED_SHORT_5_6_5
    },
    {
	"GL_VENDOR", GL_VENDOR
    },
    {
	"GL_VERSION", GL_VERSION
    },
    {
        "GL_VERTEX_ARRAY", GL_VERTEX_ARRAY
    },
    {
	"GL_XOR", GL_XOR
    },
    {
	"GL_ZERO", GL_ZERO
    },
    {
	"Bad enumeration", GL_NULL
    }
};


void AutoClearColor(GLint color)
{
    glClearColort(colorMap[color][0], colorMap[color][1], colorMap[color][2], ITOF(0));
}

void AutoColor(GLint color)
{
    glColor4t(colorMap[color][0], colorMap[color][1], colorMap[color][2], ITOF(1));
}

long AutoColorCompare(GLftype buf, GLint color)
{

    if (ABS(buf-ITOF(color)) < epsilon.zero) {
	return GL_TRUE;
    } else {
	return GL_FALSE;
    }
}

void ErrorReport(char *fileName, long line, char *errStr)
{
    char buf[80];
    long flag, i, j, k, l;

    Output(1, " failed.\n");

    Output(2, "    File - %s, line - %d.\n", fileName, line);
    if (errStr) {
	i = 0;
	for (;;) {
	    for (;;) {
		if (errStr[i] == 0x0) {
		    flag = GL_TRUE;
		    break;
		} else if (errStr[i] != ' ' && errStr[i] != '\n') {
		    flag = GL_FALSE;
		    break;
		} else {
		    i++;
		}
	    }
	    if (flag == GL_TRUE) {
		break;
	    }
	    j = i;
	    k = i;
	    for (;;) {
		for (;;) {
		    if (errStr[j] == 0x0 || errStr[j] == '\n') {
			flag = GL_TRUE;
			break;
		    } else if (errStr[j] == ' ') {
			flag = GL_FALSE;
			break;
		    } else {
			j++;
		    }
		}
		if (j-i+1 > 80-10) {
		    if (k == i) {
			k = i + 60;
			for (l = 0; l < 80-10; l++, i++) {
			    buf[l] = errStr[i];
			}
			buf[l] = 0;
			Output(2, "        %s-\n", buf);
		    } else {
			for (l = 0; i != k; l++, i++) {
			    buf[l] = errStr[i];
			}
			buf[l] = 0;
			Output(2, "        %s\n", buf);
		    }
		    i = k;
		    break;
		} else {
		    if (flag == GL_TRUE) {
			for (l = 0; i != j; l++, i++) {
			    buf[l] = errStr[i];
			}
			buf[l] = 0;
			Output(2, "        %s\n", buf);
			i = j;
			break;
		    } else {
			k = j;
			j++;
		    }
		}
	    }
	}
    }
    StateReport();
    if (machine.pathLevel != 0) {
	PathReport();
    }
}

/* Varadic printf-style version of ErrorReport */
void ErrorPrintf(char *fileName, long line, char *format, ...)
{
    char errStr[480];

    VA_LIST args;

    VA_START(args, format);
    TVSPRINTF(errStr, format, args);
    VA_END(args);

    ErrorReport(fileName, line, errStr);
}

void GetEnumName(GLenum value, char *str)
{
    long i;

    for (i = 0; enumInfo[i].value != GL_NULL; i++) {
	if (enumInfo[i].value == (long)value) {
	    STRCPY(str, enumInfo[i].name);
	    return;
	}
    }
    StrMake(str, "%d (%s)", value, enumInfo[i].name);
}

long GLErrorReport(void)
{
    char buf[40];
    GLenum error;

    error = glGetError();
    if (error != GL_NO_ERROR) {
	Output(1, " failed.\n");
	GetEnumName(error, buf);
	Output(2, "    GL Error - %s.\n", buf);
	StateReport();
	if (machine.pathLevel != 0) {
	    PathReport();
	}
	while ((error = glGetError()) != GL_NO_ERROR) {
	    GetEnumName(error, buf);
	    Output(2, "    GL Error - %s.\n", buf);
	}
	return ERROR;
    } else {
	return NO_ERROR;
    }
}

void MakeIdentMatrix(GLftype *buf)
{
    long i;

    for (i = 0; i < 16; i++) {
	buf[i] = ITOF(0);
    }
    for (i = 0; i < 4; i++) {
	buf[i*4+i] = ITOF(1);
    }
}

void Ortho2D(GLftype left, GLftype right, GLftype bottom, GLftype top)
{
    GLftype m[4][4], deltaX, deltaY;

    MakeIdentMatrix(&m[0][0]);
    deltaX = right - left;
    deltaY = top - bottom;
    m[0][0] = DIV(ITOF(2), deltaX);
    m[3][0] = DIV(-(right + left), deltaX);
    m[1][1] = DIV(ITOF(2), deltaY);
    m[3][1] = DIV(-(top + bottom), deltaY);
    m[2][2] = ITOF(-1);
    glMatrixMode(GL_PROJECTION);
    glLoadMatrixt(&m[0][0]);
    glMatrixMode(GL_MODELVIEW);
}

void Ortho3D(GLftype left, GLftype right, GLftype bottom, GLftype top,
	     GLftype zNear, GLftype zFar)
{
    GLftype m[4][4], deltaX, deltaY, deltaZ;

    MakeIdentMatrix(&m[0][0]);
    deltaX = right - left;
    deltaY = top - bottom;
    deltaZ = zFar - zNear;
    m[0][0] = DIV(ITOF(2), deltaX);
    m[3][0] = DIV(-(right + left), deltaX);
    m[1][1] = DIV(ITOF(2), deltaY);
    m[3][1] = DIV(-(top + bottom), deltaY);
    m[2][2] = DIV(ITOF(-2), deltaZ);
    m[3][2] = DIV(-(zFar + zNear), deltaZ);
    glMatrixMode(GL_PROJECTION);
    glLoadMatrixt(&m[0][0]);
    glMatrixMode(GL_MODELVIEW);
}

void Output(long level, char *format, ...)
{
    VA_LIST args;

    VA_START(args, format);
    if (machine.verboseLevel >= level) {
        char __output[1024];
        TVSPRINTF(__output, format, args);
	TPRINTF1("%s", __output);
	FFLUSH(stdout);
        if (machine.logFileHandle) {
            FPRINTF1(machine.logFileHandle, "%s", __output);
            FFLUSH(machine.logFileHandle);
        }
    }
    VA_END(args);
}

GLftype OESRandom(GLftype low, GLftype hi)
{
    GLftype x;

    x = DIV(ITOF(RAND()), ITOF(RANDOM_MAX));
    return (MUL(ITOF(1) - x, low) + MUL(x, hi));
}

#if 1

/*
** Gain back accuracy lost through conversion.
*/
#define UBTOF(c, max) DIV(ITOF((int)((2*c*max + 255) / (2*255))), ITOF(max))

void ReadScreen(GLint x, GLint y, GLsizei w, GLsizei h, GLenum type,
                GLftype *buf)
{
    GLubyte *tmpBuf;
    GLftype sum;
    long repeat, i, j, max[4];

    switch (type) {
    default:
	repeat = 0;
	break;

    case GL_AUTO_COLOR:
    case GL_ALPHA:
    case GL_LUMINANCE:
	repeat = 1;
	break;
    case GL_LUMINANCE_ALPHA:
	repeat = 2;
	break;
    case GL_RGB:
	repeat = 3;
	break;
    case GL_RGBA:
	repeat = 4;
	break;
    }

    for (i = 0; i < w*h*repeat; i++) {
	buf[i] = 0;
    }

    if (machine.failMode == GL_TRUE) {
	return;
    }

    tmpBuf = (GLubyte *)MALLOC(w*h*4*sizeof(GLubyte));
    glReadPixels(x, y, w, h, GL_RGBA, GL_UNSIGNED_BYTE, tmpBuf);

    for (i = 0; i < 4; i++) {
	if (buffer.colorBits[i] == 0) {
	    max[i] = 255;
	} else {
            max[i] = (1 << buffer.colorBits[i]) - 1;
	}
    }

    switch (type) {
    case GL_AUTO_COLOR:
	for (i = 0; i < w*h; i++) {
	    for (j = 0; j < 4; j++) {
		if (ABS(UBTOF(tmpBuf[i*4 + 0], max[0])-colorMap[j][0]) < epsilon.zero &&
		    ABS(UBTOF(tmpBuf[i*4 + 1], max[1])-colorMap[j][1]) < epsilon.zero &&
		    ABS(UBTOF(tmpBuf[i*4 + 2], max[2])-colorMap[j][2]) < epsilon.zero)
		{
		    buf[i] = ITOF(j);
		    break;
		}
	    }
	}
        break;

    case GL_ALPHA:
        for (i = 0; i != w*h; i++) {
	    buf[i] = UBTOF(tmpBuf[i*4 + 3], max[3]);
        }
        break;

    case GL_LUMINANCE:
        for (i = 0; i != w*h; i++) {
            sum = 0;
	    sum += UBTOF(tmpBuf[i*4 + 0], max[0]);
	    sum += UBTOF(tmpBuf[i*4 + 1], max[1]);
	    sum += UBTOF(tmpBuf[i*4 + 2], max[2]);
	    buf[i] = sum / 3.0;
        }
        break;

    case GL_LUMINANCE_ALPHA:
        for (i = 0; i < w*h; i++) {
            sum = 0;
	    sum += UBTOF(tmpBuf[i*4 + 0], max[0]);
	    sum += UBTOF(tmpBuf[i*4 + 1], max[1]);
	    sum += UBTOF(tmpBuf[i*4 + 2], max[2]);
	    buf[i*2 + 0] = sum / 3.0;
            buf[i*2 + 1] = UBTOF(tmpBuf[i*4 + 3], max[3]);
        }
        break;

    case GL_RGB:
        for (i = 0; i < w*h; i++) {
	    buf[i*3 + 0] = UBTOF(tmpBuf[i*4 + 0], max[0]);
	    buf[i*3 + 1] = UBTOF(tmpBuf[i*4 + 1], max[1]);
	    buf[i*3 + 2] = UBTOF(tmpBuf[i*4 + 2], max[2]);
        }
        break;

    case GL_RGBA:
        for (i = 0; i < w*h; i++) {
            for (j = 0; j < 4; j++) {
		buf[i*4 + j] = UBTOF(tmpBuf[i*4 + j], max[j]);
	    }
        }
        break;
    }   

    FREE(tmpBuf);
}

#else

void ReadScreen(GLint x, GLint y, GLsizei w, GLsizei h, GLenum type,
		GLfloat *buf)
{
    long repeat = 1, i, j;

    switch (type) {
      case GL_AUTO_COLOR:
      case GL_ALPHA:
      case GL_LUMINANCE:
	repeat = 1;
	break;
      case GL_LUMINANCE_ALPHA:
	repeat = 2;
	break;
      case GL_RGB:
	repeat = 3;
	break;
      case GL_RGBA:
	repeat = 4;
	break;
    }

    for (i = 0; i < w*h*repeat; i++) {
	buf[i] = 0.0;
    }

    if (machine.failMode == GL_TRUE) {
	return;
    }

    if (type == GL_AUTO_COLOR) {
	GLfloat *tmpBuf = (GLfloat *)MALLOC(w*h*3*sizeof(GLfloat));
	GLfloat *ptr = buf;

	glReadPixels(x, y, w, h, GL_RGB, GL_FLOAT, (GLvoid *)tmpBuf);
	for (i = 0; i < w*h*3; i += 3) {
	    for (j = 0; j < 4; j++) {
		if (ABS(tmpBuf[i]-colorMap[j][0]) < epsilon.zero &&
		    ABS(tmpBuf[i+1]-colorMap[j][1]) < epsilon.zero &&
		    ABS(tmpBuf[i+2]-colorMap[j][2]) < epsilon.zero) {
		    *ptr = (GLfloat)j;
		    break;
		}
	    }
	    ptr++;
	}
	FREE(tmpBuf);
    } else {
	glReadPixels(x, y, w, h, type, GL_FLOAT, (GLvoid *)buf);
    }
}

#endif

void ResetMatrix(void)
{
    GLftype buf[16];

    MakeIdentMatrix(buf);
    glMatrixMode(GL_MODELVIEW);
    glLoadMatrixt(buf);
    glMatrixMode(GL_PROJECTION);
    glLoadMatrixt(buf);
    glMatrixMode(GL_TEXTURE);
    glLoadMatrixt(buf);
}

long Round(ftype x)
{

    return FTOI(x + ATOF("0.5"));
}

void StrMake(char *str, char const *format, ...)
{
    VA_LIST args;

    VA_START(args, format);
    TVSPRINTF(str, format, args);
    VA_END(args);
}

ftype CalcEpsilon(long bits)
{
    ftype e;

    if (bits == 0) {
	e = epsilon.zero;
#ifdef COMMON_LITE
    } else if (bits >= 16) {
        e = FIXED_EPSILON + epsilon.zero;
#endif
    } else {
        e = DIV(ITOF(1), LDEXP(ITOF(1), bits) - ITOF(1)) + epsilon.zero;
	if (e > ITOF(1)) {
	    e = ITOF(1);
	}
    }
    return e;
}

static void Fill(GLsizei r, GLsizei g, GLsizei b, GLsizei a, GLsizei bits[4])
{
    bits[0] = r;
    bits[1] = g;
    bits[2] = b;
    bits[3] = a;
}

void GetComponentBits(GLenum format, GLenum type, GLsizei bits[4])
{
    switch (type) {
    case GL_UNSIGNED_BYTE:
	{
		switch (format) {
		default:
			Fill(0, 0, 0, 0, bits);
			break;
		case GL_ALPHA:
			Fill(0, 0, 0, 8, bits);
			break;
		case GL_RGB:
		case GL_LUMINANCE:
			Fill(8, 8, 8, 0, bits);
			break;

		case GL_RGBA:
		case GL_LUMINANCE_ALPHA:
			Fill(8, 8, 8, 8, bits);
			break;
        }
		break;
        }
    case GL_UNSIGNED_SHORT_5_6_5:
        Fill(5, 6, 5, 0, bits);
        break;

    case GL_UNSIGNED_SHORT_4_4_4_4:
	case GL_UNSIGNED_SHORT_4_4_4_4_REV_EXT:
        Fill(4, 4, 4, 4, bits);
        break;

    case GL_UNSIGNED_SHORT_5_5_5_1:
	case GL_UNSIGNED_SHORT_1_5_5_5_REV_EXT:
        Fill(5, 5, 5, 1, bits);
        break;
    }
}

void GetCompressedComponentBits(GLenum compressedFormat, GLsizei bits[4])
{
    switch (compressedFormat) {
    default:
        Fill(0, 0, 0, 0, bits);
        break;
    
    case GL_PALETTE4_R5_G6_B5_OES:
    case GL_PALETTE8_R5_G6_B5_OES:
        Fill(5, 6, 5, 0, bits);
        break;
    
    case GL_PALETTE4_RGB8_OES:
    case GL_PALETTE8_RGB8_OES:
        Fill(8, 8, 8, 0, bits);
        break;
        
    case GL_PALETTE4_RGBA4_OES:
    case GL_PALETTE8_RGBA4_OES:
        Fill(4, 4, 4, 4, bits);
        break;

    case GL_PALETTE4_RGB5_A1_OES:
    case GL_PALETTE8_RGB5_A1_OES:
        Fill(5, 5, 5, 1, bits);
        break;

    case GL_PALETTE4_RGBA8_OES:
    case GL_PALETTE8_RGBA8_OES:
        Fill(8, 8, 8, 8, bits);
        break;
    }
}

/* Calculate error epsilons to the least accurate of either
** frame buffer or texture precision. RGBA epsilons are
** returned in textureEpsilon[]. target must be a valid
** texture target.
*/
void CalcTextureEpsilon(const GLsizei textureBits[4], GLftype textureEpsilon[4])
{
    GLint i, bits;

    for (i = 0; i < 4; i++) {
        if (textureBits[i] == 0) {
	    /* 'exact' */
	    bits = 1000;
	} else {
	    bits = textureBits[i];
	}

	/* internal texture bits are 4444 */
	if (bits > 4) {
	    bits = 4;
	}

	/* frame buffer bits */
	if (bits > buffer.colorBits[i]) {
	    bits = buffer.colorBits[i];
	}

	if (bits == 0) {
	    /* infinity */
	    textureEpsilon[i] = ITOF(2);
	} else {
	    textureEpsilon[i] = CalcEpsilon(bits);
	}
    }
}

/* This function is necessary, as sometimes epsilons are treated as an 
 * array of 4 elements R, G, B, A, and sometimes as an array of
 * numcomponents elements, dependenent on format. In this case the epsilons
 * must be swizzled, so that eg Alpha appears in the first element of the array
 */
void SwizzleTextureEpsilon(GLenum format, GLenum type, GLftype texEpsilon[4])
{
	switch(type) {
    case GL_UNSIGNED_BYTE:
	{
		switch (format) {
		case GL_ALPHA:
			/* Alpha goes to component 0, zero other components */
			texEpsilon[0] = texEpsilon[3];
			texEpsilon[1] = texEpsilon[2] = texEpsilon[3] = epsilon.zero;
			break;
		case GL_LUMINANCE:
			/* Luminance is already in component 0, zero other components */
			texEpsilon[1] = texEpsilon[2] = texEpsilon[3] = epsilon.zero;
			break;
		case GL_LUMINANCE_ALPHA:
			/* Luminance is already in component 0, Alpha goes to component 1, zero other components */
			texEpsilon[1] = texEpsilon[3];
			texEpsilon[2] = texEpsilon[3] = epsilon.zero;
			break;

		/* No swizzling necessary */ 
		default:
		case GL_RGBA:
		case GL_RGB:
			break;
		}
		break;
	}
	/* No swizzling necessary */ 
    case GL_UNSIGNED_SHORT_5_6_5:
    case GL_UNSIGNED_SHORT_4_4_4_4:
    case GL_UNSIGNED_SHORT_5_5_5_1:
	default:
		break;
    }
}


/* Draws a plane with luminance value and depth z. */
void DrawPlane(GLftype value, GLclampt z)
{
    GLftype vertices[4][4] = {
        {ITOF(0), ITOF(0), ITOF(0), ITOF(1)},
        {ITOF(WINDSIZEX), ITOF(0), ITOF(0), ITOF(1)},
        {ITOF(0), ITOF(WINDSIZEY), ITOF(0), ITOF(1)},
        {ITOF(WINDSIZEX), ITOF(WINDSIZEY), ITOF(0), ITOF(1)}
    };

    vertices[0][2] =
    vertices[1][2] =
    vertices[2][2] =
    vertices[3][2] = z;

    glColor4t(value, value, value, ITOF(1));

    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(4, GL_FTYPE, 0, &vertices[0][0]);
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
    glDisableClientState(GL_VERTEX_ARRAY);
}

void DrawPoint2f(GLftype x, GLftype y)
{
    GLftype vertex[2];
    vertex[0] = x;
    vertex[1] = y;

    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(2, GL_FTYPE, 0, &vertex[0]);
    glDrawArrays(GL_POINTS, 0, 1);
    glDisableClientState(GL_VERTEX_ARRAY);
}

void DrawPoint4fv(const GLftype vertex[4])
{
    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(4, GL_FTYPE, 0, &vertex[0]);
    glDrawArrays(GL_POINTS, 0, 1);
    glDisableClientState(GL_VERTEX_ARRAY);
}

void DrawQuad2f(GLftype x1, GLftype y1, GLftype x2, GLftype y2)
{
    GLftype vertices[4][2];
    vertices[0][0] = x1; vertices[0][1] = y1;
    vertices[1][0] = x2; vertices[1][1] = y1;
    vertices[2][0] = x1; vertices[2][1] = y2;
    vertices[3][0] = x2; vertices[3][1] = y2;

    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(2, GL_FTYPE, 0, &vertices[0][0]);
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
    glDisableClientState(GL_VERTEX_ARRAY);
}

void DrawQuad2i(GLint x1, GLint y1, GLint x2, GLint y2)
{
    GLshort vertices[4][2];
    vertices[0][0] = (GLshort)x1; vertices[0][1] = (GLshort)y1;
    vertices[1][0] = (GLshort)x2; vertices[1][1] = (GLshort)y1;
    vertices[2][0] = (GLshort)x1; vertices[2][1] = (GLshort)y2;
    vertices[3][0] = (GLshort)x2; vertices[3][1] = (GLshort)y2;

    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(2, GL_SHORT, 0, &vertices[0][0]);
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
    glDisableClientState(GL_VERTEX_ARRAY);
}

void DrawTexQuad2i2f(GLint x1, GLint y1, GLint x2, GLint y2,
    GLftype s1, GLftype t1, GLftype s2, GLftype t2)
{
    GLshort vertices[4][2];
    GLftype tex_coords[4][2];

    vertices[0][0] = (GLshort)x1; vertices[0][1] = (GLshort)y1;
    vertices[1][0] = (GLshort)x2; vertices[1][1] = (GLshort)y1;
    vertices[2][0] = (GLshort)x1; vertices[2][1] = (GLshort)y2;
    vertices[3][0] = (GLshort)x2; vertices[3][1] = (GLshort)y2;

    tex_coords[0][0] = s1; tex_coords[0][1] = t1;
    tex_coords[1][0] = s2; tex_coords[1][1] = t1;
    tex_coords[2][0] = s1; tex_coords[2][1] = t2;
    tex_coords[3][0] = s2; tex_coords[3][1] = t2;

    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(2, GL_SHORT, 0, &vertices[0][0]);

    glEnableClientState(GL_TEXTURE_COORD_ARRAY);
    glTexCoordPointer(2, GL_FTYPE, 0, &tex_coords[0][0]);

    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
    glDisableClientState(GL_VERTEX_ARRAY);
}

void DrawPixels(GLint x, GLint y, GLsizei width, GLsizei height, GLenum format, GLenum type, GLvoid *pixels)
{
    GLuint texture;

    Ortho2D(ITOF(0), ITOF(WINDSIZEX), ITOF(0), ITOF(WINDSIZEY));

    glGenTextures(1, &texture);
    glBindTexture(GL_TEXTURE_2D, texture);
    glTexImage2D(GL_TEXTURE_2D, 0, format, width, height, 0, format, type, pixels);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);

    glEnable(GL_TEXTURE_2D);
    glColor4t(ITOF(1), ITOF(1), ITOF(1), ITOF(1));
    DrawTexQuad2i2f(x, y, width, height, ITOF(0), ITOF(0), ITOF(1), ITOF(1));
    glDisable(GL_TEXTURE_2D);

    glDeleteTextures(1, &texture);
}

GLenum GuessColorBufferFormat(void)
{
    if (buffer.colorBits[0] == 0) {
        return GL_ALPHA;
    } else {
        if (buffer.colorBits[1] == 0 || buffer.colorBits[2] == 0) {
            if (buffer.colorBits[3] == 0) {
                return GL_LUMINANCE;
            } else {
                return GL_LUMINANCE_ALPHA;
            }
        } else {
            if (buffer.colorBits[3] == 0) {
                return GL_RGB;
            } else {
                return GL_RGBA;
            }
        }
    }
}

GLsizei CalcCopyTextureTargetFormats(GLenum colorBufferFormat, GLenum *textureFormats)
{
    GLsizei i;

    i = 0;
    switch (colorBufferFormat) {
    case GL_ALPHA:
        textureFormats[i++] = GL_ALPHA;
        break;

    case GL_LUMINANCE:
        textureFormats[i++] = GL_LUMINANCE;
        break;

    case GL_LUMINANCE_ALPHA:
        textureFormats[i++] = GL_LUMINANCE_ALPHA;
        textureFormats[i++] = GL_LUMINANCE;
        textureFormats[i++] = GL_ALPHA;
        break;

    case GL_RGB:
        textureFormats[i++] = GL_RGB;
        textureFormats[i++] = GL_LUMINANCE;
        break;

    case GL_RGBA:
        textureFormats[i++] = GL_RGBA;
        textureFormats[i++] = GL_RGB;
        textureFormats[i++] = GL_LUMINANCE_ALPHA;
        textureFormats[i++] = GL_LUMINANCE;
        textureFormats[i++] = GL_ALPHA;
        break;
    }

    return i;
}
