/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conformshell/utilg.c,v 1.3 2005/05/11 15:21:43 artlab Exp $
*/

#include "conform.h"
#include "util.h"
#include "utilg.h"


void GetSlope(slopeRec *s, vertexRec *v0, vertexRec *v1)
{

    s->dx = v1->x - v0->x;
    s->dy = v1->y - v0->y;
}

void GetIntercept(interceptRec *i, vertexRec *v, slopeRec *s)
{

    if (s->dx != ITOF(0)) {
	i->b = v->y - DIV(MUL(v->x, s->dy), s->dx);
	i->bDx = MUL(v->y, s->dx) - MUL(v->x, s->dy);
    } else {
	i->b = ITOF(1);
	i->bDx = MUL(v->y, s->dx) - MUL(v->x, s->dy);
    }
}

long LessThan(ftype x, ftype y, slopeRec *s, interceptRec *i)
{

    return (MUL(y, s->dx) <= MUL(x, s->dy) + i->bDx);
}

long MoreThan(ftype x, ftype y, slopeRec *s, interceptRec *i)
{

    return (MUL(y, s->dx) >= MUL(x, s->dy) + i->bDx);
}

void SetVertex(vertexRec *v, ftype x, ftype y, ftype z)
{

    v->x = x;
    v->y = y;
    v->z = z;
}

void SetHVertex(hVertexRec *v, ftype x, ftype y, ftype z, ftype w)
{

    v->v.x = x;
    v->v.y = y;
    v->v.z = z;

    v->hx = MUL(x, w);
    v->hy = MUL(y, w);
    v->hz = MUL(z, w);
    v->hw = w;
}

void MakeDiag(diagEqnRec *d, vertexRec *v0, vertexRec *v1, vertexRec *vOp)
{

    GetSlope(&d->slope, v0, v1);
    GetIntercept(&d->intercept, v0, &d->slope);

    if (LessThan(vOp->x, vOp->y, &d->slope, &d->intercept)) {
	d->Inequality = LessThan;
    } else {
	d->Inequality = MoreThan;
    }
}

ftype SideLength(vertexRec *v0, vertexRec *v1)
{

    return SQRT(
       MUL((v0->x-v1->x), (v0->x-v1->x)) +
       MUL((v0->y-v1->y), (v0->y-v1->y)));
}

ftype TriangleArea(vertexRec *v0, vertexRec *v1, vertexRec *v2)
{
    ftype length0, length1, length2, s;

    length0 = SideLength(v0, v1);
    length1 = SideLength(v1, v2);
    length2 = SideLength(v0, v2);

    s = DIV(length0 + length1 + length2, ITOF(2));
    s = MUL(MUL(MUL(s, s - length0), s - length1), s - length2);

    /*
    ** Remember the triangle inequality is false for floating point numbers.
    ** This kills the possibility of dealing with anti-aliasing and
    ** means tnat only very simple primitives may be tested.
    */
    if (s >= ITOF(0)) {
	return SQRT(s);
    } else {
	return ITOF(0);
    }
}

void ArrangeComponent(ftype max[4], ftype mid[4], ftype min[4],
		      ftype oneShade[3])
{
    GLint redBits, greenBits, blueBits, maxBits, minBits, midBits;
    GLint maxChannel, minChannel, midChannel, tmp;

    glGetIntegerv(GL_RED_BITS, &redBits);
    glGetIntegerv(GL_GREEN_BITS, &greenBits);
    glGetIntegerv(GL_BLUE_BITS, &blueBits);

    /*
    ** Sort without subroutines.
    */
    if (redBits > greenBits) {
	maxBits = redBits;
	minBits = greenBits;
	maxChannel = 0;
	minChannel = 1;
    } else {
	maxBits = greenBits;
	minBits = redBits;
	maxChannel = 1;
	minChannel = 0;
    }

    midBits = blueBits;
    midChannel = 2;

    if (maxBits < midBits) {
	tmp = maxBits;
	maxBits = midBits;
	midBits = tmp;

	tmp = maxChannel;
	maxChannel = midChannel;
	midChannel = tmp;
    }

    if (minBits > midBits) {
	tmp = midBits;
	midBits = minBits;
	minBits = tmp;

	tmp = midChannel;
	midChannel = minChannel;
	midChannel = tmp;
    }

    max[0] = ITOF(0);
    max[1] = ITOF(0);
    max[2] = ITOF(0);
    max[3] = ITOF(1);
    max[maxChannel] = ITOF(1);

    mid[0] = ITOF(0);
    mid[1] = ITOF(0);
    mid[2] = ITOF(0);
    mid[3] = ITOF(1);
    mid[midChannel] = ITOF(1);

    min[0] = ITOF(0);
    min[1] = ITOF(0);
    min[2] = ITOF(0);
    min[3] = ITOF(1);
    min[minChannel] = ITOF(1);

    oneShade[0]  = DIV(ITOF(1), (LDEXP(ITOF(1), maxBits) - ITOF(1)));
    oneShade[1]  = DIV(ITOF(1), (LDEXP(ITOF(1), midBits) - ITOF(1)));
    oneShade[2]  = DIV(ITOF(1), (LDEXP(ITOF(1), minBits) - ITOF(1)));
}
