/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conformshell/utilm.c,v 1.4 2005/05/11 15:21:43 artlab Exp $
*/

#include "conform.h"
#include "util.h"
#include "utilm.h"


static long Compare(GLint x, GLint y, GLint color, GLftype *buf)
{
    long i;

    i = WINDSIZEX * y + x;
    return AutoColorCompare(buf[i], color);
}

long XFormTestBuf(GLftype *buf, GLftype *point, GLint index)
{
    GLint i = 0, j = 0;

    i = FTOI((ITOF(1) + point[0]) * WINDSIZEX / 2);
    j = FTOI((ITOF(1) + point[1]) * WINDSIZEY / 2);

    /*
    **  Test the 9 pixels around the target.
    */
    if (!(Compare(i-1, j+1, index, buf) ||
          Compare(i  , j+1, index, buf) ||
          Compare(i+1, j+1, index, buf) ||
          Compare(i-1, j  , index, buf) ||
          Compare(i  , j  , index, buf) ||
          Compare(i+1, j  , index, buf) ||
          Compare(i-1, j-1, index, buf) ||
          Compare(i  , j-1, index, buf) ||
          Compare(i+1, j-1, index, buf))) {
        return ERROR;
    }
    return NO_ERROR;
}

static void Translate(xFormRec *data, GLftype *in, GLftype *out)
{

    out[0] = ADD(in[0], data->x);
    out[1] = ADD(in[1], data->y);
    out[2] = ADD(in[2], data->z);
}

static void Scale(xFormRec *data, GLftype *in, GLftype *out)
{

    out[0] = MUL(in[0], data->x);
    out[1] = MUL(in[1], data->y);
    out[2] = MUL(in[2], data->z);
}

static void Rotate(xFormRec *data, GLftype *in, GLftype *out)
{
    ftype angleCos, angleSin;
    ftype m[3][3], axis[3], len, tmp[3];

    angleCos = COS(DIV(MUL(data->angle, PI), ITOF(180)));
    angleSin = SIN(DIV(MUL(data->angle, PI), ITOF(180)));

    len = MUL(data->x, data->x) + MUL(data->y, data->y) + MUL(data->z, data->z);
    len = SQRT(len);

    axis[0] = DIV(data->x, len);
    axis[1] = DIV(data->y, len);
    axis[2] = DIV(data->z, len);

    m[0][0] = MUL(MUL(axis[0], axis[0]), ITOF(1) - angleCos) + angleCos;
    m[0][1] = MUL(MUL(axis[0], axis[1]), ITOF(1) - angleCos) - MUL(angleSin, axis[2]); 
    m[0][2] = MUL(MUL(axis[0], axis[2]), ITOF(1) - angleCos) + MUL(angleSin, axis[1]);

    m[1][0] = MUL(MUL(axis[1], axis[0]), ITOF(1) - angleCos) + MUL(angleSin, axis[2]);
    m[1][1] = MUL(MUL(axis[1], axis[1]), ITOF(1) - angleCos) + angleCos;
    m[1][2] = MUL(MUL(axis[1], axis[2]), ITOF(1) - angleCos) - MUL(angleSin, axis[0]);

    m[2][0] = MUL(MUL(axis[2], axis[0]), ITOF(1) - angleCos) - MUL(angleSin, axis[1]);
    m[2][1] = MUL(MUL(axis[2], axis[1]), ITOF(1) - angleCos) + MUL(angleSin, axis[0]);
    m[2][2] = MUL(MUL(axis[2], axis[2]), ITOF(1) - angleCos) + angleCos;

    tmp[0] = MUL(m[0][0], in[0]) + MUL(m[0][1], in[1]) + MUL(m[0][2], in[2]);
    tmp[1] = MUL(m[1][0], in[0]) + MUL(m[1][1], in[1]) + MUL(m[1][2], in[2]);
    tmp[2] = MUL(m[2][0], in[0]) + MUL(m[2][1], in[1]) + MUL(m[2][2], in[2]);

    out[0] = tmp[0];
    out[1] = tmp[1];
    out[2] = tmp[2];
}

static void Ortho(xFormRec *data, GLftype *in, GLftype *out)
{

    out[0] = DIV(MUL(in[0], ITOF(2)), data->d1);
    out[1] = DIV(MUL(in[1], ITOF(2)), data->d2);
    out[2] = DIV(MUL(in[2], ITOF(-2)), data->d3);
}

static void Frustum(xFormRec *data, GLftype *in, GLftype *out)
{

    out[0] = MUL(in[0], DIV(2*data->d5, data->d2 - data->d1))
	   + MUL(in[2], DIV(data->d2 + data->d1, data->d2 - data->d1));
    out[1] = MUL(in[1], DIV(2*data->d5, data->d4 - data->d3))
	   + MUL(in[2], DIV(data->d4 + data->d3, data->d4 - data->d3));
    out[2] = MUL(in[3], DIV(MUL(-2*data->d6, data->d5), data->d6 - data->d5))
	   + MUL(in[2], DIV(-(data->d6 + data->d5), data->d6 - data->d5));
    out[3] = -in[2];

    out[0] = DIV(out[0], out[3]);
    out[1] = DIV(out[1], out[3]);
    out[2] = DIV(out[2], out[3]);
    out[3] = ITOF(1);
}

void XFormCalcTrue(long type, xFormRec *data, GLftype *in, GLftype *out)
{

    switch (type) {
      case XFORM_TRANSLATE:
	Translate(data, in, out);
	break;
      case XFORM_SCALE:
	Scale(data, in, out);
	break;
      case XFORM_ROTATE:
	Rotate(data, in, out);
	break;
      case XFORM_ORTHO:
	Ortho(data, in, out);
	break;
      case XFORM_FRUSTUM:
	Frustum(data, in, out);
	break;
    }
}

void XFormCalcGL(long type, xFormRec *data)
{

    switch (type) {
      case XFORM_TRANSLATE:
	glTranslatet(data->x, data->y, data->z);
	break;
      case XFORM_SCALE:
	glScalet(data->x, data->y, data->z);
	break;
      case XFORM_ROTATE:
	glRotatet(data->angle, data->x, data->y, data->z);
	break;
      case XFORM_ORTHO:
	glOrthot(-data->d1/2, data->d1/2, -data->d2/2, data->d2/2,
		-data->d3/2, data->d3/2);
	break;
      case XFORM_FRUSTUM:
	glFrustumt(data->d1, data->d2, data->d3, data->d4, data->d5, data->d6);
	break;
    }
}

void XFormMake(long *type, xFormRec *data)
{

    *type = FTOI(OESRandom(ITOF(XFORM_TRANSLATE), ITOF(XFORM_FRUSTUM)+ATOF("0.9")));
    switch (*type) {
      case XFORM_TRANSLATE:
	data->x = OESRandom(ATOF("-0.75"), ATOF("0.75"));
	data->y = OESRandom(ATOF("-0.75"), ATOF("0.75"));
	data->z = ITOF(0);
	break;
      case XFORM_SCALE:
	data->x = OESRandom(ITOF(-4), ITOF(4));
	data->y = OESRandom(ITOF(-4), ITOF(4));
	data->z = ITOF(1);
	break;
      case XFORM_ROTATE:
	data->x = OESRandom(ITOF(-2), ITOF(2));
	data->y = OESRandom(ITOF(-2), ITOF(2));
	data->z = OESRandom(ITOF(-2), ITOF(2));
	data->angle = OESRandom(ITOF(-180), ITOF(180));
	break;
      case XFORM_ORTHO:
        data->d1 = OESRandom(ITOF(2), ITOF(8));
        data->d2 = OESRandom(ITOF(2), ITOF(8)); 
        data->d3 = OESRandom(ITOF(8), ITOF(32));
	break;
      case XFORM_FRUSTUM:
	data->d1 = OESRandom(ITOF(-1), ITOF(-4));
	data->d2 = OESRandom(ITOF(+1), ITOF(+4));
	data->d3 = OESRandom(ITOF(-1), ITOF(-4));
	data->d4 = OESRandom(ITOF(+1), ITOF(+4));
	data->d5 = OESRandom(ATOF("0.5"), ATOF("1.5"));
	data->d6 = OESRandom(ITOF(5), ITOF(7));
	break;
    }
}

void XFormMakeMatrix(long type, xFormRec *data, GLftype *matrix)
{
    GLftype sine, cosine;

    MakeIdentMatrix(matrix);

    /*
    ** Note - LoadMatrix takes matrices in COLUMN major order.
    ** These matrices will be expressed as ROW major, so the numbers
    ** will appear transposed.
    */
    switch (type) {
      case XFORM_TRANSLATE:
	matrix[12] = data->x;
	matrix[13] = data->y;
	matrix[14] = data->z;
	break;
      case XFORM_SCALE:
	matrix[0] = data->x;
	matrix[5] = data->y;
	matrix[10] = data->z;
	break;
      case XFORM_ROTATE:
	/*
	** Doing only rotations around z-axis.
	*/
	cosine = COS(DIV(MUL(data->angle, PI), ITOF(180)));
	sine = SIN(DIV(MUL(data->angle, PI), ITOF(180)));
	matrix[0] = cosine;
	matrix[1] = sine;
	matrix[4] = -sine;
	matrix[5] = cosine;
	break;
      case XFORM_ORTHO:
	matrix[0] = DIV(ITOF(2), data->d1);
	matrix[5] = DIV(ITOF(2), data->d2);
	matrix[10] = DIV(ITOF(-2), data->d3);
	break;
      case XFORM_FRUSTUM:
	matrix[0] = DIV(2 * data->d5, data->d2 - data->d1);
	matrix[5] = DIV(2 * data->d5, data->d4 - data->d3);
	matrix[10] = DIV(-(data->d6 + data->d5), data->d6 - data->d5);
	matrix[8] = DIV(data->d2 + data->d1, data->d2 - data->d1);
	matrix[9] = DIV(data->d4 + data->d3, data->d4 - data->d3);
	matrix[14] = DIV(-MUL(2 * data->d5, data->d6), data->d6 - data->d5);
	matrix[11] = ITOF(-1);
	matrix[15] = ITOF(0);
	break;
    }
    return;
}

long XFormValid(long type, xFormRec *data)
{

    switch (type) {
      case XFORM_TRANSLATE:
	return NO_ERROR;
      case XFORM_SCALE:
	if (MUL(MUL(data->x, data->y), data->z) == ITOF(0)) {
	    return ERROR;
	}
	return NO_ERROR;
      case XFORM_ROTATE:
	if (MUL(data->x, data->x) + MUL(data->y, data->y) + MUL(data->z, data->z) == ITOF(0)) {
	    return ERROR;
	}
	return NO_ERROR;
      case XFORM_ORTHO:
	if (MUL(MUL(data->d1, data->d2), data->d3) == ITOF(0)) {
	    return ERROR;
	}
	return NO_ERROR;
      case XFORM_FRUSTUM:
	if (data->d1 == data->d2 || data->d3 == data->d4 ||
	    data->d5 <= 0 || data->d5 >= data->d6)
	{
	    return ERROR;
	}
	return NO_ERROR;
    }
    return NO_ERROR;
}

long XFormCompareMatrix(GLftype *m1, GLftype *m2)
{
    long i;

    for (i = 0; i < 16; i++) {
	 if (ABS(m1[i]-m2[i]) > epsilon.zero) {
	     return ERROR;
         }
    }
    return NO_ERROR;
}

void XFormMul4x4(GLftype *m, GLftype *in, GLftype *out)
{
    GLftype cp[4];
    int     i, j;

    cp[0] = in[0];
    cp[1] = in[1];
    cp[2] = in[2];
    cp[3] = in[3];

    for(i = 0; i < 4; ++i)  {
        out[i] = 0;
        for(j = 0; j < 4; ++j)
            out[i] += MUL(m[j*4 + i], cp[j]);
    }
}

void XFormMul3x3(GLftype *m, GLftype *in, GLftype *out)
{
    GLftype cp[3];
    int     i, j;

    cp[0] = in[0];
    cp[1] = in[1];
    cp[2] = in[2];

    for(i = 0; i < 3; ++i)  {
        out[i] = 0;
        for(j = 0; j < 3; ++j)
            out[i] += MUL(m[j*4 + i], cp[j]);
    }
}
