/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conformshell/utilp.c,v 1.3 2005/05/11 15:21:43 artlab Exp $
*/

#include "conform.h"
#include "util.h"
#include "utilp.h"

static const packPixRec packedPixelInfo[] = {
  { GL_UNSIGNED_SHORT_5_6_5, 3, sizeof(GLushort),
    {	5,  6,	5,  0 },
    {  11,  5,	0,  0 } },
  { GL_UNSIGNED_SHORT_4_4_4_4, 4, sizeof(GLushort),
    {	4,  4,	4,  4 },
    {  12,  8,	4,  0 } },
  { GL_UNSIGNED_SHORT_5_5_5_1, 4, sizeof(GLushort),
    {	5,  5,	5,  1 },
    {  11,  6,	1,  0 } },
  { GL_UNSIGNED_SHORT_4_4_4_4_REV_EXT, 4, sizeof(GLushort),
    {	4,  4,	4,  4 },
    {   8,  4,	0, 12 } },
  { GL_UNSIGNED_SHORT_1_5_5_5_REV_EXT, 4, sizeof(GLushort),
    {	5,  5,	5,  1 },
    {   10,  5,	0, 15 } },
};

static const GLsizei packedPixelInfo_count =
    sizeof(packedPixelInfo) / sizeof(packedPixelInfo[0]);

/* Bitmasks for components of sizes 0-10 bits */
static const GLuint componentMask[] = {
    0x000, 0x001, 0x003, 0x007, 0x00F, 0x01F, 0x03F, 0x07F, 0x0FF, 0x1FF, 0x3FF
};

/* Maximum values of unsigned bitfields of sizes 0-10 bits */
static const GLuint componentMaxValue[] = {
    0, 1, 3, 7, 15, 31, 63, 127, 255, 511, 1023
};

/* Return record describing the packed pixel format 'type' in detail */
const packPixRec *packPixInfo(GLenum type) {
    long i;

    for (i = 0; i != packedPixelInfo_count; i++) {
	if (packedPixelInfo[i].type == type)
	    return &packedPixelInfo[i];
    }
    return NULL;
}

/*
** Convert 4-component float array 'pixel' to a packed pixel described by
** 'format'. For each component:
**  Scale from float to unsigned with appropriate number of bits.
**  Mask and shift to appropriate offset in the packed pixel.
**  Bitwise or packed component into the final packed result.
*/
GLuint ConvertRGBAtoPacked(GLftype const *pixel, packPixRec const *format)
{
    GLuint numbits, ipix, result;
    long i;

    result = 0;
    for (i = 0; i < 4; i++) {
	numbits = format->size[i];
	ipix = (GLuint)FTOI(MUL(pixel[i], ITOF(componentMaxValue[numbits])));
	result |= ((ipix & componentMask[numbits]) << format->shift[i]);
    }

    return result;
}

/*
** Convert RGB components of the packed pixel described by 'format'
** in location pointed to by 'pixel' into floats in array 'out'.
** For each component:
**  Mask and shift packed pixel to extract component as unsigned with
**	appropriate number of bits.
**  Scale to float.
*/
void ConvertPackedtoRGBA(GLubyte const *pixel, GLftype *out, packPixRec const *format)
{
    GLuint numbits, ipix;
    long i;

    switch (format->datasize) {
      case sizeof(GLubyte):
	for (i = 0; i < 4; i++) {
	    numbits = format->size[i];
	    ipix = (*pixel >> format->shift[i]) & componentMask[numbits];
	    if (numbits == 0) {
		out[i] = (i == 3) ? ITOF(1) : ITOF(0);
	    } else {
	        out[i] = DIV(ITOF(ipix), ITOF(componentMaxValue[numbits]));
	    }
	}
	break;
      case sizeof(GLushort):
	for (i = 0; i < 4; i++) {
	    numbits = format->size[i];
	    ipix = (*(GLushort *)pixel >> format->shift[i]) &
		   componentMask[numbits];
	    if (numbits == 0) {
		out[i] = (i == 3) ? ITOF(1) : ITOF(0);
	    } else {
		out[i] = DIV(ITOF(ipix), ITOF(componentMaxValue[numbits]));
	    }
	}
	break;
      case sizeof(GLuint):
	for (i = 0; i < 4; i++) {
	    numbits = format->size[i];
	    ipix = (*(GLuint *)pixel >> format->shift[i]) &
		   componentMask[numbits];
	    if (numbits == 0) {
		out[i] = (i == 3) ? ITOF(1) : ITOF(0);
	    } else {
		out[i] = DIV(ITOF(ipix), ITOF(componentMaxValue[numbits]));
	    }
	}
	break;
    }
}

/* Convert the count RGBA,float pixels in inPtr to the designated
** packed pixel format in outPtr.
*/
void ConvertRGBAtoPackedImage(GLftype const *inPtr, GLubyte *outPtr,
			      packPixRec const *format, long count)
{
    GLubyte *ucPtr;
    GLushort *usPtr;
    GLuint *uiPtr;
    long i;

    switch (format->datasize) {
      case sizeof(GLubyte):
	ucPtr = (GLubyte *)outPtr;
	for (i = 0; i < count; i++) {
	    *ucPtr++ = (GLubyte)ConvertRGBAtoPacked(inPtr, format);
	    inPtr += 4;
	}
	break;
      case sizeof(GLushort):
	usPtr = (GLushort *)outPtr;
	for (i = 0; i < count; i++) {
	    *usPtr++ = (GLushort)ConvertRGBAtoPacked(inPtr, format);
	    inPtr += 4;
	}
	break;
      case sizeof(GLuint):
	uiPtr = (GLuint *)outPtr;
	for (i = 0; i < count; i++) {
	    *uiPtr++ = ConvertRGBAtoPacked(inPtr, format);
	    inPtr += 4;
	}
	break;
    }
}
