/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conformshell/utilru.c,v 1.3 2005/05/11 15:21:43 artlab Exp $
*/

#include "conform.h"
#include "util.h"
#include "utilru.h"


static const char componentName[][40] = {
    "Red",
    "Green",
    "Blue"
};
static const char errStrFmtRGB[][160] = {
    "Color is (%g, %g, %g). Green and blue components should be 0.0.",
    "Color is (%g, %g, %g). Red and blue components should be 0.0.",
    "Color is (%g, %g, %g). Red and green components should be 0.0.",
    "%s component is not monotonic. Current value is %g, last value was %g.",
    "%s component increased by more than one color shade. Step is %g, should be %g.",
    "%s component is not increasing consistantly from one shade to the next. Current step delta is %g, last step delta was %g.",
    "%s component did not reach 1.0. Value is %g.",
    "%s component does not have correct number of shades. Count is %d, should be %d."
};

void ColorError_RGBClamp(char *str, long component, ftype value)
{

    StrMake(str, errStrFmtRGB[6], componentName[component], value);
}

void ColorError_RGBCount(char *str, long component, long curCount,
			 long trueCount)
{

    StrMake(str, errStrFmtRGB[7], componentName[component], curCount,
	    trueCount);
}

void ColorError_RGBDelta(char *str, long component, ftype delta,
			 ftype lastDelta)
{

    StrMake(str, errStrFmtRGB[5], componentName[component], delta, lastDelta);
}

void ColorError_RGBMonotonic(char *str, long component, ftype curValue,
			     ftype lastValue)
{

    StrMake(str, errStrFmtRGB[3], componentName[component], curValue,
	    lastValue);
}

void ColorError_RGBStep(char *str, long component, ftype delta, ftype step)
{

    StrMake(str, errStrFmtRGB[4], componentName[component], delta, step);
}

void ColorError_RGBZero(char *str, long component, GLftype *color)
{

    switch (component) {
      case 0:
	StrMake(str, errStrFmtRGB[0], color[0], color[1], color[2]);
	break;
      case 1:
	StrMake(str, errStrFmtRGB[1], color[0], color[1], color[2]);
	break;
      case 2:
	StrMake(str, errStrFmtRGB[2], color[0], color[1], color[2]);
	break;
    }
}

long RampUtil(ftype min, ftype max, void (*Init)(void *),
	      void (*Set)(void *, ftype *), long (*Test)(void *, ftype),
	      void *data)
{
    ftype ref, x;

    glClear(GL_COLOR_BUFFER_BIT);

    if (Init) {
	(*Init)(data);
    }

    ref = min;
    for (;;) {
	x = ref;

	if (Set) {
	    (*Set)(data, &x);
	}

	if (Test) {
	    if ((*Test)(data, ref) == ERROR) {
		return ERROR;
	    }
	}

	if (ref == max) {
	    break;
	}

        if (x > max) {
	    ref = max;
	} else if (x < min) {
	    ref = min;
	} else {
	    ref = x;
	}
    }

    return NO_ERROR;
}
