/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/conformshell/utils.c,v 1.10 2005/05/16 19:57:34 artlab Exp $
*/

#include "conform.h"
#include "util.h"
#include "utils.h"
#include "pathdata.h"
#include "driver.h"


stateRec state[] = {
    {
	GL_ALPHA_BITS, STATEDATA_DATA,
	STATEDATA_DEPENDENT, 1
    },
    {
	GL_BLUE_BITS, STATEDATA_DATA,
	STATEDATA_DEPENDENT, 1
    },
    {
	GL_DEPTH_BITS, STATEDATA_DATA,
	STATEDATA_DEPENDENT, 1
    },
    {
	GL_GREEN_BITS, STATEDATA_DATA,
	STATEDATA_DEPENDENT, 1
    },
    {
	GL_SMOOTH_LINE_WIDTH_RANGE, STATEDATA_DATA,
	STATEDATA_DEPENDENT, 2
    },
    {
	GL_MAX_LIGHTS, STATEDATA_DATA,
	STATEDATA_DEPENDENT, 1
    },
    {
	GL_MAX_MODELVIEW_STACK_DEPTH, STATEDATA_DATA,
	STATEDATA_DEPENDENT, 1
    },
    {
	GL_MAX_PROJECTION_STACK_DEPTH, STATEDATA_DATA,
	STATEDATA_DEPENDENT, 1
    },
    {
	GL_MAX_TEXTURE_SIZE, STATEDATA_DATA,
	STATEDATA_DEPENDENT, 1
    },
    {
	GL_MAX_TEXTURE_STACK_DEPTH, STATEDATA_DATA,
	STATEDATA_DEPENDENT, 1
    },
    {
	GL_MAX_VIEWPORT_DIMS, STATEDATA_DATA,
	STATEDATA_DEPENDENT, 2
    },
    {
	GL_NUM_COMPRESSED_TEXTURE_FORMATS, STATEDATA_DATA,
	STATEDATA_DEPENDENT, 1
    },
    {
	GL_SMOOTH_POINT_SIZE_RANGE, STATEDATA_DATA,
	STATEDATA_DEPENDENT, 2
    },
    {
	GL_RED_BITS, STATEDATA_DATA,
	STATEDATA_DEPENDENT, 1
    },
    {
	GL_STENCIL_BITS, STATEDATA_DATA,
	STATEDATA_DEPENDENT, 1
    },
    {
	GL_SUBPIXEL_BITS, STATEDATA_DATA,
	STATEDATA_DEPENDENT, 1
    },
    {
	GL_ALIASED_POINT_SIZE_RANGE, STATEDATA_DATA,
	STATEDATA_DEPENDENT, 2
    },
    {
	GL_ALIASED_LINE_WIDTH_RANGE, STATEDATA_DATA,
	STATEDATA_DEPENDENT, 2
    },
    {
	GL_MAX_TEXTURE_UNITS, STATEDATA_DATA,
	STATEDATA_DEPENDENT, 1
    },
};

static int const state_count = sizeof(state) / sizeof(state[0]);


long StateCheck(void)
{
    long i, j;

    for (j = 0; j != state_count; j++) {
	stateRec *ptr = &state[j];

	for (i = 0; i < 2 + 1; i++) {
	    ptr->dataCur[i] = -1;
	}
        glGetIntegerv(ptr->value, ptr->dataCur);
	if (ptr->dataFlag == GL_TRUE) {
	    for (i = 0; i < ptr->dataCount; i++) {
		if (ptr->dataCur[i] != ptr->dataTrue[i]) {
		    return ERROR;
		}
	    }
	    if (ptr->dataCur[i] != -1) {
		return ERROR;
	    }
	}
    }
    return NO_ERROR;
}

long StateInit(void)
{
    long i, j;

    for (j = 0; j != state_count; j++) {
        stateRec *ptr = &state[j];

	if (ptr->dataType == STATEDATA_DEPENDENT) {
	    for (i = 0; i < 2 + 1; i++) {
		ptr->dataCur[i] = -1;
	    }
            glGetIntegerv(ptr->value, ptr->dataCur);
	    for (i = 0; i < ptr->dataCount; i++) {
		ptr->dataTrue[i] = ptr->dataCur[i];
	    }
	    ptr->dataFlag = GL_TRUE;
	    if (ptr->dataCur[i] != -1) {
		return ERROR;
	    }
	}
    }
    return NO_ERROR;
}

void StateReport(void)
{
    char name[40];
    long i, j;

    Output(2, "    State Report.\n");
    for (j = 0; j != state_count; j++) {
        stateRec *ptr = &state[j];

	if (ptr->dataCount > 0) {
            glGetIntegerv(ptr->value, ptr->dataCur);
	    for (i = 0; i < ptr->dataCount; i++) {
		if (ptr->dataCur[i] != ptr->dataTrue[i]) {
		    Output(2, "        ");
		    GetEnumName(ptr->value, name);
		    Output(2, "%s", name);
		    Output(2, " (%s)\n", "State Information");
		    switch (ptr->valueType) {
		      case STATEDATA_BOOLEAN:
			Output(2, "           ");
			for (i = 0; i < ptr->dataCount; i++) {
			    if (ptr->dataCur[i] == 0) {
				Output(2, " GL_FALSE");
			    } else if (ptr->dataCur[i] == 1) {
				Output(2, " GL_TRUE");
			    } else {
				Output(2, " %d", ptr->dataCur[i]);
			    }
			}
			Output(2, " <->");
			for (i = 0; i < ptr->dataCount; i++) {
			    if (ptr->dataTrue[i] == ITOF(0)) {
				Output(2, " GL_FALSE");
			    } else if (ptr->dataTrue[i] == ITOF(1)) {
				Output(2, " GL_TRUE");
			    } else {
				Output(2, " %d", ptr->dataTrue[i]);
			    }
			}
			Output(2, "\n");
			break;
		      case STATEDATA_DATA:
			Output(2, "           ");
			for (i = 0; i < ptr->dataCount; i++) {
			    Output(2, " %d", ptr->dataCur[i]);
			}
			Output(2, " <->");
			for (i = 0; i < ptr->dataCount; i++) {
			    Output(2, " %d", ptr->dataTrue[i]);
			}
			Output(2, "\n");
			break;
		    }
		    break;
		}
	    }
	}
    }
}

void StateSave(void)
{
    glViewport(0, 0, WINDSIZEX, WINDSIZEY);
    glScissor(0, 0, WINDSIZEX, WINDSIZEY);
}

long StateReset(void)
{
    static const GLftype fog_color[4] = {ITOF(0), ITOF(0), ITOF(0), ITOF(0)};
                 GLftype ambient_scene[4] = {-1, -1,  -1, ITOF(1)};
    static const GLftype light_ambient[4] = {0, 0, 0, ITOF(1)};
    static const GLftype light_diffuse_0[4] = {ITOF(1), ITOF(1), ITOF(1), ITOF(1)};
    static const GLftype light_diffuse_n[4] = {0, 0, 0, 0};
    static const GLftype light_specular_0[4] = {ITOF(1), ITOF(1), ITOF(1), ITOF(1)};
    static const GLftype light_specular_n[4] = {0, 0, 0, 0};
    static const GLftype light_position[4] = {0, 0, ITOF(1), 0};
    static const GLftype spot_direction[3] = {0, 0, ITOF(-1)};
                 GLftype material_ambient[4] = {-1, -1, -1, ITOF(1)};
                 GLftype material_diffuse[4] = {-1, -1, -1, ITOF(1)};
    static const GLftype material_specular[4] = {0, 0, 0, ITOF(1)};
    static const GLftype material_emission[4] = {0, 0, 0, ITOF(1)};
    static const GLftype texture_env_color[4] = {0, 0, 0, 0};

    int i;
    GLint max_lights;
    GLint max_texture_units;
    GLint projection_stack_depth;
    GLint texture_stack_depth;
    GLint modelview_stack_depth;
#ifdef GL_OES_VERSION_1_1
    GLint max_clip_planes;
    static const GLftype point_params[3] = {ITOF(1), ITOF(0), ITOF(0)};
    GLftype aliased_point_size_range[2];
    GLftype smooth_point_size_range[2];
    GLftype max_point_size;
#endif

    /* fixup initial values */

    ambient_scene[0] =
    ambient_scene[1] =
    ambient_scene[2] = ATOF("0.2");

    material_ambient[0] =
    material_ambient[1] =
    material_ambient[2] = ATOF("0.2");

    material_diffuse[0] =
    material_diffuse[1] =
    material_diffuse[2] = ATOF("0.8");

    glGetIntegerv(GL_MAX_LIGHTS, &max_lights);
    glGetIntegerv(GL_MAX_TEXTURE_UNITS, &max_texture_units);
    glGetIntegerv(GL_MAX_PROJECTION_STACK_DEPTH, &projection_stack_depth);
    glGetIntegerv(GL_MAX_TEXTURE_STACK_DEPTH, &texture_stack_depth);
    glGetIntegerv(GL_MAX_MODELVIEW_STACK_DEPTH, &modelview_stack_depth);
#ifdef GL_OES_VERSION_1_1
    glGetIntegerv(GL_MAX_CLIP_PLANES, &max_clip_planes);
    glGetFloatvt(GL_ALIASED_POINT_SIZE_RANGE, aliased_point_size_range);
    glGetFloatvt(GL_SMOOTH_POINT_SIZE_RANGE, smooth_point_size_range);
    max_point_size = (aliased_point_size_range[1] > smooth_point_size_range[1]) ? aliased_point_size_range[1] : smooth_point_size_range[1];
#endif

    for (i = 0; i < max_texture_units; i++) {
	int j;
        glActiveTexture(GL_TEXTURE0 + i);
	glMatrixMode(GL_TEXTURE);
	for (j = 1; j < texture_stack_depth; j++) {
	    glPopMatrix();
	}
	glLoadIdentity();
	glDisable(GL_TEXTURE_2D);
	glTexEnvt(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glTexEnvtv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, texture_env_color);
        glBindTexture(GL_TEXTURE_2D, 0);
	glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_LINEAR);
	glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParametert(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

        glClientActiveTexture(GL_TEXTURE0 + i);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	/* should always be GL_FLOAT but does not exist for CL */
	glTexCoordPointer(4, GL_FTYPE, 0, NULL);
    } 

    glMatrixMode(GL_PROJECTION);
    for (i = 1; i < projection_stack_depth; i++)
	glPopMatrix();
    glLoadIdentity();

    glMatrixMode(GL_MODELVIEW);
    for (i = 1; i < modelview_stack_depth; i++)
	glPopMatrix();
    glLoadIdentity();

    /* Set GL state to known values */

    glDisable(GL_ALPHA_TEST);
    glDisable(GL_BLEND);
#ifdef GL_OES_VERSION_1_1
    for (i=0; i<max_clip_planes; ++i)
	{
		static const GLftype equation[4] = { ITOF(0), ITOF(0), ITOF(0), ITOF(0) };
		glClipPlanet(GL_CLIP_PLANE0 + i, equation);
        glDisable(GL_CLIP_PLANE0 + i);
	}
#endif
    glDisable(GL_COLOR_LOGIC_OP);
    glDisable(GL_COLOR_MATERIAL);
    glDisable(GL_CULL_FACE);
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_FOG);
    for (i = 0; i < max_lights; i++) {
	glDisable(GL_LIGHT0 + i);
    }
    glDisable(GL_LIGHTING);
    glDisable(GL_LINE_SMOOTH);
    glDisable(GL_NORMALIZE);
    glDisable(GL_POINT_SMOOTH);
    glDisable(GL_POLYGON_OFFSET_FILL);
    glDisable(GL_RESCALE_NORMAL);
    glDisable(GL_SAMPLE_ALPHA_TO_COVERAGE);
    glDisable(GL_SAMPLE_ALPHA_TO_ONE);
    glDisable(GL_SAMPLE_COVERAGE);
    glDisable(GL_SCISSOR_TEST);
    glDisable(GL_STENCIL_TEST);
#ifdef GL_OES_VERSION_1_1
    glDisable(GL_POINT_SPRITE_OES);
#endif

    glEnable(GL_DITHER);
    glEnable(GL_MULTISAMPLE);

    glDisableClientState(GL_VERTEX_ARRAY);
    glDisableClientState(GL_NORMAL_ARRAY);
    glDisableClientState(GL_COLOR_ARRAY);
#ifdef GL_OES_VERSION_1_1
    glDisableClientState(GL_POINT_SIZE_ARRAY_OES);
#endif

    glActiveTexture(GL_TEXTURE0);
    glAlphaFunct(GL_ALWAYS, ITOF(0));
#ifdef GL_OES_VERSION_1_1
    glBindBuffer(GL_ARRAY_BUFFER, 0);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
#endif
    glBindTexture(GL_TEXTURE_2D, 0);
    glBlendFunc(GL_ONE, GL_ZERO);
    glClearColort(ITOF(0), ITOF(0), ITOF(0), ITOF(0));
    glClearDeptht(ITOF(1));
    glClearStencil(0);
    glClientActiveTexture(GL_TEXTURE0);
    glColor4t(ITOF(1), ITOF(1), ITOF(1), ITOF(1));
    glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
    /* should be GL_FLOAT but does not exist for CL */
    glColorPointer(4, GL_FTYPE, 0, NULL);
    glCullFace(GL_BACK);
    glDepthFunc(GL_LESS);
    glDepthMask(GL_TRUE);
    glDepthRanget(ITOF(0), ITOF(1));
    glFogt(GL_FOG_MODE, GL_EXP);
    glFogt(GL_FOG_DENSITY, ITOF(1));
    glFogt(GL_FOG_START, ITOF(0));
    glFogt(GL_FOG_END, ITOF(1));
    glFogtv(GL_FOG_COLOR, fog_color);
    glFrontFace(GL_CCW);
    glHint(GL_FOG_HINT, GL_DONT_CARE);
    glHint(GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_DONT_CARE);
    glHint(GL_POINT_SMOOTH_HINT, GL_DONT_CARE);
    glLightModeltv(GL_LIGHT_MODEL_AMBIENT, ambient_scene);
    glLightModelt(GL_LIGHT_MODEL_TWO_SIDE, ITOF(0));

    for (i = 0; i < max_lights; i++) {
        glLighttv(GL_LIGHT0 + i, GL_AMBIENT, light_ambient);
        if (i == 0) {
            glLighttv(GL_LIGHT0 + i, GL_DIFFUSE, light_diffuse_0);
            glLighttv(GL_LIGHT0 + i, GL_SPECULAR, light_specular_0);
        } else {
            glLighttv(GL_LIGHT0 + i, GL_DIFFUSE, light_diffuse_n);
            glLighttv(GL_LIGHT0 + i, GL_SPECULAR, light_specular_n);
        }
        glLighttv(GL_LIGHT0 + i, GL_POSITION, light_position);
        glLightt(GL_LIGHT0 + i, GL_SPOT_CUTOFF, ITOF(180));
        glLightt(GL_LIGHT0 + i, GL_SPOT_EXPONENT, ITOF(0));
        glLighttv(GL_LIGHT0 + i, GL_SPOT_DIRECTION, spot_direction);
        glLightt(GL_LIGHT0 + i, GL_CONSTANT_ATTENUATION, ITOF(1));
        glLightt(GL_LIGHT0 + i, GL_LINEAR_ATTENUATION, ITOF(0));
        glLightt(GL_LIGHT0 + i, GL_QUADRATIC_ATTENUATION, ITOF(0));
    }

    glLineWidtht(ITOF(1));
    glLogicOp(GL_COPY);
    glMaterialtv(GL_FRONT_AND_BACK, GL_AMBIENT, material_ambient);
    glMaterialtv(GL_FRONT_AND_BACK, GL_DIFFUSE, material_diffuse);
    glMaterialtv(GL_FRONT_AND_BACK, GL_SPECULAR, material_specular);
    glMaterialtv(GL_FRONT_AND_BACK, GL_EMISSION, material_emission);
    glMaterialt(GL_FRONT_AND_BACK, GL_SHININESS, ITOF(0));
    glMatrixMode(GL_MODELVIEW);
    for (i = 0; i != max_texture_units; ++i) {
        glMultiTexCoord4t(GL_TEXTURE0 + i, ITOF(0), ITOF(0), ITOF(0), ITOF(1));
    }
    glNormal3t(ITOF(0), ITOF(0), ITOF(1));
    /* should be GL_FLOAT but does not exist for CL */
    glNormalPointer(GL_FTYPE, 0, NULL);
    glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
    glPixelStorei(GL_PACK_ALIGNMENT, 4);

#ifdef GL_OES_VERSION_1_1
    glPointParametertv(GL_POINT_DISTANCE_ATTENUATION, point_params);
    glPointParametertv(GL_POINT_FADE_THRESHOLD_SIZE, point_params);
    glPointParametert(GL_POINT_SIZE_MIN, ITOF(0));
    glPointParametert(GL_POINT_SIZE_MAX, max_point_size);
#endif

    glPointSizet(ITOF(1));
    glPolygonOffsett(ITOF(0), ITOF(0));
    glSampleCoveraget(ITOF(1), GL_FALSE);
    glScissor(0, 0, WINDSIZEX, WINDSIZEY);
    glShadeModel(GL_SMOOTH);
    glStencilFunc(GL_ALWAYS, 0, ~0);
    glStencilMask(~0);
    glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
    /* should be GL_FLOAT but does not exist for CL */
    glVertexPointer(4, GL_FTYPE, 0, NULL);
    glViewport(0, 0, WINDSIZEX, WINDSIZEY);

    /*Check for any errors while making GL calls (including resetting state)*/
    while (glGetError());

    if (machine.stateCheckFlag == GL_TRUE) {
	if (StateCheck() == ERROR) {
	    return ERROR_STATE;
	} else {
	    return NO_ERROR;
	}
    } else {
	return NO_ERROR;
    }
}

void StateSetup(void)
{
    long i;

    for (i = 0; i != state_count; i++) {
        stateRec *ptr = &state[i];

	if (ptr->dataType == STATEDATA_LOCKED) {
	    ptr->dataFlag = GL_TRUE;
	} else {
	    ptr->dataFlag = GL_FALSE;
	}
    }
}
