/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/covegl/enum.c,v 1.3 2005/05/11 15:21:43 artlab Exp $
*/

#include "shell.h"

#define ENUM(name, value) \
    { \
	#name, name, value \
    }

static struct EnumCheckRec {
    char *name;
    long value;
    long true_;
} const enumCheck[] = {
    /* errors */
    ENUM(EGL_SUCCESS, 0x3000),
    ENUM(EGL_NOT_INITIALIZED, 0x3001),
    ENUM(EGL_BAD_ACCESS, 0x3002),
    ENUM(EGL_BAD_ALLOC, 0x3003),
    ENUM(EGL_BAD_ATTRIBUTE, 0x3004),
    ENUM(EGL_BAD_CONFIG, 0x3005),
    ENUM(EGL_BAD_CONTEXT, 0x3006),
    ENUM(EGL_BAD_CURRENT_SURFACE, 0x3007),
    ENUM(EGL_BAD_DISPLAY, 0x3008),
    ENUM(EGL_BAD_MATCH, 0x3009),
    ENUM(EGL_BAD_NATIVE_PIXMAP, 0x300A),
    ENUM(EGL_BAD_NATIVE_WINDOW, 0x300B),
    ENUM(EGL_BAD_PARAMETER, 0x300C),
    ENUM(EGL_BAD_SURFACE, 0x300D),

    /* config attributes  */
    ENUM(EGL_BUFFER_SIZE, 0x3020),
    ENUM(EGL_ALPHA_SIZE, 0x3021),
    ENUM(EGL_BLUE_SIZE, 0x3022),
    ENUM(EGL_GREEN_SIZE, 0x3023),
    ENUM(EGL_RED_SIZE, 0x3024),
    ENUM(EGL_DEPTH_SIZE, 0x3025),
    ENUM(EGL_STENCIL_SIZE, 0x3026),
    ENUM(EGL_CONFIG_CAVEAT, 0x3027),
    ENUM(EGL_CONFIG_ID, 0x3028),
    ENUM(EGL_LEVEL, 0x3029),
    ENUM(EGL_MAX_PBUFFER_HEIGHT, 0x302A),
    ENUM(EGL_MAX_PBUFFER_PIXELS, 0x302B),
    ENUM(EGL_MAX_PBUFFER_WIDTH, 0x302C),
    ENUM(EGL_NATIVE_RENDERABLE, 0x302D),
    ENUM(EGL_NATIVE_VISUAL_ID, 0x302E),
    ENUM(EGL_NATIVE_VISUAL_TYPE, 0x302F),
    ENUM(EGL_SAMPLES, 0x3031),
    ENUM(EGL_SAMPLE_BUFFERS, 0x3032),
    ENUM(EGL_SURFACE_TYPE, 0x3033),
    ENUM(EGL_TRANSPARENT_TYPE, 0x3034),
    ENUM(EGL_TRANSPARENT_BLUE_VALUE, 0x3035),
    ENUM(EGL_TRANSPARENT_GREEN_VALUE, 0x3036),
    ENUM(EGL_TRANSPARENT_RED_VALUE, 0x3037),

    /* config attribute and values */
    ENUM(EGL_NONE, 0x3038),

    ENUM(EGL_SLOW_CONFIG, 0x3050),
    ENUM(EGL_NON_CONFORMANT_CONFIG, 0x3051),
    ENUM(EGL_TRANSPARENT_RGB, 0x3052),

    /* config values */
    ENUM(EGL_DONT_CARE, -1),
    ENUM(EGL_PBUFFER_BIT, 0x01),
    ENUM(EGL_PIXMAP_BIT, 0x02),
    ENUM(EGL_WINDOW_BIT, 0x04),

    /* surface attributes */
    ENUM(EGL_HEIGHT, 0x3056),
    ENUM(EGL_WIDTH, 0x3057),
    ENUM(EGL_LARGEST_PBUFFER, 0x3058),

    /* surface types */
    ENUM(EGL_DRAW, 0x3059),
    ENUM(EGL_READ, 0x305A),

    /* engines */
    ENUM(EGL_CORE_NATIVE_ENGINE, 0x305B),
};


void VerifyEnums(void)
{
    struct EnumCheckRec const *p, *end;

    p = enumCheck;
    end = p + (sizeof(enumCheck) / sizeof(struct EnumCheckRec));

    Output(1, "Enumeration check.\n");
    while (p < end) {
	Output(1, "\t%s (%d) = %d.\n", p->name, p->true_, p->value);
	if (p->value != p->true_) {
	    Output(0, "covegl failed.");
            Output(0, "");
	    EXIT(EXIT_FAILURE);
	}    
	p++;
    }
    Output(1, "\n");
}
