/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/covegl/native.c,v 1.3 2005/05/11 15:21:43 artlab Exp $
*/

#include <X11/Xutil.h>
#include <X11/keysym.h>
#include "native.h"

static Display *display;
static long screen;
static XVisualInfo *visual;
static Colormap colormap;
static Window window;

Display *nativeCreateDisplay(void)
{
    display = XOpenDisplay(NULL);
    screen = DefaultScreen(display);
    return display;
}

void nativeDestroyDisplay(Display *display_)
{
    XFree((char *)visual);
    XCloseDisplay(display_);
}

static Bool WaitForMapNotify(Display *d, XEvent *e, char *arg)
{

    if (e->type == MapNotify && e->xmap.window == window) {
	return True;
    }
    return False;
}

Window nativeCreateWindow(const char *title, int width, int height, EGLint visualId)
{
    XVisualInfo template_;
    XSetWindowAttributes wa;
    XSizeHints sh;
    XEvent e;
    int nVisuals;
    unsigned long mask;

    template_.visualid = visualId;
    template_.screen = screen;
    visual = XGetVisualInfo(display, VisualIDMask | VisualScreenMask,
	&template_, &nVisuals);

    colormap = XCreateColormap(display, RootWindow(display, screen),
			       visual->visual, AllocNone);
    wa.colormap = colormap;
    wa.background_pixel = 0xFFFFFFFF;
    wa.border_pixel = 0;
    wa.event_mask = StructureNotifyMask | ExposureMask;

    mask = CWBackPixel | CWBorderPixel | CWEventMask | CWColormap;

    window = XCreateWindow(display, RootWindow(display, screen), 0, 0,
			   width, height, 0, visual->depth, InputOutput,
			   visual->visual, mask, &wa);

    sh.flags = USPosition;
    sh.x = 10;
    sh.y = 10;
    XSetStandardProperties(display, window, title, title, None, 0, 0, &sh);
    
    XMapWindow(display, window);
    XIfEvent(display, &e, WaitForMapNotify, 0);

    XSetWMColormapWindows(display, window, &window, 1);

    XFlush(display);

    return window;
}

void nativeDestroyWindow(Window window)
{
    XDestroyWindow(display, window);
    XFreeColormap(display, colormap);
}

Pixmap nativeCreatePixmap(int width, int height, EGLint visualId)
{
    XVisualInfo template_;
    int nVisuals;

    template_.visualid = visualId;
    template_.screen = screen;
    visual = XGetVisualInfo(display, VisualIDMask | VisualScreenMask,
	&template_, &nVisuals);

    return XCreatePixmap(display, RootWindow(display, screen), width, height, visual->depth);
}

void nativeDestroyPixmap(Pixmap pixmap)
{
    XFreePixmap(display, pixmap);
}
