/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/covegl/test.c,v 1.5 2005/05/24 11:07:08 artlab Exp $
*/

#include "native.h"
#include "shell.h"

static NativeDisplay nativeDisplay;

static EGLDisplay display;
static EGLConfig config;
static EGLSurface surface;
static EGLContext context;

#define ENUM(name) \
    {name, #name}

static const enumTestRec enum_QueryString[] = {
    ENUM(EGL_VENDOR),
    ENUM(EGL_VERSION),
    ENUM(EGL_EXTENSIONS),
    {-1, "End of List"}
};

static const enumTestRec enum_GetConfig[] = {
    ENUM(EGL_CONFIG_ID),
    ENUM(EGL_BUFFER_SIZE),
    ENUM(EGL_LEVEL),
    ENUM(EGL_RED_SIZE),
    ENUM(EGL_GREEN_SIZE),
    ENUM(EGL_BLUE_SIZE),
    ENUM(EGL_ALPHA_SIZE),
    ENUM(EGL_DEPTH_SIZE),
    ENUM(EGL_STENCIL_SIZE),
    ENUM(EGL_SAMPLE_BUFFERS),
    ENUM(EGL_SAMPLES),
    ENUM(EGL_SURFACE_TYPE),
    ENUM(EGL_NATIVE_RENDERABLE),
    ENUM(EGL_NATIVE_VISUAL_ID),
    ENUM(EGL_NATIVE_VISUAL_TYPE),
    ENUM(EGL_CONFIG_CAVEAT),
    ENUM(EGL_TRANSPARENT_TYPE),
    ENUM(EGL_TRANSPARENT_RED_VALUE),
    ENUM(EGL_TRANSPARENT_GREEN_VALUE),
    ENUM(EGL_TRANSPARENT_BLUE_VALUE),
    ENUM(EGL_MAX_PBUFFER_WIDTH),
    ENUM(EGL_MAX_PBUFFER_HEIGHT),
    ENUM(EGL_MAX_PBUFFER_PIXELS),
    {-1, "End of List"}
};

static const enumTestRec enum_QuerySurface[] = {
    ENUM(EGL_CONFIG_ID),
    ENUM(EGL_LARGEST_PBUFFER),
    ENUM(EGL_WIDTH),
    ENUM(EGL_HEIGHT),
    {-1, "End of List"}
};

static const enumTestRec enum_QueryContext[] = {
    ENUM(EGL_CONFIG_ID),
    {-1, "End of List"}
};

static const enumTestRec enum_GetCurrentSurface[] = {
    ENUM(EGL_DRAW),
    ENUM(EGL_READ),
    {-1, "End of List"}
};

static const enumTestRec enum_WaitNative[] = {
    ENUM(EGL_CORE_NATIVE_ENGINE),
    {-1, "End of List"}
};

static void FailAndDie(void)
{

    Output(1, "\n");
    Output(0, "covegl failed.\n");
    Output(0, "\n");
    EXIT(EXIT_FAILURE);
}

static void ProbeError(void)
{
    switch (eglGetError()) {
    case EGL_NOT_INITIALIZED:
    case EGL_BAD_DISPLAY:
    case EGL_BAD_ACCESS:
    case EGL_BAD_ATTRIBUTE:
    case EGL_BAD_PARAMETER:
    case EGL_BAD_SURFACE:
        FailAndDie();
        break;
    }
}

static void CallWindow(void)
{
    NativeWindow native_window;
    EGLSurface window;
    EGLConfig window_config;
    EGLint visualId;
    EGLint num_window_config;
    EGLint config_attrib_list[] = {EGL_SURFACE_TYPE, EGL_WINDOW_BIT, EGL_NONE};

    if (!eglChooseConfig(display, config_attrib_list, &window_config, 1,
        &num_window_config) || num_window_config == 0) {
        eglGetError();
        return;
    }

    (void) eglGetConfigAttrib(display, window_config, EGL_NATIVE_VISUAL_ID,
        &visualId);
    ProbeError();

    native_window = nativeCreateWindow("name", WINDSIZEX, WINDSIZEY, visualId);

    Output(1, "eglCreateWindowSurface\n");
    window = eglCreateWindowSurface(display, window_config, 
                                    native_window, NULL);

    if (window == EGL_NO_SURFACE) {
        eglGetError();
    } else {
        ProbeError();
        Output(1, "eglDestroySurface\n");
        eglDestroySurface(display, window);
        ProbeError();
    }

    Output(1, "eglSwapBuffers\n");
    eglSwapBuffers(display, surface);
    ProbeError();

    nativeDestroyWindow(native_window);
}

static void CallPbuffer(void)
{
    EGLSurface pbuffer;
    EGLConfig pbuffer_config;
    EGLint num_pbuffer_config;
    EGLint config_attrib_list[] = {EGL_SURFACE_TYPE, EGL_PBUFFER_BIT, EGL_NONE};
    EGLint pbuffer_attrib_list[] = {
        EGL_WIDTH, WINDSIZEX,
        EGL_HEIGHT, WINDSIZEY,
        EGL_LARGEST_PBUFFER, EGL_TRUE,
        EGL_NONE
    };

    if (!eglChooseConfig(display, config_attrib_list, &pbuffer_config, 1,
        &num_pbuffer_config) || num_pbuffer_config == 0) {
        eglGetError();
        return;
    }

    Output(1, "eglCreatePbufferSurface\n");
    pbuffer = eglCreatePbufferSurface(display, pbuffer_config,
        pbuffer_attrib_list);

    if (pbuffer == EGL_NO_SURFACE) {
        eglGetError();
    } else {
        ProbeError();
        Output(1, "eglDestroySurface\n");
        eglDestroySurface(display, pbuffer);
        ProbeError();
    }
}

static void CallPixmap(void)
{
    NativePixmap native_pixmap;
    EGLSurface pixmap;
    EGLConfig pixmap_config;
    EGLint num_pixmap_config;
    EGLint visualId;
    EGLint config_attrib_list[] = {EGL_SURFACE_TYPE, EGL_PIXMAP_BIT, EGL_NONE};

    if (!eglChooseConfig(display, config_attrib_list, &pixmap_config, 1,
        &num_pixmap_config) || num_pixmap_config == 0) {
        eglGetError();
        return;
    }

    (void) eglGetConfigAttrib(display, pixmap_config, EGL_NATIVE_VISUAL_ID,
        &visualId);
    ProbeError();

    native_pixmap = nativeCreatePixmap(WINDSIZEX, WINDSIZEY, visualId);

    Output(1, "eglCreatePixmapSurface\n");
    pixmap = eglCreatePixmapSurface(display, pixmap_config, 
                    (EGLNativePixmapType)native_pixmap, NULL);
    ProbeError();

    if (pixmap != EGL_NO_SURFACE) {
        Output(1, "eglDestroySurface\n");
        eglDestroySurface(display, pixmap);
        ProbeError();
    }

    Output(1, "eglCopyBuffers\n");
    (void) eglCopyBuffers(display, surface, (EGLNativePixmapType)native_pixmap);
    (void) eglGetError();

    nativeDestroyPixmap(native_pixmap);
}

static void CallQueryString(void)
{
    int i;

    Output(1, "eglQueryString\n");
    for (i = 0; enum_QueryString[i].value != -1; i++) {
        Output(1, "\t%s\n", enum_QueryString[i].name);
        (void)eglQueryString(display, enum_QueryString[i].value);
        ProbeError();
    }
}

static void CallGetConfigAttrib(void)
{
    EGLint x;
    int i;

    Output(1, "eglGetConfigAttrib\n");
    for (i = 0; enum_GetConfig[i].value != -1; i++) {
        Output(1, "\t%s\n", enum_GetConfig[i].name);
        (void)eglGetConfigAttrib(display, config,
            enum_GetConfig[i].value, &x);
        ProbeError();
    }
}

static void CallQuerySurface(void)
{
    EGLint x;
    int i;

    Output(1, "eglQuerySurface\n");
    for (i = 0; enum_QuerySurface[i].value != -1; i++) {
        Output(1, "\t%s\n", enum_QuerySurface[i].name);
        (void)eglQuerySurface(display, surface,
            enum_QuerySurface[i].value, &x);
        ProbeError();
    }
}

static void CallQueryContext(void)
{
    EGLint x;
    int i;

    Output(1, "eglQueryContext\n");
    for (i = 0; enum_QueryContext[i].value != -1; i++) {
        Output(1, "\t%s\n", enum_QueryContext[i].name);
        (void)eglQueryContext(display, context, enum_QueryContext[i].value,
            &x);
        ProbeError();
    }
}

static void CallGetCurrentSurface(void)
{
    int i;

    Output(1, "eglGetCurrentSurface\n");
    for (i = 0; enum_GetCurrentSurface[i].value != -1; i++) {
        Output(1, "\t%s\n", enum_GetCurrentSurface[i].name);
        (void)eglGetCurrentSurface(enum_GetCurrentSurface[i].value);
        ProbeError();
    }
}

static void CallWaitNative(void)
{
    int i;

    Output(1, "eglWaitNative\n");
    for (i = 0; enum_WaitNative[i].value != -1; i++) {
        Output(1, "\t%s\n", enum_WaitNative[i].name);
        (void)eglWaitNative(enum_WaitNative[i].value);
        ProbeError();
    }
}

void DoTest(void)
{
    EGLint attrib_list[] = {
        EGL_SURFACE_TYPE, 0,
        EGL_RED_SIZE, 1,
        EGL_DEPTH_SIZE, 12,
        EGL_NONE
    };
    EGLint visualId;
    EGLContext tmpContext;
    EGLint num_config;
    EGLint major, minor;
    NativeWindow nativeWindow = (NativeWindow)NULL;
    NativePixmap nativePixmap = (NativePixmap)NULL;


    nativeDisplay = nativeCreateDisplay();

    Output(1, "eglGetError\n");
    (void) eglGetError();

    Output(1, "eglGetProcAddress with empty string\n");
    (void) eglGetProcAddress("");
    ProbeError();

    Output(1, "eglGetDisplay\n");
    display = eglGetDisplay((EGLNativeDisplayType)nativeDisplay);
    ProbeError();

    Output(1, "eglInitialize\n");
    (void) eglInitialize(display, &major, &minor);
    ProbeError();

    CallQueryString();

    Output(1, "eglGetConfigs NULL config\n");
    (void) eglGetConfigs(display, NULL, 0, &num_config);
    ProbeError();

    Output(1, "eglGetConfigs with config\n");
    (void) eglGetConfigs(display, &config, 1, &num_config);
    ProbeError();

    Output(1, "eglChooseConfig NULL config\n");
    (void) eglChooseConfig(display, attrib_list, NULL, 1, &num_config);
    ProbeError();

    Output(1, "eglChooseConfig NULL attrib_list\n");
    (void) eglChooseConfig(display, NULL, &config, 1, &num_config);
    ProbeError();

    Output(1, "eglChooseConfig with attrib_list and config\n");
    (void) eglChooseConfig(display, attrib_list, &config, 1, &num_config);
    ProbeError();

    tmpContext = eglCreateContext(display, config, EGL_NO_CONTEXT, NULL);
    Output(1, "eglDestroyContext\n");
    eglDestroyContext(display, tmpContext);
    ProbeError();

	attrib_list[1] = EGL_WINDOW_BIT;
	(void)eglChooseConfig(display, attrib_list, &config, 1, &num_config);
	ProbeError();

	Output(1, "eglGetConfigAttrib\n");
	(void)eglGetConfigAttrib(display, config, EGL_NATIVE_VISUAL_ID,
		&visualId);
	ProbeError();

    if (num_config > 0) {
        nativeWindow = nativeCreateWindow("covegl", WINDSIZEX, WINDSIZEY, visualId);

        Output(1, "eglCreateWindowSurface\n");
        surface = eglCreateWindowSurface(display, config, 
                                         nativeWindow, NULL);
    }

    if (surface == EGL_NO_SURFACE) {
        attrib_list[1] = EGL_PBUFFER_BIT;
        (void) eglChooseConfig(display, attrib_list, &config, 1, &num_config);
        ProbeError();

        if (num_config > 0) {
            EGLint pbuffer_attributes[] = {
                EGL_WIDTH, WINDSIZEX,
                EGL_HEIGHT, WINDSIZEY,
                EGL_NONE
            };

            Output(1, "eglCreatePbufferSurface\n");
            surface = eglCreatePbufferSurface(display, config, pbuffer_attributes);
        }
    }

    if (surface == EGL_NO_SURFACE) {
        attrib_list[1] = EGL_PIXMAP_BIT;
        (void) eglChooseConfig(display, attrib_list, &config, 1, &num_config);
        ProbeError();

        if (num_config > 0) {
            (void) eglGetConfigAttrib(display, config, EGL_NATIVE_VISUAL_ID,
                &visualId);
            ProbeError();

            nativePixmap = nativeCreatePixmap(WINDSIZEX, WINDSIZEY, visualId);

            Output(1, "eglCreatePixmapSurface\n");
            surface = eglCreatePixmapSurface(display, config, 
                         (EGLNativePixmapType)nativePixmap, NULL);
        }
    }

    if (surface == EGL_NO_SURFACE) {
        FailAndDie();
    }

    ProbeError();

    Output(1, "eglCreateContext\n");
    context = eglCreateContext(display, config, EGL_NO_CONTEXT, NULL);
    ProbeError();

    Output(1, "eglMakeCurrent\n");
    (void) eglMakeCurrent(display, surface, surface, context);
    ProbeError();

    Output(1, "eglGetCurrentDisplay\n");
    (void) eglGetCurrentDisplay();
    ProbeError();

    CallGetCurrentSurface();

    Output(1, "eglGetCurrentContext\n");
    (void) eglGetCurrentContext();
    ProbeError();

    CallGetConfigAttrib();
    CallQuerySurface();
    CallQueryContext();

    switch (attrib_list[1]) {
    case EGL_WINDOW_BIT:
        CallPbuffer();
        CallPixmap();
        break;

    case EGL_PBUFFER_BIT:
        CallWindow();
        CallPixmap();
        break;

    case EGL_PIXMAP_BIT:
        Output(1, "eglCopyBuffers\n");
        (void) eglCopyBuffers(display, surface, (EGLNativePixmapType)nativePixmap);
        (void) eglGetError();

        CallWindow();
        CallPbuffer();
        break;
    }

    Output(1, "eglSwapBuffers\n");
    (void) eglSwapBuffers(display, surface);
    ProbeError();

    Output(1, "eglWaitGL\n");
    (void) eglWaitGL();
    ProbeError();

    CallWaitNative();

    switch (attrib_list[1]) {
    case EGL_WINDOW_BIT:
        nativeDestroyWindow(nativeWindow);
        break;

    case EGL_PIXMAP_BIT:
        nativeDestroyPixmap(nativePixmap);
        break;
    }

    Output(1, "eglMakeCurrent NULL\n");
    (void) eglMakeCurrent(display, EGL_NO_SURFACE, EGL_NO_SURFACE,
        EGL_NO_CONTEXT);
    ProbeError();

    Output(1, "eglDestroySurface\n");
    (void) eglDestroySurface(display, surface);
    ProbeError();

    Output(1, "eglDestroyContext\n");
    (void) eglDestroyContext(display, context);
    ProbeError();

    Output(1, "eglTerminate\n");
    (void) eglTerminate(display);
    ProbeError();

    nativeDestroyDisplay(nativeDisplay);

    Output(1, "\n");
}
