/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/covgl/c.c,v 1.6 2005/05/11 15:21:44 artlab Exp $
*/

#include "shell.h"


void CallClear(void)
{
    long i;

    Output(1, "glClear\n");
    for (i = 0; enum_ClearBufferMask[i].value != -1; i++) {
    Output(1, "\t%s\n", enum_ClearBufferMask[i].name);
    glClear(enum_ClearBufferMask[i].value);
    ProbeEnum();
    }
    Output(1, "\n");
}

void CallClearColor(void)
{

    Output(1, "glClearColorx\n");
    glClearColorx(0, 0, 0, FIXED_ONE);
#ifdef COMMON
    Output(1, "glClearColor\n");
    glClearColor(0.0, 0.0, 0.0, 1.0);
#endif
    Output(1, "\n");
}

void CallClearDepth(void)
{

    Output(1, "glClearDepthx\n");
    glClearDepthx(0);
#ifdef COMMON
    Output(1, "glClearDepthf\n");
    glClearDepthf(0.0);
#endif
    Output(1, "\n");
}

void CallClearStencil(void)
{

    Output(1, "glClearStencil\n");
    glClearStencil(0);
    Output(1, "\n");
}

void CallClientActiveTexture(void)
{
    long i;
    GLint n;

    Output(1, "glClientActiveTexture\n");

    glGetIntegerv(GL_MAX_TEXTURE_UNITS, &n);

    for (i = 0; i < n; i++) {

        Output(1, "\t%s\n", enum_MultiTextureUnits[i].name);
        glClientActiveTexture(enum_MultiTextureUnits[i].value);
        ProbeEnum();
    }

    Output(1, "\n");
}

void CallColor(void)
{

    Output(1, "glColor4x\n");
    glColor4x(FIXED_ONE, FIXED_ONE, FIXED_ONE, FIXED_ONE);
#ifdef COMMON
    Output(1, "glColor4f\n");
    glColor4f(1.0, 1.0, 1.0, 1.0);
#endif

#ifdef GL_OES_VERSION_1_1
    Output(1, "glColor4ub\n");
    glColor4ub(1, 1, 1, 1);
#endif /* GL_OES_VERSION_1_1 */

    Output(1, "\n");
}

void CallColorMask(void)
{

    Output(1, "glColorMask\n");
    glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_TRUE);
    Output(1, "\n");
}

void CallColorPointer(void)
{

    Output(1, "glColorPointer\n");
    {
        GLubyte buf[4];
        buf[0] = 0;
        buf[1] = 0;
        buf[2] = 0;
        buf[3] = 255;
        Output(1, "\tGL_UNSIGNED_BYTE\n");
        glColorPointer(4, GL_UNSIGNED_BYTE, 0, buf);
        ProbeEnum();
    }
    {
        GLfixed buf[4];
        buf[0] = 0;
        buf[1] = 0;
        buf[2] = 0;
        buf[3] = FIXED_ONE;
        Output(1, "\tGL_FIXED\n");
        glColorPointer(4, GL_FIXED, 0, buf);
        ProbeEnum();
    }
#ifdef COMMON
    {
        GLfloat buf[4];
        buf[0] = 0.0f;
        buf[1] = 0.0f;
        buf[2] = 0.0f;
        buf[3] = 1.0f;
        Output(1, "\tGL_FLOAT\n");
        glColorPointer(4, GL_FLOAT, 0, buf);
        ProbeEnum();
    }
#endif

    Output(1, "\n");
}


static GLsizei GetNumPaletteEntries(GLenum format)
{
    switch (format) {
    default:
        return 0;

#ifdef GL_OES_compressed_paletted_texture
    case GL_PALETTE4_R5_G6_B5_OES:
    case GL_PALETTE4_RGB8_OES:
    case GL_PALETTE4_RGBA4_OES:
    case GL_PALETTE4_RGB5_A1_OES:
    case GL_PALETTE4_RGBA8_OES:
        return 16;

    case GL_PALETTE8_R5_G6_B5_OES:
    case GL_PALETTE8_RGB8_OES:
    case GL_PALETTE8_RGBA4_OES:
    case GL_PALETTE8_RGB5_A1_OES:
    case GL_PALETTE8_RGBA8_OES:
        return 256;
#endif /* GL_OES_compressed_paletted_texture */
    }
}

static GLsizei GetPixelSize(GLenum format)
{
    switch (format) {
    default:
        return 0;

#ifdef GL_OES_compressed_paletted_texture
    case GL_PALETTE4_R5_G6_B5_OES:
    case GL_PALETTE4_RGBA4_OES:
    case GL_PALETTE4_RGB5_A1_OES:
    case GL_PALETTE8_R5_G6_B5_OES:
    case GL_PALETTE8_RGBA4_OES:
    case GL_PALETTE8_RGB5_A1_OES:
        return 2;

    case GL_PALETTE4_RGB8_OES:
    case GL_PALETTE8_RGB8_OES:
        return 3;

    case GL_PALETTE4_RGBA8_OES:
    case GL_PALETTE8_RGBA8_OES:
        return 4;
#endif /* GL_OES_compressed_paletted_texture */
    }
}

static GLsizei
getCompressedPaletteSize(GLenum internalformat)
{
    return GetPixelSize(internalformat) * GetNumPaletteEntries(internalformat);
}

static GLsizei
getCompressedPixelsSize(GLenum internalformat, GLsizei width, GLsizei height, GLsizei border)
{
    GLsizei pixels;

    pixels = (width + border * 2) * (height + border * 2);

    switch (internalformat) {
#ifdef GL_OES_compressed_paletted_texture
    case GL_PALETTE4_RGB8_OES:
    case GL_PALETTE4_RGBA8_OES:
    case GL_PALETTE4_R5_G6_B5_OES:
    case GL_PALETTE4_RGBA4_OES:
    case GL_PALETTE4_RGB5_A1_OES:
    return ((pixels % 2) == 0) ? (pixels / 2) : (pixels / 2 + 1);

    case GL_PALETTE8_RGB8_OES:
    case GL_PALETTE8_RGBA8_OES:
    case GL_PALETTE8_R5_G6_B5_OES:
    case GL_PALETTE8_RGBA4_OES:
    case GL_PALETTE8_RGB5_A1_OES:
    return pixels;
#endif /* GL_OES_compressed_paletted_texture */

    default:
    return 0;
    }
}

static GLsizei
getCompressedTextureSize(GLenum internalformat, GLsizei width, GLsizei height, GLsizei border)
{
    return
    getCompressedPaletteSize(internalformat) +
    getCompressedPixelsSize(internalformat, width, height, border);
}

void CallCompressedTexImage2D(void)
{
#ifdef GL_OES_compressed_paletted_texture
    static const GLenum formats[] = {
    GL_PALETTE4_RGB8_OES,
    GL_PALETTE4_RGBA8_OES,
    GL_PALETTE4_R5_G6_B5_OES,
    GL_PALETTE4_RGBA4_OES,
    GL_PALETTE4_RGB5_A1_OES,
    GL_PALETTE8_RGB8_OES,
    GL_PALETTE8_RGBA8_OES,
    GL_PALETTE8_R5_G6_B5_OES,
    GL_PALETTE8_RGBA4_OES,
    GL_PALETTE8_RGB5_A1_OES
    };
    GLubyte buf[1025]; /* big enough for 256*1 byte palette + 1 byte image */
    GLsizei size;
    int i, j;

    Output(1, "glCompressedTexImage2D\n");
    for (i = 0; i != 10; i++) {
    size = getCompressedTextureSize(formats[i], 1, 1, 0);

        Output(1, "\t%s, with texture\n",
               GetEnumName(formats[i]));
        for (j = 0; j != sizeof(buf); j++) {
        buf[j] = 0;
        }
        glCompressedTexImage2D(GL_TEXTURE_2D, 0,
            formats[i], 1, 1, 0, size, buf);
        ProbeEnum();

        Output(1, "\t%s, NULL texture\n",
                   GetEnumName(formats[i]));
        glCompressedTexImage2D(GL_TEXTURE_2D, 0,
            formats[i], 1, 1, 0,
            size, (const GLvoid *)NULL);
        ProbeEnum();
    }
    Output(1, "\n");

#endif /* GL_OES_compressed_paletted_texture */
}

void CallCompressedTexSubImage2D(void)
{
    GLubyte buf[1000];
    GLenum format;
    GLsizei sub_size;
    int i;

#ifdef GL_OES_compressed_paletted_texture

    Output(1, "glCompressedTexSubImage2D\n");
    /* choose a format and size */
    format = 42;
    sub_size = 1;

    for (i = 0; i != sub_size; ++i) {
    buf[i] = 0;
    }

    Output(1, "\t<dummy format> with texture\n");
    glCompressedTexSubImage2D(GL_TEXTURE_2D, 0,
        0, 0, 1, 1, format, sub_size, buf);
    while (glGetError() != GL_NO_ERROR);
    ProbeEnum();

    Output(1, "\t<dummy format> NULL texture\n");
    glCompressedTexSubImage2D(GL_TEXTURE_2D, 0,
            0, 0, 1, 1, format, sub_size, buf);
    while (glGetError() != GL_NO_ERROR);
    ProbeEnum();

    Output(1, "\n");

#endif /* GL_OES_compressed_paletted_texture */
}

static GLenum GuessColorBufferFormat(void)
{
    GLsizei colorBits[4];

    glGetIntegerv(GL_RED_BITS, &colorBits[0]);
    glGetIntegerv(GL_GREEN_BITS, &colorBits[1]);
    glGetIntegerv(GL_BLUE_BITS, &colorBits[2]);
    glGetIntegerv(GL_ALPHA_BITS, &colorBits[3]);

    if (colorBits[0] == 0) {
        return GL_ALPHA;
    } else {
        if (colorBits[1] == 0 || colorBits[2] == 0) {
            if (colorBits[3] == 0) {
                return GL_LUMINANCE;
            } else {
                return GL_LUMINANCE_ALPHA;
            }
        } else {
            if (colorBits[3] == 0) {
                return GL_RGB;
            } else {
                return GL_RGBA;
            }
        }
    }
}

static GLsizei CalcTargetFormats(GLenum colorBufferFormat, GLenum *textureFormats)
{
    GLsizei i;

    i = 0;
    switch (colorBufferFormat) {
    case GL_ALPHA:
        textureFormats[i++] = GL_ALPHA;
        break;

    case GL_LUMINANCE:
        textureFormats[i++] = GL_LUMINANCE;
        break;

    case GL_LUMINANCE_ALPHA:
        textureFormats[i++] = GL_LUMINANCE;
        textureFormats[i++] = GL_LUMINANCE_ALPHA;
        textureFormats[i++] = GL_ALPHA;
        break;

    case GL_RGB:
        textureFormats[i++] = GL_RGB;
        textureFormats[i++] = GL_LUMINANCE;
        break;

    case GL_RGBA:
        textureFormats[i++] = GL_RGB;
        textureFormats[i++] = GL_RGBA;
        textureFormats[i++] = GL_LUMINANCE;
        textureFormats[i++] = GL_LUMINANCE_ALPHA;
        textureFormats[i++] = GL_ALPHA;
        break;
    }

    return i;
}

void CallCopyTexImage2D(void)
{
    GLenum colorBufferFormat, targetFormats[5];
    GLsizei numTargetFormats;
    int i;

    colorBufferFormat = GuessColorBufferFormat();
    numTargetFormats = CalcTargetFormats(colorBufferFormat, targetFormats);

    Output(1, "glCopyTexImage2D\n");
    for (i = 0; i != numTargetFormats; i++) {
        Output(1, "\t%s\n", GetEnumName(targetFormats[i]));
        glCopyTexImage2D(GL_TEXTURE_2D, 0,
                         targetFormats[i],
                         0, 0, 1, 1, 0);
    }
    ProbeEnum();
    Output(1, "\n");
}

void CallCopyTexSubImage2D(void)
{
    GLubyte buf[1000];
    GLenum colorBufferFormat, targetFormats[5];
    GLsizei numTargetFormats;
    int i;

    colorBufferFormat = GuessColorBufferFormat();
    numTargetFormats = CalcTargetFormats(colorBufferFormat, targetFormats);

    Output(1, "glCopyTexSubImage2D\n");
    ZeroBuf(GL_UNSIGNED_BYTE, 100, buf);

    for (i = 0; i != numTargetFormats; i++) {
        Output(1, "\t%s\n", GetEnumName(targetFormats[i]));
        Output(1, "\tnon-NULL Texture\n");
        glTexImage2D(GL_TEXTURE_2D, 0,  targetFormats[i], 1, 1, 0,
                 targetFormats[i], GL_UNSIGNED_BYTE, buf);
        glCopyTexSubImage2D(GL_TEXTURE_2D, 0,
                            0, 0, 0, 0, 1, 1);
        ProbeEnum();

        Output(1, "\tNULL Texture\n");
        glTexImage2D(GL_TEXTURE_2D, 0, targetFormats[i], 1, 1, 0,
                 targetFormats[i], GL_UNSIGNED_BYTE,
                 (const GLvoid *)NULL);
        glCopyTexSubImage2D(GL_TEXTURE_2D, 0,
                            0, 0, 0, 0, 1, 1);
        ProbeEnum();
    }
    Output(1, "\n");
}

void CallCullFace(void)
{
    long i;

    Output(1, "glCullFace\n");
    for (i = 0; enum_CullFaceMode[i].value != -1; i++) {
    Output(1, "\t%s\n", enum_CullFaceMode[i].name);
    glCullFace(enum_CullFaceMode[i].value);
    ProbeEnum();
    }
    Output(1, "\n");
}

#ifdef GL_OES_matrix_palette

void CallCurrentPaletteMatrixOES(void)
{
    Output(1, "glCurrentPaletteMatrixOES\n");
    glCurrentPaletteMatrixOES(0);
    ProbeEnum();
    Output(1, "\n");
}

#endif /* GL_OES_matrix_palette */

#ifdef GL_OES_VERSION_1_1
void CallClipPlane(void)
{
    GLint n;
    int i;
    GLfixed xEq[4] = { 0, 0, 0, 0 };
#ifdef COMMON
    GLfloat fEq[4] = { 0.f, 0.f, 0.f, 0.f };
#endif

    glGetIntegerv(GL_MAX_CLIP_PLANES, &n);

    Output(1, "glClipPlanex\n");
#ifdef COMMON
    Output(1, "glClipPlanef\n");
#endif

    for (i = 0; i < n && enum_ClipPlanes[i].value != -1; i ++)
    {
        Output(1, "\t%s\n", enum_ClipPlanes[i].name);
        glClipPlanex(enum_ClipPlanes[i].value, xEq);
#ifdef COMMON
        glClipPlanef(enum_ClipPlanes[i].value, fEq);
#endif
        ProbeEnum();
    }

    Output(1, "\n");
}
#endif
