/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/primtest/driver.c,v 1.3 2005/05/11 15:21:44 artlab Exp $
*/

#include "tproto.h"
#include "driver.h"
#include "shell.h"

driverRec driver[] = {
    {
	TEST_HINT,
	HintInit, HintSet, HintStatus, HintUpdate
    },
    {
	TEST_ALIAS,
	AliasInit, AliasSet, AliasStatus, AliasUpdate
    },
    {
	TEST_ALPHA,
	AlphaInit, AlphaSet, AlphaStatus, AlphaUpdate
    },
    {
	TEST_BLEND,
	BlendInit, BlendSet, BlendStatus, BlendUpdate
    },
    {
	TEST_DEPTH,
	DepthInit, DepthSet, DepthStatus, DepthUpdate
    },
    {
	TEST_DITHER,
	DitherInit, DitherSet, DitherStatus, DitherUpdate
    },
    {
	TEST_FOG,
	FogInit, FogSet, FogStatus, FogUpdate
    },
    {
	TEST_LIGHT,
	LightInit, LightSet, LightStatus, LightUpdate
    },
    {
	TEST_LOGICOP,
	LogicOpInit, LogicOpSet, LogicOpStatus, LogicOpUpdate
    },
    {
	TEST_SCISSOR,
	ScissorInit, ScissorSet, ScissorStatus, ScissorUpdate
    },
    {
	TEST_SHADE,
	ShadeInit, ShadeSet, ShadeStatus, ShadeUpdate
    },
    {
	TEST_STENCIL,
	StencilInit, StencilSet, StencilStatus, StencilUpdate
    },
    {
	TEST_TEXTURE,
	TextureInit, TextureSet, TextureStatus, TextureUpdate
    },
    {
	TEST_NULL
    }
};


static long TestProc(void)
{
    long i;

    i = 0;
    while (driver[i].test != TEST_NULL) {
	i++;
    }
    if (driver[i-1].enabled == 1) {
	return 0;
    }

    i = 0;
    for (;;) {
	if (driver[i].test == TEST_NULL) {
	    driver[0].enabled = 1;
	    return 1;
	} else if (driver[i].enabled == 1) {
	    driver[i].enabled = 0;
	    driver[i+1].enabled = 1;
	    return 1;
	}
	i++;
    }
}

long Driver(long op)
{
    long flag, i;

    switch (op) {
	case DRIVER_INIT:
	    for (i = 0; driver[i].test != TEST_NULL; i++) {
		(*driver[i].funcInit)((void *)&driver[i].data);
		driver[i].enabled = 0;
		driver[i].finish = 0;
		if (glGetError() != GL_NO_ERROR) {
		    Output(0, "primtest failed (driver init).\n\n");
		    return 0;
		}
	    }
	    return 1;
	case DRIVER_END:
		for (i = 0; driver[i].test != TEST_NULL; i++)
		{
			if(driver[i].data)
				FREE(driver[i].data);
		}
		return 1;
	case DRIVER_SET:
	    for (i = 0; driver[i].test != TEST_NULL; i++) {
		(*driver[i].funcSet)(driver[i].enabled, driver[i].data);
		if (glGetError() != GL_NO_ERROR) {
		    Output(0, "primtest failed (driver set).\n\n");
		    return 0;
		}
	    }
	    return 1;
	case DRIVER_STATUS:
	    for (i = 0; driver[i].test != TEST_NULL; i++) {
		(*driver[i].funcStatus)(driver[i].enabled, driver[i].data);
		if (glGetError() != GL_NO_ERROR) {
		    Output(0, "primtest failed (driver status).\n\n");
		    return 0;
		}
	    }
	    Output(1, "\n");
	    return 1;
	case DRIVER_UPDATE:
	    for (i = 0; driver[i].test != TEST_NULL; i++) {
		if (driver[i].enabled) {
		    driver[i].finish = (*driver[i].funcUpdate)(driver[i].data);
		    if (glGetError() != GL_NO_ERROR) {
			Output(0, "primtest failed (driver update).\n\n");
			return 0;
		    }
		} else {
		    driver[i].finish = 1;
		}
	    }
	    flag = 1;
	    for (i = 0; driver[i].test != TEST_NULL; i++) {
		if (driver[i].enabled && driver[i].finish == 0) {
		    flag = 0;
		}
	    }
	    if (flag) {
		return TestProc();
	    } else {
		return 1;
	    }
    }
    return 0;
}
