/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/primtest/hint.c,v 1.3 2005/05/11 15:21:44 artlab Exp $
*/

#include "shell.h"


typedef struct _hintRec {
    long index[2];
    GLenum value[2];
    char *name[2];
    enumRec *enumTable[2];
} hintRec;

#define ENUM(name) \
	{name, #name}

enumRec enum_HintTarget[] = {
    ENUM(GL_POINT_SMOOTH_HINT),
    ENUM(GL_LINE_SMOOTH_HINT),
    ENUM(GL_PERSPECTIVE_CORRECTION_HINT),
    ENUM(GL_FOG_HINT),
    {-1, "End of List"}
};

enumRec enum_HintMode[] = {
    ENUM(GL_NICEST),
    ENUM(GL_FASTEST),
    {-1, "End of List"}
};


void HintInit(void *data)
{
    hintRec *ptr;
    long i;

    ptr = (hintRec *)MALLOC(sizeof(hintRec));    
    *((void **)data) = (void *)ptr;

    ptr->enumTable[0] = enum_HintTarget;
    ptr->enumTable[1] = enum_HintMode;

    for (i = 0; i < 2; i++) {
	ptr->index[i] = 0;
	ptr->value[i] = ptr->enumTable[i]->value;
	ptr->name[i] = ptr->enumTable[i]->name;
    }
}

long HintUpdate(void *data)
{
    hintRec *ptr = (hintRec *)data;
    long flag, i;

    flag = 1;
    for (i = 1; i >= 0; i--) {
	if (flag) {
	    ptr->index[i] += 1;
	    if (ptr->enumTable[i][ptr->index[i]].value == -1) {
		ptr->index[i] = 0;
		ptr->value[i] = ptr->enumTable[i]->value;
		ptr->name[i] = ptr->enumTable[i]->name;
		flag = 1;
	    } else {
		ptr->value[i] = ptr->enumTable[i][ptr->index[i]].value;
		ptr->name[i] = ptr->enumTable[i][ptr->index[i]].name;
		flag = 0;
	    }
	}
    }

    return flag;
}

void HintSet(long enabled, void *data)
{
    hintRec *ptr = (hintRec *)data;

    if (enabled) {
	glHint(ptr->value[0], ptr->value[1]);
    } else {
	glHint(GL_POINT_SMOOTH_HINT, GL_DONT_CARE);
	glHint(GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_DONT_CARE);
	glHint(GL_FOG_HINT, GL_DONT_CARE);
    }
    Probe();
}

void HintStatus(long enabled, void *data)
{
    hintRec *ptr = (hintRec *)data;

    if (enabled) {
	Output(1, "Hint.\n");
	Output(1, "\t%s, %s.\n", ptr->name[0], ptr->name[1]);
    } else {
	Output(1, "Hint GL_DONT_CARE.\n");
    }
}
