/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/primtest/light.c,v 1.3 2005/05/11 15:21:44 artlab Exp $
*/

#include "shell.h"


typedef struct _lightRec {
    long index[2];
    GLenum value[2];
    char *name[2];
    enumRec *enumTable[2];
} lightRec;

#define ENUM(name) \
	{name, #name}

enumRec enum_Light_LocalViewer[] = {
    ENUM(GL_TRUE),
    ENUM(GL_FALSE),
    {-1, "End of List"}
};

enumRec enum_Light_TwoSided[] = {
    ENUM(GL_TRUE),
    ENUM(GL_FALSE),
    {-1, "End of List"}
};


void LightInit(void *data)
{
    static const GLenum buf1[] = {
	GL_LIGHT0, GL_LIGHT1, GL_LIGHT2, GL_LIGHT3,
	GL_LIGHT4, GL_LIGHT5, GL_LIGHT6, GL_LIGHT7
    };
    static const GLenum buf2[] = {
	GL_FRONT_AND_BACK
    };
    GLftype lm_ambient[] = {ITOF(42), ITOF(42), ITOF(42), ITOF(1)};
    GLftype l_ambient[] = {ITOF(42), ITOF(42), ITOF(42), ITOF(1)};
    GLftype l_diffuse[] = {ITOF(1)/2, ITOF(1), ITOF(1), ITOF(1)};
    GLftype l_specular[] = {ITOF(1)/2, ITOF(1), ITOF(1), ITOF(1)};
    GLftype l_position[] = {ITOF(BOXW), ITOF(BOXH), ITOF(-100), ITOF(1)};
    GLftype l_spot_expon[] = {ITOF(1)};
    GLftype l_spot_cutoff[] = {0};
    GLftype l_atten_const[] = {ITOF(1)};
    GLftype l_atten_lin[] = {ITOF(1)};
    GLftype l_atten_quad[] = {ITOF(1)};
    GLftype mat_shininess[] = {ITOF(30)};
    GLftype mat_specular[] = {ITOF(42), ITOF(42), ITOF(42), ITOF(1)};
    GLftype mat_diffuse[] = {ITOF(1)/2, ITOF(42), ITOF(42), ITOF(1)};
    GLftype mat_ambient[] = {ITOF(42), ITOF(42), ITOF(42), ITOF(1)};
    GLftype mat_emission[] = {0, 0, 0, ITOF(1)};
    lightRec *ptr;
    long i;

    lm_ambient[0] =
    lm_ambient[1] =
    lm_ambient[2] = ATOF("0.2");

    l_ambient[0] =
    l_ambient[1] =
    l_ambient[2] = ATOF("0.1");

    mat_specular[0] =
    mat_specular[1] =
    mat_specular[2] = ATOF("0.2");

    mat_diffuse[1] = ATOF("0.28");
    mat_diffuse[2] = ATOF("0.38");

    mat_ambient[0] =
    mat_ambient[1] =
    mat_ambient[2] = ATOF("0.2");

    ptr = (lightRec *)MALLOC(sizeof(lightRec));    
    *((void **)data) = (void *)ptr;

    ptr->enumTable[0] = enum_Light_LocalViewer;
    ptr->enumTable[1] = enum_Light_TwoSided;

    for (i = 0; i < 2; i++) {
	ptr->index[i] = 0;
	ptr->value[i] = ptr->enumTable[i]->value;
	ptr->name[i] = ptr->enumTable[i]->name;
    }

    glLightModeltv(GL_LIGHT_MODEL_AMBIENT, lm_ambient);
    for (i = 0; i < 1; i++) {
	glMaterialtv(buf2[i], GL_SHININESS, mat_shininess);
	glMaterialtv(buf2[i], GL_SPECULAR, mat_specular);
	glMaterialtv(buf2[i], GL_DIFFUSE, mat_diffuse);
	glMaterialtv(buf2[i], GL_EMISSION, mat_emission);
	glMaterialtv(buf2[i], GL_AMBIENT, mat_ambient);
    }
    for (i = 0; i < 8; i++) {
	glLighttv(buf1[i], GL_AMBIENT, l_ambient);
	glLighttv(buf1[i], GL_DIFFUSE, l_diffuse);
	glLighttv(buf1[i], GL_SPECULAR, l_specular);
	glLighttv(buf1[i], GL_POSITION, l_position);
	glLighttv(buf1[i], GL_SPOT_DIRECTION, l_position);
	glLighttv(buf1[i], GL_SPOT_EXPONENT, l_spot_expon);
	glLighttv(buf1[i], GL_SPOT_CUTOFF, l_spot_cutoff);
	glLighttv(buf1[i], GL_CONSTANT_ATTENUATION, l_atten_const);
	glLighttv(buf1[i], GL_LINEAR_ATTENUATION, l_atten_lin);
	glLighttv(buf1[i], GL_QUADRATIC_ATTENUATION, l_atten_quad);
	glEnable(buf1[i]);
    }
    Probe();
}

long LightUpdate(void *data)
{
    lightRec *ptr = (lightRec *)data;
    long flag, i;

    flag = 1;
    for (i = 1; i >= 0; i--) {
	if (flag) {
	    ptr->index[i] += 1;
	    if (ptr->enumTable[i][ptr->index[i]].value == -1) {
		ptr->index[i] = 0;
		ptr->value[i] = ptr->enumTable[i]->value;
		ptr->name[i] = ptr->enumTable[i]->name;
		flag = 1;
	    } else {
		ptr->value[i] = ptr->enumTable[i][ptr->index[i]].value;
		ptr->name[i] = ptr->enumTable[i][ptr->index[i]].name;
		flag = 0;
	    }
	}
    }

    return flag;
}

void LightSet(long enabled, void *data)
{
    lightRec *ptr = (lightRec *)data;
    GLftype buf[1];

    if (enabled) {
	buf[0] = ITOF(ptr->value[1]);
	glLightModeltv(GL_LIGHT_MODEL_TWO_SIDE, buf);
	glEnable(GL_LIGHTING);
    } else {
	glDisable(GL_LIGHTING);
    }
    Probe();
}

void LightStatus(long enabled, void *data)
{
    lightRec *ptr = (lightRec *)data;

    if (enabled) {
	Output(1, "Lighting on.\n");
	Output(1, "\t%s, %s.\n", ptr->name[0], ptr->name[1]);
    } else {
	Output(1, "Lighting off.\n");
    }
}
