/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/primtest/shell.c,v 1.5 2005/07/08 09:19:52 artlab Exp $
*/

#include "shell.h"
#include "driver.h"


long verbose;
long logFile;
char logFileName[80];
FILEHANDLE logFileHandle;

GLftype const rgbColorMap[][3] = {
    {ITOF(0), ITOF(0), ITOF(0)},
    {ITOF(1), ITOF(0), ITOF(0)},
    {ITOF(0), ITOF(1), ITOF(0)},
    {ITOF(1), ITOF(1), ITOF(0)},
    {ITOF(0), ITOF(0), ITOF(1)},
    {ITOF(1), ITOF(0), ITOF(1)},
    {ITOF(0), ITOF(1), ITOF(1)},
    {ITOF(1), ITOF(1), ITOF(1)},
    {ITOF(1)/2, ITOF(1)/2, ITOF(1)/2}
};


static void MakeIdentMatrix(GLftype *buf)
{
    long i;

    for (i = 0; i < 16; i++) {
	buf[i] = 0;
    }
    for (i = 0; i < 4; i++) {
	buf[i*4+i] = ITOF(1);
    }
}

static void Ortho2D(GLftype left, GLftype right, GLftype bottom, GLftype top)
{
    GLftype m[4][4], deltaX, deltaY;

    MakeIdentMatrix(&m[0][0]);
    deltaX = right - left;
    deltaY = top - bottom;
    m[0][0] = DIV(ITOF(2), deltaX);
    m[3][0] = -DIV((right + left), deltaX);
    m[1][1] = DIV(ITOF(2), deltaY);
    m[3][1] = -DIV((top + bottom), deltaY);
    m[2][2] = -ITOF(1);
    glMatrixMode(GL_PROJECTION);
    glLoadMatrixt(&m[0][0]);
    glMatrixMode(GL_MODELVIEW);
}

void Output(long level, char *format, ...)
{
    char	__output[1024];
    VA_LIST args;

    VA_START(args, format);
    if (verbose >= level) {
	VSPRINTF(__output, format, args);
        PRINTF1("%s", __output);
	FFLUSH(stdout);
        if (logFileHandle) {
            FPRINTF1(logFileHandle, "%s", __output);
            FFLUSH(logFileHandle);
        }
    }
    VA_END(args);
}

void Probe(void)
{

    if (glGetError() != GL_NO_ERROR) {
	Output(1, "\n");
#ifdef COMMON_LITE
	Output(0, "primtest (fixed point) failed.\n");
#endif
#ifdef COMMON
	Output(0, "primtest (single precision) failed.\n");
#endif
	Output(0, "\n");
	tkQuit();
    }
}

long Exec(TK_EventRec *ptr)
{
    unsigned long x;

    if (ptr->event == TK_EVENT_EXPOSE) {
	glViewport(0, 0, BOXW*2, BOXH*2);
	glScissor(0, 0, BOXW*2, BOXH*2);

	Ortho2D(-ITOF(BOXW), ITOF(BOXW), -ITOF(BOXH), ITOF(BOXH));
	glClearColort(0, 0, 0, ITOF(1));
	glClear(GL_COLOR_BUFFER_BIT  | GL_DEPTH_BUFFER_BIT);

	x = 0;
	Driver(DRIVER_INIT);
	do {
	    Driver(DRIVER_STATUS);
	    Driver(DRIVER_SET);
	    DrawPrims();
	    x++;
	} while (Driver(DRIVER_UPDATE));
	
	Driver(DRIVER_END);
	
	if (verbose) {
	    PRINTF1("\n%lu Combinations.\n", x);
	}

#ifdef COMMON_LITE
	Output(0, "primtest (fixed point) passed.\n");
#endif
#ifdef COMMON
	Output(0, "primtest (single precision) passed.\n");
#endif
	Output(0, "\n");

	return 0;
    }
    return 1;
}

static long Init(int argc, char **argv)
{
    long i;

    verbose = 0;

    for (i = 1; i < argc; i++) {
	if (STRCMP(argv[i], "-h") == 0) {
	    Output(0, "Options:\n");
	    Output(0, "\t-h\t\tPrint this help screen.\n");
	    Output(0, "\t-l\t<file>\tCreate a logfile.\n");
	    Output(0, "\t-v\t\tVerbose mode ON.\n");
	    Output(0, "\n");
        FFLUSH(stdout);
	    return 1;
	} else if (STRCMP(argv[i], "-v") == 0) {
	    verbose = 1;
	} else if (STRCMP(argv[i], "-l") == 0) {
            if (i+1 >= argc || argv[i+1][0] == '-') {
                Output(0, "-l (No logfile name).\n\n");
                return GL_TRUE;
            } else {
	    	logFile = 1;
                STRCPY(logFileName, argv[++i]);
                logFileHandle = FOPEN(logFileName, "w");
                if (logFileHandle != NULL) {
                    FCLOSE(logFileHandle);
                    Output(0, "Logfile %s created.\n\n", argv[i]);
                } else {
                    Output(0, "-l (Bad logfile name).\n\n");
                }
            }
	} else {
	    PRINTF1("%s (Bad option).\n", argv[i]);
	    FFLUSH(stdout);
	    return 1;
	}
    }
    return 0;
}

#if defined(CONFORM_ALTERNATE_MAIN)
  int conformain_prim(int argc, char **argv)
#elif defined(WINCE)
  int _tmain(int argc, _TCHAR* _argv[])
#else
  int main(int argc, char **argv)
#endif

{
#if defined(WINCE)
	int j=0;
	char* argv[64];
#endif
    TK_WindowRec wind;

#if defined(WINCE)
	for(j=0; j<argc; j++){
		int size;
		char* buff;
		size = WideCharToMultiByte(CP_ACP, 0, _argv[j], -1, NULL, 0, NULL, NULL);
		buff = malloc(size);
		WideCharToMultiByte(CP_ACP, 0, _argv[j], -1, buff, size, NULL, NULL);
		argv[j] = buff;
	}
#endif

    if (Init(argc, argv)) {
#if defined(WINCE)
	for(j=0; j<argc; j++){
		free(argv[j]);
	}
#endif
	tkQuit();
	return EXIT_FAILURE;
    }

    if (logFile) {
         logFileHandle = FOPEN(logFileName, "a");
    }

    Output(0, "OpenGL ES Primitives Test.\n");
    Output(0, "Version CONFORM_VERSION, CONFORM_DATE\n");
    Output(0, "\n");

    STRCPY(wind.name, "Primitives Test");

    wind.x = 0;
    wind.y = 50;
    wind.width = BOXW*2*10;
    wind.height = BOXH*2*10;

    wind.type = TK_WIND_REQUEST;

    wind.info = TK_WIND_STENCIL;
    if (tkNewWindow(&wind)) {
	tkExec(Exec);
    } else {
	wind.info = 0;
	if (tkNewWindow(&wind)) {
	    tkExec(Exec);
	} else {
	    Output(0, "Config requested not found.\n");
	}
    }
#if defined(WINCE)
	for(j=0; j<argc; j++){
		free(argv[j]);
	}
#endif
    tkQuit();
    FFLUSH(stdout);

    if (logFileHandle) {
        FCLOSE(logFileHandle);
    }
    return EXIT_SUCCESS;
}
