/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/primtest/stencil.c,v 1.3 2005/05/11 15:21:44 artlab Exp $
*/

#include "shell.h"


typedef struct _stencilRec {
    long index[4];
    GLenum value[4];
    char *name[4];
    enumRec *enumTable[4];
} stencilRec;

#define ENUM(name) \
	{name, #name}

enumRec enum_StencilFunc[] = {
    ENUM(GL_EQUAL),
    ENUM(GL_NOTEQUAL),
    ENUM(GL_LESS),
    ENUM(GL_LEQUAL),
    ENUM(GL_GREATER),
    ENUM(GL_GEQUAL),
    ENUM(GL_ALWAYS),
    ENUM(GL_NEVER),
    {-1, "End of List"}
};

enumRec enum_StencilOp[] = {
    ENUM(GL_KEEP),
    ENUM(GL_REPLACE),
    ENUM(GL_INCR),
    ENUM(GL_DECR),
    ENUM(GL_INVERT),
    ENUM(GL_ZERO),
    {-1, "End of List"}
};


void StencilInit(void *data)
{
    stencilRec *ptr;
    long i;

    ptr = (stencilRec *)MALLOC(sizeof(stencilRec));    
    *((void **)data) = (void *)ptr;

    ptr->enumTable[0] = enum_StencilFunc;
    ptr->enumTable[1] = enum_StencilOp;
    ptr->enumTable[2] = enum_StencilOp;
    ptr->enumTable[3] = enum_StencilOp;

    for (i = 0; i < 4; i++) {
	ptr->index[i] = 0;
	ptr->value[i] = ptr->enumTable[i]->value;
	ptr->name[i] = ptr->enumTable[i]->name;
    }
}

long StencilUpdate(void *data)
{
    stencilRec *ptr = (stencilRec *)data;
    long flag, i;

    flag = 1;
    for (i = 3; i >= 0; i--) {
	if (flag) {
	    ptr->index[i] += 1;
	    if (ptr->enumTable[i][ptr->index[i]].value == -1) {
		ptr->index[i] = 0;
		ptr->value[i] = ptr->enumTable[i]->value;
		ptr->name[i] = ptr->enumTable[i]->name;
		flag = 1;
	    } else {
		ptr->value[i] = ptr->enumTable[i][ptr->index[i]].value;
		ptr->name[i] = ptr->enumTable[i][ptr->index[i]].name;
		flag = 0;
	    }
	}
    }

    return flag;
}

void StencilSet(long enabled, void *data)
{
    stencilRec *ptr = (stencilRec *)data;

    if (enabled) {
	glStencilFunc(ptr->value[0], 0, ~0);
	glStencilOp(ptr->value[1], ptr->value[2], ptr->value[3]);
	glStencilMask(~0);
	glEnable(GL_STENCIL_TEST);
    } else {
	glDisable(GL_STENCIL_TEST);
    }
    Probe();
}

void StencilStatus(long enabled, void *data)
{
    stencilRec *ptr = (stencilRec *)data;

    if (enabled) {
	Output(1, "Stencil on.\n");
	Output(1, "\t%s, %s, %s, %s.\n", ptr->name[0], ptr->name[1],
	       ptr->name[2], ptr->name[3]);
    } else {
	Output(1, "Stencil off.\n");
    }
}
