/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/conform/primtest/texture.c,v 1.3 2005/05/11 15:21:44 artlab Exp $
*/

#include "shell.h"


typedef struct _textureRec {
    long index[8];
    GLenum value[8];
    char *name[8];
    enumRec *enumTable[8];
} textureRec;

#define ENUM(name) \
	{name, #name}

enumRec enum_TextureTarget[] = {
    ENUM(GL_TEXTURE_2D),
    {-1, "End of List"}
};

enumRec enum_TextureEnv[] = {
    ENUM(GL_MODULATE),
    ENUM(GL_DECAL),
    ENUM(GL_BLEND),
    {-1, "End of List"}
};

enumRec enum_S_TextureWrapMode[] = {
    ENUM(GL_REPEAT),
    ENUM(GL_CLAMP_TO_EDGE),
    {-1, "End of List"}
};

enumRec enum_T_TextureWrapMode[] = {
    ENUM(GL_REPEAT),
    ENUM(GL_CLAMP_TO_EDGE),
    {-1, "End of List"}
};

enumRec enum_TextureMinFilter[] = {
    ENUM(GL_NEAREST),
    ENUM(GL_LINEAR),
    ENUM(GL_NEAREST_MIPMAP_NEAREST),
    ENUM(GL_LINEAR_MIPMAP_NEAREST),
    ENUM(GL_NEAREST_MIPMAP_LINEAR),
    ENUM(GL_LINEAR_MIPMAP_LINEAR),
    {-1, "End of List"}
};

enumRec enum_TextureMagFilter[] = {
    ENUM(GL_NEAREST),
    ENUM(GL_LINEAR),
    {-1, "End of List"}
};

enumRec enum_TexImage_PixelFormat[] = {
    ENUM(GL_ALPHA),
    ENUM(GL_RGB),
    ENUM(GL_RGBA),
    ENUM(GL_LUMINANCE),
    ENUM(GL_LUMINANCE_ALPHA),
    {-1, "End of List"}
};

enumRec enum_TexImage_PixelType[] = {
    ENUM(GL_UNSIGNED_BYTE),
    ENUM(GL_UNSIGNED_BYTE),
    ENUM(GL_UNSIGNED_BYTE),
    ENUM(GL_UNSIGNED_BYTE),
    ENUM(GL_UNSIGNED_BYTE),
/* XXX what about other format-tied types? */
#if 0
    GL_BYTE, "GL_BYTE",
    GL_UNSIGNED_SHORT, "GL_UNSIGNED_SHORT",
    GL_SHORT, "GL_SHORT",
    GL_FLOAT, "GL_FLOAT",
#endif
    {-1, "End of List"}
};


static void TextureMake(GLenum format, GLenum type, long size, GLubyte const *src, GLubyte *dest)
{
    long extract, stride, i, j;

    size /= 4;
    switch (format) {
	default:
	    extract = 0;
	    stride = 0;
	    break;
	case GL_ALPHA:
	    src += 3;
	    extract = 1;
	    stride = 3;
	    break;
	case GL_RGB:
	    src += 0;
	    extract = 3;
	    stride = 1;
	    break;
	case GL_RGBA:
	    src += 0;
	    extract = 4;
	    stride = 0;
	    break;
	case GL_LUMINANCE:
	    src += 0;
	    extract = 1;
	    stride = 3;
	    break;
	case GL_LUMINANCE_ALPHA:
	    src += 0;
	    extract = 2;
	    stride = 2;
	    break;
    }
    switch (type) {
	case GL_UNSIGNED_BYTE:
	    {
		unsigned char *ptr = (unsigned char *)dest;
		for (i = 0; i < size; i++) {
		    for (j = 0; j < extract; j++) {
			*ptr++ = (unsigned char)(*src++);
		    }
		    src += stride;
		}
	    }
	    break;
        /* XXX should add 4_4_4_4, 5_5_5_1, 5_6_5 here */
    }
}

void TextureInit(void *data)
{
    static const GLftype color[4] = {ITOF(1)/2, ITOF(1)/2, ITOF(1)/2, ITOF(1)/2};
    textureRec *ptr;
    long i;

    ptr = (textureRec *)MALLOC(sizeof(textureRec));    
    *((void **)data) = (void *)ptr;

    ptr->enumTable[0] = enum_TextureTarget;
    ptr->enumTable[1] = enum_TextureEnv;
    ptr->enumTable[2] = enum_S_TextureWrapMode;
    ptr->enumTable[3] = enum_T_TextureWrapMode;
    ptr->enumTable[4] = enum_TextureMinFilter;
    ptr->enumTable[5] = enum_TextureMagFilter;
    ptr->enumTable[6] = enum_TexImage_PixelFormat;
    ptr->enumTable[7] = enum_TexImage_PixelType;

    for (i = 0; i < 8; i++) {
	ptr->index[i] = 0;
	ptr->value[i] = ptr->enumTable[i]->value;
	ptr->name[i] = ptr->enumTable[i]->name;
    }

    glTexEnvtv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, color);
    Probe();
}

long TextureUpdate(void *data)
{
    textureRec *ptr = (textureRec *)data;
    long flag, i;

    flag = 1;
    for (i = 7; i >= 0; i--) {
	if (flag) {
	    ptr->index[i] += 1;
	    if (ptr->enumTable[i][ptr->index[i]].value == -1) {
		ptr->index[i] = 0;
		ptr->value[i] = ptr->enumTable[i]->value;
		ptr->name[i] = ptr->enumTable[i]->name;
		flag = 1;
	    } else {
		ptr->value[i] = ptr->enumTable[i][ptr->index[i]].value;
		ptr->name[i] = ptr->enumTable[i][ptr->index[i]].name;
		flag = 0;
	    }
	}
    }

    return flag;
}

void TextureSet(long enabled, void *data)
{
    static const GLubyte textureSrc[4*4*4] = {
	  0,  0,  0,128, 255,255,255,128,   0,  0,  0,128, 255,255,255,128,
	255,255,255,128,   0,  0,  0,128, 255,255,255,128,   0,  0,  0,128,
	  0,  0,  0,128, 255,255,255,128,   0,  0,  0,128, 255,255,255,128,
	255,255,255,128,   0,  0,  0,128, 255,255,255,128,   0,  0,  0,128
    };
    textureRec *ptr = (textureRec *)data;
    GLftype buf[1];
    GLubyte textureDest[1000];
    GLsizei size;

    if (enabled) {
	buf[0] = (GLftype)ptr->enumTable[1]->value;
	glTexEnvtv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, buf);

	buf[0] = (GLftype)ptr->enumTable[2]->value;
	glTexParametert(ptr->enumTable[0]->value, GL_TEXTURE_WRAP_S, buf[0]);
	buf[0] = (GLftype)ptr->enumTable[3]->value;
	glTexParametert(ptr->enumTable[0]->value, GL_TEXTURE_WRAP_T, buf[0]);
	buf[0] = (GLftype)ptr->enumTable[4]->value;
	glTexParametert(ptr->enumTable[0]->value, GL_TEXTURE_MIN_FILTER, buf[0]);
	buf[0] = (GLftype)ptr->enumTable[5]->value;
	glTexParametert(ptr->enumTable[0]->value, GL_TEXTURE_MAG_FILTER, buf[0]);

	TextureMake(ptr->enumTable[6]->value, ptr->enumTable[7]->value, 32, textureSrc, textureDest);

        size = 4;
	switch (ptr->enumTable[0]->value) {
	    case GL_TEXTURE_2D:
		glTexImage2D(ptr->enumTable[0]->value, 0, ptr->enumTable[6]->value, size, size, 0, ptr->enumTable[6]->value, ptr->enumTable[7]->value, textureDest);
		glEnable(GL_TEXTURE_2D);
		break;
	}
    } else {
	glDisable(GL_TEXTURE_2D);
    }
    Probe();
}

void TextureStatus(long enabled, void *data)
{
    textureRec *ptr = (textureRec *)data;

    if (enabled) {
	Output(1, "Texture on.\n");
	Output(1, "\t%s.\n", ptr->name[1]);
	Output(1, "\tGL_S, %s, %s, GL_T, %s, %s.\n", ptr->name[2], ptr->name[4],
	       ptr->name[3], ptr->name[5]);
	Output(1, "\t%s, %s.\n", ptr->name[6], ptr->name[7]);
	Output(1, "\t%s.\n", ptr->name[0]);
    } else {
	Output(1, "Texture off.\n");
    }
}
