/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/ctk/ctke.c,v 1.3 2005/05/11 15:22:45 artlab Exp $
*/

#ifdef _WIN32
#include <windows.h>
#elif defined(OS_ANDROID) || defined(OS_LE__LINUX_CBSP_2_0)
    //Nothing...
#else
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#endif

#include <GLES/egl.h>
#include "ctk.h"

#if defined(__cplusplus) || defined(c_plusplus)
#define class c_class
#endif

/******************************************************************************/

#ifdef _WIN32
static HINSTANCE hInstance     = NULL;
static HWND      hWnd        = NULL;
static HWND      hWndParent    = NULL;
#elif defined(OS_ANDROID) || defined(OS_LE__LINUX_CBSP_2_0)
    //Nothing...
#else
static Display *display = 0;
static int screen = 0;
static XVisualInfo *visual = 0;
static Colormap colorMap;
static Window window = 0;
#endif

static EGLDisplay egl_display = EGL_NO_DISPLAY;
static EGLSurface egl_window = EGL_NO_SURFACE;
static EGLContext context = EGL_NO_CONTEXT;
static EGLConfig config;

static TK_EventRec event;

/******************************************************************************/

static long DisplayInit(void)
{
    int majorVersion, minorVersion;

#ifdef _WIN32
    hInstance = GetModuleHandle(NULL);
    egl_display = eglGetDisplay(EGL_DEFAULT_DISPLAY);
#elif defined(OS_ANDROID) || defined(OS_LE__LINUX_CBSP_2_0)
    egl_display = eglGetDisplay(EGL_DEFAULT_DISPLAY);
#else
    if (display)
    {
        return 1;
    }

    display = XOpenDisplay(NULL);
    if (display == NULL)
    {
        FPRINT(stderr, "Can't open display!\n");
        return 0;
    }

    screen = DefaultScreen(display);

    egl_display = eglGetDisplay(display);
#endif
    if (egl_display == EGL_NO_DISPLAY)
    {
        FPRINT(stderr, "Can't get display!\n");
        return 0;
    }

    if (!eglInitialize(egl_display, &majorVersion, &minorVersion))
    {
        FPRINT(stderr, "Can't initialize EGL display\n");
        return 0;
    }

    return 1;
}

static EGLConfig FindConfig(long type)
{
    EGLConfig config;
    EGLint num_config;
    EGLint list[32];
    int i;

    i = 0;

    list[i++] = EGL_LEVEL;
    list[i++] = 0;

    list[i++] = EGL_RED_SIZE;
    list[i++] = 1;

    list[i++] = EGL_DEPTH_SIZE;
    list[i++] = 12;

    if (type & TK_WIND_STENCIL) {
        list[i++] = EGL_STENCIL_SIZE;
        list[i++] = 1;
    }

    list[i++] = EGL_NONE;

    if (!eglChooseConfig(egl_display, list, &config, 1, &num_config) || num_config != 1) {
        /* XXX what is a null config? */
        return 0;
    }

    return config;
}

static long MakeConfigType(EGLConfig config)
{
    int stencilSize, depthSize;
    long mask;

    mask = 0;

    eglGetConfigAttrib(egl_display, config, EGL_STENCIL_SIZE, &stencilSize);
    if (stencilSize > 0) {
        mask |= TK_WIND_STENCIL;
    }

    eglGetConfigAttrib(egl_display, config, EGL_DEPTH_SIZE, &depthSize);
    if (depthSize > 0) {
        mask |= TK_WIND_Z;
    }

    return mask;
}

#if !defined(_WIN32) && !defined(OS_ANDROID) && !defined(OS_LE__LINUX_CBSP_2_0)

static Bool WaitForMapNotify(Display *d, XEvent *e, char *arg)
{
    if (e->type == MapNotify && e->xmap.window == window) {
        return GL_TRUE;
    }
    return GL_FALSE;
}
#endif

long tkNewWindow(TK_WindowRec *wind)
{
#ifdef _WIN32
    static int windowClassRegistered = 0;
#elif defined(OS_ANDROID) || defined(OS_LE__LINUX_CBSP_2_0)

#else
    XSetWindowAttributes wa;
    XTextProperty tp;
    XSizeHints sh;
    XEvent e;
    char *ptr;
#endif

    if (!DisplayInit()) 
    {
        return 0;
    }

    if (wind->type == TK_WIND_REQUEST)
    {
        config = FindConfig(wind->info);
        if (config == 0)
        {
            return 0;
        }
    }
    else
    {
        GLint attrib_list[3];
        EGLint count;

        attrib_list[0] = EGL_CONFIG_ID;
        attrib_list[1] = wind->info;
        attrib_list[2] = EGL_NONE;

        if (!eglChooseConfig(egl_display, attrib_list, &config, 1, &count) || count != 1)
        {
            return 0;
        }
    }
    wind->type = TK_WIND_REQUEST;
    wind->info = MakeConfigType(config);
#ifdef _WIN32
    {
        WNDCLASS    wc;
        RECT        clientRect;
        DWORD        dwStyle;
#if !defined(_WIN32_WCE)
        char        className[] = "GerberaClass";
#else
        wchar_t        className[] = TEXT("GerberaClass");
#endif

        if (!windowClassRegistered)
        {
            windowClassRegistered = 1;

            /* create class */
            wc.style            = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
            wc.lpfnWndProc        = DefWindowProc;
            wc.cbClsExtra        = 0;
            wc.cbWndExtra        = 0;
            wc.hInstance        = hInstance;
#if defined(_WIN32_WCE)
            wc.hIcon            = NULL;
#else
            wc.hIcon            = LoadIcon (NULL, IDI_APPLICATION);
#endif
            wc.hCursor            = LoadCursor (NULL, IDC_ARROW);
            wc.hbrBackground    = (HBRUSH)GetStockObject (BLACK_BRUSH);
            wc.lpszMenuName     = NULL;
            wc.lpszClassName    = className;

            /* make sure the class registers OK */
            if (!RegisterClass (&wc))
            {
                FPRINT(stderr, "Unable to register class!");
                return 0;
            }
        }

        /* calculate window size for default size client rectangle */
        clientRect.left     = wind->x;
        clientRect.top        = wind->y;
        clientRect.right    = wind->width;
        clientRect.bottom    = wind->height;

        dwStyle = /*WS_VISIBLE |*/ WS_CAPTION | WS_BORDER | WS_SYSMENU | WS_MINIMIZEBOX | WS_MAXIMIZEBOX | WS_THICKFRAME;

#if defined(_WIN32_WCE)
        AdjustWindowRectEx(&clientRect, dwStyle, 0, 0);
#else
        AdjustWindowRect(&clientRect, dwStyle, 0);
#endif

        /* position window to top left corner */
        clientRect.right  -= clientRect.left;
        clientRect.bottom -= clientRect.top;
        clientRect.left = 0;
        clientRect.top    = 0;

        /* create window */
        hWndParent = CreateWindowEx(0,
            className,
            TEXT("OpenGL ES Conformance Test"),
            dwStyle,
            clientRect.left,
            clientRect.top,
            clientRect.right-clientRect.left,
            clientRect.bottom-clientRect.top,
            NULL,            // no parent handle
            NULL,            // no menu handle
            hInstance,        // main program instance
            NULL);            // creation parameters

        if (!hWndParent)
        {
            FPRINT(stderr, "Unable to create parent window!");
            return 0;
        }

        hWnd = CreateWindowEx(0,
            className,
            TEXT(""),
            WS_CHILD,
            0,0,
            wind->width, wind->height,
            hWndParent,
            NULL,
            hInstance,
            NULL);

        if (!hWnd)
        {
            FPRINT(stderr, "Unable to create child window!");
            return 0;
        }
    }

    egl_window = eglCreateWindowSurface(egl_display, config, hWnd, NULL);
    if(egl_window == EGL_NO_SURFACE)
    {
        FPRINT(stderr, "Can't create EGL window\n");
        return 0;
    }
#elif defined(OS_ANDROID) || defined(OS_LE__LINUX_CBSP_2_0)

#ifdef USE_PBUFFER_SURFACE
        // If PBUFFER Surface is used then make sure that PBUFFER_BIT support
        // is enabled for the selected config ID
        GLint attrib_list[5];
        EGLint count;

        attrib_list[0] = EGL_CONFIG_ID;
        attrib_list[1] = wind->info;
        attrib_list[2] = EGL_SURFACE_TYPE;
        attrib_list[3] = EGL_PBUFFER_BIT;
        attrib_list[4] = EGL_NONE;

        static EGLint pbuffer_attrib_list[] =
        {
            EGL_WIDTH,  0,
            EGL_HEIGHT, 0,
            EGL_NONE
        };

        pbuffer_attrib_list[1] = wind->width;
        pbuffer_attrib_list[3] = wind->height;

        egl_window = eglCreatePbufferSurface(egl_display, config, pbuffer_attrib_list);
        if(egl_window == EGL_NO_SURFACE) {
            FPRINT(stderr, "Can't create EGL surface\n");
            return 0;
        }

#else

#if defined(OS_ANDROID)
    egl_window = eglCreateWindowSurface(egl_display, config, android_createDisplaySurface(), NULL);
#elif defined(OS_LE__LINUX_CBSP_2_0)
    egl_window = eglCreateWindowSurface(egl_display, config, updater_get_native_window(), NULL);
#endif

#endif //USE_PBUFFER_SURFACE

#else
    {
        EGLint visualId;
        XVisualInfo template_;
        int nVisuals;

        if (! eglGetConfigAttrib(egl_display, config, EGL_NATIVE_VISUAL_ID, &visualId)) {
            FPRINT(stderr, "Can't get visual ID\n");
        }

        template_.visualid = visualId;
        template_.screen = screen;
        visual = XGetVisualInfo(display, VisualIDMask | VisualScreenMask, &template_, &nVisuals);
        if (visual == NULL) {
            FPRINT(stderr, "Can't get visual\n");
        }
    }

    colorMap = XCreateColormap(display, RootWindow(display, screen),
        visual->visual, AllocNone);
    wa.colormap = colorMap;
    wa.background_pixmap = None;
    wa.border_pixel = 0;
    wa.event_mask = StructureNotifyMask | ExposureMask;

    window = XCreateWindow(display, RootWindow(display, screen), (int)wind->x,
        (int)wind->y, (unsigned int)wind->width,
        (unsigned int)wind->height, 0, visual->depth,
        InputOutput, visual->visual,
        CWBackPixmap|CWBorderPixel|CWEventMask|CWColormap,
        &wa);

    /*
    ** Set up window hints.
    */
    ptr = &wind->name[0];
    XStringListToTextProperty(&ptr, 1, &tp);

    sh.flags = USPosition | USSize | PMinSize | PMaxSize;
    sh.x = 0;
    sh.y = 0;
    sh.width = wind->width;
    sh.height = wind->height;
    sh.min_width = wind->width;
    sh.min_height = wind->height;
    sh.max_width = wind->width;
    sh.max_height = wind->height;
    XSetWMProperties(display, window, &tp, &tp, 0, 0, &sh, 0, 0);

    /*
    ** Map window and then wait for the window system to get around to it.
    */
    XMapWindow(display, window);
    XIfEvent(display, &e, WaitForMapNotify, 0);

    XSetWMColormapWindows(display, window, &window, 1);

    egl_window = eglCreateWindowSurface(egl_display, config, window, NULL);
    if (egl_window == EGL_NO_SURFACE) {
        FPRINT(stderr, "Can't create EGL window\n");
        return 0;
    }
#endif

    context = eglCreateContext(egl_display, config, NULL, NULL);
    if (!context)
    {
        FPRINT(stderr, "Can't create a context!\n");
        return 0;
    }

    if (!eglMakeCurrent(egl_display, egl_window, egl_window, context)) {
        FPRINT(stderr, "Can't make context current\n");
        return 0;
    }
    tkInitDispatch();
#if !defined(_WIN32) && !defined(OS_ANDROID) && !defined(OS_LE__LINUX_CBSP_2_0)
    XFlush(display);
#endif

    return 1;
}

void tkCloseWindow(void)
{
    if (!eglMakeCurrent(egl_display, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT))
        FPRINT(stderr, "Can't release current context\n");

    eglDestroySurface(egl_display, egl_window);
    eglDestroyContext(egl_display, context);

#ifndef USE_PBUFFER_SURFACE

#ifdef _WIN32
    DestroyWindow(hWnd);
    DestroyWindow(hWndParent);
#elif defined(OS_LE__LINUX_CBSP_2_0)
    updater_destroy_native_window(NULL);
#elif defined(OS_ANDROID)
    //Nothing...
#else
    XDestroyWindow(display, window);
    XFreeColormap(display, colorMap);
    XFree(visual);
#endif

#endif //!USE_PBUFFER_SURFACE
}

void tkSwapBuffers(void)
{
    eglSwapBuffers(egl_display, egl_window);
}

void tkQuit(void)
{
    eglTerminate(egl_display);
    EXIT(EXIT_SUCCESS);
}

#if defined(_WIN32) || defined(OS_ANDROID) || defined(OS_LE__LINUX_CBSP_2_0)
void tkExec(long (*Func)(TK_EventRec* ptr))
{
    event.event = 0;
    event.data[0] = 0;
    event.data[1] = 0;
    event.data[2] = 0;
    event.data[3] = 0;
    event.event = TK_EVENT_EXPOSE;
    for(;;)
    {
        static int cnt = 0;
        if(Func(&event) == 0)
            break;
        /* HG_ASSERT(0); */
    }
    return;
}
#else
static void GetNextEvent(void)
{
    XEvent current, ahead;

    event.event = 0;
    event.data[0] = 0;
    event.data[1] = 0;
    event.data[2] = 0;
    event.data[3] = 0;

    XNextEvent(display, &current);
    switch (current.type) {
      case MappingNotify:
          XRefreshKeyboardMapping((XMappingEvent *)&current);
          break;

      case Expose:
          while (XEventsQueued(current.xexpose.display, QueuedAfterReading) > 0) {
              XPeekEvent(current.xexpose.display, &ahead);
              if (ahead.type != Expose) {
                  break;
              }
              if (ahead.xexpose.window != current.xexpose.window) {
                  break;
              }
              XNextEvent(display, &current);
          }
          if (current.xexpose.count == 0) {
              event.event = TK_EVENT_EXPOSE;
              event.data[TK_WINDOWX] = current.xexpose.width;
              event.data[TK_WINDOWY] = current.xexpose.height;
          }
          break;
    }
}

void tkExec(long (*Func)(TK_EventRec *ptr))
{

    for (;;) {
        if (XPending(display)) {
            GetNextEvent();
            if (event.event == TK_EVENT_EXPOSE) {
                if ((*Func)(&event) == 0) {
                    break;
                }
            }
        }
    }
}
#endif

void tkGetConfigs(TK_ConfigsRec *ptr)
{
    GLint num_config;
    EGLConfig *configs;
    EGLint configId, caveatValue;
    int i;

    ptr->count = 0;
    ptr->nonConformantCount = 0;

    if (!DisplayInit()) {
        return;
    }

    eglGetConfigs(egl_display, NULL, 0, &num_config);
    configs = (EGLConfig *)MALLOC(num_config * sizeof(EGLConfig));
    eglGetConfigs(egl_display, configs, num_config, &num_config);

    for (i = 0; i < num_config; i++) {
        eglGetConfigAttrib(egl_display, configs[i], EGL_CONFIG_ID, &configId);
        eglGetConfigAttrib(egl_display, configs[i], EGL_CONFIG_CAVEAT, &caveatValue);

        if (caveatValue == EGL_NON_CONFORMANT_CONFIG) {
            ptr->nonConformantIDs[ptr->nonConformantCount++] = configId;
#ifdef OS_ANDROID
        } else if (caveatValue == EGL_SLOW_CONFIG) {
            ptr->nonConformantIDs[ptr->nonConformantCount++] = configId;
#endif
        } else {
            ptr->IDs[ptr->count++] = configId;
        }
    }

    FREE(configs);
}
