/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/ctk/ctkw.c,v 1.4 2005/05/11 15:22:45 artlab Exp $
*/

#define WIN32_LEAN_AND_MEAN 1
#include <windows.h>
#include <GLES/egl.h>
#include "ctk.h"

#if defined(__cplusplus) || defined(c_plusplus)
#define class c_class
#endif

/******************************************************************************/

static HDC hdc = 0;
static HWND hwnd = 0;

static EGLDisplay egl_display = EGL_NO_DISPLAY;
static EGLSurface egl_window = EGL_NO_SURFACE;
static EGLContext context = EGL_NO_CONTEXT;
static EGLConfig config;
static int majorVersion, minorVersion;

static TK_EventRec event;

/******************************************************************************/

static EGLConfig FindConfig(long type)
{
    EGLConfig config;
    EGLint num_config;
    EGLint list[32];
    int i;

    i = 0;

    list[i++] = EGL_LEVEL;
    list[i++] = 0;

    list[i++] = EGL_RED_SIZE;
    list[i++] = 1;

    list[i++] = EGL_DEPTH_SIZE;
    list[i++] = 12;

    if (type & TK_WIND_STENCIL) {
	list[i++] = EGL_STENCIL_SIZE;
	list[i++] = 1;
    }

    list[i++] = EGL_NONE;

    if (!eglChooseConfig(egl_display, list, &config, 1, &num_config) || num_config != 1) {
	/* XXX what is a null config? */
	return 0;
    }

    return config;
}

static long MakeConfigType(EGLConfig config)
{
    int stencilSize, depthSize;
    long mask;

    mask = 0;

    eglGetConfigAttrib(egl_display, config, EGL_STENCIL_SIZE, &stencilSize);
    if (stencilSize > 0) {
	mask |= TK_WIND_STENCIL;
    }

    eglGetConfigAttrib(egl_display, config, EGL_DEPTH_SIZE, &depthSize);
    if (depthSize > 0) {
	mask |= TK_WIND_Z;
    }

    return mask;
}

static long
NoOpExecFunc( TK_EventRec *pEvent ) {
    return(1);
}

static long (*ExecFunc)(TK_EventRec *pEvent ) = NoOpExecFunc;

static long tkWndProc(HWND hWnd, UINT message, DWORD wParam, LONG lParam)
{
    PAINTSTRUCT Paint;

    switch (message) {
    case WM_PAINT:
	BeginPaint( hWnd, &Paint );
	event.event = TK_EVENT_EXPOSE;
	event.data[TK_WINDOWX] = 0;
	event.data[TK_WINDOWY] = 0;
	if (!(*ExecFunc)(&event)) {
	    tkCloseWindow();
	}
	EndPaint( hWnd, &Paint );
	break;

    case WM_GETMINMAXINFO:
	{
	    LPMINMAXINFO lpmmi = (LPMINMAXINFO) lParam;
	    lpmmi->ptMinTrackSize.x = 1;
	    lpmmi->ptMinTrackSize.y = 1;
	}
	return 0;

    case WM_DESTROY:
#if 0
	CleanUp();
#endif
	PostQuitMessage(TRUE);
	return(DefWindowProc( hWnd, message, wParam, lParam));
    }
    return(DefWindowProc( hWnd, message, wParam, lParam));
}

long
tkNewWindow(TK_WindowRec *wind) {
    WNDCLASS wndclass;
    RECT WinRect;
    HANDLE hInstance;

    hInstance = GetModuleHandle(NULL);
    wndclass.style          = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
    wndclass.lpfnWndProc    = (WNDPROC)tkWndProc;
    wndclass.cbClsExtra     = 0;
    wndclass.cbWndExtra     = 0;
    wndclass.hInstance      = hInstance;
    wndclass.hIcon          = LoadIcon(NULL, IDI_APPLICATION);
    wndclass.hCursor        = LoadCursor(NULL, IDC_ARROW);
    wndclass.hbrBackground  = GetStockObject(WHITE_BRUSH);
    wndclass.lpszMenuName   = NULL;
    wndclass.lpszClassName  = "ctkLibWClass";

    RegisterClass(&wndclass);

    /* Make window large enough to hold a client area as large as wind */
    WinRect.left   = (wind->x == CW_USEDEFAULT) ? 0 : wind->x +10;
    WinRect.top    = (wind->y == CW_USEDEFAULT) ? 0 : wind->y +10;
    WinRect.right  = WinRect.left + wind->width;
    WinRect.bottom = WinRect.top + wind->height;

    AdjustWindowRect(&WinRect, WS_OVERLAPPEDWINDOW, FALSE);

    hwnd = CreateWindowEx(
                WS_EX_TOPMOST,
                wndclass.lpszClassName,
                wind->name,
                WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
                (wind->x == CW_USEDEFAULT ? CW_USEDEFAULT : WinRect.left),
                (wind->y == CW_USEDEFAULT ? CW_USEDEFAULT : WinRect.top),
                WinRect.right  - WinRect.left,
                WinRect.bottom - WinRect.top,
                NULL,
                NULL,
                hInstance,
                NULL);
     if (!hwnd) return 0;

    /*
     * Fixup window size in case minimum tracking size did something.
     */
    GetClientRect(hwnd, &WinRect);
    wind->width  = WinRect.right;
    wind->height = WinRect.bottom;

    hdc = GetDC(hwnd);
    egl_display = eglGetDisplay(hdc);
    if (!eglInitialize(egl_display, &majorVersion, &minorVersion)) {
	FPRINT(stderr, "eglInitialize failed!\n");
	goto out;
    }

    if (wind->type == TK_WIND_REQUEST) {
	config = FindConfig(wind->info);
	if (config == 0) {
	    return 0;
	}
    } else {
        GLint attrib_list[3];
	EGLint count;

	attrib_list[0] = EGL_CONFIG_ID;
	attrib_list[1] = wind->info;
	attrib_list[2] = EGL_NONE;

	if (!eglChooseConfig(egl_display, attrib_list, &config, 1, &count) || count != 1) {
	    goto out;
	}
    }
    wind->type = TK_WIND_REQUEST;
    wind->info = MakeConfigType(config);

    context = eglCreateContext(egl_display, config, EGL_NO_CONTEXT, NULL);
    if (!context) {
	FPRINT(stderr, "Can't create a context!\n");
	goto out;
    }

    ShowWindow(hwnd, SW_SHOWDEFAULT);
    egl_window = eglCreateWindowSurface(egl_display, config, hwnd, NULL);
    if (egl_window == EGL_NO_SURFACE) {
	FPRINT(stderr, "Can't create EGL window\n");
	goto out;
    }

    if (!eglMakeCurrent(egl_display, egl_window, egl_window, context)) {
	FPRINT(stderr, "Can't make context current\n");
	goto out;
    }
    tkInitDispatch();
    return 1;

out:
    ReleaseDC(hwnd, hdc);
    DestroyWindow(hwnd);
    /*
     *  Process all the messages
     */

    tkExec( (long (*)(TK_EventRec *pEvent))NULL );
    return 0;
}

void tkCloseWindow(void)
{
    ReleaseDC(hwnd, hdc);
    DestroyWindow(hwnd);
    eglDestroyContext(egl_display, context);
}

void tkSwapBuffers(void)
{
    eglSwapBuffers(egl_display, egl_window);
}

void tkQuit(void)
{
    eglTerminate(egl_display);
    EXIT(EXIT_SUCCESS);
}

void tkExec( long (*Func)(TK_EventRec *pEvent) )
{
    MSG Message;
    /* WM_SIZE gets delivered before we get here! */

    if ( NULL != Func ) {
        ExecFunc = Func;        /* save a pointer to the drawing function */
    } else {
        ExecFunc = NoOpExecFunc;
    }

    while (GL_TRUE) {
        if (GetMessage(&Message, NULL, 0, 0) ) {
            TranslateMessage(&Message);
            DispatchMessage(&Message);
        } else {
            break;
        }
    }
}

void tkGetConfigs(TK_ConfigsRec *ptr)
{
    GLint num_config;
    EGLConfig *configs;
    EGLint configId, caveatValue;
    int i;

    ptr->count = 0;
    ptr->nonConformantCount = 0;

    hdc = GetDC(NULL);
    egl_display = eglGetDisplay(hdc);
    eglInitialize(egl_display, &majorVersion, &minorVersion);

    eglGetConfigs(egl_display, NULL, 0, &num_config);
    configs = (EGLConfig *)MALLOC(num_config * sizeof(EGLConfig));
    eglGetConfigs(egl_display, configs, num_config, &num_config);

    for (i = 0; i < num_config; i++) {
	eglGetConfigAttrib(egl_display, configs[i], EGL_CONFIG_ID, &configId);
	eglGetConfigAttrib(egl_display, configs[i], EGL_CONFIG_CAVEAT, &caveatValue);

	if (caveatValue == EGL_NON_CONFORMANT_CONFIG) {
	    ptr->nonConformantIDs[ptr->nonConformantCount++] = configId;
	} else {
	    ptr->IDs[ptr->count++] = configId;
	}
    }

    FREE(configs);

    eglTerminate(egl_display);
    ReleaseDC(NULL, hdc);
}
