/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/ctk/glext.c,v 1.3 2005/05/11 15:22:45 artlab Exp $
*/

/* Some functions defined by OpenGL ES extensions may only be available
** dynamically at runtime. tkInitDispatch sets up function pointers
** to these functions; wrappers use these pointers to dispatch
** to the "real" GL entry points while allowing conform to statically
** link against the extensions.
**
** Each extension represented here is included only if a corresponding
** preprocessor symbol, TK_DISPATCH_<extension name>, is #defined. The
** currently known extensions are:
**
**      TK_DISPATCH_OES_query_matrix - OES_query_matrix wrappers
**      TK_DISPATCH_OES_matrix_palette - OES_matrix_palette wrappers
**      TK_DISPATCH_OES_draw_texture - OES_draw_texture wrappers
**
*/

#include "ctk.h"
#include <GLES/egl.h>

#ifdef TK_DISPATCH_OES_query_matrix
typedef GLbitfield (GL_APIENTRY*QueryMatrixxOES_func)(GLfixed [16], GLint [16]);
#endif

#ifdef TK_DISPATCH_OES_matrix_palette
typedef void (GL_APIENTRY*CurrentPaletteMatrixOES_func)(GLuint matrixpaletteindex);
typedef void (GL_APIENTRY*LoadPaletteFromModelViewMatrixOES_func)(void);
typedef void (GL_APIENTRY*MatrixIndexPointerOES_func)(GLint size, GLenum type, GLsizei stride, const GLvoid *pointer);
typedef void (GL_APIENTRY*WeightPointerOES_func)(GLint size, GLenum type, GLsizei stride, const GLvoid *pointer);
#endif

#ifdef TK_DISPATCH_OES_draw_texture
typedef void (GL_APIENTRY*DrawTexsOES_func)(GLshort x, GLshort y, GLshort z, GLshort width, GLshort height);
typedef void (GL_APIENTRY*DrawTexiOES_func)(GLint x, GLint y, GLint z, GLint width, GLint height);
typedef void (GL_APIENTRY*DrawTexxOES_func)(GLfixed x, GLfixed y, GLfixed z, GLfixed width, GLfixed height);

typedef void (GL_APIENTRY*DrawTexsvOES_func)(const GLshort *coords);
typedef void (GL_APIENTRY*DrawTexivOES_func)(const GLint *coords);
typedef void (GL_APIENTRY*DrawTexxvOES_func)(const GLfixed *coords);

typedef void (GL_APIENTRY*DrawTexfOES_func)(GLfloat x, GLfloat y, GLfloat z, GLfloat width, GLfloat height);
typedef void (GL_APIENTRY*DrawTexfvOES_func)(const GLfloat *coords);
#endif

#define FUNCTION(name) \
    name ## _func name

struct {
    int dummy; /* Work around compiler problem w/empty structure */

#ifdef TK_DISPATCH_OES_query_matrix
    FUNCTION(QueryMatrixxOES);
#endif /* TK_DISPATCH_OES_query_matrix */

#ifdef TK_DISPATCH_OES_matrix_palette
    FUNCTION(CurrentPaletteMatrixOES);
    FUNCTION(LoadPaletteFromModelViewMatrixOES);
    FUNCTION(MatrixIndexPointerOES);
    FUNCTION(WeightPointerOES);
#endif /* TK_DISPATCH_OES_matrix_palette */

#ifdef TK_DISPATCH_OES_draw_texture
    FUNCTION(DrawTexsOES);
    FUNCTION(DrawTexsvOES);
    FUNCTION(DrawTexiOES);
    FUNCTION(DrawTexivOES);
    FUNCTION(DrawTexfOES);
    FUNCTION(DrawTexfvOES);
    FUNCTION(DrawTexxOES);
    FUNCTION(DrawTexxvOES);
#endif /* TK_DISPATCH_OES_draw_texture */ 

} dispatch;

GLboolean tkCheckExtension(const char *name)
{
    const char *p = (const char *) glGetString(GL_EXTENSIONS);

    while ((p = STRSTR(p, name)) != NULL) {
        const char *q = p + STRLEN(name);

        if (*q == ' ' || *q == '\0') {
            return GL_TRUE;
        }
        p = q;
    }
    return GL_FALSE;
}

#define GETPROCADDRESS(name) \
    dispatch.name = (name ## _func)eglGetProcAddress("gl" #name);

void tkInitDispatch(void)
{
#ifdef TK_DISPATCH_OES_query_matrix
    if (tkCheckExtension("GL_OES_query_matrix")) {
        GETPROCADDRESS(QueryMatrixxOES);
    }
#endif /* TK_DISPATCH_OES_query_matrix */

#ifdef TK_DISPATCH_OES_matrix_palette
    if (tkCheckExtension("GL_OES_matrix_palette")) {
        GETPROCADDRESS(CurrentPaletteMatrixOES);
        GETPROCADDRESS(LoadPaletteFromModelViewMatrixOES);
        GETPROCADDRESS(MatrixIndexPointerOES);
        GETPROCADDRESS(WeightPointerOES);
    }
#endif /* TK_DISPATCH_OES_matrix_palette */

#ifdef TK_DISPATCH_OES_draw_texture
    if (tkCheckExtension("GL_OES_draw_texture")) {
        GETPROCADDRESS(DrawTexsOES);
        GETPROCADDRESS(DrawTexsvOES);
        GETPROCADDRESS(DrawTexiOES);
        GETPROCADDRESS(DrawTexivOES);
        GETPROCADDRESS(DrawTexfOES);
        GETPROCADDRESS(DrawTexfvOES);
        GETPROCADDRESS(DrawTexxOES);
        GETPROCADDRESS(DrawTexxvOES);
    }
#endif /* TK_DISPATCH_OES_draw_texture */
}

#define DISPATCH(func) (*dispatch.func)

#ifdef TK_DISPATCH_OES_query_matrix

/* OES_query_matrix entry points */

GLbitfield GL_APIENTRY glQueryMatrixxOES(GLfixed mantissa[16], GLint exponent[16])
{
    return DISPATCH(QueryMatrixxOES)(mantissa, exponent);
}

#endif /* TK_DISPATCH_OES_query_matrix */

#ifdef TK_DISPATCH_OES_matrix_palette

/* OES_matrix_palette entry points */

void GL_APIENTRY glCurrentPaletteMatrixOES(GLuint matrixpaletteindex)
{
    DISPATCH(CurrentPaletteMatrixOES)(matrixpaletteindex);
}

void GL_APIENTRY glLoadPaletteFromModelViewMatrixOES(void)
{
    DISPATCH(LoadPaletteFromModelViewMatrixOES)();
}

void GL_APIENTRY glMatrixIndexPointerOES(GLint size, GLenum type, GLsizei stride, const GLvoid *pointer)
{
    DISPATCH(MatrixIndexPointerOES)(size, type, stride, pointer);
}

void GL_APIENTRY glWeightPointerOES(GLint size, GLenum type, GLsizei stride, const GLvoid *pointer)
{
    DISPATCH(WeightPointerOES)(size, type, stride, pointer);
}

#endif /* TK_DISPATCH_OES_matrix_palette */

#ifdef TK_DISPATCH_OES_draw_texture

void GL_APIENTRY glDrawTexsOES (GLshort x, GLshort y, GLshort z, GLshort width, GLshort height)
{
    DISPATCH(DrawTexsOES)(x, y, z, width, height);
}

void GL_APIENTRY glDrawTexiOES (GLint x, GLint y, GLint z, GLint width, GLint height)
{
    DISPATCH(DrawTexiOES)(x, y, z, width, height);
}

void GL_APIENTRY glDrawTexxOES (GLfixed x, GLfixed y, GLfixed z, GLfixed width, GLfixed height)
{
    DISPATCH(DrawTexxOES)(x, y, z, width, height);
}

void GL_APIENTRY glDrawTexsvOES (const GLshort *coords)
{
    DISPATCH(DrawTexsvOES)(coords);
}

void GL_APIENTRY glDrawTexivOES (const GLint *coords)
{
    DISPATCH(DrawTexivOES)(coords);
}

void GL_APIENTRY glDrawTexxvOES (const GLfixed *coords)
{
    DISPATCH(DrawTexxvOES)(coords);
}

void GL_APIENTRY glDrawTexfOES (GLfloat x, GLfloat y, GLfloat z, GLfloat width, GLfloat height)
{
    DISPATCH(DrawTexfOES)(x, y, z, width, height);
}

void GL_APIENTRY glDrawTexfvOES (const GLfloat *coords)
{
    DISPATCH(DrawTexfvOES)(coords);
}

#endif /* TK_DISPATCH_OES_draw_texture */
