/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/fixed/fixed.h,v 1.6 2005/05/11 15:22:46 artlab Exp $
*/

#ifndef XMATH_H
#define XMATH_H

#include <platform/platform_stdlib.h>
#include <platform/platform_types.h>

typedef int32_t fixed;

/* limits */
#define FIXED_MIN       ((fixed) 0x80000000)
#define FIXED_MAX       (0x7FFFFFFF)
#define FIXED_NAN       (FIXED_MIN)
#define FIXED_EPSILON   (0x00000001)

/* constants */
#define FIXED_ZERO      (0x00000000)
#define FIXED_ONE       (0x00010000)
#define FIXED_HALF      (0x00008000)
#define FIXED_E         (0x0002B7E1)
#define FIXED_LOG2E     (0x00017154)
#define FIXED_LOG10E    (0x00006F2D)
#define FIXED_LN2       (0x0000B171)
#define FIXED_LN10      (0x00024D75)
#define FIXED_PI        (0x0003243F)
#define FIXED_PI_2      (0x0001921F)
#define FIXED_PI_4      (0x0000C90F)
#define FIXED_1_PI      (0x0000517C)
#define FIXED_2_PI      (0x0000A2FA)
#define FIXED_2_SQRTPI  (0x000120DC)
#define FIXED_SQRT2     (0x00016A09)
#define FIXED_SQRT1_2   (0x0000B504)

/* errors */
#define FIXED_NONE         0
#define FIXED_DOMAIN       1
#define FIXED_SINGULARITY  2
#define FIXED_OVERFLOW     3
#define FIXED_UNDERFLOW    4
#define FIXED_TOTAL_LOSS   5
#define FIXED_PARTIAL_LOSS 6

/* unchecked */
#define ITOX(i) ((i) << 16)
#define XTOI(f) ((f) >> 16)
#define XNEG(f) (-(f))
#define XADD(x, y) ((x) + (y))
#define XSUB(x, y) ((x) - (y))

struct xmath_exception {
    int type;
    const char * name;
    fixed arg1, arg2, retval;
};
typedef int (* xmatherr)(struct xmath_exception * const exception);

#ifdef __cplusplus
extern "C" {
#endif

extern xmatherr xsetmatherr(xmatherr const handler);

/* print */
extern int xprintf(char const * const format, ...);
extern int xfprintf(FILEHANDLE const file, char const * const format, ...);
extern int xsprintf(char * const string, char const * const format, ...);
extern int xvprintf(char const * const format, VA_LIST arg);
extern int xvfprintf(FILEHANDLE const file, char const * const format, VA_LIST arg);
extern int xvsprintf(char * const string, char const * const format, VA_LIST arg);

/* conversion functions */
extern fixed atox(char const * const s);
extern fixed itox(int const i);
extern int xtoi(fixed const x);
extern fixed xldexp(fixed const value, int const exp);
extern fixed xfrexp(fixed const value, int * const exp);

extern fixed xabs(fixed const f);
extern fixed xneg(fixed const f);
extern fixed xfloor(fixed const x);
extern fixed xceil(fixed const x);

/* basic arithmetic functions */
extern fixed xadd(fixed const x, fixed const y);
extern fixed xsub(fixed const x, fixed const y);
extern fixed xmul(fixed const x, fixed const y);
extern fixed xdiv(fixed const x_, fixed const y_);
extern fixed xmod(fixed const x, fixed const y);
extern fixed xmodx(const fixed value, fixed * const ip);

/* exponential and logarithm functions */
extern fixed xsqrt(fixed const x);
extern fixed xpow(fixed const x, fixed const y);
extern fixed xexp(fixed const value);
extern fixed xlog(fixed const x);
extern fixed xlog10(fixed const x);

/* trigonometric functions */
extern fixed xsin(fixed const angle);
extern fixed xcos(fixed const angle);
extern fixed xtan(fixed const angle);
extern fixed xsinh(fixed const arg);
extern fixed xcosh(fixed const arg);
extern fixed xtanh(fixed const arg);

/* inverse trigonometric functions */
extern fixed xasin(fixed const x);
extern fixed xacos(fixed const x);
extern fixed xatan(fixed const x);
extern fixed xatan2(fixed const y, fixed const x);

#ifdef __cplusplus
}
#endif

#endif /* ! XMATH_H */
