/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/fixed/float64.c,v 1.4 2005/05/11 15:22:46 artlab Exp $
*/

#include <assert.h>
#include <platform/platform.h>
#include "int32.h"
#include "int64.h"
#include "float64.h"

float64 const FLOAT64_ZERO    = {0x00000000, 0x00000000};
float64 const FLOAT64_ONE     = {0x00000001, 0x00000000};
float64 const FLOAT64_HALF    = {0x00000001, -1};
float64 const FLOAT64_EPSILON = {0x00000001, -30};
float64 const FLOAT64_LN2     = {0x2C5C85FD, -30};

static int
sprintf64_(char * const string, float64 const x, int const left_aligned, char const prefix, int const alternative, int const min_width, int accuracy, char format)
{
    int32_t ip;
    float64 y, z;
    int i, j, precision, exponent;

    assert(string);

    /* XXX too big for fixed */
    if (x.mantissa == INT32_MIN) {
        return SPRINTF1(string, "-2147483648*2^%d", x.exponent);
    }

    j = 0;

    if (x.mantissa < 0) {
        j += SPRINT(&string[j], "-");
    } else {
        switch (prefix) {
        case ' ':
        case '+':
            j += SPRINTF1(&string[j], "%c", prefix);
            break;
        }
    }

    y = fabs64(x);

    exponent = 0;

    switch (format) {
    case 'e':
    case 'E':
    case 'g':
    case 'G':
        z = y;

        if (z.mantissa == 0) {
            z.exponent = 0;
            exponent = 0;

        } else {
            while (fcompare64(z, itof64(10)) >= 0) {
                z = fdiv64(z, itof64(10));
                ++exponent;
            }

            while (fcompare64(z, FLOAT64_ONE) < 0) {
                z = fmul64(z, itof64(10));
                --exponent;
            }
        }
        break;
    }

    switch (format) {
    case 'e':
    case 'E':
        y = z;
        break;

    case 'g':
    case 'G':
        precision = (accuracy == 0) ? 1 : accuracy;

        if (z.mantissa != 0 && (exponent < -4 || exponent >= precision)) {
            /* convert G to E, g to e */
            format -= 2;
            accuracy = precision - 1;
            y = z;
        } else {
            format = 'f';
            if (z.mantissa == 0 && !alternative) {
                precision = 1;
            }
            accuracy = precision - 1;
            if (exponent < 0) {
                accuracy -= exponent;
            }
        }
        break;
    }

    ip = ishift32(y.mantissa, y.exponent);
    j += SPRINTF1(&string[j], "%d", ip);
    y = fsub64(y, itof64(ip));

    for (i = 0; i != accuracy; ++i) {
        if (i == 0) {
            j += SPRINT(&string[j], ".");
        }
        y = fmul64(y, itof64(10));
        ip = ishift32(y.mantissa, y.exponent);
        j += SPRINTF1(&string[j], "%d", ip);
        y = fsub64(y, itof64(ip));
    }

    if (accuracy == 0 && alternative) {
        j += SPRINT(&string[j], ".");
    }

    switch (format) {
    case 'e':
    case 'E':
        j += SPRINTF1(&string[j], "%c", format);
        j += SPRINTF1(&string[j], "%0+3d", exponent);
        break;
    }

    if (j < min_width) {
        if (left_aligned) {
            while (j != min_width) {
                j += SPRINT(&string[j], " ");
            }
        } else {
            memmove(&string[min_width - j], &string[0], j + 1);
            for (i = 0; i != min_width - j; ++i) {
                string[i] = ' ';
            }
            j = min_width;
        }
    }

    assert(j >= min_width);

    return j;
}

int
fsprintf64(char * const string, char const * const format, float64 const x)
{
    int i, left_aligned, alternative, min_width, accuracy;
    char prefix, format_;

    i = 0;

    assert(format[i] == '%');

    ++i;

    left_aligned = 0;
    prefix = '\0';
    alternative = 0;
    switch (format[i]) {
    case '-':
        left_aligned = 1;
        ++i;
        break;

    case '+':
    case ' ':
        prefix = format[i];
        ++i;
        break;

    case '#':
        alternative = 1;
        ++i;
        break;

    default:
        break;
    }

    min_width = 0;
    while (ISDIGIT((int)format[i])) {
        min_width = min_width * 10 + format[i] - '0';
        ++i;
    }

    if (format[i] == '.') {
        ++i;
        accuracy = 0;
        while (ISDIGIT((int)format[i])) {
            accuracy = accuracy * 10 + format[i] - '0';
            ++i;
        }
    } else {
        accuracy = 6;
    }

    switch (format[i]) {
    case 'F':
    case 'N':
    case 'h':
    case 'I':
    case 'L':
        ++i;
        break;
    }

    format_ = format[i];

    return sprintf64_(string, x, left_aligned, prefix, alternative, min_width, accuracy, format_);
}

#if 1
void
printf64(float64 const x)
{
    char string[256];
    sprintf64_(string, x, 0, '\0', 0, 0, 6, 'f');
    PRINTF1("%s", string);
}
#endif

float64
itof64(int const i)
{
    float64 x;

    x.mantissa = i;
    x.exponent = 0;

    return x;
}

float64
atof64(char const * const s)
{
    static const int base = 10;
    float64 x, y, scale;
    int i, period = 0;

    assert(s);

    i = 0;
    if (s[i] == '-' || s[i] == '+')
        ++i;

    x = FLOAT64_ZERO;

    for (; s[i] != '\0'; ++i) {

        char const character = s[i];

        if  (ISDIGIT((int)character)) {

            int const digit = (int) character - (int) '0';

            if (period) {
                scale = fmul64(scale, itof64(base));
                y = fdiv64(itof64(digit), scale);
                x = fadd64(x, y);
            } else {
                x = fmul64(x, itof64(base));
                x = fadd64(x, itof64(digit));
            }

        } else if (character == '.') {
            if (period)
                break;
            period = 1;
            scale = FLOAT64_ONE;
        }
    }

    if (s[0] == '-')
        return fneg64(x);

    return x;
}

int
f64toi(float64 const x)
{
    return ishift32(x.mantissa, x.exponent);
}

/*static */int shift_left(float64 * const x);

#if 0
int
fcompare64(float64 x, float64 y)
{
    shift_left(&x);
    shift_left(&y);

/* XXX broken for 0x7FFFF */

    if (x.mantissa == 0)
        return -y.mantissa;

    if (y.mantissa == 0)
        return x.mantissa;

    if (x.mantissa < 0 && y.mantissa > 0)
        return -1;

    if (y.mantissa < 0 && x.mantissa > 0)
        return +1;

    if (x.exponent < y.exponent) {
        if (x.mantissa < 0)
            return +1;
        else
            return -1;

    } else if (y.exponent < x.exponent) {
        if (y.mantissa < 0)
            return -1;
        else
            return +1;

    } else {
        return x.mantissa - y.mantissa;
    }
}

#else
/* XXX broken */
int
fcompare64(float64 const x, float64 const y)
{
    float64 z;

    z = fsub64(x, y);

    return z.mantissa;
}
#endif

float64
fneg64(float64 const x)
{
    float64 negated_x;

    if (x.mantissa == 0) {
        negated_x = x;
    } else if (x.mantissa == -x.mantissa) {
	negated_x.mantissa = -(x.mantissa / 2);
	negated_x.exponent = x.exponent + 1;
    } else {
	negated_x.mantissa = -x.mantissa;
	negated_x.exponent = x.exponent;
    }

    return negated_x;
}

float64
fabs64(float64 const x)
{
    float64 y;

    if (x.mantissa >= 0)
        return x;

    y.mantissa = -x.mantissa;
    y.exponent = x.exponent;

    return y;
}

float64
fshift64(float64 const x, int const count)
{
    float64 y;

    y.mantissa = x.mantissa;
    y.exponent = x.exponent + count;

    return y;
}

/*static */int
shift_left(float64 * const x)
{
    int32_t mantissa2;
    int i;

    assert(x);

    if (x->mantissa == 0) {
        x->exponent = 0;
        return 0;
    }

    mantissa2 = x->mantissa << 1;
    i = 0;

    while (!((x->mantissa ^ (x->mantissa << 1)) & (1 << 31))) {
/*    while ((x->mantissa > 0) == (mantissa2 > 0)) {*/
        x->mantissa = mantissa2;
        --x->exponent;
        mantissa2 = x->mantissa << 1;
        ++i;
    }

    return i;        
}

static int
shift_right(float64 * const x)
{
    int i;

    assert(x);

    if (x->mantissa == 0) {
        x->exponent = 0;
        return 0;
    }

    i = 0;
    while ((x->mantissa % 2) == 0) {
        x->mantissa = ishift32(x->mantissa, -1);
        ++x->exponent;
        ++i;
    }

    return i;
}

float64
fadd64(float64 const x_, float64 const y_)
{
    int32_t x_mantissa, y_mantissa, carry_bits;
    float64 x, y, z;

    x = x_;
    y = y_;

    if (x.mantissa == 0) {
        shift_right(&y);
        return y;
    }

    if (y.mantissa == 0) {
        shift_right(&x);
        return x;
    }

    shift_left(&x);
    shift_left(&y);

    if (x.exponent < y.exponent) {

        z.exponent = y.exponent;
        x_mantissa = ishift32_round(x.mantissa, x.exponent - y.exponent);
        y_mantissa = y.mantissa;

    } else if (y.exponent < x.exponent) {

        z.exponent = x.exponent;
        x_mantissa = x.mantissa;
        y_mantissa = ishift32_round(y.mantissa, y.exponent - x.exponent);

    } else {

        z.exponent = x.exponent;
        x_mantissa = x.mantissa;
        y_mantissa = y.mantissa;
    }

    z.mantissa = x_mantissa + y_mantissa;

    /* handle overflow */
    carry_bits = icarry32(x_mantissa, y_mantissa);
    if (ioverflow32(carry_bits)) {
        ++z.exponent;
        z.mantissa = ((uint32_t) z.mantissa >> 1) | (carry_bits & (1 << 31));
    }

    shift_right(&z);

    return z;
}

float64
fsub64(float64 const x, float64 const y)
{
    return fadd64(x, fneg64(y));
}

float64
fmul64(float64 const x, float64 const y)
{
    float64 z;
    int64 mantissa, mantissa2;

    z.exponent = x.exponent + y.exponent;
    mantissa = imul64(x.mantissa, y.mantissa);

    mantissa2 = ishift64(mantissa, 1);

    while (icompare64(mantissa, INT64_ZERO)
        * icompare64(mantissa2, INT64_ZERO) > 0 && 
        mantissa.lo != 0)
    {
        mantissa = mantissa2;
        --z.exponent;
        mantissa2 = ishift64(mantissa, 1);
    }

    z.mantissa = mantissa.hi;
    z.exponent += 32;

#if 1
    /* round */
    if (z.mantissa >= 0 && (mantissa.lo & (1 << 31))) {
        float64 one;
        one.mantissa = 1;
        one.exponent = z.exponent;
        z = fadd64(z, one);
    }
#endif

    if (z.mantissa == 0)
        z.exponent = 0;

    return z;
}

float64
fdiv64(float64 /*const*/ x, float64 /*const*/ y)
{
    float64 z;
    int64 x_, mantissa, mantissa2;

    /* improve accuracy */
    shift_left(&x);
    shift_right(&y);

    /* XXX kludge */
    if (x.mantissa == INT32_MIN) {
        x.mantissa /= 2;
        x.exponent += 1;
    }

    z.exponent = x.exponent - y.exponent;

    x_.hi = x.mantissa;
    x_.lo = 0;
    mantissa = idiv64(x_, y.mantissa);

    mantissa2 = ishift64(mantissa, 1);

    while (icompare64(mantissa, INT64_ZERO)
        * icompare64(mantissa2, INT64_ZERO) > 0 &&
        mantissa.lo != 0)
    {
        mantissa = mantissa2;
        --z.exponent;
        mantissa2 = ishift64(mantissa, 1);
    }

    z.mantissa = mantissa.hi;

#if 0
    /* round */
    if (z.mantissa >= 0 && (mantissa.lo & (1 << 31))) {
        float64 one;
        one.mantissa = 1;
        one.exponent = z.exponent;
        z = fadd64(z, one);
    }
#endif

    return z;
}

static float64
fexp_naive(float64 const x)
{   
    float64 s, sn, a;
    int i; 
        
    assert(fcompare64(x, fneg64(FLOAT64_ONE)) > 0);

    sn = a = FLOAT64_ONE;
    i = 0;

    do {
        ++i;
        s = sn;
        a = fmul64(a, x);
        a = fdiv64(a, itof64(i));
        sn = fadd64(sn, a);
    } while (fcompare64(sn, s) != 0);
    
    return sn;
}

float64
fexp64(float64 const x)
{
    float64 sn, z;
    int m;
 
    m = 0;
    z = x;

    while (fcompare64(fabs64(z), fshift64(FLOAT64_LN2, -1)) > 0) {
        if (z.mantissa < 0) {
            --m;
            z = fadd64(z, FLOAT64_LN2);
        } else {
            ++m;
            z = fsub64(z, FLOAT64_LN2);
        }
    }

    sn = fexp_naive(z);
    sn = fshift64(sn, m);

    return sn;
}

/* limited to (0, 1] */
static float64
flog_naive(float64 const value)
{
    float64 sn, s, a, x, d;
    int i;

    assert(value.mantissa > 0 && fcompare64(value, FLOAT64_ONE) <= 0);

    sn = FLOAT64_ZERO;
    x = fsub64(FLOAT64_ONE, value);
    d = FLOAT64_ONE;
    i = 0;

    do {
        ++i;
        s = sn;
        d = fmul64(d, x);
        a = fdiv64(d, itof64(i));
        sn = fsub64(s, a);

    } while (fcompare64(sn, s) != 0);

    return sn;
}

#ifndef OLD_LOG_IMPLEMENTATION
float64
flog64(float64 x)
{
    int negate = 0;
    float64 y, exponent, fix;

    /* bring value into range (0, 1] */
    if (fcompare64(x, FLOAT64_ONE) > 0) {
        x = fdiv64(FLOAT64_ONE, x);
        negate = 1;
    }

    /* bring value into range [0.5, 1] */
    exponent.mantissa = 0;
    exponent.exponent = 0;
    while (fcompare64(x, FLOAT64_HALF) < 0) {
        ++x.exponent;
        ++exponent.mantissa;
    }

    fix = fmul64(FLOAT64_LN2, exponent);
    y = fsub64(flog_naive(x), fix);

    return negate ? fneg64(y) : y;
}
#else
float64
flog64(float64 const x)
{
    float64 y;

    /* bring value into range (0, 1] */
    if (fcompare64(x, FLOAT64_ONE) > 0) {
        y = fdiv64(FLOAT64_ONE, x);
        return fneg64(flog_naive(y));

    } else {
        return flog_naive(x);
    }
}
#endif

float64
flog1064(float64 const x)
{
    return fdiv64(flog64(x), flog64(itof64(10)));
}

float64
fpow64(float64 const x, float64 const y)
{
    if (x.mantissa == 0) {
        if (y.mantissa == 0)
            return FLOAT64_ONE;
        else
            return FLOAT64_ZERO;
    }

    return fexp64(fmul64(y, flog64(x)));
}
