/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/fixed/int32.c,v 1.2 2005/05/11 15:22:46 artlab Exp $
*/

#include <platform/platform.h>
#include "int32.h"

void
iprintb32(int32_t const x)
{
   int i;

    for (i = 31; i >= 0; --i) {
        if (i == 15)
            PRINT(".");
        PRINT(x & (1 << i) ? "1" : "0");
    }
}

int32_t
icarry32(int32_t const x, int32_t const y)
{
    uint32_t mask, xor_, and_, carry_bits;
    int i, carry;

    xor_ = x ^ y;
    and_ = x & y;

    carry = 0;
    carry_bits = 0;
    mask = 1;

    for (i = 0; i != 32; ++i) {
        carry = (mask & ((carry ? xor_ : 0) | and_)) ? 1 : 0;
        carry_bits |= (carry ? mask : 0);
        mask <<= 1;
    }

    return carry_bits;
}

int
ioverflow32(int32_t carry_bits)
{
    return (carry_bits ^ (carry_bits << 1)) & (1 << 31);
}

int
icompare32(int32_t const x, int32_t const y)
{
    if (x < y)
        return -1;
    else if (y < x)
        return +1;
    else
        return 0;
}

int32_t
ishift32(int32_t const x, int const count)
{
    if (count == 0)
        return x;

    if (count <= -32)
        return 0;

    if (count >= 32)
        return 0;

    if (count < 0) {
        if (x < 0) {
            if (x == -x)
                return ishift32(x / 2, count + 1);
            else {
                return (((uint32_t) x) >> -count) | ((1 << -count) - 1) << (32 + count);
            }
        } else
            return x >> -count;

    } else if (count > 0) {
        return x << count;

    } else {
        return x;
    }
}

int32_t
ishift32_round(int32_t const x, int const count)
{
    int32_t y;

    y = ishift32(x, count);

    if (x <= 0 || count >= 0 || count < -32)
        return y;

    if (x & (1 << (-count - 1)))
        return y + 1;

    return y;
}
