/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/fixed/int64.c,v 1.3 2005/05/11 15:22:46 artlab Exp $
*/

#include <assert.h>
#include "int32.h"
#include "int64.h"

/* constants */
int64 const INT64_MIN     = {0x80000000, 0x00000000};
int64 const INT64_MAX     = {0x7FFFFFFF, 0xFFFFFFFF};
int64 const INT64_ZERO    = {0x00000000, 0x00000000};
int64 const INT64_ONE     = {0x00000000, 0x00000001};

int64
itoi64(int const x)
{
    int64 y;

    y.hi = (x & (1 << 31)) ? ~0 : 0;
    y.lo = x;

    return y;
}

int
i64toi(int64 const x)
{
    return x.lo;
}

int64
ineg64(int64 const x)
{
    int64 neg_x;

    neg_x.hi = ~x.hi;
    neg_x.lo = ~x.lo;

    return iadd64(neg_x, INT64_ONE);
}

int64
iabs64(int64 const x)
{
    if ((int32_t) x.hi >= 0)
        return x;

    return ineg64(x);
}

int64
ishift64(int64 x, int count)
{
    if (count == 0)
        return x;

    if (count <= -64)
        return INT64_ZERO;

    if (count >= 64)
        return INT64_ZERO;

    if (count >= 32) {
        x.hi = x.lo;
        x.lo = 0;
        return ishift64(x, count - 32);
    }

    if (count <= -32) {
        x.lo = x.hi;
        x.hi = (x.hi & (1 << 31)) ? 0xFFFFFFFF : 0;
        return ishift64(x, count + 32);
    }

    if (count < 0) {
        uint32_t const mask = (1 << -count) - 1;

        count = -count;
        x.lo >>= count;
        x.lo |= (mask & x.hi) << (32 - count);

        if ((int32_t) x.hi < 0) {
            x.hi = (uint32_t) x.hi >> count;
            x.hi |= mask << (32 - count);
        } else {
            x.hi >>= count;
        }

    } else if (count > 0) {
        if (count == 32) {
            x.hi = x.lo;
            x.lo = 0; 

        } else {
            uint32_t const mask = (1 << count) - 1;

            x.hi <<= count;
            x.hi |= ((mask << (32 - count)) & x.lo) >> (32 - count);
            x.lo <<= count;
        }
    }

    return x;
}

static int
compare_unsigned(uint32_t const x, uint32_t const y)
{
    if (x < y)
        return -1;
    else if (y < x)
        return +1;
    else
        return 0;
}

int
icompare64(int64 const x, int64 const y)
{
    if (x.hi == y.hi)
        return compare_unsigned(x.lo, y.lo);
    else
        return icompare32((int32_t) x.hi, (int32_t) y.hi);
}

int64
iadd64(int64 const x, int64 const y)
{
    int64 z;
    int32_t carry_bits;

    carry_bits = icarry32((int32_t) x.lo, (int32_t) y.lo);
    z.hi = x.hi + y.hi + ((carry_bits & (1 << 31)) ? 1 : 0);
        /*ioverflow32((int32_t) x.lo, (int32_t) y.lo);*/
    z.lo = x.lo + y.lo;

    return z;
}

int64
isub64(int64 const x, int64 const y)
{
    return iadd64(x, ineg64(y));
}

int64
imul64(int32_t const x, int32_t const y)
{   
    uint32_t x0, x1, y0, y1;
    int64 z, t;

    x0 = ishift32(x, -16);
    x1 = x & 0x0000FFFF;

    y0 = ishift32(y, -16);
    y1 = y & 0x0000FFFF;

    z.hi = x0 * y1 + x1 * y0;
    z.lo = 0;
    z = ishift64(z, -16); 

    t.hi = x0 * y0;
    t.lo = 0;
    z = iadd64(z, t);

    t.hi = 0;
    t.lo = x1 * y1;
    z = iadd64(z, t);

    return z;
}

#ifndef OLD_IDIV64_IMPLEMENTATION
/* Hybrid's replacement idiv64() routine */
#include "hgdiv64.c"

#else 

int64
idiv64(int64 const x_, int32_t const y_)
{
    int64 x, y, c, d, delta_c, delta_d, delta_d2;

    x = iabs64(x_);

    y.hi = y_;
    y.lo = 0;

    y = iabs64(y);
    y = ishift64(y, -32);

    d = x;
    c.hi = c.lo = 0;

    /* invariant: x = y * c + d */
    while (icompare64(d, y) >= 0) {
        delta_d = y;
        delta_c.hi = 0;
        delta_c.lo = 1;

        delta_d2 = delta_d;
        delta_d2 = ishift64(delta_d2, 1);

        while ((delta_d2.hi & (1 << 31)) == 0 && icompare64(delta_d2, d) <= 0) {
            delta_d = delta_d2;
            delta_c = ishift64(delta_c, 1);

            delta_d2 = delta_d;
            delta_d2 = ishift64(delta_d2, 1);
        }
        /* post cond: delta_d <= d */

        d = isub64(d, delta_d);
        c = iadd64(c, delta_c);
    }

    if (((int32_t) x_.hi < 0) == (y_ < 0))
        return c;
    else
        return ineg64(c);
}
#endif 
