/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/fixed/mandel.c,v 1.3 2005/05/11 15:22:46 artlab Exp $
*/

#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include "fixed.h"

static const fixed FOUR = ITOX(4);

static int
error_handler(struct xmath_exception * const exception)
{
    assert(exception != NULL);

    /* ignore underflow */
    if (exception->type == FIXED_UNDERFLOW)
        return 1;

    return 0;
}

int
mandelbrot(const fixed cr, const fixed ci, const int n)
{
    int i = -1;
    fixed zr = 0, zi = 0, a, b;

    do { 
        i++;
        a = xmul(zr, zr); 
        b = xmul(zi, zi);
        zi = xadd(ci, xmul(zr, zi) * 2);
        zr = xadd(cr, xsub(a, b));
    } 
    while (i < n && xadd(a, b) < FOUR);

    return i;
}

int
main()
{
    static const int width = 80;
    static const int height = 24;
    static const int n = 64;
    int i, j;

    xsetmatherr(error_handler);

    for (i = 0; i != height; ++i) {
        const fixed y = xsub(itox(i) * 2 / height, itox(1));

        for (j = 0; j != width; ++j) {
            const fixed x = xsub(itox(j) * 4 / width, itox(2));
            const int value = mandelbrot(x, y, n);
            if (value == n) {
                printf(" ");
            } else {
                printf("%c", '0' + value);
            }
        }
        printf("\n");
    }

    assert(errno == 0);
    return EXIT_SUCCESS;
}
