/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/fixed/test.c,v 1.3 2005/05/11 15:22:46 artlab Exp $
*/

#undef NDEBUG

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include "int32.h"
#include "fixed.h"

static void
printValue(char const * const name, fixed const x)
{
    printf("%-10s = ", name);
    iprintb32(x);
    xprintf(" (%0#10X, %f)\n", x, x);
    /* XXX is leading zero ANSI C89 or not? */
}

static void
assert_range_(const int line, const fixed actual, const fixed expected, const fixed tolerance)
{
    if (xabs(actual - expected) <= tolerance)
        return;

    printf("line = %d\n", line);
    printValue("actual", actual);
    printValue("expected", expected);
    printValue("tolerance", tolerance);
}

#define assert_equals(actual, expected) assert_range_(__LINE__, actual, expected, 0)
#define assert_range(actual, expected, tolerance) assert_range_(__LINE__, actual, expected, tolerance)

static int error = FIXED_NONE; 

static int
error_handler(struct xmath_exception * exception)
{
    assert(exception != NULL);
    error = exception->type;
    return 1;
}

static void
test_atox(void)
{
    error = FIXED_NONE;
    assert_equals(atox("0"), itox(0));
    assert_equals(atox("42"), itox(42));
    assert_equals(atox("-42"), itox(-42));
    assert_equals(atox(".5") * 2, itox(1));
    assert_equals(atox("+3.75"), xdiv(itox(375), itox(100)));
    assert_equals(atox("+3.75"), itox(+15) / 4);
    assert_equals(atox("-3.75"), itox(-15) / 4);
    assert(atox("0.3333") < itox(1));
    assert(atox("0.33333") < itox(1));
    assert(error == FIXED_NONE);

    atox("100000.0");
    assert(error == FIXED_OVERFLOW);

    error = FIXED_NONE;
    atox("-100000.0");
    assert(error == FIXED_OVERFLOW);
}

static void
test_printf(void)
{
    char string[256];
    size_t length;

    length = (size_t)xsprintf(string, "%d", 42);
    assert(length == STRLEN(string) && STRCMP(string, "42") == 0);

    length = (size_t)xsprintf(string, "%f", itox(42));
    assert(length == STRLEN(string) && STRCMP(string, "42.000000") == 0);

    length = (size_t)xsprintf(string, "%e", itox(1));
    assert(length == STRLEN(string) && STRCMP(string, "1.000000e+00") == 0);

    length = (size_t)xsprintf(string, "%E", itox(1));
    assert(length == STRLEN(string) && STRCMP(string, "1.000000E+00") == 0);

    length = (size_t)xsprintf(string, "%e", itox(10));
    assert(length == STRLEN(string) && STRCMP(string, "1.000000e+01") == 0);

    length = (size_t)xsprintf(string, "%e", itox(1)/2);
    assert(length == STRLEN(string) && STRCMP(string, "5.000000e-01") == 0);

#if 0
    /* rounding error */
    length = xsprintf(string, "%e", xldexp(itox(1), -13));
    assert(length == STRLEN(string) && STRCMP(string, "1.22i0000e-04");
#endif

    length = (size_t)xsprintf(string, "%f", xldexp(itox(1), -13));
    assert(length == STRLEN(string) && STRCMP(string, "0.000122") == 0);

#if 0
    /* rounding error */
    length = xsprintf(string, "%g", xldexp(itox(1), -13));
    assert(length == STRLEN(string) && STRCMP(string, "0.000122") == 0);
#endif

    length = (size_t)xsprintf(string, "%.3g", xldexp(itox(1), -13));
    assert(length == STRLEN(string) && STRCMP(string, "0.000122") == 0);

#if 0
    length = xsprintf(string, "%e", itox(42));
    assert(length == STRLEN(string) && STRCMP(string, "4.200000e+01") == 0);

    length = xsprintf(string, "%E", itox(42));
    assert(length == STRLEN(STRING) && STRCMP(string, "4.200000E+01") == 0);
#endif

    {
        static int const values[4] = {1, 2, 3, 4};
        length = (size_t)xsprintf(string, "(%d, %d, %d, %d)",
	    values[0], values[1], values[2], values[3]);
	assert(length == STRLEN(string));
	assert(STRCMP(string, "(1, 2, 3, 4)") == 0);
    }

    length = (size_t)xsprintf(string, "%g", atof(".0"));
    assert(length == STRLEN(string) && STRCMP(string, "0") == 0);

    length = (size_t)xsprintf(string, "%#g", atof(".0"));
    assert(length == STRLEN(string) && STRCMP(string, "0.00000") == 0);
}

static void
test_constants(void)
{
    assert_equals(FIXED_MIN, 0x80000000);
    assert_equals(FIXED_MAX, 0x7FFFFFFF);
    assert_equals(FIXED_EPSILON, 0x00000001);
    assert_equals(FIXED_ONE, 0x00010000);
    assert_equals(FIXED_ZERO, 0x00000000);
    assert_equals(FIXED_HALF, 0x00008000);
    assert_range(FIXED_ZERO,     atox("0.00000000000000000000"), FIXED_EPSILON);
    assert_range(FIXED_ONE,      atox("1.00000000000000000000"), FIXED_EPSILON);
    assert_range(FIXED_HALF,     atox("0.50000000000000000000"), FIXED_EPSILON);
    assert_range(FIXED_E,        atox("2.7182818284590452354"), FIXED_EPSILON);
    assert_range(FIXED_LOG2E,    atox("1.4426950408889634074"), FIXED_EPSILON);
    assert_range(FIXED_LOG10E,   atox("0.43429448190325182765"), FIXED_EPSILON);
    assert_range(FIXED_LN2,      atox("0.69314718055994530942"), FIXED_EPSILON);
    assert_range(FIXED_LN10,     atox("2.30258509299404568402"), FIXED_EPSILON);
    assert_range(FIXED_PI,       atox("3.14159265358979323846"), FIXED_EPSILON);
    assert_range(FIXED_PI_2,     atox("1.57079632679489661923"), FIXED_EPSILON);
    assert_range(FIXED_PI_4,     atox("0.78539816339744830962"), FIXED_EPSILON);
    assert_range(FIXED_1_PI,     atox("0.31830988618379067154"), FIXED_EPSILON);
    assert_range(FIXED_2_PI,     atox("0.63661977236758134308"), FIXED_EPSILON);
    assert_range(FIXED_2_SQRTPI, atox("1.12837916709551257390"), FIXED_EPSILON);
    assert_range(FIXED_SQRT2,    atox("1.41421356237309504880"), FIXED_EPSILON);
    assert_range(FIXED_SQRT1_2,  atox("0.70710678118654752440"), FIXED_EPSILON);
}

static void
test_itox(void)
{
    error = FIXED_NONE;
    assert_equals(itox(1), FIXED_ONE);
    assert(itox(42) != 42);
    assert(itox(42) == 42 << 16);
    assert(xtoi(42) != 42);
    assert(xtoi(itox(42)) == 42);
    assert(error == FIXED_NONE);
}

static void
test_neg(void)
{
    error = FIXED_NONE;

    assert_equals(xneg(itox(42)), itox(-42));
    assert_equals(xneg(xneg(itox(42))), itox(42));

    assert_equals(xneg(xneg(FIXED_MAX)), FIXED_MAX);
    assert(error == FIXED_NONE);

    error = FIXED_NONE;
    xneg(FIXED_MIN);
    assert(error == FIXED_OVERFLOW);
}

static void
test_abs(void)
{
    error = FIXED_NONE;
    assert_equals(xabs(itox(0)), itox(0));
    assert_equals(xabs(itox(42)), itox(42));
    assert_equals(xabs(itox(-42)), itox(42));
    assert(error == FIXED_NONE);

    assert_equals(xabs(FIXED_MIN), FIXED_MIN);
    assert(error == FIXED_OVERFLOW);
}

static void
test_add(void)
{
    error = FIXED_NONE;

    assert_equals(xadd(itox(7), itox(13)), itox(20));
    assert_equals(xadd(itox(-7), itox(13)), itox(6));
    assert_equals(xadd(itox(-7), itox(-13)), itox(-20));
    assert_equals(xadd(itox(7), atox("2.5")), atox("9.5"));
    assert_equals(xadd(itox(-7), atox("2.5")), atox("-4.5"));

    error = FIXED_NONE;
    xadd(FIXED_MAX, FIXED_EPSILON);
    assert(error == FIXED_OVERFLOW);

    error = FIXED_NONE;
    xadd(FIXED_MAX - itox(10), itox(100));
    assert(error == FIXED_OVERFLOW);

    error = FIXED_NONE;
    xadd(FIXED_MAX - itox(100), FIXED_MAX - itox(100));
    assert(error == FIXED_OVERFLOW);

    error = FIXED_NONE;
    xadd(FIXED_MAX - itox(100), itox(100));
    assert(error == FIXED_NONE);
}

static void
test_mul(void)
{
    error = FIXED_NONE;
    assert_equals(xtoi(xmul(itox(7),itox(13))), 7 * 13);
    assert_equals(xtoi(xmul(itox(-7),itox(13))), -7 * 13);
    assert_equals(xtoi(xmul(itox(3) / 2, itox(3) / 2) * 4), 9);
    assert_equals(xtoi(xmul(itox(-3) / 2, itox(-3) / 2) * 4), 9);
    assert_equals(xtoi(xmul(itox(7) / 4, itox(7) / 4) * 16), 49);
    assert_equals(xtoi(xmul(itox(-7) / 4, itox(-7) / 4) * 16), 49);
    assert(error == FIXED_NONE);

    assert_equals(xmul(atox("0.75"), FIXED_EPSILON), 0);
    assert(error == FIXED_UNDERFLOW);

    error = FIXED_NONE;
    assert_equals(xmul(FIXED_MAX, atox("1.5")), FIXED_MAX);
    assert(error == FIXED_OVERFLOW);

    error = FIXED_NONE;
    assert_equals(xmul(FIXED_MIN, atox("1.5")), FIXED_MIN);
    assert(error == FIXED_OVERFLOW);
}

static void
test_div(void)
{
    error = FIXED_NONE;

    assert_equals(xdiv(itox(0),itox(1)), 0);
    assert_equals(xdiv(itox(42),itox(1)), itox(42));
    assert_equals(xdiv(itox(-42),itox(1)), itox(-42));
    assert_equals(xdiv(itox(-2),itox(1)), itox(-2));
    assert_equals(xtoi(xdiv(itox(42),itox(2))), 21);
    assert_equals(xtoi(xdiv(itox(-42),itox(2))), -21);
    assert_equals(xtoi(xdiv(itox(42),itox(-2))), -21);
    assert_equals(xtoi(xdiv(itox(-42),itox(-2))), 21);
    assert_equals(xdiv(itox(21),itox(2)), atox("10.5"));
    assert_range(xdiv(itox(1), itox(2)), atox("0.50"), 0);
    assert_range(xdiv(itox(1), itox(3)), atox("0.33333"), FIXED_EPSILON);
    assert_range(xdiv(itox(1), itox(4)), atox("0.25"), 0);
    assert_range(xdiv(itox(1), itox(5)), atox("0.20"), 0);
    assert_range(xdiv(itox(1), itox(6)), atox("0.16666"), FIXED_EPSILON);
    assert_range(xdiv(itox(1), itox(7)), atox("0.14285"), FIXED_EPSILON);
    assert_range(xdiv(itox(1), itox(8)), atox("0.125"), 0);
    assert_range(xdiv(itox(1), itox(9)), atox("0.11111"), FIXED_EPSILON);
    assert_range(xdiv(itox(1), itox(1)), atox("1.0"), 0);
    assert_range(xdiv(itox(3), itox(2)), atox("1.5"), 0);
    assert_range(xneg(xdiv(itox(22), itox(7))), xdiv(xneg(itox(22)), itox(7)), 0);
    assert_range(xdiv(itox(22), itox(7)), atox("3.14285"), FIXED_EPSILON);
    assert_range(xdiv(itox(-22), itox(7)), atox("-3.14285"), FIXED_EPSILON);
    assert_range(xdiv(itox(22), itox(-7)), atox("-3.14285"), FIXED_EPSILON);
    assert_range(xdiv(itox(-22), itox(-7)), atox("3.14285"), FIXED_EPSILON);
    assert_range(xdiv(atox("42.721359"), atox("42.721359")), itox(1), 0);
    assert_range(xmul(xdiv(itox(1), itox(7)), itox(7)), atox("1"), FIXED_EPSILON * 2);
    assert(error == FIXED_NONE);

    error = FIXED_NONE;
    assert_equals(xdiv(itox(42), itox(0)), FIXED_MAX);
    assert(error == FIXED_SINGULARITY);

    error = FIXED_NONE;
    assert_equals(xdiv(itox(-42), itox(0)), FIXED_MIN);
    assert(error == FIXED_SINGULARITY);

    error = FIXED_NONE;
    assert_equals(xdiv(FIXED_EPSILON, itox(2)), 0);
    assert(error == FIXED_UNDERFLOW);

    error = FIXED_NONE;
    assert_equals(xdiv(atox("0.05"), FIXED_MAX), 0);
    assert(error == FIXED_UNDERFLOW);

}

static void
test_sqrt(void)
{
    int i;

    error = FIXED_NONE;

    assert_equals(xsqrt(FIXED_ZERO), FIXED_ZERO);
    assert_equals(xsqrt(FIXED_ONE), FIXED_ONE);
    assert_range(xsqrt(itox(2)), atox("1.4142136"), FIXED_EPSILON);

    for (i = 0; i != 100; ++i) {
        const fixed f = itox(i);
        assert(xsqrt(xmul(f, f)) == f);
    }

    assert(error == FIXED_NONE);

    assert_equals(xsqrt(xneg(FIXED_ONE)), FIXED_NAN);

    assert(error == FIXED_DOMAIN);
}

static void
test_ldexp(void)
{
    error = FIXED_NONE;
    assert_equals(xldexp(itox(1), 1), itox(2));
    assert_equals(xldexp(itox(1), -1), atox("0.5"));
    assert_equals(xldexp(itox(-42), 2), itox(-42 * 4));
    assert(error == FIXED_NONE);

    xldexp(itox(1), 16);
    assert(error == FIXED_OVERFLOW);

    error = FIXED_NONE;
    assert(xldexp(itox(1), -16) != 0);
    assert(error == FIXED_NONE);

    assert_equals(xldexp(itox(1), -17), 0);
    assert(error == FIXED_UNDERFLOW);
}

static void
test_modx(void)
{
    fixed ip;

    error = FIXED_NONE;

    assert_equals(xmodx(itox(42), &ip), itox(0));
    assert_equals(ip, itox(42));

    assert_equals(xmodx(atox("1.75"), &ip), atox("0.75"));
    assert_equals(ip, itox(1));

    assert_equals(xmodx(atox("-1.75"), &ip), atox("-0.75"));
    assert_equals(ip, itox(-1));

    xmodx(FIXED_MAX, &ip);
    xmodx(FIXED_MIN, &ip);

    assert(error == FIXED_NONE);
}

static void
test_mod(void)
{
    error = FIXED_NONE;

    assert_equals(xmod(itox(42), itox(2)), 0);
    assert_equals(xmod(itox(41), itox(2)), itox(1));

    assert_equals(xmod(atox("2.5"), itox(1)), atox("0.5"));
    assert_equals(xmod(atox("2.5"), itox(-1)), atox("0.5"));
    assert_equals(xmod(atox("-2.5"), itox(1)), atox("-0.5"));
    assert_equals(xmod(atox("-2.5"), itox(-1)), atox("-0.5"));

    assert_equals(xmod(atox("2.25"), itox(1)), atox("0.25"));
    assert_equals(xmod(atox("2.25"), itox(-1)), atox("0.25"));
    assert_equals(xmod(atox("-2.25"), itox(1)), atox("-0.25"));
    assert_equals(xmod(atox("-2.25"), itox(-1)), atox("-0.25"));

    assert_equals(xmod(atox("2.25"), atox("1.5")), atox("0.75"));
    assert_equals(xmod(atox("2.25"), atox("-1.5")), atox("0.75"));
    assert_equals(xmod(atox("-2.25"), atox("1.5")), atox("-0.75"));
    assert_equals(xmod(atox("-2.25"), atox("-1.5")), atox("-0.75"));

    assert(error == FIXED_NONE);
}

static void
test_frexp(void)
{
    int exp;

    error = FIXED_NONE;

    exp = 42;
    assert_equals(xfrexp(FIXED_ZERO, &exp), FIXED_ZERO);
    assert(exp == 0);

    exp = 42;
    assert_equals(xldexp(xfrexp(FIXED_MIN, &exp), 16), FIXED_MIN);
    assert(exp == 16);
    
    exp = 42;
    assert_equals(xfrexp(FIXED_ONE, &exp), FIXED_HALF);
    assert(exp == 1);
    
    exp = 42;
    assert_equals(xfrexp(FIXED_HALF, &exp), FIXED_HALF);
    assert(exp == 0);
    
    exp = 42;
    assert_equals(xfrexp(itox(2), &exp), FIXED_HALF);
    assert(exp == 2);
    
    exp = 42;
    assert_equals(xfrexp(itox(-2), &exp), atox("-0.5"));
    assert(exp == 2);
    
    exp = 42;
    assert_equals(xfrexp(atox("-0.125"), &exp), atox("-0.5"));
    assert(exp == -2);
    
    exp = 42;
    assert(xfrexp(atox("35.129"), &exp) < FIXED_ONE);
    assert(xfrexp(atox("35.129"), &exp) >= FIXED_HALF);
    assert(exp == 6);
    
    assert(error == FIXED_NONE);
}

static void
test_floor(void)
{
    error = FIXED_NONE;

    assert_equals(xfloor(itox(42)), itox(42));
    assert_equals(xfloor(atox("42.2")), itox(42));
    assert_equals(xfloor(atox("-42.2")), itox(-43));

    assert(error == FIXED_NONE);
}

static void
test_ceil(void)
{
    error = FIXED_NONE;

    assert_equals(xceil(itox(42)), itox(42));
    assert_equals(xceil(atox("42.2")), itox(43));
    assert_equals(xceil(atox("-42.2")), itox(-42));

    assert(error == FIXED_NONE);
}

static void
test_exp(void)
{
    error = FIXED_NONE;

    xexp(FIXED_ONE);
    assert_range(xexp(FIXED_ONE), FIXED_E, 0);
    assert_range(xexp(itox(2)), atox("7.389056"), 0);
    assert_range(xexp(itox(4)), atox("54.59815"), 0);
    assert_range(xexp(itox(8)), atox("2980.957987"), FIXED_EPSILON * 2981);
    assert_range(xexp(itox(10)), atox("22026.46579"), FIXED_EPSILON * 22027);

    assert(error == FIXED_NONE || error == FIXED_UNDERFLOW);
}

static void
test_log(void)
{
    error = FIXED_NONE;

    assert_range(xlog(FIXED_HALF), atox("-0.69314718"), FIXED_EPSILON);
    assert_range(xlog(FIXED_ONE), FIXED_ZERO, FIXED_EPSILON);
    assert_range(xlog(FIXED_E), FIXED_ONE, FIXED_EPSILON);
    assert_range(xlog(itox(2)), FIXED_LN2, FIXED_EPSILON);
    assert_range(xlog(itox(10)), FIXED_LN10, xmul(FIXED_EPSILON, FIXED_LN10));

    assert(error == FIXED_NONE);
}

static void
test_log10(void)
{
    error = FIXED_NONE;
    assert_range(xlog10(itox(1)), 0, FIXED_EPSILON);
    assert_range(xlog10(itox(10)), FIXED_ONE, FIXED_EPSILON);
    assert_range(xlog10(itox(100)), itox(2), FIXED_EPSILON * 100); /* very slow */
    assert(error == FIXED_NONE);
}

static void
test_pow(void)
{
    error = FIXED_NONE;
    assert_equals(xpow(itox(0), itox(0)), FIXED_ONE); 
    assert_equals(xpow(itox(0), itox(42)), FIXED_ZERO); 
    assert_equals(xpow(itox(42), itox(0)), FIXED_ONE); 
    assert_range(xpow(itox(42), itox(1)), itox(42), FIXED_EPSILON); 
    assert_range(xpow(itox(10), itox(2)), itox(100), FIXED_EPSILON * 100); 
    assert(error == FIXED_NONE);
}

static void
test_sin(void)
{
    error = FIXED_NONE;

    assert_range(xsin(FIXED_ZERO), FIXED_ZERO, FIXED_EPSILON);
    assert_range(xsin(FIXED_PI_2), FIXED_ONE, FIXED_EPSILON); /* underflow */
    assert_range(xsin(FIXED_PI), FIXED_ZERO, FIXED_EPSILON);
    assert_range(xsin(FIXED_PI * 3 / 2), xneg(FIXED_ONE), FIXED_EPSILON);
    assert_range(xsin(FIXED_PI * 2), FIXED_ZERO, FIXED_EPSILON);

    assert_range(xsin(FIXED_ONE), atox("0.84147098"), FIXED_EPSILON);
    assert_range(xsin(FIXED_HALF), atox("0.4794255"), FIXED_EPSILON);
    assert_range(xsin(xneg(FIXED_HALF)), atox("-0.4794255"), FIXED_EPSILON);

    assert(error == FIXED_NONE || error == FIXED_UNDERFLOW);
}

static void
test_cos(void)
{
    error = FIXED_NONE;

    assert_range(xcos(FIXED_ZERO), FIXED_ONE, FIXED_EPSILON);
    assert_range(xcos(FIXED_PI_2), FIXED_ZERO, FIXED_EPSILON);
    assert_range(xcos(FIXED_PI), xneg(FIXED_ONE), FIXED_EPSILON);
    assert_range(xcos(FIXED_PI * 3 / 2), FIXED_ZERO, FIXED_EPSILON);
    assert_range(xcos(FIXED_PI * 2), FIXED_ONE, FIXED_EPSILON);

    assert_range(xcos(FIXED_ONE), atox("0.5403023"), FIXED_EPSILON);
    assert_range(xcos(FIXED_HALF), atox("0.87758256"), FIXED_EPSILON);
    assert_range(xcos(FIXED_HALF), xcos(xneg(FIXED_HALF)), FIXED_EPSILON);

    assert(error == FIXED_NONE || error == FIXED_UNDERFLOW);
}

static void
test_tan(void)
{
    error = FIXED_NONE;

    assert_range(xtan(FIXED_ZERO), FIXED_ZERO, FIXED_EPSILON);
    assert_range(xtan(FIXED_PI / 6), xsqrt(itox(3)) / 3, FIXED_EPSILON);
    assert_range(xtan(FIXED_PI / 3), xsqrt(itox(3)), FIXED_EPSILON);
    assert_range(xtan(FIXED_PI / 4), FIXED_ONE, FIXED_EPSILON);

    assert(error == FIXED_NONE || error == FIXED_UNDERFLOW);

    /* trick: FIXED_MIN - FIXED_MAX ==  FIXED_EPSILON */
    assert_range(xtan(FIXED_PI_2), FIXED_NAN, FIXED_EPSILON);
}

static void
test_atan(void)
{
    error = FIXED_NONE;

    assert_range(xatan(FIXED_ZERO), FIXED_ZERO, FIXED_EPSILON);
    assert_range(xatan(FIXED_ONE), FIXED_PI_4, FIXED_EPSILON);
#if 0
    assert_range(xatan(xdiv(FIXED_ONE, itox(5))) * 4 + xatan(xdiv(FIXED_ONE, itox(239))),
        FIXED_PI_4, FIXED_EPSILON);
#endif

    assert(error == FIXED_NONE || error == FIXED_UNDERFLOW);
}

static void
test_atan2(void)
{
    error = FIXED_NONE;

    assert_equals(xatan2(FIXED_ZERO, FIXED_ZERO), FIXED_ZERO);
    assert_range(xatan2(FIXED_ZERO, FIXED_ONE), FIXED_ZERO, FIXED_EPSILON);
    assert_range(xatan2(FIXED_ONE, FIXED_ONE), FIXED_PI_4, FIXED_EPSILON);
    assert_range(xatan2(FIXED_ONE, FIXED_ZERO), FIXED_PI_2, FIXED_EPSILON);
    assert_range(xatan2(FIXED_ZERO, xneg(FIXED_ONE)), FIXED_PI, FIXED_EPSILON);
    assert_range(xatan2(xneg(FIXED_ONE), FIXED_ZERO), xneg(FIXED_PI_2), FIXED_EPSILON);
#if 0
    assert_range(xatan(xdiv(FIXED_ONE, itox(5))) * 4 + xatan(xdiv(FIXED_ONE, itox(239))),
        FIXED_PI_4, FIXED_EPSILON);
#endif

    assert(error == FIXED_NONE || error == FIXED_UNDERFLOW);
}

static void
test_asin(void)
{
    error = FIXED_NONE;

    assert_range(xasin(xneg(FIXED_ONE)), xneg(FIXED_PI_2), FIXED_EPSILON);
    assert_range(xasin(FIXED_ZERO), FIXED_ZERO, FIXED_EPSILON);
    assert_range(xasin(FIXED_ONE), FIXED_PI_2, FIXED_EPSILON);

    assert(error == FIXED_NONE || error == FIXED_UNDERFLOW);
}

static void
test_acos(void)
{
    error = FIXED_NONE;

    assert_range(xacos(FIXED_ONE), FIXED_ZERO, FIXED_EPSILON);
    assert_range(xacos(FIXED_ZERO), FIXED_PI_2, FIXED_EPSILON);
    assert_range(xacos(xneg(FIXED_ONE)), FIXED_PI, FIXED_EPSILON);

    assert(error == FIXED_NONE || error == FIXED_UNDERFLOW);
}

int
main()
{
    xsetmatherr(error_handler);

    test_itox();
    test_atox();
    test_printf();
    test_constants();
    test_neg();
    test_abs();
    test_add();
    test_mul();
    test_div();
    test_sqrt();
    test_ldexp();
    test_frexp();
    test_modx();
    test_mod();
    test_floor();
    test_ceil();
    test_exp();
    test_log();
    test_log10();
    test_pow();
    test_sin();
    test_cos();
    test_tan();
    test_atan();
    test_atan2();
    test_asin();
    test_acos();

    printf("passed.\n");

    return EXIT_SUCCESS;
}
