/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/fixed/test_f64.c,v 1.3 2005/05/11 15:22:46 artlab Exp $
*/

#undef NDEBUG

#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include "int32.h"
#include "float64.h"

extern int shift_left(float64 * const x);
extern int shift_right(float64 * const x);

static void
print_mantissa(int32_t mantissa)
{
    int i;

    for (i = 31; i >= 0; --i) {
        printf("%d", (mantissa & (1 << i)) ? 1 : 0);
    }
}

static void
assert_range_(int const line, float64 /*const*/ actual, float64 /*const*/ expected, float64 const tolerance)
{
    if (fcompare64(fabs64(fsub64(actual, expected)), tolerance) <= 0)
        return;

    shift_left(&actual);
    shift_left(&expected);

    printf("line = %d\n", line);

    printf("actual    = ");
    print_mantissa(actual.mantissa); printf(" 2^%d\n", actual.exponent);

    printf("expected  = ");
    print_mantissa(expected.mantissa); printf(" 2^%d\n", expected.exponent);

    printf("tolerance = ");
    print_mantissa(tolerance.mantissa); printf(" 2^%d\n", tolerance.exponent);

    printf("\n");
}

#define assert_equals(actual, expected) \
    assert_range_(__LINE__, actual, expected, FLOAT64_ZERO)

#define assert_range(actual, expected, tolerance) \
    assert_range_(__LINE__, actual, expected, tolerance)

static void
test_itof64(void)
{
    float64 x;

    x = itof64(0);
    assert(x.mantissa == 0);

    x = itof64(1);
    assert(x.mantissa == 1 && x.exponent == 0);

    x = itof64(-1);
    assert(x.mantissa == -1 && x.exponent == 0);

    x = itof64(42);
    assert(x.mantissa == 42 && x.exponent == 0);

    x = itof64(-42);
    assert(x.mantissa == -42 && x.exponent == 0);
}

static void
test_f64toi(void)
{
    assert(f64toi(itof64(0)) == 0);
    assert(f64toi(itof64(1)) == 1);
    assert(f64toi(itof64(42)) == 42);
    assert(f64toi(itof64(-1)) == -1);
    assert(f64toi(itof64(-42)) == -42);
}

static void
test_constants(void)
{
    assert(FLOAT64_ZERO.mantissa == 0x00000000);
    assert(FLOAT64_ZERO.exponent == 0x00000000);
    assert(FLOAT64_ONE.mantissa == 0x00000001);
    assert(FLOAT64_ONE.exponent == 0x00000000);
    assert(FLOAT64_EPSILON.mantissa == 0x00000001);
    assert(FLOAT64_EPSILON.exponent == -30);
    assert(fcompare64(FLOAT64_LN2, atof64("0.69314718056")) == 0);
}

static void
test_fcompare64(void)
{
    float64 x;

    assert(fcompare64(FLOAT64_ZERO, FLOAT64_ZERO) == 0);
    assert(fcompare64(FLOAT64_ONE, FLOAT64_ONE) == 0);
    assert(fcompare64(FLOAT64_ZERO, FLOAT64_ONE) < 0);
    assert(fcompare64(FLOAT64_ONE, FLOAT64_ZERO) > 0);
    assert(fcompare64(itof64(-1), itof64(-1)) == 0);
    assert(fcompare64(itof64(42), itof64(42)) == 0);
    assert(fcompare64(itof64(7), itof64(12)) < 0);
    assert(fcompare64(itof64(12), itof64(7)) > 0);
    assert(fcompare64(itof64(-12), itof64(7)) < 0);
    assert(fcompare64(itof64(12), itof64(-7)) > 0);
    assert(fcompare64(itof64(-12), itof64(-7)) < 0);
    assert(fcompare64(itof64(-7), itof64(-12)) > 0);

    x.mantissa = INT32_MAX;
    x.exponent = -16;
    assert(fcompare64(FLOAT64_ZERO, x) < 0);

    x.mantissa = INT32_MIN;
    x.exponent = -16;
    assert(fcompare64(FLOAT64_ZERO, x) > 0);
}

static void
test_fneg64(void)
{
    assert(fcompare64(fneg64(FLOAT64_ZERO), FLOAT64_ZERO) == 0);
    assert(fcompare64(fneg64(FLOAT64_ONE), FLOAT64_ONE) != 0);
    assert(fcompare64(fneg64(itof64(42)), itof64(-42)) == 0);
    assert(fcompare64(fneg64(fneg64(itof64(42))), itof64(42)) == 0);
}

static void
test_fshift64(void)
{
    float64 x;

    assert(fcompare64(fshift64(FLOAT64_ZERO, 7), FLOAT64_ZERO) == 0);
    assert(fcompare64(fshift64(FLOAT64_ONE, 1), itof64(2)) == 0);
    assert(fcompare64(fshift64(FLOAT64_ONE, -1), FLOAT64_ZERO) != 0);
    assert(fcompare64(fshift64(fshift64(FLOAT64_ONE, -1), +1), FLOAT64_ONE) == 0);

    x = fshift64(FLOAT64_ONE, 32);
    assert(x.mantissa == 1 && x.exponent == 32);

    x = fshift64(x, -32);
    assert(x.mantissa == 1 && x.exponent == 0);
}

static void
test_fadd64(void)
{
    float64 x, y;

    assert(fcompare64(fadd64(itof64(7), itof64(13)), itof64(20)) == 0);
    assert(fcompare64(fadd64(itof64(-7), itof64(13)), itof64(6)) == 0);
    assert(fcompare64(fadd64(itof64(-7), itof64(-13)), itof64(-20)) == 0);
    assert(fcompare64(fadd64(itof64(7), atof64("2.5")), atof64("9.5")) == 0);
    assert(fcompare64(fadd64(itof64(-7), atof64("2.5")), atof64("-4.5")) == 0);

    x.mantissa = INT32_MAX;
    x.exponent = 0;
    x = fadd64(x, FLOAT64_ONE);

    y.mantissa = 1;
    y.exponent = 31;
    assert(fcompare64(x, y) == 0);

    x.mantissa = 0x00000005;
    x.exponent = 3;
    assert(fcompare64(x, itof64(42)) != 0);
    x = fadd64(x, itof64(2));
    assert(fcompare64(x, itof64(42)) == 0);
    assert(f64toi(x) == 42);
}

static void
test_fsub64(void)
{
    float64 x, y, z;

    assert(fcompare64(fsub64(itof64(7), itof64(13)), itof64(-6)) == 0);
    assert(fcompare64(fsub64(itof64(-7), itof64(13)), itof64(-20)) == 0);
    assert(fcompare64(fsub64(itof64(-7), itof64(-13)), itof64(6)) == 0);

    x.mantissa = 1;
    x.exponent = 1;
    x = fsub64(x, FLOAT64_ONE);
    assert(x.mantissa = 1 && x.exponent == 0);

    x.mantissa = 1;
    x.exponent = 2;
    x = fsub64(x, FLOAT64_ONE);
    assert(x.mantissa = 3 && x.exponent == 0);

    x.mantissa = 1;
    x.exponent = 30;
    x = fsub64(x, FLOAT64_ONE);
    assert(x.mantissa = 0x3FFFFFFF && x.exponent == 0);

    /* test for rounding error */
    x.mantissa = 1;
    x.exponent = 31;
    x = fsub64(x, FLOAT64_ONE);
/* XXX rounding problem */
#if 0
/*    assert(x.mantissa = 1 && x.exponent == 31);*/
    assert(x.mantissa = 0x3FFFFFFE && x.exponent == 1);
#endif

    x.mantissa = INT32_MAX;
    x.exponent = -31;
    y.mantissa = 0x40000000;
    y.exponent = -30;
    z = fsub64(x, y);
    assert((z.mantissa == 0 && z.exponent == 0) || (z.mantissa == -1 && z.exponent == -31));
}

static void
test_fmul64(void)
{
    assert(f64toi(fmul64(itof64(7), itof64(13))) == 7 * 13);
    assert(f64toi(fmul64(itof64(-7), itof64(13))) == -7 * 13);
    assert(f64toi(fmul64(itof64(3), itof64(3))) == 9);
    assert(f64toi(fmul64(itof64(-3), itof64(-3))) == 9);
    assert(f64toi(fmul64(itof64(7), itof64(7))) == 49);
    assert(f64toi(fmul64(itof64(-7), itof64(-7))) == 49);
    assert(fcompare64(fmul64(itof64(1 << 20), itof64(1 << 20)), fshift64(itof64(1 << 20), 20)) == 0);
    assert(f64toi(fmul64(itof64(42), itof64(10))) == 420);
    assert(fcompare64(fmul64(itof64(42), itof64(10)), itof64(420)) == 0);
    assert(f64toi(fmul64(itof64(0), itof64(10))) == 0);
}

static void
test_fdiv64(void)
{
    assert(f64toi(fdiv64(itof64(0), itof64(1))) == 0);
    assert(f64toi(fdiv64(itof64(42), itof64(1))) == 42);
    assert(f64toi(fdiv64(itof64(-42), itof64(1))) == -42);
    assert(f64toi(fdiv64(itof64(-2), itof64(1))) == -2);
    assert(f64toi(fdiv64(itof64(42), itof64(2))) == 21);
    assert(f64toi(fdiv64(itof64(-42), itof64(2))) == -21);
    assert(f64toi(fdiv64(itof64(42), itof64(-2))) == -21);
    assert(f64toi(fdiv64(itof64(-42), itof64(-2))) ==  21);
    assert_equals(fdiv64(itof64(21), itof64(2)), atof64("10.5"));
    assert_equals(fdiv64(FLOAT64_ONE, itof64(2)), atof64("0.50"));
    assert_range(fdiv64(FLOAT64_ONE, itof64(3)), atof64("0.3333333333"), FLOAT64_EPSILON);
    assert_range(fdiv64(FLOAT64_ONE, itof64(4)), atof64("0.25"), FLOAT64_ZERO);
    assert_range(fdiv64(FLOAT64_ONE, itof64(5)), atof64("0.20"), FLOAT64_ZERO);
    assert_range(fdiv64(FLOAT64_ONE, itof64(6)), atof64("0.1666666666"), FLOAT64_EPSILON);
    assert_range(fdiv64(FLOAT64_ONE, itof64(7)), atof64("0.14285714286"), FLOAT64_EPSILON);
    assert_range(fdiv64(FLOAT64_ONE, itof64(8)), atof64("0.125"), FLOAT64_ZERO);
    assert_range(fdiv64(FLOAT64_ONE, itof64(9)), atof64("0.1111111111"), FLOAT64_EPSILON);
    assert_range(fdiv64(FLOAT64_ONE, itof64(1)), atof64("1.0"), FLOAT64_ZERO);
    assert_range(fdiv64(itof64(3), itof64(2)), atof64("1.5"), FLOAT64_ZERO);
    assert_range(fneg64(fdiv64(itof64(22), itof64(7))), fdiv64(fneg64(itof64(22)), itof64(7)), fmul64(FLOAT64_EPSILON, atof64("3.14285714286")));
    assert_range(fdiv64(itof64(22), itof64(7)), atof64("3.14285714286"), FLOAT64_EPSILON);
    assert_range(fdiv64(itof64(-22), itof64(7)), atof64("-3.14285714286"), fmul64(FLOAT64_EPSILON, atof64("3.14285714286")));
    assert_range(fdiv64(itof64(22), itof64(-7)), atof64("-3.14285714286"), fmul64(FLOAT64_EPSILON, atof64("3.14285714286")));
    assert_range(fdiv64(itof64(-22), itof64(-7)), atof64("3.14285714286"), FLOAT64_EPSILON);
    assert_range(fdiv64(atof64("42.721359"), atof64("42.721359")), FLOAT64_ONE, FLOAT64_ZERO);
    assert_range(fmul64(fdiv64(itof64(1), itof64(7)), itof64(7)), atof64("1"), FLOAT64_EPSILON);
}

static void
test_atof64(void)
{
    assert_equals(atof64("0.0"), FLOAT64_ZERO);
    assert_equals(atof64("1.0"), FLOAT64_ONE);
    assert_equals(atof64("42.0"), itof64(42));
    assert_equals(atof64("42"), itof64(42));
    assert_equals(atof64("-42"), itof64(-42));
    assert_equals(atof64("42.00000001"), itof64(42));
    assert(fcompare64(atof64("42.0000001"), itof64(42)) != 0);
    assert(fcompare64(atof64("43"), itof64(42)) != 0);
    assert_equals(atof64("3.75"), fdiv64(itof64(375), itof64(100)));
}

static void
test_printf64(void)
{
    printf("pi = "); printf64(atof64("3.141592653589793238")); printf("\n");
    printf("-pi = "); printf64(atof64("-3.141592653589793238")); printf("\n");
    printf("ln2 = "); printf64(atof64(".6931471805599453094")); printf("\n");
}

int
main()
{
    assert(f64toi(fdiv64(itof64(-2), itof64(1))) == -2);
    test_itof64();
    test_f64toi();
    test_constants();
    test_fcompare64();
    test_fneg64();
    test_fshift64();
    test_fadd64();
    test_fsub64();
    test_fmul64();
    test_fdiv64();
    test_atof64();
    test_printf64();

    printf("passed.\n");

    return EXIT_SUCCESS;
}
