/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/fixed/test_i32.c,v 1.3 2005/05/11 15:22:46 artlab Exp $
*/

#undef NDEBUG

#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include "int32.h"

static void
test_constants(void)
{
    assert(INT32_MAX == (int32_t)0x7FFFFFFF);
    assert(INT32_MIN == (int32_t)0x80000000);
}

static void
test_ioverflow32(void)
{
    int32_t x, y, carry_bits;

    x = 42;
    y = 7;
    carry_bits = icarry32(x, y);
    assert(!ioverflow32(carry_bits));

    x = (int32_t)0x7FFFFFFE;
    y = (int32_t)0x00000001;
    carry_bits = icarry32(x, y);
    assert(!ioverflow32(carry_bits));

    x = INT32_MAX;
    y = (int32_t)0x00000001;
    carry_bits = icarry32(x, y);
    assert(ioverflow32(carry_bits));

    x = (int32_t)0xFFFFFFFF;
    y = (int32_t)0xFFFFFFFF;
    carry_bits = icarry32(x, y);
    assert(!ioverflow32(carry_bits));

    x = INT32_MIN;
    y = (int32_t)0xFFFFFFFF;
    carry_bits = icarry32(x, y);
    assert(ioverflow32(carry_bits));
}

static void
test_ishift32(void)
{
    assert(ishift32(0, 1) == 0);
    assert(ishift32(1, 1) == 2);
    assert(ishift32(42, 0) == 42);
    assert(ishift32(42, 32) == 0);
    assert(ishift32(42, -1) == 21);
    assert(ishift32(-42, -1) == -21);
    assert(ishift32(-42, +1) == -84);
    assert(ishift32(1, 8) == 256); 
    assert(ishift32((int32_t)0x7FFFFFFF, -1) == (int32_t)0x3FFFFFFF);
    assert(ishift32((int32_t)0x80000000, -1) == (int32_t)0xC0000000);
    assert(ishift32((int32_t)0x00000001, -1) == 0);
    assert(ishift32((int32_t)0x7FFFFFFF, +1) == (int32_t)0xFFFFFFFE);
    assert(ishift32((int32_t)0x80000000, +1) == 0);
    assert(ishift32((int32_t)0x80000001, -1) == (int32_t)0xC0000000);
    assert(ishift32((int32_t)0x7FFF0000, -16) == (int32_t)0x00007FFF);
    assert(ishift32((int32_t)0x7FFF0000, -20) == (int32_t)0x000007FF);

    assert(ishift32_round((int32_t)0x00000001, -1) == (int32_t)0x00000001);
    assert(ishift32_round((int32_t)0x00000001, -2) == (int32_t)0x00000000);
    assert(ishift32_round(5, -1) == 3);
} 

int
main()
{
    test_constants();
    test_ioverflow32();
    test_ishift32();

    printf("passed.\n");

    return EXIT_SUCCESS;
}
