/*
** Copyright (c) 2008 The Khronos Group Inc.
** 
** All Rights Reserved.  This code is protected by copyright laws and 
** contains material proprietary to the Khronos Group, Inc.  This is 
** UNPUBLISHED PROPRIETARY SOURCE CODE that may not be disclosed in whole 
** or in part to third parties, and may not be reproduced, republished, 
** distributed, transmitted, displayed, broadcast or otherwise exploited 
** in any manner without the express prior written permission of Khronos 
** Group.
** 
** The receipt or possession of this code does not convey any rights to 
** reproduce, disclose, or distribute its contents, or to manufacture, 
** use, or sell anything that it may describe, in whole or in part other 
** than under the terms of the Khronos Adopters Agreement or Khronos 
** Conformance Test Source License Agreement as executed between Khronos 
** and the recipient.
** 
** For the avoidance of doubt, this code when provided: a) under the 
** Khronos Conformance Test Source License Agreement is for the sole 
** purpose of creating conformance tests for delivery to Khronos and does 
** not provide for formally testing products or use of Khronos trademarks 
** on conformant products; b) under the Khronos Adopters Agreement is for 
** the sole purpose of formally administering tests to products pursuant 
** to the Khronos Conformance Process Document.
** 
** Khronos, OpenKODE, OpenVG, OpenWF, glFX, OpenMAX and OpenSL ES are 
** trademarks of the Khronos Group Inc.  COLLADA is a trademark of Sony 
** Computer Entertainment Inc. used by permission by Khronos.  OpenGL and 
** OpenML are registered trademarks and the OpenGL ES logo is a trademark 
** of Silicon Graphics Inc. used by permission by Khronos. 
** 
** Use, duplication or disclosure by the Government is subject to 
** restrictions as set forth in subdivision (c)(1)(ii) of the Rights in 
** Technical Data and Computer Software clause at DFARS 252.227-7013, and/
** or in similar or successor clauses in the FAR, DOD or NASA FAR 
** Supplement. Unpublished rights reserved under the Copyright Laws of 
** the United States and other countries.
*/

/*
** Copyright 1999-2005, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /home/cvs/OpenGL-ES/1.0/ogles/conform/fixed/test_i64.c,v 1.3 2005/05/11 15:22:46 artlab Exp $
*/

#undef NDEBUG

#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include "int64.h"

void
test_itoi64(void)
{
    int64 x;

    x = itoi64(0);
    assert(x.hi == 0 && x.lo == 0);

    x = itoi64(1);
    assert(x.hi == 0 && x.lo == 1);

    x = itoi64(-1);
    assert(x.hi == (uint32_t)-1 && x.lo == (uint32_t)-1);

    x = itoi64(42);
    assert(x.hi == 0 && x.lo == 42);

    x = itoi64(-42);
    assert(x.hi == (uint32_t)-1 && x.lo == (uint32_t)-42);
}

void
test_i64toi(void)
{
    assert(i64toi(itoi64(0)) == 0);
    assert(i64toi(itoi64(1)) == 1);
    assert(i64toi(itoi64(42)) == 42);
    assert(i64toi(itoi64(-1)) == -1);
    assert(i64toi(itoi64(-42)) == -42);
}

void
test_constants(void)
{
    assert(INT64_MAX.hi == 0x7FFFFFFF);
    assert(INT64_MAX.lo == 0xFFFFFFFF);
    assert(INT64_MIN.hi == 0x80000000);
    assert(INT64_MIN.lo == 0x00000000);
    assert(INT64_ZERO.hi == 0x00000000);
    assert(INT64_ZERO.lo == 0x00000000);
    assert(INT64_ONE.hi == 0x00000000);
    assert(INT64_ONE.lo == 0x00000001);
}

void
test_icompare64(void)
{
    assert(icompare64(INT64_ZERO, INT64_ZERO) == 0);
    assert(icompare64(INT64_ONE, INT64_ONE) == 0);
    assert(icompare64(INT64_ZERO, INT64_ONE) < 0);
    assert(icompare64(INT64_ONE, INT64_ZERO) > 0);
    assert(icompare64(INT64_MAX, INT64_ZERO) > 0);
    assert(icompare64(INT64_MIN, INT64_ZERO) < 0);
    assert(icompare64(INT64_MAX, INT64_MIN) > 0);
    assert(icompare64(itoi64(-1), itoi64(-1)) == 0);
    assert(icompare64(itoi64(42), itoi64(42)) == 0);
    assert(icompare64(itoi64(7), itoi64(12)) < 0);
    assert(icompare64(itoi64(12), itoi64(7)) > 0);
    assert(icompare64(itoi64(-12), itoi64(7)) < 0);
    assert(icompare64(itoi64(12), itoi64(-7)) > 0);
    assert(icompare64(itoi64(-12), itoi64(-7)) < 0);
    assert(icompare64(itoi64(-7), itoi64(-12)) > 0);
}

void
test_ineg64(void)
{
    assert(icompare64(ineg64(INT64_ZERO), INT64_ZERO) == 0);
    assert(icompare64(ineg64(INT64_ONE), INT64_ONE) != 0);
    assert(icompare64(ineg64(itoi64(42)), itoi64(-42)) == 0);
    assert(icompare64(ineg64(ineg64(itoi64(42))), itoi64(42)) == 0);
}

void
test_ishift64(void)
{
    int64 x;

    assert(icompare64(ishift64(INT64_ZERO, 7), INT64_ZERO) == 0);
    assert(icompare64(ishift64(INT64_ONE, 1), itoi64(2)) == 0);
    assert(icompare64(ishift64(INT64_ONE, -1), INT64_ZERO) == 0);

    x = ishift64(INT64_ONE, 32);
    assert(x.hi == 1 && x.lo == 0);

    x = ishift64(x, -32);
    assert(x.hi == 0 && x.lo == 1);

    x.hi = 0xFFFFFFFF;
    x.lo = 0;
    x = ishift64(x, -16);
    assert(x.hi == 0xFFFFFFFF && x.lo == 0xFFFF0000);

    x.hi = 0x0000FFFF;
    x.lo = 0;
    x = ishift64(x, -16);
    assert(x.hi == 0 && x.lo == 0xFFFF0000);
}

void
test_iadd64(void)
{
    int64 x;

    assert(icompare64(iadd64(itoi64(7), itoi64(13)), itoi64(20)) == 0);
    assert(icompare64(iadd64(itoi64(-7), itoi64(13)), itoi64(6)) == 0);
    assert(icompare64(iadd64(itoi64(-7), itoi64(-13)), itoi64(-20)) == 0);

    x.hi = 1;
    x.lo = 0xFFFFFFFF;
    x = iadd64(x, INT64_ONE);
    assert(x.hi == 2 && x.lo == 0);
}

void
test_isub64(void)
{
    int64 x;

    assert(icompare64(isub64(itoi64(7), itoi64(13)), itoi64(-6)) == 0);
    assert(icompare64(isub64(itoi64(-7), itoi64(13)), itoi64(-20)) == 0);
    assert(icompare64(isub64(itoi64(-7), itoi64(-13)), itoi64(6)) == 0);

    x.hi = 2;
    x.lo = 0;
    x = isub64(x, INT64_ONE);
    assert(x.hi == 1 && x.lo == 0xFFFFFFFF);
}

void
test_imul64(void)
{
    int64 x;

    assert(i64toi(imul64(7, 13)) == 7 * 13);
    assert(i64toi(imul64(-7, 13)) == -7 * 13);
    assert(i64toi(imul64(3, 3)) == 9);
    assert(i64toi(imul64(-3, -3)) == 9);
    assert(i64toi(imul64(7 , 7)) == 49);
    assert(i64toi(imul64(-7, -7)) == 49);
    assert(icompare64(imul64(1 << 20, 1 << 20), ishift64(itoi64(1 << 20), 20)) == 0);

    x = imul64(0x40000000, 0x00020000);
    assert(x.hi == 0x00008000 && x.lo == 0);
}

void
test_idiv64(void)
{
    int64 x, y;

    assert(i64toi(idiv64(itoi64(0), 1)) == 0);
    assert(i64toi(idiv64(itoi64(42), 2)) == 21);
    assert(i64toi(idiv64(itoi64(-42), 2)) == -21);
    assert(i64toi(idiv64(itoi64(42), -2)) == -21);
    assert(i64toi(idiv64(itoi64(-42), -2)) ==  21);
    assert(i64toi(idiv64(itoi64(7), 3)) == 2);
    assert(i64toi(idiv64(itoi64(-7), 3)) == -2);

    x.hi = 0x40000000;
    x.lo = 0;
    y = idiv64(x, 2);
    assert(y.hi == 0x20000000 && y.lo == 0);

    x.hi = 0x7FFFFFFF;
    x.lo = 0;
    y = idiv64(x, 2);
    assert(y.hi == 0x3FFFFFFF && y.lo == 0x80000000);
}

int
main()
{
    test_itoi64();
    test_i64toi();
    test_constants();
    test_icompare64();
    test_ineg64();
    test_ishift64();
    test_iadd64();
    test_isub64();
    test_imul64();
    test_idiv64();

    printf("passed.\n");

    return EXIT_SUCCESS;
}
